Package: diagFDR
Title: Verifiable FDR Diagnostics for Proteomics
Version: 0.1.1
Authors@R: c(person("Quentin", "Giai Gianetto", email = "quentin.giaigianetto@pasteur.fr", role = c("aut", "cre")))
Description: Provides methods to compute verifiable false discovery rate (FDR) diagnostic checks for workflows based on target-decoy competition and related confidence measures. Implements calibration, stability and tail diagnostics, including tail support, threshold elasticity, posterior error probability (PEP) reliability, and equal-chance checks. If you used this package in your research, please cite the associated preprint <doi:10.64898/2026.04.16.718468>. Detailed examples of using this package can also be found on the GitHub repository (<https://github.com/Jacky11/diagFDR>).
URL: https://www.biorxiv.org/content/10.64898/2026.04.16.718468
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.2.0)
Imports: dplyr, tibble, ggplot2, purrr, rlang, readr, xml2, cp4p,
        tidyr, scales, data.table
Suggests: arrow, knitr, officer, rvg, testthat (>= 3.0.0), withr,
        rmarkdown, jsonlite
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-27 12:05:13 UTC; qgiaigia
Author: Quentin Giai Gianetto [aut, cre]
Maintainer: Quentin Giai Gianetto <quentin.giaigianetto@pasteur.fr>
Repository: CRAN
Date/Publication: 2026-04-27 18:30:08 UTC
Built: R 4.6.0; ; 2026-05-01 22:43:25 UTC; unix

Package: diagFDR
Title: Verifiable FDR Diagnostics for Proteomics
Version: 0.1.1
Authors@R: c(person("Quentin", "Giai Gianetto", email = "quentin.giaigianetto@pasteur.fr", role = c("aut", "cre")))
Description: Provides methods to compute verifiable false discovery rate (FDR) diagnostic checks for workflows based on target-decoy competition and related confidence measures. Implements calibration, stability and tail diagnostics, including tail support, threshold elasticity, posterior error probability (PEP) reliability, and equal-chance checks. If you used this package in your research, please cite the associated preprint <doi:10.64898/2026.04.16.718468>. Detailed examples of using this package can also be found on the GitHub repository (<https://github.com/Jacky11/diagFDR>).
URL: https://www.biorxiv.org/content/10.64898/2026.04.16.718468
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.2.0)
Imports: dplyr, tibble, ggplot2, purrr, rlang, readr, xml2, cp4p,
        tidyr, scales, data.table
Suggests: arrow, knitr, officer, rvg, testthat (>= 3.0.0), withr,
        rmarkdown, jsonlite
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-27 12:05:13 UTC; qgiaigia
Author: Quentin Giai Gianetto [aut, cre]
Maintainer: Quentin Giai Gianetto <quentin.giaigianetto@pasteur.fr>
Repository: CRAN
Date/Publication: 2026-04-27 18:30:08 UTC
Built: R 4.5.2; ; 2026-04-27 21:24:49 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,dfdr_tbl)
export(as_dfdr_tbl)
export(dfdr_bh_diagnostics)
export(dfdr_bh_elasticity)
export(dfdr_curve_stability)
export(dfdr_elasticity)
export(dfdr_equal_chance_qbands)
export(dfdr_headline)
export(dfdr_local_tail)
export(dfdr_p_calibration)
export(dfdr_pep_decoy_sanity)
export(dfdr_pep_reliability)
export(dfdr_pep_reliability_tdc)
export(dfdr_pi0_storey)
export(dfdr_plot_bh_elasticity)
export(dfdr_plot_cv)
export(dfdr_plot_dalpha)
export(dfdr_plot_dwin)
export(dfdr_plot_elasticity)
export(dfdr_plot_equal_chance)
export(dfdr_plot_fdrhat_pi0)
export(dfdr_plot_p_calibration)
export(dfdr_plot_p_calibration2)
export(dfdr_plot_p_density_by_decoy)
export(dfdr_plot_pep_density_by_decoy)
export(dfdr_plot_pep_reliability)
export(dfdr_plot_pep_reliability_tdc)
export(dfdr_plot_pi0)
export(dfdr_plot_scope_disagreement_matrix)
export(dfdr_plot_score_distributions)
export(dfdr_render_report)
export(dfdr_run_all)
export(dfdr_run_jaccard)
export(dfdr_scope_disagreement)
export(dfdr_summary_headline)
export(dfdr_sumpep)
export(dfdr_write_manifest)
export(dfdr_write_readme)
export(dfdr_write_report)
export(diann_global_minrunq)
export(diann_global_precursor)
export(diann_runxprecursor)
export(flag_headline)
export(mokapot_competed_universe)
export(read_dfdr_maxquant_msms)
export(read_dfdr_mzid)
export(read_diann_parquet)
export(read_mokapot_psms)
export(read_spectronaut)
export(read_spectronaut_efficient)
export(score_to_pvalue)
export(spectronaut_runxprecursor)
importFrom(dplyr,filter)
importFrom(rlang,.data)
importFrom(rlang,abort)
importFrom(tibble,tibble)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(xml2,read_xml)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_parent)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5
)

## ----toy----------------------------------------------------------------------
library(diagFDR)

set.seed(10)

n <- 8000
toy <- data.frame(
  id = as.character(seq_len(n)),
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.98, 0.02)),
  run = sample(paste0("run", 1:3), n, replace = TRUE),
  score = c(rnorm(n * 0.98, mean = 7, sd = 1), rnorm(n * 0.02, mean = 5, sd = 1))[seq_len(n)],
  pep = NA_real_
)

# Reconstruct q-values by simple TDC from score (for toy data only):
# Here we mimic a typical "higher score is better" setting.
toy <- toy[order(toy$score, decreasing = TRUE), ]
toy$D_cum <- cumsum(toy$is_decoy)
toy$T_cum <- cumsum(!toy$is_decoy)
toy$FDR_hat <- (toy$D_cum + 1) / pmax(toy$T_cum, 1)
toy$q <- rev(cummin(rev(toy$FDR_hat)))
toy <- toy[, c("id","is_decoy","q","pep","run","score")]

x_toy <- as_dfdr_tbl(
  toy,
  unit = "psm",
  scope = "global",
  q_source = "toy TDC from score",
  q_max_export = 1,
  provenance = list(tool = "toy")
)

diag <- dfdr_run_all(xs = list(MaxQuant_PSM = x_toy), alpha_main = 0.01)

## ----headline-----------------------------------------------------------------
diag$tables$headline

## ----stability-plots----------------------------------------------------------
diag$plots$dalpha
diag$plots$cv

## ----boundary-stability-------------------------------------------------------
diag$plots$dwin
diag$plots$elasticity

## ----equal-chance-------------------------------------------------------------
diag$tables$equal_chance_pooled
diag$plots$equal_chance__MaxQuant_PSM

## ----real-maxquant, eval=FALSE------------------------------------------------
# library(diagFDR)
# 
# mq_path <- "path/to/msms.txt"
# 
# # Read msms.txt and reconstruct TDC q-values using MaxQuant Score and Reverse indicator.
# # - Reverse == "+" is treated as a decoy indicator.
# # - Score is assumed "higher is better".
# # - q-values are computed using FDR(i) = (D(i)+add_decoy)/T(i) and q(i)=min_{j>=i} FDR(j).
# x_mq <- read_dfdr_maxquant_msms(
#   path = mq_path,
#   pep_mode = "sanitize",          # or "drop" if PEP contains values >1
#   exclude_contaminants = TRUE,
#   add_decoy = 1L,
#   unit = "psm",
#   scope = "global",
#   provenance = list(tool = "MaxQuant", file = basename(mq_path))
# )
# 
# # Run diagnostics
# diag <- dfdr_run_all(
#   xs = list(MaxQuant_PSM = x_mq),
#   alpha_main = 0.01,
#   alphas = c(1e-4, 5e-4, 1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
#   eps = 0.2,
#   win_rel = 0.2,
#   truncation = "warn_drop",
#   low_conf = c(0.2, 0.5)
# )
# 
# # Inspect headline diagnostics
# diag$tables$headline
# 
# # Export tables/plots + human-readable summary
# dfdr_write_report(
#   diag,
#   out_dir = "diagFDR_maxquant_out",
#   formats = c("csv", "png", "manifest", "readme", "summary")
# )
# 
# # Optional: render a single HTML report (requires rmarkdown in Suggests)
# dfdr_render_report(diag, out_dir = "diagFDR_maxquant_out")
# 
# 
# library(diagFDR)
# 
# mq_path <- "path/to/msms.txt"
# 
# # Read msms.txt and reconstruct TDC q-values
# x_mq <- read_dfdr_maxquant_msms(
#   path = mq_path,
#   pep_mode = "sanitize",
#   exclude_contaminants = TRUE,
#   add_decoy = 1L,
#   unit = "psm",
#   scope = "global",
#   provenance = list(tool = "MaxQuant", file = basename(mq_path))
# )
# 
# # Run diagnostics with automatic pseudo p-value computation
# # This will use score_to_pvalue(method="decoy_ecdf") internally
# diag <- dfdr_run_all(
#   xs = list(MaxQuant_PSM = x_mq),
#   alpha_main = 0.01,
#   compute_pseudo_pvalues = TRUE,   # generates pseudo p-values from scores
#   pseudo_pvalue_method = "decoy_ecdf",  # Most defensible method for arbitrary scores
#   p_stratify = "run"  # Optional: stratify p-value diagnostics by run
# )
# 
# # diag will contain:
# # - All standard target-decoy diagnostics
# # - PLUS p-value calibration plots and tables
# # - PLUS Storey pi0 diagnostics
# # - PLUS BH comparison diagnostics
# 
# # Inspect headline + p-value calibration
# diag$tables$headline
# diag$tables$p_calibration_summary
# 
# # Export everything
# dfdr_write_report(
#   diag,
#   out_dir = "diagFDR_maxquant_out",
#   formats = c("csv", "png", "manifest", "readme", "summary")
# )
# 
# # Render HTML report (will include p-value diagnostics section)
# dfdr_render_report(diag, out_dir = "diagFDR_maxquant_out")

## ----pseudo-p, eval=FALSE-----------------------------------------------------
# # Create pseudo-p-values from the decoy score tail and rerun diagnostics with p-value checks.
# x_mq$p <- score_to_pvalue(
#   score = x_mq$score,
#   method = "decoy_ecdf",
#   is_decoy = x_mq$is_decoy
# )
# attr(x_mq, "meta")$p_source <- "score_to_pvalue(method='decoy_ecdf' on MaxQuant Score)"
# 
# diag_p <- dfdr_run_all(
#   xs = list(MaxQuant_PSM = x_mq),
#   alpha_main = 0.01,
#   p_stratify = "run"   # optional stratification if run column is meaningful
# )
# 
# dfdr_write_report(diag_p, out_dir = "diagFDR_maxquant_out_with_p", formats = c("csv","png","manifest","readme","summary"))
# dfdr_render_report(diag_p, out_dir = "diagFDR_maxquant_out_with_p")

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5
)

## ----toy----------------------------------------------------------------------
library(diagFDR)

set.seed(2)

n <- 6000
toy <- data.frame(
  id = paste0("run1||scan", seq_len(n)),
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.98, 0.02)),
  q = pmin(1, runif(n)^3),                                  # many small q-values
  pep = pmin(1, pmax(0, runif(n)^3 + rnorm(n, sd = 0.02))), # correlated toy PEP
  run = "run1",
  score = rnorm(n)
)

x <- as_dfdr_tbl(
  toy,
  unit = "psm",
  scope = "global",
  q_source = "toy mokapot",
  q_max_export = 1
)

diag <- dfdr_run_all(
  xs = list(mokapot = x),
  alpha_main = 0.01,
  alphas = c(5e-4, 1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
  low_conf = c(0.2, 0.5)
)

## ----headline-----------------------------------------------------------------
diag$tables$headline

## ----stability-plots----------------------------------------------------------
diag$plots$dalpha
diag$plots$cv

## ----boundary-stability-------------------------------------------------------
diag$plots$dwin
diag$plots$elasticity

## ----equal-chance-------------------------------------------------------------
diag$tables$equal_chance_pooled
diag$plots$equal_chance__mokapot

## ----pep----------------------------------------------------------------------
diag$tables$pep_IPE
diag$plots$pep_reliability__mokapot

## ----sumpep-------------------------------------------------------------------
# sumpep is a list-of-tibbles (one per list)
diag$tables$sumpep$mokapot

## ----real-mokapot, eval=FALSE-------------------------------------------------
# library(diagFDR)
# 
# # Read mokapot outputs (targets + decoys)
# raw <- read_mokapot_psms(
#   target_path = "path/to/your_output.mokapot.psms.txt",
#   decoy_path  = "path/to/your_output.mokapot.decoy.psms.txt"
# )
# 
# # Construct competed winners (1 per run+spectrum_id; max mokapot score)
# x <- mokapot_competed_universe(
#   raw,
#   id_mode = "runxid",
#   unit = "psm",
#   scope = "global",
#   q_source = "mokapot q-value",
#   q_max_export = 1
# )
# 
# # Run diagnostics
# diag <- dfdr_run_all(
#   xs = list(mokapot = x),
#   alpha_main = 0.01,
#   alphas = c(5e-4, 1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
#   low_conf = c(0.2, 0.5)
# )
# 
# # Write outputs to disk (tables + plots + summary + manifest + README)
# dfdr_write_report(
#   diag,
#   out_dir = "diagFDR_mokapot_out",
#   formats = c("csv", "png", "manifest", "readme", "summary")
# )
# 
# # Render a single HTML report (requires rmarkdown in Suggests)
# dfdr_render_report(diag, out_dir = "diagFDR_mokapot_out")

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5
)

## ----toy-data-----------------------------------------------------------------
library(diagFDR)

set.seed(1)

n <- 3000
toy_global <- data.frame(
  id = paste0("P", seq_len(n)),
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.97, 0.03)),
  q = pmin(1, runif(n)^3),       # skew toward small q-values
  pep = NA_real_,
  run = NA_character_,
  score = NA_real_
)

x_global <- as_dfdr_tbl(
  toy_global,
  unit = "precursor",
  scope = "global",
  q_source = "toy",
  q_max_export = 0.5
)

diag <- dfdr_run_all(
  xs = list(global = x_global),
  alpha_main = 0.01,
  alphas = c(1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
  low_conf = c(0.2, 0.5)
)

## ----headline-----------------------------------------------------------------
diag$tables$headline

## ----plots-stability----------------------------------------------------------
diag$plots$dalpha
diag$plots$cv

## ----plot-dwin----------------------------------------------------------------
diag$plots$dwin

## ----plot-elasticity----------------------------------------------------------
diag$plots$elasticity

## ----equal-chance-------------------------------------------------------------
diag$tables$equal_chance_pooled
diag$plots$equal_chance__global

## ----real-diann, eval=FALSE---------------------------------------------------
# # Requires arrow
# rep <- read_diann_parquet("path/to/report.parquet")
# 
# # (A) Global precursor list using Global.Q.Value
# # Recommended for experiment-wide (pooled) lists.
# x_global_gq <- diann_global_precursor(
#   rep,
#   q_col = "Global.Q.Value",
#   q_max_export = 0.5,
#   unit = "precursor",
#   scope = "global",
#   q_source = "Global.Q.Value"
# )
# 
# # (B) Run×precursor universe using run-wise Q.Value
# # Recommended for per-run decisions / QC.
# x_runx <- diann_runxprecursor(
#   rep,
#   q_col = "Q.Value",
#   q_max_export = 0.5,
#   id_mode = "runxid",
#   unit = "runxprecursor",
#   scope = "runwise",
#   q_source = "Q.Value"
# )
# 
# # (C) Scope misuse comparator: min run-wise q over runs per precursor (anti-pattern)
# # Useful for demonstrating/diagnosing scope mismatch.
# x_minrun <- diann_global_minrunq(
#   rep,
#   q_col = "Q.Value",
#   q_max_export = 0.5,
#   unit = "precursor",
#   scope = "aggregated",
#   q_source = "min_run(Q.Value)"
# )
# 
# diag <- dfdr_run_all(
#   xs = list(global = x_global_gq, runx = x_runx, minrun = x_minrun),
#   alpha_main = 0.01,
#   compute_pseudo_pvalues = TRUE  # <-- This adds p-value diagnostics
# )
# 
# # Compare accepted lists across scopes (Jaccard overlap across alpha)
# scope_tbl <- dfdr_scope_disagreement(
#   x1 = x_global_gq,
#   x2 = x_minrun,
#   alphas = c(1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2),
#   label1 = "Global.Q.Value",
#   label2 = "min_run(Q.Value)"
# )
# 
# # Write outputs to disk (tables + plots; optionally PPTX)
# dfdr_write_report(diag, out_dir = "diagFDR_diann_out", formats = c("csv", "png", "manifest", "readme", "summary"))
# 
# # Render a single HTML report (requires rmarkdown in Suggests)
# dfdr_render_report(diag, out_dir = "diagFDR_diann_out")

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5
)

## ----toy----------------------------------------------------------------------
library(diagFDR)

set.seed(3)

n <- 7000
pi_decoy <- 0.03

# Decoy indicator
is_decoy <- sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(1 - pi_decoy, pi_decoy))

# Targets are a mixture: some null-like (close to decoys), some true (higher score)
# This yields realistic separation and non-trivial discoveries at 1% FDR.
is_true_target <- (!is_decoy) & (runif(n) < 0.30)  # 30% of targets are "true"
is_null_target <- (!is_decoy) & (!is_true_target)

score <- numeric(n)
score[is_decoy]       <- rnorm(sum(is_decoy), mean = 0.0, sd = 1.0)
score[is_null_target] <- rnorm(sum(is_null_target), mean = 0.2, sd = 1.0)
score[is_true_target] <- rnorm(sum(is_true_target), mean = 3.0, sd = 1.0)

toy <- data.frame(
  id = paste0("psm", seq_len(n)),
  is_decoy = is_decoy,
  run = sample(paste0("run", 1:4), n, replace = TRUE),
  score = score,
  pep = NA_real_
)

# Score-based TDC q-values (higher score is better)
toy <- toy[order(toy$score, decreasing = TRUE), ]
toy$D_cum <- cumsum(toy$is_decoy)
toy$T_cum <- cumsum(!toy$is_decoy)
toy$FDR_hat <- (toy$D_cum + 1) / pmax(toy$T_cum, 1)
toy$q <- rev(cummin(rev(toy$FDR_hat)))
toy <- toy[, c("id","is_decoy","q","pep","run","score")]

x_toy <- as_dfdr_tbl(
  toy,
  unit = "psm",
  scope = "global",
  q_source = "toy TDC from score",
  q_max_export = 1,
  provenance = list(tool = "toy")
)

diag <- dfdr_run_all(
  xs = list(mzid_PSM = x_toy),
  alpha_main = 0.01,
  alphas = c(1e-4, 5e-4, 1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
  eps = 0.2,
  win_rel = 0.2,
  truncation = "warn_drop",
  low_conf = c(0.2, 0.5)
)

## ----headline-----------------------------------------------------------------
diag$tables$headline

if (nrow(diag$tables$headline) > 0 && diag$tables$headline$T_alpha[1] == 0) {
  cat("\nNote: No discoveries at alpha_main for this toy run. ",
      "For demonstration, consider using alpha_main = 0.02.\n", sep = "")
}

## ----stability-plots----------------------------------------------------------
diag$plots$dalpha
diag$plots$cv

## ----boundary-stability-------------------------------------------------------
diag$plots$dwin
diag$plots$elasticity

## ----equal-chance-------------------------------------------------------------
diag$tables$equal_chance_pooled
diag$plots$equal_chance__mzid_PSM

## ----real-mzid, eval=FALSE----------------------------------------------------
# library(diagFDR)
# 
# mzid_path <- "path/to/search_results.mzid"
# 
# x_mzid <- read_dfdr_mzid(
#   mzid_path = mzid_path,
#   rank = 1L,  # competed universe: take rank-1 SpectrumIdentificationItem
# 
#   # Choose a score CV term (priority list) and interpret its direction
#   score_accession_preference = c(
#     "MS:1002257", # MS-GF:RawScore (higher is better)
#     "MS:1001330", # Mascot:score (higher is better)
#     "MS:1001328", # SEQUEST:xcorr (higher is better)
#     "MS:1002052",
#     "MS:1002049",
#     "MS:1001331",
#     "MS:1001171",
#     "MS:1001950",
#     "MS:1002466"
#   ),
#   score_direction = "auto",  # or "higher_better"/"lower_better" if auto fails
# 
#   # TDC correction: FDR_hat = (D + add_decoy)/T
#   add_decoy = 1L,
# 
#   # Strict by default: require score for all PSMs (set <1 to allow missing)
#   min_score_coverage = 1.0,
# 
#   # Fallback decoy inference if PeptideEvidence@isDecoy is not informative
#   decoy_regex = "(^##|_REVERSED$|^REV_|^DECOY_)",
# 
#   unit = "psm",
#   scope = "global",
#   provenance = list(file = basename(mzid_path))
# )
# 
# diag <- dfdr_run_all(
#   xs = list(mzid_PSM = x_mzid),
#   alpha_main = 0.01,
#   alphas = c(1e-4, 5e-4, 1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
#   eps = 0.2,
#   win_rel = 0.2,
#   truncation = "warn_drop",
#   low_conf = c(0.2, 0.5)
# )
# 
# # Export outputs
# dfdr_write_report(
#   diag,
#   out_dir = "diagFDR_mzid_out",
#   formats = c("csv", "png", "manifest", "readme", "summary")
# )
# 
# # Render a single HTML report (requires rmarkdown in Suggests)
# dfdr_render_report(diag, out_dir = "diagFDR_mzid_out")

## ----pseudo-p, eval=FALSE-----------------------------------------------------
# x_mzid$p <- score_to_pvalue(
#   score = x_mzid$score,
#   method = "decoy_ecdf",
#   is_decoy = x_mzid$is_decoy
# )
# attr(x_mzid, "meta")$p_source <- "score_to_pvalue(method='decoy_ecdf' on mzid score)"
# 
# diag_p <- dfdr_run_all(
#   xs = list(mzid_PSM = x_mzid),
#   alpha_main = 0.01
# )
# 
# dfdr_write_report(
#   diag_p,
#   out_dir = "diagFDR_mzid_out_with_p",
#   formats = c("csv","png","manifest","readme","summary")
# )
# dfdr_render_report(diag_p, out_dir = "diagFDR_mzid_out_with_p")

#' Internal: assert required columns exist
#'
#' Checks that a data frame contains the required column names.
#'
#' @param x A data.frame / tibble.
#' @param cols Character vector of required column names.
#'
#' @return
#' Returns \code{invisible(TRUE)} if all columns are present; otherwise throws an error.
#'
#' @keywords internal
.check_has_cols <- function(x, cols) {
  miss <- setdiff(cols, colnames(x))
  if (length(miss)) rlang::abort(paste0("Missing required column(s): ", paste(miss, collapse = ", ")))
}

#' Internal: validate data-frame input
#'
#' Asserts that \code{x} is a data.frame (including tibble). Used to provide a
#' consistent error message for user-facing functions that accept data frames.
#'
#' @param x Object to check.
#'
#' @return
#' Returns \code{x} unchanged if it is a data.frame; otherwise throws an error.
#'
#' @keywords internal
.safe_df <- function(x) {
  if (!inherits(x, "data.frame")) rlang::abort("`x` must be a data.frame / tibble (preferably dfdr_tbl).")
  x
}

#' Internal: extract stratification columns
#'
#' Returns the subset of \code{x} corresponding to columns named in \code{stratify}.
#' Used by stratified diagnostics.
#'
#' @param x A data.frame / tibble.
#' @param stratify Optional character vector of column names used for stratification.
#'
#' @return
#' If \code{stratify} is \code{NULL} or empty, returns \code{NULL}. Otherwise
#' returns a data.frame with the selected stratification columns (in the order
#' given by \code{stratify}). Throws an error if any requested columns are missing.
#'
#' @keywords internal
.get_strata_df <- function(x, stratify = NULL) {
  if (is.null(stratify) || length(stratify) == 0) return(NULL)
  if (!all(stratify %in% names(x))) {
    rlang::abort(paste0("Unknown `stratify` column(s): ", paste(setdiff(stratify, names(x)), collapse = ", ")))
  }
  x[stratify]
}

#' Internal: build stratum identifiers
#'
#' Builds a human-readable stratum identifier per row from a data.frame of
#' stratification columns. Each identifier is of the form
#' \code{"col1=... | col2=..."}.
#'
#' @param strata_df A data.frame of stratification columns (typically produced by
#'   \code{.get_strata_df}). If \code{NULL}, a single stratum \code{"all"} is used.
#'
#' @return
#' A character vector of stratum identifiers (length \code{nrow(strata_df)} if not
#' \code{NULL}, otherwise \code{"all"}).
#'
#' @keywords internal
.make_stratum_id <- function(strata_df) {
  if (is.null(strata_df)) return(rep("all", 1))
  apply(strata_df, 1, function(row) {
    paste(paste0(names(strata_df), "=", ifelse(is.na(row), "NA", as.character(row))), collapse = " | ")
  })
}

#' Internal: Jaccard similarity of two sets
#'
#' Computes the Jaccard index \eqn{|A \cap B| / |A \cup B|} on unique values.
#' Returns 1 when both sets are empty.
#'
#' @param a Vector representing set A.
#' @param b Vector representing set B.
#'
#' @return Numeric scalar in \eqn{[0,1]} giving the Jaccard similarity.
#'
#' @keywords internal
.jaccard <- function(a, b) {
  a <- unique(a); b <- unique(b)
  if (length(a) == 0 && length(b) == 0) return(1)
  length(intersect(a, b)) / length(union(a, b))
}

#' Internal: Benjamini--Hochberg rejection threshold
#'
#' Computes the BH cutoff \eqn{t_\alpha} for a vector of p-values \code{p} at level
#' \code{alpha}. Returns \code{NA} if no hypotheses are rejected (or no finite p-values).
#'
#' @param p Numeric vector of p-values (only finite values are used).
#' @param alpha Numeric scalar in \eqn{(0,1)}.
#'
#' @return Numeric scalar BH threshold \eqn{t_\alpha}, or \code{NA_real_} if no
#'   rejection threshold exists.
#'
#' @keywords internal
.bh_threshold <- function(p, alpha) {
  ok <- is.finite(p)
  p <- p[ok]
  m <- length(p)
  if (m == 0) return(NA_real_)
  o <- order(p)
  p_sorted <- p[o]
  crit <- (seq_len(m) / m) * alpha
  idx <- which(p_sorted <= crit)
  if (length(idx) == 0) return(NA_real_)
  p_sorted[max(idx)]
}

utils::globalVariables(c(
  ".data",
  # common column names referenced non-standardly
  "alpha","D_alpha","CV_hat","D_alpha_win","jaccard",
  "pep","pep_mean","decoy_rate","decoy_frac","q_mean","qbin",
  "n","n_decoy","bin",
  "id","run","score","spectrum_id","precursor","is_target","is_decoy"
))

#' Flag diagnostic values based on heuristic thresholds
#'
#' Adds simple, ASCII-only flags and a coarse overall status to a headline
#' diagnostics table (typically produced by \code{\link{dfdr_headline}} or the
#' \code{headline} table inside \code{\link{dfdr_run_all}}). The heuristics are
#' intended for quick triage in reports; they do not replace manual review.
#'
#' Rendering as icons (if desired) can be handled downstream (e.g. in an HTML
#' template); this function returns plain text labels for portability.
#'
#' @param headline_tbl A data.frame/tibble of headline diagnostics. It may contain
#'   one or multiple rows. If some expected columns are missing, they are created
#'   and filled with \code{NA}.
#'
#' @return
#' A \link[tibble:tibble]{tibble} (or data frame) with the same rows as
#' \code{headline_tbl} and additional columns:
#' \describe{
#'   \item{flag_Dalpha}{Flag based on \code{D_alpha} (decoy support).}
#'   \item{flag_CV}{Flag based on \code{CV_hat} (stability/variability).}
#'   \item{flag_Dwin}{Flag based on \code{D_alpha_win} (local tail support).}
#'   \item{flag_IPE}{Flag based on \code{IPE} (internal PEP calibration error).}
#'   \item{flag_FDR}{Flag comparing \code{FDR_hat} to the nominal \code{alpha}.}
#'   \item{flag_equalchance}{Flag based on \code{effect_abs} (equal-chance deviation).}
#'   \item{status}{Overall triage status: \code{"VALID"}, \code{"CAUTION"}, or
#'   \code{"REVIEW_NEEDED"}.}
#'   \item{interpretation}{A short human-readable interpretation string.}
#' }
#'
#' @examples
#' library(tibble)
#'
#' headline_tbl <- tibble(
#'   list = c("A", "B"),
#'   alpha = 0.01,
#'   D_alpha = c(5, 80),
#'   CV_hat = c(0.35, 0.10),
#'   D_alpha_win = c(2, 30),
#'   IPE = c(0.12, 0.02),
#'   FDR_hat = c(0.02, 0.009),
#'   effect_abs = c(0.18, 0.05)
#' )
#'
#' flag_headline(headline_tbl)
#'
#' @export
flag_headline <- function(headline_tbl) {
  if (!is.data.frame(headline_tbl)) {
    rlang::abort("`headline_tbl` must be a data.frame/tibble.")
  }

  # Ensure required columns exist (some diagnostics are optional)
  ensure_col <- function(df, nm, default = NA_real_) {
    if (!nm %in% names(df)) df[[nm]] <- default
    df
  }

  headline_tbl <- headline_tbl |>
    ensure_col("D_alpha", NA_real_) |>
    ensure_col("CV_hat", NA_real_) |>
    ensure_col("D_alpha_win", NA_real_) |>
    ensure_col("IPE", NA_real_) |>
    ensure_col("FDR_hat", NA_real_) |>
    ensure_col("alpha", NA_real_) |>
    ensure_col("effect_abs", NA_real_)

  headline_tbl |>
    dplyr::mutate(
      flag_Dalpha = dplyr::case_when(
        is.na(D_alpha) ~ "NA",
        D_alpha < 10 ~ "PROBLEM_granular",
        D_alpha < 50 ~ "CAUTION",
        TRUE ~ "OK"
      ),
      flag_CV = dplyr::case_when(
        is.na(CV_hat) ~ "NA",
        CV_hat > 0.30 ~ "PROBLEM_granular",
        CV_hat > 0.15 ~ "CAUTION",
        TRUE ~ "OK"
      ),
      flag_Dwin = dplyr::case_when(
        is.na(D_alpha_win) ~ "NA",
        D_alpha_win < 5 ~ "PROBLEM_very_sparse",
        D_alpha_win < 20 ~ "CAUTION_sparse",
        TRUE ~ "OK"
      ),
      flag_IPE = dplyr::case_when(
        is.na(IPE) ~ "NA",
        IPE > 0.10 ~ "PROBLEM_serious",
        IPE > 0.05 ~ "CAUTION",
        TRUE ~ "OK"
      ),
      flag_FDR = dplyr::case_when(
        is.na(FDR_hat) ~ "NA",
        is.na(alpha) ~ "NA",
        FDR_hat > alpha * 1.5 ~ "PROBLEM_inflated",
        FDR_hat > alpha * 1.2 ~ "CAUTION_slightly_high",
        TRUE ~ "OK"
      ),
      flag_equalchance = dplyr::case_when(
        is.na(effect_abs) ~ "NA",
        effect_abs > 0.15 ~ "PROBLEM_imbalance",
        effect_abs > 0.10 ~ "CAUTION",
        TRUE ~ "OK"
      ),

      status = dplyr::case_when(
        grepl(
          "PROBLEM",
          paste(flag_Dalpha, flag_CV, flag_Dwin, flag_IPE, flag_FDR, flag_equalchance)
        ) ~ "REVIEW_NEEDED",
        grepl(
          "CAUTION",
          paste(flag_Dalpha, flag_CV, flag_Dwin, flag_IPE, flag_FDR, flag_equalchance)
        ) ~ "CAUTION",
        TRUE ~ "VALID"
      ),

      interpretation = dplyr::case_when(
        flag_FDR == "PROBLEM_inflated" ~ "Empirical FDR exceeds nominal threshold - likely scope misuse",
        flag_Dalpha == "PROBLEM_granular" & flag_CV == "PROBLEM_granular" ~ "Sparse decoy support - threshold potentially unstable",
        flag_IPE == "PROBLEM_serious" ~ "PEPs seriously miscalibrated",
        flag_equalchance == "PROBLEM_imbalance" ~ "Equal-chance assumption potentially violated - check if selection effects were used (two-pass learning, etc.)",
        status == "VALID" ~ "No major issues detected",
        TRUE ~ "Minor issues - review flagged metrics"
      )
    )
}

#' BH list elasticity (Jaccard) across alpha values
#'
#' Computes the stability (elasticity) of Benjamini--Hochberg (BH) discovery sets
#' under a multiplicative perturbation of the nominal FDR level. For each
#' \code{alpha} in \code{alphas}, the function compares the BH discovery set at
#' \code{alpha} to the discovery set at \code{(1+eps)*alpha} using the Jaccard
#' index.
#'
#' @param x A \code{dfdr_tbl} (or data frame) with columns \code{id} and \code{p}.
#'   Only finite \code{p}-values are used.
#' @param alphas Numeric vector of BH levels in \eqn{(0,1)}.
#' @param eps Numeric scalar \eqn{\epsilon > 0}. Relative perturbation (default
#'   0.2), i.e. compares \code{alpha} vs \code{(1+eps)*alpha}.
#'
#' @return
#' A \link[tibble:tibble]{tibble} with one row per element of \code{alphas}. Columns include:
#' \describe{
#'   \item{alpha}{The nominal BH level.}
#'   \item{alpha_perturbed}{The perturbed level \code{min((1+eps)*alpha, 1)}.}
#'   \item{t_alpha}{BH rejection threshold at \code{alpha} (NA if no rejections).}
#'   \item{t_alpha_perturbed}{BH rejection threshold at \code{alpha_perturbed} (NA if none).}
#'   \item{R_alpha}{Number of discoveries at \code{alpha}.}
#'   \item{R_alpha_perturbed}{Number of discoveries at \code{alpha_perturbed}.}
#'   \item{jaccard}{Jaccard similarity between the two discovery ID sets.}
#' }
#' This output is intended to quantify how sensitive BH discoveries are to small
#' changes in the chosen FDR level.
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 5000
#' x <- tibble(
#'   id = as.character(seq_len(n)),
#'   # mixture: mostly null p-values + a small enriched component
#'   p = c(stats::runif(4500), stats::rbeta(500, 0.3, 1))
#' )
#'
#' dfdr_bh_elasticity(x, alphas = c(1e-3, 5e-3, 1e-2, 2e-2), eps = 0.2)
#'
#' @export
dfdr_bh_elasticity <- function(x,
                               alphas = c(1e-04, 5e-04, 0.001, 0.002, 0.005, 0.01, 0.02, 0.05),
                               eps = 0.2) {
  x <- .safe_df(x)
  .check_has_cols(x, c("id", "p"))

  if (!is.numeric(alphas) || any(!is.finite(alphas)) || any(alphas <= 0) || any(alphas >= 1)) {
    rlang::abort("`alphas` must be finite values in (0,1).")
  }
  if (!is.numeric(eps) || length(eps) != 1 || !is.finite(eps) || eps <= 0) {
    rlang::abort("`eps` must be a single positive number.")
  }

  ok <- is.finite(x$p)
  p <- x$p[ok]
  ids <- x$id[ok]

  out <- vector("list", length(alphas))

  for (i in seq_along(alphas)) {
    a <- alphas[i]
    a2 <- min((1 + eps) * a, 1 - 1e-12)

    t1 <- .bh_threshold(p, a)
    t2 <- .bh_threshold(p, a2)

    A1 <- if (is.finite(t1)) ids[p <= t1] else character()
    A2 <- if (is.finite(t2)) ids[p <= t2] else character()

    out[[i]] <- tibble::tibble(
      alpha = a,
      alpha_perturbed = a2,
      t_alpha = if (is.finite(t1)) t1 else NA_real_,
      t_alpha_perturbed = if (is.finite(t2)) t2 else NA_real_,
      R_alpha = as.integer(length(A1)),
      R_alpha_perturbed = as.integer(length(A2)),
      jaccard = .jaccard(A1, A2)
    )
  }

  dplyr::bind_rows(out)
}


#' Plot BH elasticity (Jaccard) vs alpha
#'
#' Convenience plotting function for the output of \code{\link{dfdr_bh_elasticity}}.
#' The x-axis is \code{log10(alpha)} and the y-axis is the Jaccard overlap between
#' BH discovery sets at \code{alpha} and \code{(1+eps)*alpha}.
#'
#' @param el_tbl A tibble as returned by \code{\link{dfdr_bh_elasticity}}.
#' @param xlab Character. X-axis label.
#' @param title Character. Plot title.
#'
#' @return
#' A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 2000
#' x <- tibble(
#'   id = as.character(seq_len(n)),
#'   p = c(stats::runif(1800), stats::rbeta(200, 0.3, 1))
#' )
#'
#' el <- dfdr_bh_elasticity(x, alphas = c(1e-3, 5e-3, 1e-2, 2e-2), eps = 0.2)
#' p <- dfdr_plot_bh_elasticity(el)
#' p
#'
#' @export
dfdr_plot_bh_elasticity <- function(el_tbl,
                                    xlab = "alpha (log10)",
                                    title = "BH elasticity: Jaccard(alpha, (1+eps)alpha)") {
  .check_has_cols(el_tbl, c("alpha", "jaccard"))
  ggplot2::ggplot(el_tbl, ggplot2::aes(x = log10(alpha), y = jaccard)) +
    ggplot2::geom_line() + ggplot2::geom_point(size = 1.2) +
    ggplot2::labs(x = xlab, y = "Jaccard", title = title) +
    ggplot2::ylim(0, 1)
}

#' Convert identification scores to p-values or pseudo-p-values
#'
#' Converts identification scores to:
#' \itemize{
#'   \item \strong{p-values} when the score has a known p-value definition (e.g. Mascot score),
#'   \item \strong{pseudo-p-values} when no such definition exists (e.g. MaxQuant/Andromeda scores).
#' }
#'
#' Pseudo-p-values are useful as inputs to plausibility checks (e.g. calibration/ECDF diagnostics),
#' but they are not guaranteed to be valid null p-values unless the method is explicitly null-based
#' (e.g. \code{method = "decoy_ecdf"} with a reliable decoy set).
#'
#' @param score Numeric vector. Identification scores. Higher-is-better is assumed for
#'   \code{method = "rank"} and \code{method = "decoy_ecdf"} (flip sign upstream if needed).
#' @param method Character. Conversion method:
#'   \describe{
#'     \item{\code{"mascot"}}{Treat \code{score} as Mascot score \eqn{S=-10\log_{10}(p)}; returns \eqn{p=10^{-S/10}}.}
#'     \item{\code{"neglog10p"}}{Generic \eqn{S=-10\log_{10}(p)}; same transform as \code{"mascot"}.}
#'     \item{\code{"evalue"}}{Treat \code{score} as an expectation/e-value-like quantity and return it as a p-like value.}
#'     \item{\code{"rank"}}{Rank-based pseudo-p-values \eqn{p_i=\mathrm{rank}(-s_i)/(n+1)}.}
#'     \item{\code{"decoy_ecdf"}}{Decoy-ECDF pseudo-p-values from the empirical right-tail of decoy scores;
#'       requires \code{is_decoy}.}
#'   }
#' @param is_decoy Optional logical vector (same length as \code{score}). Required for
#'   \code{method = "decoy_ecdf"}. TRUE=decoy, FALSE=target.
#' @param eps Numeric scalar > 0. Lower bound to avoid exact 0 (default \code{.Machine$double.xmin}).
#' @param clamp Logical. If \code{TRUE} (default), clamp output to \code{[0,1]}.
#' @param ties Character. Tie-handling for \code{method="rank"}; passed to \code{rank()}.
#'
#' @return
#' A numeric vector of p-values/pseudo-p-values of the same length as \code{score}.
#' Non-finite scores yield \code{NA_real_}. For \code{method="decoy_ecdf"}, returned
#' values estimate the decoy right-tail probability \eqn{P(S_{decoy}\ge s)} (with a
#' small +1 smoothing), which can be interpreted as a null-calibrated pseudo-p-value
#' when decoys provide a good null sample.
#'
#' @examples
#' set.seed(1)
#' score <- rnorm(10)
#'
#' # Mascot-like transformation: S = -10 log10(p)
#' S <- c(10, 20, 30)
#' score_to_pvalue(S, method = "mascot")
#'
#' # Rank-based pseudo-p-values (higher score => smaller p)
#' score_to_pvalue(score, method = "rank")
#'
#' # Decoy-ECDF pseudo-p-values (requires decoys)
#' n <- 200
#' score2 <- c(rnorm(n, mean = 0), rnorm(n, mean = 1))  # targets shifted higher
#' is_decoy <- c(rep(TRUE, n), rep(FALSE, n))
#' p2 <- score_to_pvalue(score2, method = "decoy_ecdf", is_decoy = is_decoy)
#' summary(p2)
#'
#' @export
score_to_pvalue <- function(score,
                            method = c("mascot", "neglog10p", "evalue", "rank", "decoy_ecdf"),
                            is_decoy = NULL,
                            eps = .Machine$double.xmin,
                            clamp = TRUE,
                            ties = c("average", "first", "random", "max", "min")) {
  method <- match.arg(method)
  ties <- match.arg(ties)

  if (!is.numeric(score)) rlang::abort("`score` must be numeric.")
  if (!is.numeric(eps) || length(eps) != 1 || !is.finite(eps) || eps <= 0) {
    rlang::abort("`eps` must be a single positive finite number.")
  }
  if (!is.logical(clamp) || length(clamp) != 1) rlang::abort("`clamp` must be TRUE/FALSE.")

  n <- length(score)
  ok <- is.finite(score)

  p <- rep(NA_real_, n)

  if (method %in% c("mascot", "neglog10p")) {
    # p = 10^(-S/10)
    p[ok] <- 10^(-score[ok] / 10)

  } else if (method == "evalue") {
    # p-like monotone mapping
    p[ok] <- score[ok]

  } else if (method == "rank") {
    # rank-based pseudo p-values, best score -> smallest p
    # rank(-score): largest score gets rank 1
    r <- rep(NA_real_, n)
    r[ok] <- rank(-score[ok], ties.method = ties)
    p[ok] <- r[ok] / (sum(ok) + 1)

  } else if (method == "decoy_ecdf") {
    if (is.null(is_decoy)) rlang::abort("`is_decoy` is required for method='decoy_ecdf'.")
    if (!is.logical(is_decoy) || length(is_decoy) != n) {
      rlang::abort("`is_decoy` must be a logical vector of the same length as `score`.")
    }
    if (anyNA(is_decoy)) rlang::abort("`is_decoy` contains NA; please fix upstream.")

    dec_scores <- score[ok & is_decoy]
    if (length(dec_scores) < 50) {
      rlang::abort("Too few finite decoy scores for method='decoy_ecdf' (need at least ~50).")
    }

    # Right-tail empirical survival function:
    # p(s) = P(S_decoy >= s) estimated from decoys
    # Use ranks among decoys to compute tail probability with +1 smoothing.
    dec_sorted <- sort(dec_scores, decreasing = FALSE) # ascending for findInterval
    # For each s, count decoys < s, then tail = (n_dec - count_lt + 1)/(n_dec + 1)
    count_lt <- findInterval(score[ok], dec_sorted, left.open = TRUE)  # number of decoys <=? approx
    n_dec <- length(dec_sorted)
    tail <- (n_dec - count_lt + 1) / (n_dec + 1)
    p[ok] <- tail
  }

  # sanitize numeric issues
  p[is.finite(p)] <- pmax(p[is.finite(p)], eps)
  if (isTRUE(clamp)) p[is.finite(p)] <- pmin(p[is.finite(p)], 1)

  p
}



#' Add pseudo p-values to \code{dfdr_tbl} objects that have scores but no p-values
#'
#' Iterates over a named list of \code{dfdr_tbl} objects and, where a finite \code{p}
#' column is missing but \code{score} and \code{is_decoy} are available, adds a new
#' \code{p} column computed by \code{\link{score_to_pvalue}}. Updates the metadata
#' attribute \code{p_source}.
#'
#' @param xs Named list of \code{dfdr_tbl} objects.
#' @param method Character. Method passed to \code{\link{score_to_pvalue}} (default \code{"decoy_ecdf"}).
#'
#' @return
#' A list of \code{dfdr_tbl} objects of the same length/order as \code{xs}, with a
#' \code{p} column added where applicable.
#'
#' @keywords internal
add_pseudo_pvalues_to_list <- function(xs, method = "decoy_ecdf") {
  purrr::map(xs, function(x) {
    # Skip if already has p-values
    if ("p" %in% names(x) && any(is.finite(x$p))) {
      return(x)
    }

    # Skip if no score column
    if (!"score" %in% names(x) || all(is.na(x$score))) {
      return(x)
    }

    # Compute pseudo p-values
    x$p <- score_to_pvalue(
      score = x$score,
      method = method,
      is_decoy = x$is_decoy
    )

    # Update metadata
    meta <- attr(x, "meta") %||% list()
    meta$p_source <- sprintf("score_to_pvalue(method='%s')", method)
    attr(x, "meta") <- meta

    x
  })
}

#' Internal: robust coercion of mokapot \code{is_target} column to logical
#'
#' @param x Vector containing mokapot \code{is_target} values (logical, numeric,
#'   or character).
#'
#' @return A logical vector of the same length as \code{x}.
#'
#' @keywords internal
mokapot_is_target_to_logical <- function(x) {
  # mokapot writes is_target as TRUE/FALSE (logical) or strings
  if (is.logical(x)) return(x)
  if (is.numeric(x)) return(x == 1)
  z <- tolower(as.character(x))
  z %in% c("true", "t", "1", "yes", "y")
}

#' Compete winners by maximum score within keys (one per run+spectrum_id)
#'
#' @param df A data frame.
#' @param keys Character vector of column names defining groups to compete within.
#' @param score_col Character scalar giving the score column name.
#'
#' @return A data frame with at most one row per unique combination of \code{keys},
#'   retaining the row with the maximum score (ties broken by taking a single row).
#'
#' @keywords internal
compete_max_score <- function(df, keys, score_col) {
  df |>
    dplyr::group_by(dplyr::across(dplyr::all_of(keys))) |>
    dplyr::slice_max(.data[[score_col]], n = 1, with_ties = FALSE) |>
    dplyr::ungroup()
}

#' Read mokapot PSM text outputs (targets + decoys) and combine
#'
#' Reads the mokapot PSM output tables for targets and decoys (typically
#' \code{*.mokapot.psms.txt} and \code{*.mokapot.decoy.psms.txt}) and concatenates
#' them into a single table.
#'
#' @param target_path Character scalar. Path to the mokapot target PSM table
#'   (e.g. \code{*.mokapot.psms.txt}).
#' @param decoy_path Character scalar. Path to the mokapot decoy PSM table
#'   (e.g. \code{*.mokapot.decoy.psms.txt}).
#'
#' @return
#' A \link[tibble:tibble]{tibble} containing all rows from the target and decoy
#' tables. Column names and content are determined by mokapot.
#'
#' @examples
#' if (requireNamespace("readr", quietly = TRUE)) {
#'   # Create tiny example mokapot-like target/decoy tables and read them back
#'   tf1 <- tempfile(fileext = ".txt")
#'   tf2 <- tempfile(fileext = ".txt")
#'   txt1 <- "run\tspectrum_id\tmokapot score\tmokapot q-value\tmokapot PEP
#'   \tis_target\nr1\t1\t2.0\t0.01\t0.02\tTRUE\n"
#'   txt2 <- "run\tspectrum_id\tmokapot score\tmokapot q-value\tmokapot PEP
#'   \tis_target\nr1\t1\t1.0\t0.50\t0.60\tFALSE\n"
#'   writeLines(txt1, tf1)
#'   writeLines(txt2, tf2)
#'
#'   raw <- read_mokapot_psms(tf1, tf2)
#'   raw
#' }
#'
#' @export
read_mokapot_psms <- function(target_path, decoy_path) {
  if (!file.exists(target_path)) rlang::abort(paste0("File not found: ", target_path))
  if (!file.exists(decoy_path)) rlang::abort(paste0("File not found: ", decoy_path))

  T <- readr::read_tsv(target_path, show_col_types = FALSE)
  D <- readr::read_tsv(decoy_path, show_col_types = FALSE)

  dplyr::bind_rows(T, D)
}

#' Create a competed-winner universe from mokapot PSM outputs
#'
#' Constructs a "postprocessor universe" by selecting (competing) the single best
#' scoring entry (target or decoy) per \code{run + spectrum_id} from mokapot PSM
#' outputs. This yields one row per spectrum per run and can be used for downstream
#' target-decoy diagnostics.
#'
#' @param raw A combined mokapot table, typically the output of
#'   \code{\link{read_mokapot_psms}}.
#' @param run_col Character. Column name for run (default \code{"run"}).
#' @param spectrum_id_col Character. Column name for spectrum identifier (default
#'   \code{"spectrum_id"}).
#' @param score_col Character. Column name for mokapot score used for competition
#'   (default \code{"mokapot score"}; higher is better).
#' @param q_col Character. Column name for mokapot q-value (default
#'   \code{"mokapot q-value"}).
#' @param pep_col Character. Column name for mokapot PEP (default
#'   \code{"mokapot PEP"}).
#' @param id_mode Character. If \code{"runxid"} (default), sets \code{id} to
#'   \code{"run||spectrum_id"}. If \code{"id"}, sets \code{id = spectrum_id}.
#' @param unit Character. Unit metadata stored in the returned object.
#' @param scope Character. Scope metadata stored in the returned object.
#' @param q_source Character. Source label stored in metadata.
#' @param q_max_export Numeric. Export ceiling for q-values (typically 1 for mokapot).
#'
#' @return
#' An object of class \code{dfdr_tbl} (a tibble subclass) with one row per competed
#' \code{run + spectrum_id}. The returned table contains the standard columns
#' required by \code{diagFDR}: \code{id}, \code{is_decoy}, \code{q}, \code{pep},
#' \code{run}, and \code{score}. Metadata are stored in \code{attr(x, "meta")}.
#'
#' @examples
#' library(tibble)
#'
#' raw <- tibble(
#'   run = c("r1", "r1"),
#'   spectrum_id = c("1001", "1001"),
#'   `mokapot score` = c(2.0, 1.0),      # target wins
#'   `mokapot q-value` = c(0.01, 0.5),
#'   `mokapot PEP` = c(0.02, 0.6),
#'   is_target = c(TRUE, FALSE)
#' )
#'
#' x <- mokapot_competed_universe(raw, id_mode = "runxid")
#' x
#'
#' @export
mokapot_competed_universe <- function(raw,
                                      run_col = "run",
                                      spectrum_id_col = "spectrum_id",
                                      score_col = "mokapot score",
                                      q_col = "mokapot q-value",
                                      pep_col = "mokapot PEP",
                                      id_mode = c("runxid", "id"),
                                      unit = "psm",
                                      scope = "global",
                                      q_source = "mokapot q-value",
                                      q_max_export = 1) {
  id_mode <- match.arg(id_mode)

  req <- c(run_col, spectrum_id_col, score_col, q_col, pep_col, "is_target")
  miss <- setdiff(req, names(raw))
  if (length(miss)) rlang::abort(paste0("Missing required mokapot columns: ", paste(miss, collapse = ", ")))

  df <- raw |>
    dplyr::transmute(
      run = as.character(.data[[run_col]]),
      spectrum_id = as.character(.data[[spectrum_id_col]]),
      score = suppressWarnings(as.numeric(.data[[score_col]])),
      q = suppressWarnings(as.numeric(.data[[q_col]])),
      pep = suppressWarnings(as.numeric(.data[[pep_col]])),
      is_target = mokapot_is_target_to_logical(.data[["is_target"]])
    ) |>
    dplyr::mutate(
      is_decoy = !is_target
    ) |>
    dplyr::filter(
      !is.na(run), !is.na(spectrum_id),
      is.finite(score),
      is.finite(q), q >= 0, q <= 1,
      is.finite(pep), pep >= 0, pep <= 1
    )

  # compete winners per run+spectrum_id
  comp <- compete_max_score(df, keys = c("run", "spectrum_id"), score_col = "score")

  out <- comp |>
    dplyr::mutate(
      id = if (id_mode == "runxid") paste(run, spectrum_id, sep = "||") else spectrum_id
    ) |>
    dplyr::select(id, is_decoy, q, pep, run, score)

  as_dfdr_tbl(
    out,
    unit = unit,
    scope = scope,
    q_source = q_source,
    q_max_export = q_max_export,
    provenance = list(tool = "mokapot", id_mode = id_mode)
  )
}

#' Validate a \code{dfdr_tbl}
#'
#' Performs structural checks required by \code{diagFDR} diagnostics. This
#' function is intended for internal use and is called by constructors and
#' downstream diagnostics.
#'
#' The required columns are \code{id}, \code{is_decoy}, \code{q}, \code{pep},
#' \code{run}, and \code{score}. If a \code{p} column is present, it is also
#' validated.
#'
#' @param x A data frame (typically a tibble) to validate.
#'
#' @return
#' The input \code{x} invisibly (unchanged) if validation succeeds. Otherwise an
#' error is raised describing the first detected problem.
#'
#' @keywords internal
validate_dfdr_tbl <- function(x) {
  stopifnot(is.data.frame(x))

  req <- c("id", "is_decoy", "q", "pep", "run", "score")
  miss <- setdiff(req, names(x))
  if (length(miss)) {
    rlang::abort(paste0("Missing required columns: ", paste(miss, collapse = ", ")))
  }

  if (!is.character(x$id)) rlang::abort("`id` must be character.")

  if (!is.logical(x$is_decoy)) rlang::abort("`is_decoy` must be logical (TRUE/FALSE).")
  if (anyNA(x$is_decoy)) rlang::abort("`is_decoy` contains NA; please fix upstream.")

  if (!is.numeric(x$q)) rlang::abort("`q` must be numeric.")
  if (any(!is.finite(x$q))) rlang::abort("`q` contains non-finite values.")
  if (any(x$q < 0 | x$q > 1)) rlang::abort("`q` must be in [0,1].")

  if (!is.numeric(x$pep)) rlang::abort("`pep` must be numeric (use NA if unavailable).")
  if (any(is.finite(x$pep) & (x$pep < 0 | x$pep > 1))) {
    rlang::abort("`pep` must be in [0,1] when present.")
  }

  if (!is.character(x$run)) rlang::abort("`run` must be character (use NA if not applicable).")

  if (!is.numeric(x$score)) rlang::abort("`score` must be numeric (use NA if unavailable).")

  # Optional p-values / pseudo-p-values
  if ("p" %in% names(x)) {
    if (!is.numeric(x$p)) rlang::abort("`p` must be numeric (use NA if unavailable).")
    if (any(is.finite(x$p) & (x$p < 0 | x$p > 1))) {
      rlang::abort("`p` must be in [0,1] when present.")
    }
  }

  x
}

#' Create a \code{dfdr_tbl}
#'
#' Coerces input to a tibble, standardizes the \code{is_decoy} column to logical,
#' validates required columns and types, attaches metadata, and returns an S3
#' object of class \code{dfdr_tbl} for downstream diagnostics.
#'
#' @param x A data.frame with columns \code{id}, \code{is_decoy}, \code{q},
#'   \code{pep}, \code{run}, \code{score} (where \code{pep}/\code{run}/\code{score}
#'   may be \code{NA} if unavailable). Optionally may contain \code{p}.
#' @param unit Character. Statistical unit, e.g. \code{"psm"}, \code{"precursor"},
#'   \code{"peptide"}, \code{"protein"}.
#' @param scope Character. Scope of FDR control, e.g. \code{"runwise"},
#'   \code{"global"}, \code{"aggregated"}.
#' @param q_source Character. Description of where q-values came from (e.g. a
#'   column name or tool).
#' @param q_max_export Numeric. Optional export ceiling used to generate q (if known).
#' @param p_source Optional character describing where \code{p} came from.
#' @param provenance Named list carrying provenance information (tool, version,
#'   parameters, command, etc.).
#'
#' @return
#' An S3 object of class \code{dfdr_tbl} that inherits from
#' \link[tibble:tibble]{tbl_df} (and \code{data.frame}). The returned object:
#' \itemize{
#'   \item contains the validated input data with \code{is_decoy} coerced to
#'   logical (\code{TRUE}/\code{FALSE});
#'   \item carries metadata in \code{attr(x, "meta")} with entries \code{unit},
#'   \code{scope}, \code{q_source}, \code{q_max_export}, \code{p_source}, and
#'   \code{provenance}.
#' }
#'
#' @examples
#' library(tibble)
#'
#' df <- tibble(
#'   id = as.character(1:6),
#'   run = c("run1","run1","run1","run2","run2","run2"),
#'   is_decoy = c(FALSE, FALSE, TRUE, FALSE, TRUE, FALSE),
#'   score = c(10, 9, 8, 7, 6, 5),
#'   q = c(0.01, 0.02, 0.02, 0.05, 0.06, 0.07),
#'   pep = c(0.01, 0.02, NA, 0.05, NA, 0.07)
#' )
#'
#' x <- as_dfdr_tbl(
#'   df,
#'   unit = "psm",
#'   scope = "global",
#'   q_source = "toy_q",
#'   provenance = list(tool = "toy")
#' )
#'
#' x
#' attr(x, "meta")
#'
#' @export
as_dfdr_tbl <- function(x,
                        unit = NA_character_,
                        scope = NA_character_,
                        q_source = NA_character_,
                        q_max_export = NA_real_,
                        p_source = NA_character_,
                        provenance = list()) {
  x <- tibble::as_tibble(x)

  if (!"id" %in% names(x)) rlang::abort("Missing required column: id")
  if (!"is_decoy" %in% names(x)) rlang::abort("Missing required column: is_decoy")

  # Coerce is_decoy if needed
  if (is.numeric(x$is_decoy) || is.integer(x$is_decoy)) {
    u <- sort(unique(stats::na.omit(x$is_decoy)))
    if (!all(u %in% c(0, 1))) {
      rlang::abort("Numeric `is_decoy` must contain only 0/1 (no NAs).")
    }
    x$is_decoy <- x$is_decoy == 1
  }
  if (is.character(x$is_decoy)) {
    if (all(stats::na.omit(x$is_decoy) %in% c("0","1"))) {
      x$is_decoy <- x$is_decoy == "1"
    } else {
      x$is_decoy <- as.logical(x$is_decoy)
    }
  }

  x <- validate_dfdr_tbl(x)

  attr(x, "meta") <- list(
    unit = unit,
    scope = scope,
    q_source = q_source,
    q_max_export = q_max_export,
    p_source = p_source,
    provenance = provenance
  )
  class(x) <- c("dfdr_tbl", class(x))
  x
}

#' Print a \code{dfdr_tbl}
#'
#' Prints a compact header with attached metadata (unit, scope, q-source, etc.)
#' and then falls back to the default tibble/data.frame print method.
#'
#' @param x A \code{dfdr_tbl}.
#' @param ... Passed to the next print method.
#'
#' @return
#' Returns \code{x} invisibly (called for its printing side effect).
#'
#' @examples
#' library(tibble)
#'
#' df <- tibble(
#'   id = as.character(1:3),
#'   run = "run1",
#'   is_decoy = c(FALSE, TRUE, FALSE),
#'   score = c(10, 9, 8),
#'   q = c(0.01, 0.02, 0.03),
#'   pep = c(0.01, NA, 0.03)
#' )
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy_q")
#' x
#'
#' @export
print.dfdr_tbl <- function(x, ...) {
  meta <- attr(x, "meta") %||% list()
  cat("<dfdr_tbl>\n")
  cat("  unit:    ", meta$unit %||% NA_character_, "\n", sep = "")
  cat("  scope:   ", meta$scope %||% NA_character_, "\n", sep = "")
  cat("  q_from:  ", meta$q_source %||% NA_character_, "\n", sep = "")
  if (!is.null(meta$p_source) && !is.na(meta$p_source)) {
    cat("  p_from:  ", meta$p_source, "\n", sep = "")
  }
  cat("  n:       ", nrow(x), "\n", sep = "")
  NextMethod()
}

#' Run a standard set of FDR QC diagnostics
#'
#' Runs stability, boundary-support, and plausibility diagnostics on a named list
#' of \code{dfdr_tbl} objects and returns both summary tables and plots.
#'
#' Diagnostics include (depending on available columns):
#' \itemize{
#'   \item Target/decoy headline diagnostics at \code{alpha_main} (counts, FDR estimate, stability proxies)
#'   \item Stability curves across \code{alphas}
#'   \item Local tail decoy support near each \code{alpha}
#'   \item Elasticity (Jaccard overlap under threshold perturbation)
#'   \item PEP reliability / IPE and PEP sanity summaries (if \code{pep} is present)
#'   \item Equal-chance by q-bands (always attempted; may be not applicable if q-range is truncated)
#'   \item (Pseudo-)p-value calibration and Storey \eqn{\pi_0(\lambda)} (if \code{p} is present or computed)
#'   \item BH diagnostics (if \code{p} is present)
#' }
#'
#' @param xs Named list of \code{dfdr_tbl} objects.
#' @param alpha_main Numeric scalar in \eqn{(0,1]}. Headline threshold (e.g. 0.01).
#' @param alphas Numeric vector of thresholds in \eqn{(0,1]} used to build curves.
#' @param eps Numeric scalar. Relative perturbation used in elasticity (e.g. 0.2).
#' @param win_rel Numeric scalar. Relative window width for local tail support (e.g. 0.2).
#' @param truncation Character. Truncation policy for local tail windows; see \code{\link{dfdr_local_tail}}.
#' @param k_fixed Integer. Fixed +/-K used in sensitivity intervals.
#' @param k_sqrt_mult Numeric. Multiplier for adaptive +/-\eqn{\lceil mult\sqrt{D}\rceil} intervals.
#' @param qband_breaks Numeric vector. Cutpoints for q-band equal-chance diagnostic.
#' @param low_conf Length-2 numeric vector. Pooled low-confidence region for the equal-chance test.
#' @param min_N_equalchance Integer. Minimum pooled N for equal-chance test.
#' @param pep_binwidth Numeric. Bin width for PEP reliability bins.
#' @param pep_n_min Integer. Minimum bin size to include PEP bins in IPE summary/plots.
#' @param pep_max Numeric. Max mean PEP included in IPE summary.
#' @param pep_sanity_thresholds Numeric vector. Thresholds for decoy PEP sanity summaries.
#' @param pep_density_max Numeric. Max PEP displayed in target/decoy density plot.
#' @param pep_rel_tdc_breaks Numeric vector. Breaks used for the TDC-style PEP reliability bins.
#' @param pep_rel_tdc_add_decoy Integer. Additive correction used in D/T for TDC-style reliability.
#' @param compute_pseudo_pvalues Logical. If \code{TRUE}, attempt to compute pseudo-p-values
#'   when \code{p} is missing.
#' @param pseudo_pvalue_method Character. Method passed to \code{\link{score_to_pvalue}}
#'   (default \code{"decoy_ecdf"}).
#' @param p_u_grid Numeric vector in \eqn{(0,1]}. Grid for p-value ECDF calibration diagnostics.
#' @param p_u_max Numeric scalar in \eqn{(0,1]}. Upper bound of \code{u} used for p-value inflation summaries.
#' @param p_stratify Optional character vector of columns for stratified p-value diagnostics.
#' @param p_min_n_calib Integer. Minimum n for p-value calibration per stratum.
#' @param p_lambdas Numeric vector in \eqn{[0,1)}. Lambdas for Storey \eqn{\pi_0(\lambda)} diagnostic.
#' @param p_min_n_pi0 Integer. Minimum n for \eqn{\pi_0(\lambda)} diagnostic per stratum.
#' @param bh_boundary Character. Boundary window type for BH diagnostics (\code{"mult"} or \code{"add"}).
#' @param bh_win Numeric. Relative window size for BH boundary support when \code{bh_boundary="mult"}.
#' @param bh_delta Numeric. Additive window size when \code{bh_boundary="add"}.
#'
#' @return
#' A list with components:
#' \describe{
#'   \item{tables}{A named list of result tables (tibbles) for the computed diagnostics
#'   (e.g. \code{headline}, \code{stability}, \code{local_tail}, \code{elasticity},
#'   and optional PEP/p-value/BH-related tables).}
#'   \item{plots}{A named list of \link[ggplot2:ggplot]{ggplot} objects produced from
#'   the diagnostics.}
#'   \item{objects}{The input \code{xs}, possibly augmented with computed pseudo-p-values
#'   if \code{compute_pseudo_pvalues=TRUE}.}
#'   \item{params}{A list of parameter values used to compute the diagnostics.}
#' }
#'
#' @examples
#' \donttest{
#' library(tibble)
#'
#' set.seed(1)
#' n <- 2000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = sample(c("run1", "run2"), n, replace = TRUE),
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   # Toy q/pep/p for demonstration purposes
#'   q = pmin(1, rank(-score) / n),
#'   pep = stats::runif(n),
#'   p = stats::runif(n)
#' )
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy", p_source = "toy")
#'
#' diag <- dfdr_run_all(
#'   xs = list(toy = x),
#'   alpha_main = 0.01,
#'   compute_pseudo_pvalues = FALSE
#' )
#'
#' names(diag$tables)
#' names(diag$plots)
#' diag$tables$headline
#' }
#' @export
dfdr_run_all <- function(xs,
                         alpha_main = 0.01,
                         alphas = c(1e-4, 5e-4, 1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
                         eps = 0.2,
                         win_rel = 0.2,
                         truncation = "warn_drop",
                         k_fixed = 10,
                         k_sqrt_mult = 2,
                         qband_breaks = c(0, 0.01, 0.02, 0.05, 0.1, 0.2, 0.5),
                         low_conf = c(0.2, 0.5),
                         min_N_equalchance = 2000,
                         pep_binwidth = 0.05,
                         pep_n_min = 200,
                         pep_max = 0.5,
                         # ---- NEW: PEP sanity + alt reliability (TDC-style)
                         pep_sanity_thresholds = c(0.01, 0.05, 0.1, 0.2),
                         pep_density_max = 0.5,
                         pep_rel_tdc_breaks = seq(0, 0.5, by = 0.05),
                         pep_rel_tdc_add_decoy = 0L,
                         # ---- p-value diagnostics defaults
                         compute_pseudo_pvalues = TRUE,
                         pseudo_pvalue_method = "decoy_ecdf",
                         p_u_grid = c(seq(0.001, 0.1, by = 0.001), seq(0.11, 1, by = 0.01)),
                         p_u_max = 0.1,
                         p_stratify = NULL,
                         p_min_n_calib = 200,
                         p_lambdas = seq(0.5, 0.95, by = 0.05),
                         p_min_n_pi0 = 2000,
                         bh_boundary = c("mult", "add"),
                         bh_win = 0.2,
                         bh_delta = NULL) {

  bh_boundary <- match.arg(bh_boundary)

  if (!is.list(xs) || is.null(names(xs)) || any(names(xs) == "")) {
    rlang::abort("`xs` must be a *named* list of dfdr_tbl objects.")
  }
  purrr::walk(xs, validate_dfdr_tbl)

  # ---- Add pseudo p-values if requested
  if (compute_pseudo_pvalues) {
    xs <- add_pseudo_pvalues_to_list(xs, method = pseudo_pvalue_method)
  }

  # ---- Core target/decoy tables (stacked with list label)
  headline <- purrr::imap_dfr(xs, ~dplyr::mutate(dfdr_headline(.x, alpha_main, k_fixed, k_sqrt_mult), list = .y))

  # ----

  stability <- purrr::imap_dfr(xs, ~dplyr::mutate(dfdr_curve_stability(.x, alphas, k_fixed, k_sqrt_mult), list = .y))
  dwin <- purrr::imap_dfr(xs, ~dplyr::mutate(dfdr_local_tail(.x, alphas, win_rel, truncation), list = .y))
  elasticity <- purrr::imap_dfr(xs, ~dplyr::mutate(dfdr_elasticity(.x, alphas, eps), list = .y))

  # ---- Calibration: PEP reliability and sumpep (only for those with PEP)
  pep_rel <- list()
  sumpep <- list()

  # ---- PEP sanity + alternative (TDC-style) reliability
  pep_sanity <- list()
  pep_rel_tdc <- list()

  # ---- Equal-chance (q-band) diagnostics (always computed)
  eqchance <- list()

  # ---- p-value diagnostics (only if p exists and has finite values)
  p_calib <- list()
  p_pi0 <- list()
  bh_head <- list()
  bh_elast <- list()

  for (nm in names(xs)) {
    x <- xs[[nm]]

    # --- PEP-based diagnostics
    has_pep <- ("pep" %in% names(x)) && any(is.finite(x$pep))

    if (has_pep) {
      pr <- dfdr_pep_reliability(x, binwidth = pep_binwidth, n_min = pep_n_min, pep_max = pep_max)
      pep_rel[[nm]] <- pr
      sumpep[[nm]] <- dfdr_sumpep(x, alphas)

      # Sanity: decoys getting small PEP?
      pep_sanity[[nm]] <- dfdr_pep_decoy_sanity(x, thresholds = pep_sanity_thresholds)

      # Alternative reliability: D/T proxy within PEP bins
      pep_rel_tdc[[nm]] <- dfdr_pep_reliability_tdc(
        x,
        breaks = pep_rel_tdc_breaks,
        add_decoy = pep_rel_tdc_add_decoy
      )
    }

    # --- Equal-chance by q-bands
    ec <- dfdr_equal_chance_qbands(x, breaks = qband_breaks, low_conf = low_conf, min_N = min_N_equalchance)
    eqchance[[nm]] <- ec

    # --- p-value diagnostics (only if p exists and usable)
    if ("p" %in% names(x) && any(is.finite(x$p))) {
      p_calib[[nm]] <- dfdr_p_calibration(
        x,
        u_grid = p_u_grid,
        u_max = p_u_max,
        stratify = p_stratify,
        min_n = p_min_n_calib
      )

      p_pi0[[nm]] <- dfdr_pi0_storey(
        x,
        lambdas = p_lambdas,
        stratify = p_stratify,
        min_n = p_min_n_pi0,
        clamp = TRUE
      )

      bh_head[[nm]] <- dfdr_bh_diagnostics(
        x,
        alpha = alpha_main,
        boundary = bh_boundary,
        win = bh_win,
        delta = bh_delta
      )

      bh_elast[[nm]] <- dfdr_bh_elasticity(
        x,
        alphas = alphas,
        eps = eps
      )

    }
  }

  # ---- merge additional columns into headline before flagging

  # Merge D_alpha_win from local tail (at alpha_main)
  dwin_main <- dwin |>
    dplyr::filter(alpha == alpha_main) |>
    dplyr::select(list, D_alpha_win)

  headline <- headline |>
    dplyr::left_join(dwin_main, by = "list")

  # Merge IPE from PEP reliability
  if (length(pep_rel) > 0) {
    ipe_tbl <- purrr::imap_dfr(pep_rel, ~tibble::tibble(list = .y, IPE = .x$IPE))
    headline <- headline |>
      dplyr::left_join(ipe_tbl, by = "list")
  }

  # Merge effect_abs from equal-chance pooled
  if (length(eqchance) > 0) {
    effect_tbl <- purrr::imap_dfr(eqchance, ~dplyr::mutate(.x$pooled, list = .y)) |>
      dplyr::select(.data$list, .data$effect_abs)
    headline <- headline |> dplyr::left_join(effect_tbl, by = "list")
  }

  # ---- flag (with columns present but possibly NA)
  headline <- flag_headline(headline)

  # ---- Core plots (common plots for multi-table)
  plots <- list(
    dalpha = dfdr_plot_dalpha(stability),
    cv = dfdr_plot_cv(stability),
    dwin = dfdr_plot_dwin(dwin[!is.na(dwin$D_alpha_win), , drop = FALSE]),
    elasticity = dfdr_plot_elasticity(elasticity)
  )

  # ---- Scope disagreement matrix (if multiple lists provided)
  if (length(xs) > 1) {
    plots$scope_disagreement <- dfdr_plot_scope_disagreement_matrix(xs, alpha_main)
  }

  # ---- Score distribution plots per list
  for (nm in names(xs)) {
    plots[[paste0("score_dist__", nm)]] <-
      dfdr_plot_score_distributions(xs[[nm]], title = paste0("Score distributions: ", nm))
  }

  # ---- PEP reliability plots per list (if available)
  for (nm in names(pep_rel)) {
    plots[[paste0("pep_reliability__", nm)]] <-
      dfdr_plot_pep_reliability(pep_rel[[nm]]$bins, title = paste0("PEP reliability: ", nm))
  }

  # ---- PEP density sanity plots (targets vs decoys)
  for (nm in names(pep_sanity)) {
    plots[[paste0("pep_density__", nm)]] <-
      dfdr_plot_pep_density_by_decoy(
        xs[[nm]],
        pep_max = pep_density_max,
        title = paste0("PEP density (targets vs decoys): ", nm)
      )
  }

  # ---- PEP reliability (TDC-style) plots per list
  for (nm in names(pep_rel_tdc)) {
    plots[[paste0("pep_reliability_tdc__", nm)]] <-
      dfdr_plot_pep_reliability_tdc(
        pep_rel_tdc[[nm]],
        title = paste0("PEP reliability (TDC-style): ", nm)
      )
  }

  # ---- Equal-chance plots per list
  for (nm in names(eqchance)) {
    plots[[paste0("equal_chance__", nm)]] <-
      dfdr_plot_equal_chance(eqchance[[nm]]$bands, title = paste0("Equal-chance by q-band: ", nm))
  }

  # ---- p-value / pseudo-p-value density plots per list (if p exists)
  for (nm in names(xs)) {
    x <- xs[[nm]]
    if ("p" %in% names(x) && any(is.finite(x$p))) {
      plots[[paste0("p_density__", nm)]] <-
        dfdr_plot_p_density_by_decoy(
          x,
          p_max = 1,
          log10_x = FALSE,
          title = paste0("(Pseudo-)p-value density (targets vs decoys): ", nm)
        )
    }
  }

  # ---- p-value plots per list (if available)
  for (nm in names(p_calib)) {
    x<-xs[[nm]]
    if ("p" %in% names(x) && any(is.finite(x$p))) {
    plots[[paste0("Calibration_pvalues__", nm)]] <-
      dfdr_plot_p_calibration2(x,sel="all")
    }
  }
  for (nm in names(p_calib)) {
    x<-xs[[nm]]
    if ("p" %in% names(x) && any(is.finite(x$p))) {
      plots[[paste0("Calibration_pvalues_decoys__", nm)]] <-
        dfdr_plot_p_calibration2(x,sel="decoy")
    }
  }
  for (nm in names(p_calib)) {
    x<-xs[[nm]]
    if ("p" %in% names(x) && any(is.finite(x$p))) {
      plots[[paste0("Calibration_pvalues_targets__", nm)]] <-
        dfdr_plot_p_calibration2(x,sel="target")
    }
  }
  for (nm in names(p_calib)) {
    x<-xs[[nm]]
    if ("p" %in% names(x) && any(is.finite(x$p))) {
      plots[[paste0("Estimated_fdr__", nm)]] <-
        dfdr_plot_fdrhat_pi0(x,sel="all")$plot
    }
  }
  for (nm in names(p_calib)) {
    x<-xs[[nm]]
    if ("p" %in% names(x) && any(is.finite(x$p))) {
      plots[[paste0("Estimated_fdr_decoys__", nm)]] <-
        dfdr_plot_fdrhat_pi0(x,sel="decoy")$plot
    }
  }
  for (nm in names(p_calib)) {
    x<-xs[[nm]]
    if ("p" %in% names(x) && any(is.finite(x$p))) {
      plots[[paste0("Estimated_fdr_targets__", nm)]] <-
        dfdr_plot_fdrhat_pi0(x,sel="target")$plot
    }
  }
  # for (nm in names(p_calib)) {
  #   plots[[paste0("Super_uniformity_pvalues__", nm)]] <-
  #     dfdr_plot_p_calibration(p_calib[[nm]]$ecdf, title = paste0("P-value calibration: ", nm))
  # }
  for (nm in names(p_pi0)) {
    plots[[paste0("pi0_Storey__", nm)]] <-
      dfdr_plot_pi0(p_pi0[[nm]]$pi0, title = paste0("Storey pi0(lambda): ", nm))
  }
  for (nm in names(bh_elast)) {
    plots[[paste0("BH_elasticity__", nm)]] <-
      dfdr_plot_bh_elasticity(bh_elast[[nm]], title = paste0("BH elasticity: ", nm))
  }

  # ---- Stack BH headline tables (if any)
  bh_headline_tbl <- NULL
  if (length(bh_head)) {
    bh_headline_tbl <- purrr::imap_dfr(bh_head, ~dplyr::mutate(.x$headline, list = .y))
  } else {
    bh_headline_tbl <- tibble::tibble()
  }

  # ---- Stack BH elasticity tables (if any)
  bh_elasticity_tbl <- NULL
  if (length(bh_elast)) {
    bh_elasticity_tbl <- purrr::imap_dfr(bh_elast, ~dplyr::mutate(.x, list = .y))
  } else {
    bh_elasticity_tbl <- tibble::tibble()
  }

  list(
    tables = list(
      # target/decoy
      headline = headline,  # contains flags
      stability = stability,
      local_tail = dwin,
      elasticity = elasticity,

      # PEP
      sumpep = sumpep,
      pep_reliability_bins = purrr::imap(pep_rel, ~.x$bins),
      pep_IPE = purrr::imap_dfr(pep_rel, ~tibble::tibble(list = .y, IPE = .x$IPE)),

      # PEP sanity + alt reliability
      pep_decoy_sanity = purrr::imap_dfr(pep_sanity, ~dplyr::mutate(.x, list = .y)),
      pep_reliability_tdc = purrr::imap_dfr(pep_rel_tdc, ~dplyr::mutate(.x, list = .y)),

      # equal-chance
      equal_chance_bands = purrr::imap(eqchance, ~.x$bands),
      equal_chance_pooled = purrr::imap_dfr(eqchance, ~dplyr::mutate(.x$pooled, list = .y)),

      # p-value calibration + pi0
      p_calibration_ecdf = purrr::imap(p_calib, ~.x$ecdf),
      p_calibration_summary = purrr::imap_dfr(p_calib, ~dplyr::mutate(.x$summary, list = .y)),
      pi0_storey = purrr::imap(p_pi0, ~.x$pi0),
      pi0_storey_summary = purrr::imap_dfr(p_pi0, ~dplyr::mutate(.x$summary, list = .y)),

      # BH
      bh_headline = bh_headline_tbl,
      bh_elasticity = bh_elasticity_tbl
    ),
    plots = plots,
    objects = xs,
    params = list(
      alpha_main = alpha_main,
      alphas = alphas,
      eps = eps,
      win_rel = win_rel,
      truncation = truncation,
      # PEP params
      pep_sanity_thresholds = pep_sanity_thresholds,
      pep_density_max = pep_density_max,
      pep_rel_tdc_breaks = pep_rel_tdc_breaks,
      pep_rel_tdc_add_decoy = pep_rel_tdc_add_decoy,
      # p-value params
      p_u_max = p_u_max,
      p_stratify = p_stratify,
      p_min_n_calib = p_min_n_calib,
      p_lambdas = p_lambdas,
      p_min_n_pi0 = p_min_n_pi0,
      bh_boundary = bh_boundary,
      bh_win = bh_win,
      bh_delta = bh_delta
    )
  )
}

#' Scope disagreement between two lists
#'
#' Computes the Jaccard overlap of accepted target IDs between two
#' \code{dfdr_tbl} objects across a set of thresholds. This is useful for comparing
#' results obtained under different FDR scopes (e.g. run-wise vs global) or from
#' different pipelines, provided that both inputs use compatible \code{id}
#' semantics.
#'
#' @param x1 First \code{dfdr_tbl}.
#' @param x2 Second \code{dfdr_tbl}.
#' @param alphas Numeric vector of thresholds in \eqn{(0,1]}.
#' @param label1 Character label for \code{x1} in the output (default \code{"A"}).
#' @param label2 Character label for \code{x2} in the output (default \code{"B"}).
#' @param normalize_ids Logical. If \code{TRUE} (default), extracts a "base" ID by
#'   stripping any run prefix of the form \code{"<run>||<id>"} (everything up to
#'   and including \code{"||"}). Set to \code{FALSE} if IDs are already comparable
#'   between \code{x1} and \code{x2}.
#'
#' @return
#' A \link[tibble:tibble]{tibble} with one row per \code{alpha} and columns:
#' \describe{
#'   \item{alpha}{The threshold.}
#'   \item{label1,label2}{The labels identifying the two inputs.}
#'   \item{n1,n2}{Numbers of accepted target IDs in \code{x1} and \code{x2} at \code{alpha}.}
#'   \item{jaccard}{Jaccard similarity between the two accepted target ID sets.}
#' }
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 4000
#'
#' # Two "lists" with slightly different q-values for the same base IDs
#' base_ids <- as.character(seq_len(n))
#'
#' df1 <- tibble(
#'   id = paste0("run1||", base_ids),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   q = stats::runif(n, 0, 0.05),
#'   pep = NA_real_
#' )
#'
#' df2 <- tibble(
#'   id = base_ids,              # no run prefix here
#'   run = "all",
#'   is_decoy = df1$is_decoy,    # same decoy labels for simplicity
#'   score = df1$score + rnorm(n, sd = 0.2),
#'   q = pmin(1, df1$q * 1.2),   # slightly perturbed q-values
#'   pep = NA_real_
#' )
#'
#' x1 <- as_dfdr_tbl(df1, unit = "psm", scope = "runwise", q_source = "toy")
#' x2 <- as_dfdr_tbl(df2, unit = "psm", scope = "global",  q_source = "toy")
#'
#' dfdr_scope_disagreement(x1, x2, alphas = c(0.005, 0.01, 0.02), normalize_ids = TRUE)
#'
#' @export
dfdr_scope_disagreement <- function(x1, x2, alphas,
                                    label1 = "A", label2 = "B",
                                    normalize_ids = TRUE) {
  validate_dfdr_tbl(x1)
  validate_dfdr_tbl(x2)

  # Check if ID formats differ
  has_pipe_x1 <- any(grepl("\\|\\|", x1$id[1:min(100, nrow(x1))]))
  has_pipe_x2 <- any(grepl("\\|\\|", x2$id[1:min(100, nrow(x2))]))

  if (has_pipe_x1 != has_pipe_x2 && !normalize_ids) {
    rlang::warn(
      "ID formats appear to differ. Consider setting `normalize_ids = TRUE`."
    )
  }

  # Helper to extract base precursor ID
  extract_base_id <- function(ids) {
    if (!normalize_ids) return(ids)
    ifelse(grepl("\\|\\|", ids),
           sub("^.*\\|\\|", "", ids),
           ids)
  }

  purrr::map_dfr(alphas, function(a) {
    A <- unique(extract_base_id(x1$id[!x1$is_decoy & x1$q <= a]))
    B <- unique(extract_base_id(x2$id[!x2$is_decoy & x2$q <= a]))
    tibble::tibble(
      alpha = a,
      label1 = label1,
      label2 = label2,
      n1 = length(A),
      n2 = length(B),
      jaccard = jaccard_vec(A, B)
    )
  })
}

#' Plot decoy support \eqn{D_\alpha} versus \code{alpha}
#'
#' Plots the number of accepted decoys at each threshold \code{alpha}. This is a
#' key stability indicator: very small \eqn{D_\alpha} implies granular/unstable
#' behaviour of target-decoy based estimates.
#'
#' @param stab_tbl A stability table (e.g. from \code{\link{dfdr_curve_stability}})
#'   containing columns \code{alpha}, \code{D_alpha}, and \code{list}.
#' @param xlab Character. X-axis label.
#' @param title Character. Plot title.
#'
#' @return A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' stab_tbl <- tibble(
#'   list = rep(c("A", "B"), each = 4),
#'   alpha = rep(c(1e-3, 2e-3, 5e-3, 1e-2), times = 2),
#'   D_alpha = c(5, 8, 20, 40, 2, 3, 6, 10)
#' )
#' dfdr_plot_dalpha(stab_tbl)
#'
#' @export
dfdr_plot_dalpha <- function(stab_tbl, xlab = "alpha (log10)", title = "Decoy support D_alpha vs alpha") {
  ggplot2::ggplot(stab_tbl, ggplot2::aes(x = alpha, y = D_alpha, color = .data$list)) +
    ggplot2::geom_line() + ggplot2::geom_point() +
    ggplot2::scale_x_continuous(trans = "log10") +
    ggplot2::theme_bw() +
    ggplot2::labs(title = title, x = xlab, y = "D_alpha", color = "")
}

#' Plot \code{CV_hat} versus \code{alpha}
#'
#' Plots a stability proxy (\code{CV_hat}) against the threshold \code{alpha} for
#' one or more lists.
#'
#' @param stab_tbl A stability table (e.g. from \code{\link{dfdr_curve_stability}})
#'   with columns \code{alpha}, \code{CV_hat}, and \code{list}.
#' @param xlab Character. X-axis label.
#' @param title Character. Plot title.
#'
#' @return A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' stab_tbl <- tibble(
#'   list = rep(c("A", "B"), each = 4),
#'   alpha = rep(c(1e-3, 2e-3, 5e-3, 1e-2), times = 2),
#'   CV_hat = c(0.30, 0.22, 0.15, 0.12, 0.40, 0.28, 0.20, 0.18)
#' )
#' dfdr_plot_cv(stab_tbl)
#'
#' @export
dfdr_plot_cv <- function(stab_tbl, xlab = "alpha (log10)", title = "Stability proxy CV_hat vs alpha") {
  ggplot2::ggplot(stab_tbl, ggplot2::aes(x = alpha, y = CV_hat, color = .data$list)) +
    ggplot2::geom_line() + ggplot2::geom_point() +
    ggplot2::scale_x_continuous(trans = "log10") +
    ggplot2::theme_bw() +
    ggplot2::labs(title = title, x = xlab, y = "CV_hat", color = "")
}

#' Plot local decoy support \code{D_alpha_win} versus \code{alpha}
#'
#' Plots the number of decoys in a right-neighborhood above each threshold
#' (as computed by \code{\link{dfdr_local_tail}}). This approximates how well
#' supported the boundary is by nearby decoys.
#'
#' @param dwin_tbl A local-tail table (e.g. from \code{\link{dfdr_local_tail}})
#'   containing columns \code{alpha}, \code{D_alpha_win}, and \code{list}.
#' @param xlab Character. X-axis label.
#' @param title Character. Plot title.
#'
#' @return A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' dwin_tbl <- tibble(
#'   list = rep(c("A", "B"), each = 4),
#'   alpha = rep(c(1e-3, 2e-3, 5e-3, 1e-2), times = 2),
#'   D_alpha_win = c(1, 2, 6, 15, 0, 1, 2, 4)
#' )
#' dfdr_plot_dwin(dwin_tbl)
#'
#' @export
dfdr_plot_dwin <- function(dwin_tbl, xlab = "alpha (log10)", title = "Local decoy support D_alpha_win vs alpha") {
  ggplot2::ggplot(dwin_tbl, ggplot2::aes(x = alpha, y = D_alpha_win, color = .data$list)) +
    ggplot2::geom_line() + ggplot2::geom_point() +
    ggplot2::scale_x_continuous(trans = "log10") +
    ggplot2::theme_bw() +
    ggplot2::labs(title = title, x = xlab, y = "D_alpha_win", color = "")
}

#' Plot threshold elasticity (Jaccard) versus \code{alpha}
#'
#' Plots Jaccard overlap values (as returned by \code{\link{dfdr_elasticity}})
#' against \code{alpha}. Lower overlap indicates greater sensitivity of the
#' accepted set to small changes of the threshold.
#'
#' @param el_tbl An elasticity table (e.g. from \code{\link{dfdr_elasticity}})
#'   containing columns \code{alpha}, \code{jaccard}, and \code{list}.
#' @param xlab Character. X-axis label.
#' @param title Character. Plot title.
#'
#' @return A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' el_tbl <- tibble(
#'   list = rep(c("A", "B"), each = 4),
#'   alpha = rep(c(1e-3, 2e-3, 5e-3, 1e-2), times = 2),
#'   jaccard = c(0.95, 0.93, 0.90, 0.88, 0.92, 0.90, 0.86, 0.82)
#' )
#' dfdr_plot_elasticity(el_tbl)
#'
#' @export
dfdr_plot_elasticity <- function(el_tbl, xlab = "alpha (log10)", title = "Threshold elasticity (Jaccard) vs alpha") {
  ggplot2::ggplot(el_tbl, ggplot2::aes(x = alpha, y = jaccard, color = .data$list)) +
    ggplot2::geom_line() + ggplot2::geom_point() +
    ggplot2::scale_x_continuous(trans = "log10") +
    ggplot2::theme_bw() +
    ggplot2::labs(title = title, x = xlab, y = "Jaccard", color = "")
}

#' Plot PEP reliability
#'
#' Visualises PEP reliability by plotting the observed decoy fraction against the
#' mean predicted PEP in each bin (from \code{\link{dfdr_pep_reliability}}). Point
#' size reflects bin counts.
#'
#' @param bins_tbl A PEP reliability bins table (e.g. \code{out$bins} from
#'   \code{\link{dfdr_pep_reliability}}) containing columns \code{pep_mean},
#'   \code{decoy_rate}, and \code{n}.
#' @param title Character. Plot title.
#'
#' @return A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' bins_tbl <- tibble(
#'   pep_mean = c(0.05, 0.15, 0.25, 0.35),
#'   decoy_rate = c(0.06, 0.14, 0.30, 0.40),
#'   n = c(500, 400, 250, 120)
#' )
#' dfdr_plot_pep_reliability(bins_tbl)
#'
#' @export
dfdr_plot_pep_reliability <- function(bins_tbl, title = "PEP reliability") {
  ggplot2::ggplot(bins_tbl, ggplot2::aes(x = pep_mean, y = decoy_rate, size = n)) +
    ggplot2::geom_point(alpha = 0.8) +
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = 2) +
    ggplot2::coord_equal(xlim = c(0, 1), ylim = c(0, 1)) +
    ggplot2::theme_bw() +
    ggplot2::labs(title = title, x = "Mean predicted PEP", y = "Observed decoy fraction", size = "n")
}

#' Plot equal-chance plausibility by q-value bands
#'
#' Visualises decoy fractions by q-value band (as produced by
#' \code{\link{dfdr_equal_chance_qbands}}). A horizontal reference at 0.5 indicates
#' the expected decoy fraction under an "equal-chance" region assumption.
#'
#' @param bands_tbl A q-band table (e.g. \code{out$bands} from
#'   \code{\link{dfdr_equal_chance_qbands}}) containing columns \code{q_mean},
#'   \code{decoy_frac}, and \code{n}.
#' @param title Character. Plot title.
#'
#' @return A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' bands_tbl <- tibble(
#'   q_mean = c(0.05, 0.15, 0.30, 0.45),
#'   decoy_frac = c(0.10, 0.20, 0.45, 0.52),
#'   n = c(2000, 1500, 800, 400)
#' )
#' dfdr_plot_equal_chance(bands_tbl)
#'
#' @export
dfdr_plot_equal_chance <- function(bands_tbl, title = "Equal-chance plausibility: decoy fraction by q-band") {
  ggplot2::ggplot(bands_tbl, ggplot2::aes(x = q_mean, y = decoy_frac, size = n)) +
    ggplot2::geom_point(alpha = 0.85) +
    ggplot2::geom_line(alpha = 0.5) +
    ggplot2::geom_hline(yintercept = 0.5, linetype = 2) +
    ggplot2::theme_bw() +
    ggplot2::labs(title = title, x = "Mean q-value in band", y = "Decoy fraction", size = "n")
}


#' Plot target vs decoy score distributions
#'
#' Plots density distributions of scores (if available) or q-values (fallback) for
#' targets vs decoys. This provides a quick sanity check that decoys are shifted
#' toward lower scores (or higher q-values).
#'
#' @param x A \code{dfdr_tbl}.
#' @param title Character. Plot title.
#'
#' @return A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 4000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   q = pmin(1, rank(-score) / n),
#'   pep = NA_real_
#' )
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")
#' dfdr_plot_score_distributions(x)
#'
#' @export
dfdr_plot_score_distributions <- function(x, title = "Target vs Decoy Score Distributions") {
  validate_dfdr_tbl(x)

  use_score <- "score" %in% names(x) && !all(is.na(x$score))

  if (use_score) {
    plot_data <- x |>
      dplyr::filter(is.finite(.data$score)) |>
      dplyr::mutate(
        type = dplyr::if_else(.data$is_decoy, "Decoy", "Target"),
        value = .data$score
      )
    x_label <- "Score"
    plot_title <- title
    use_log <- FALSE
  } else {
    plot_data <- x |>
      dplyr::filter(is.finite(.data$q), .data$q > 0) |>
      dplyr::mutate(
        type = dplyr::if_else(.data$is_decoy, "Decoy", "Target"),
        value = .data$q
      )
    x_label <- "q-value"
    plot_title <- sub("Score [Dd]istributions?", "q-value distributions", title)
    use_log <- TRUE
  }

  if (nrow(plot_data) == 0) {
    rlang::warn("No valid data for distribution plot; returning empty plot.")
    return(ggplot2::ggplot() + ggplot2::labs(title = paste(plot_title, "(no data)")))
  }

  p <- ggplot2::ggplot(plot_data, ggplot2::aes(x = .data$value, fill = .data$type, color = .data$type)) +
    ggplot2::geom_density(alpha = 0.3, linewidth = 0.8) +
    ggplot2::scale_fill_manual(values = c("Target" = "#2E86AB", "Decoy" = "#A23B72")) +
    ggplot2::scale_color_manual(values = c("Target" = "#2E86AB", "Decoy" = "#A23B72")) +
    ggplot2::labs(
      title = plot_title,
      x = x_label,
      y = "Density",
      fill = "",
      color = ""
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      legend.position = "top",
      plot.title = ggplot2::element_text(face = "bold")
    )

  if (use_log) {
    p <- p +
      ggplot2::scale_x_log10(labels = scales::label_number(accuracy = 0.001)) +
      ggplot2::annotation_logticks(sides = "b")
  }

  p
}

#' Plot scope disagreement as a Jaccard overlap heatmap
#'
#' Computes accepted target ID sets for each list at a fixed \code{alpha} and
#' visualises pairwise Jaccard overlaps as a heatmap. IDs are compared at the
#' "base" level by stripping any \code{"run||"} prefix.
#'
#' @param xs Named list of \code{dfdr_tbl} objects.
#' @param alpha Numeric threshold used to define accepted targets (\code{q <= alpha}).
#'
#' @return A \link[ggplot2:ggplot]{ggplot} object (heatmap).
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 2000
#'
#' make_x <- function(label) {
#'   df <- tibble(
#'     id = paste0("run1||", as.character(seq_len(n))),
#'     run = "run1",
#'     is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'     score = rnorm(n),
#'     q = stats::runif(n, 0, 0.05),
#'     pep = NA_real_
#'   )
#'   as_dfdr_tbl(df, unit = "psm", scope = label, q_source = "toy")
#' }
#'
#' xs <- list(A = make_x("A"), B = make_x("B"), C = make_x("C"))
#' dfdr_plot_scope_disagreement_matrix(xs, alpha = 0.01)
#'
#' @export
dfdr_plot_scope_disagreement_matrix <- function(xs, alpha = 0.01) {
  extract_base_id <- function(ids) {
    ifelse(grepl("\\|\\|", ids), sub("^.*\\|\\|", "", ids), ids)
  }

  accepted <- purrr::map(xs, ~{
    .x |>
      dplyr::filter(!.data$is_decoy, .data$q <= alpha) |>
      dplyr::pull(.data$id) |>
      extract_base_id() |>
      unique()
  })

  n_lists <- length(accepted)
  list_names <- names(accepted)

  jaccard_mat <- matrix(NA_real_, n_lists, n_lists, dimnames = list(list_names, list_names))

  for (i in seq_len(n_lists)) {
    for (j in seq_len(n_lists)) {
      if (i == j) {
        jaccard_mat[i, j] <- 1.0
      } else {
        inter <- length(intersect(accepted[[i]], accepted[[j]]))
        uni <- length(union(accepted[[i]], accepted[[j]]))
        jaccard_mat[i, j] <- if (uni > 0) inter / uni else 0
      }
    }
  }

  jaccard_df <- as.data.frame(jaccard_mat) |>
    tibble::rownames_to_column("list1") |>
    tidyr::pivot_longer(
      cols = -dplyr::all_of("list1"),
      names_to = "list2",
      values_to = "jaccard"
    )

  ggplot2::ggplot(jaccard_df, ggplot2::aes(x = .data$list1, y = .data$list2, fill = .data$jaccard)) +
    ggplot2::geom_tile(color = "white", linewidth = 1) +
    ggplot2::geom_text(ggplot2::aes(label = sprintf("%.2f", .data$jaccard)),
                       size = 5, fontface = "bold") +
    ggplot2::scale_fill_gradient2(
      low = "#d73027", mid = "#fee08b", high = "#1a9850",
      midpoint = 0.85, limits = c(0, 1),
      name = "Jaccard\noverlap"
    ) +
    ggplot2::labs(
      title = sprintf("Scope disagreement at alpha = %.2g", alpha),
      subtitle = "Jaccard overlap of accepted target sets (precursor-level comparison)",
      x = "", y = ""
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 11),
      axis.text.y = ggplot2::element_text(size = 11),
      plot.title = ggplot2::element_text(face = "bold", size = 14),
      plot.subtitle = ggplot2::element_text(size = 11)
    ) +
    ggplot2::coord_fixed()
}

#' Plot PEP density by decoy/target
#'
#' Plots density curves of PEP values for targets vs decoys, truncated to
#' \code{pep <= pep_max}.
#'
#' @param x A \code{dfdr_tbl} with a \code{pep} column.
#' @param pep_max Numeric. Maximum PEP displayed (default 0.5).
#' @param bw Bandwidth passed to \code{ggplot2::geom_density()}.
#' @param adjust Numeric smoothing adjustment passed to \code{geom_density()}.
#' @param title Character. Plot title.
#'
#' @return A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 4000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   q = stats::runif(n, 0, 0.05),
#'   pep = stats::runif(n)
#' )
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")
#' dfdr_plot_pep_density_by_decoy(x, pep_max = 0.5)
#'
#' @export
dfdr_plot_pep_density_by_decoy <- function(x, pep_max = 0.5,
                                           bw = "nrd0",
                                           adjust = 1,
                                           title = "PEP distribution (density): targets vs decoys") {
  validate_dfdr_tbl(x)
  if (!"pep" %in% names(x)) rlang::abort("`x` must contain a `pep` column.")

  df <- x |>
    dplyr::filter(is.finite(.data$pep), .data$pep >= 0, .data$pep <= pep_max) |>
    dplyr::mutate(class = dplyr::if_else(.data$is_decoy, "Decoy", "Target"))

  ggplot2::ggplot(df, ggplot2::aes(x = .data$pep, color = .data$class, fill = .data$class)) +
    ggplot2::geom_density(linewidth = 1, alpha = 0.25, bw = bw, adjust = adjust, na.rm = TRUE) +
    ggplot2::scale_color_manual(values = c(Target = "#2b8cbe", Decoy = "#d7301f")) +
    ggplot2::scale_fill_manual(values = c(Target = "#2b8cbe", Decoy = "#d7301f")) +
    ggplot2::labs(
      title = title,
      subtitle = sprintf("Density truncated to pep <= %.2f", pep_max),
      x = "PEP", y = "Density",
      color = NULL, fill = NULL
    ) +
    ggplot2::theme_minimal()
}

#' Plot target-focused PEP reliability (TDC-style)
#'
#' Visualises the output of \code{\link{dfdr_pep_reliability_tdc}} by plotting the
#' estimated target error proxy \code{err_hat_target} against the mean predicted
#' target PEP per bin. Point size reflects the number of targets per bin.
#'
#' @param rel_tbl Output of \code{\link{dfdr_pep_reliability_tdc}}.
#' @param title Character. Plot title.
#'
#' @return A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' rel_tbl <- tibble(
#'   bin = c("(0,0.1]", "(0.1,0.2]", "(0.2,0.3]"),
#'   n_target = c(800, 500, 200),
#'   n_decoy = c(20, 30, 40),
#'   pep_mean_targets = c(0.05, 0.15, 0.25),
#'   pep_mean_all = c(0.06, 0.17, 0.28),
#'   err_hat_target = c(0.03, 0.06, 0.20),
#'   pep_bin_mid = c(0.05, 0.15, 0.25)
#' )
#' dfdr_plot_pep_reliability_tdc(rel_tbl)
#'
#' @export
dfdr_plot_pep_reliability_tdc <- function(rel_tbl,
                                          title = "PEP reliability (targets; TDC-style D/T error proxy)") {
  ggplot2::ggplot(rel_tbl, ggplot2::aes(x = .data$pep_mean_targets, y = .data$err_hat_target)) +
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed", color = "grey50") +
    ggplot2::geom_point(ggplot2::aes(size = .data$n_target), alpha = 0.8) +
    ggplot2::scale_size_continuous(range = c(2, 10)) +
    ggplot2::coord_cartesian(
      xlim = c(0, max(rel_tbl$pep_mean_targets, na.rm = TRUE)),
      ylim = c(0, 1)
    ) +
    ggplot2::labs(
      title = title,
      x = "Mean predicted PEP (targets in bin)",
      y = "Estimated false fraction among targets (min(1, (D+add)/T))",
      size = "Targets/bin"
    ) +
    ggplot2::theme_minimal()
}


#' Plot (pseudo-)p-value density by decoy/target
#'
#' Plots density curves of (pseudo-)p-values for targets vs decoys. Optionally
#' plots \eqn{-\log_{10}(p)} to emphasise small p-values.
#'
#' @param x A \code{dfdr_tbl} with columns \code{p} and \code{is_decoy}.
#' @param p_max Numeric. Maximum p-value displayed (default 1).
#' @param bw Bandwidth passed to \code{ggplot2::geom_density()}.
#' @param adjust Numeric smoothing adjustment passed to \code{geom_density()}.
#' @param title Character. Plot title.
#' @param log10_x Logical. If \code{TRUE}, plot \eqn{-\log_{10}(p)} instead of \code{p}.
#' @param eps Numeric. Lower bound used to avoid \code{Inf} when \code{log10_x = TRUE}.
#'
#' @return A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 4000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   q = stats::runif(n, 0, 0.05),
#'   pep = NA_real_,
#'   p = stats::runif(n)
#' )
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy", p_source = "toy")
#' dfdr_plot_p_density_by_decoy(x, log10_x = TRUE)
#'
#' @export
dfdr_plot_p_density_by_decoy <- function(x,
                                         p_max = 1,
                                         bw = "nrd0",
                                         adjust = 1,
                                         title = "(Pseudo-)p-value distribution (density): targets vs decoys",
                                         log10_x = FALSE,
                                         eps = 1e-300) {
  validate_dfdr_tbl(x)
  if (!"p" %in% names(x)) rlang::abort("`x` must contain a `p` column.")

  df <- x |>
    dplyr::filter(is.finite(.data$p), .data$p > 0, .data$p <= p_max) |>
    dplyr::mutate(
      class = dplyr::if_else(.data$is_decoy, "Decoy", "Target"),
      xval = if (log10_x) -log10(pmax(.data$p, eps)) else .data$p
    )

  xlab <- if (log10_x) expression(-log[10](p)) else "p"

  ggplot2::ggplot(df, ggplot2::aes(x = .data$xval, color = .data$class, fill = .data$class)) +
    ggplot2::geom_density(linewidth = 1, alpha = 0.25, bw = bw, adjust = adjust, na.rm = TRUE) +
    ggplot2::scale_color_manual(values = c(Target = "#2b8cbe", Decoy = "#d7301f")) +
    ggplot2::scale_fill_manual(values = c(Target = "#2b8cbe", Decoy = "#d7301f")) +
    ggplot2::labs(
      title = title,
      subtitle = if (log10_x) sprintf("Using -log10(p); p truncated to p <= %.2g", p_max)
      else sprintf("p truncated to p <= %.2g", p_max),
      x = xlab, y = "Density",
      color = NULL, fill = NULL
    ) +
    ggplot2::theme_minimal()
}


#' Plot Storey-style estimated FDR curve: \eqn{\hat\pi_0 m t / R(t)}
#'
#' Given p-values (or pseudo-p-values), estimates \eqn{\pi_0} using
#' \code{cp4p::estim.pi0()} and plots
#' \deqn{\widehat{\mathrm{FDR}}(t) = \hat\pi_0 \, m \, t / R(t),}
#' where \eqn{R(t) = \#\{p \le t\}} and \eqn{m} is the number of finite p-values.
#'
#' @param x A \code{dfdr_tbl} (or data.frame) containing columns \code{id},
#'   \code{p}, and \code{is_decoy}.
#' @param sel Character. Subset to use: \code{"all"}, \code{"target"}, or \code{"decoy"}.
#' @param pi0.method Character. Method passed to \code{cp4p::estim.pi0()} (not \code{"ALL"}).
#' @param nbins Integer. Passed to \code{cp4p::estim.pi0()}.
#' @param pz Numeric. Passed to \code{cp4p::estim.pi0()}.
#' @param t_grid Numeric vector of thresholds in \eqn{(0,1]} at which to evaluate the curve.
#' @param r_min Numeric. Minimum value used to determine when to compute the curve
#'   to avoid division by zero at tiny \code{t}.
#' @param cap_one Logical. If \code{TRUE}, caps \code{fdr_hat} at 1.
#'
#' @return
#' A list with components:
#' \describe{
#'   \item{plot}{A \link[ggplot2:ggplot]{ggplot} object.}
#'   \item{data}{A \link[tibble:tibble]{tibble} with columns \code{t}, \code{R}, and \code{fdr_hat}.}
#'   \item{pi0_hat}{Numeric scalar \eqn{\hat\pi_0}.}
#'   \item{m}{Integer. Number of finite p-values used.}
#' }
#'
#' @examples
#' library(tibble)
#'
#' if (requireNamespace("cp4p", quietly = TRUE)) {
#'   set.seed(1)
#'   n <- 4000
#'   df <- tibble(
#'     id = as.character(seq_len(n)),
#'     is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'     p = c(stats::runif(3600), stats::rbeta(400, 0.3, 1))
#'   )
#'   out <- dfdr_plot_fdrhat_pi0(df, sel = "all", pi0.method = "pounds", nbins = 10)
#'   out$pi0_hat
#'   out$plot
#' }
#'
#' @export
dfdr_plot_fdrhat_pi0 <- function(x,
                                 sel = c("all","target","decoy"),
                                 pi0.method = "pounds",
                                 nbins = 20,
                                 pz = 0.05,
                                 t_grid = 10^seq(-6, 0, length.out = 400),
                                 r_min = 1,
                                 cap_one = TRUE) {
  x <- .safe_df(x)
  .check_has_cols(x, c("id","p","is_decoy"))

  sel <- match.arg(tolower(sel), c("all","target","decoy"))
  xx <- x
  if (sel == "target") xx <- dplyr::filter(xx, !.data$is_decoy)
  if (sel == "decoy")  xx <- dplyr::filter(xx,  .data$is_decoy)

  p <- suppressWarnings(as.numeric(xx$p))
  p <- p[is.finite(p) & p >= 0 & p <= 1]
  m <- length(p)
  if (m < 50) rlang::abort("Too few valid p-values to estimate pi0/FDR curve reliably.")

  r <- cp4p::estim.pi0(p = p, pi0.method = pi0.method, nbins = nbins, pz = pz)
  pi0_hat <- as.numeric(r$pi0)

  p_sorted <- sort(p)
  t_grid <- t_grid[is.finite(t_grid) & t_grid > 0 & t_grid <= 1]
  R <- findInterval(t_grid, p_sorted, rightmost.closed = TRUE)

  fdr_hat <- rep(NA_real_, length(t_grid))
  ok <- R >= r_min
  fdr_hat[ok] <- (pi0_hat * m * t_grid[ok]) / R[ok]
  if (cap_one) fdr_hat <- pmin(1, fdr_hat)

  df <- tibble::tibble(t = t_grid, R = R, fdr_hat = fdr_hat)

  g <- ggplot2::ggplot(df, ggplot2::aes(t, fdr_hat)) +
    ggplot2::geom_line(linewidth = 0.9, na.rm = TRUE) +
    ggplot2::scale_x_log10() +
    ggplot2::coord_cartesian(ylim = c(0,1)) +
    ggplot2::labs(
      title = expression(hat(FDR)(t) == frac(hat(pi)[0] * m * t, R(t))),
      subtitle = paste0("subset=", sel, " | pi0_hat=", signif(pi0_hat,3),
                        " | m=", m, " | r_min=", r_min),
      x = "t (p-value threshold, log scale)",
      y = expression(hat(FDR)(t))
    ) +
    ggplot2::theme_minimal()

  list(plot = g, data = df, pi0_hat = pi0_hat, m = m)
}

#' Detect Spectronaut run column name
#' @keywords internal
spectronaut_detect_run_col <- function(df, candidates = c("R.FileName", "R.Condition", "File.Name")) {
  nm <- names(df)
  hit <- candidates[candidates %in% nm]
  if (length(hit) == 0) return(NA_character_)
  hit[[1]]
}

#' Validate that required Spectronaut columns exist
#' @keywords internal
spectronaut_require_cols <- function(df, cols) {
  miss <- setdiff(cols, names(df))
  if (length(miss)) rlang::abort(paste0("Missing required Spectronaut columns: ", paste(miss, collapse = ", ")))
  invisible(TRUE)
}

#' Parse Spectronaut boolean decoy column
#' @keywords internal
spectronaut_parse_decoy <- function(x) {
  # Spectronaut uses "True"/"False" or TRUE/FALSE
  if (is.logical(x)) return(x)
  tolower(as.character(x)) %in% c("true", "1")
}

#' Recompute q-values from scores using target-decoy competition
#'
#' When Spectronaut exports NaN q-values for decoys, we can recompute them
#' from scores. Higher scores are better (more confident).
#'
#' @param df Data frame with columns: is_decoy, score
#' @return Data frame with added column: q_from_score
#' @keywords internal
spectronaut_recompute_q_from_score <- function(df) {
  df |>
    dplyr::arrange(dplyr::desc(.data$score)) |>
    dplyr::mutate(
      rank = dplyr::row_number(),
      cumsum_decoy = cumsum(.data$is_decoy),
      cumsum_total = dplyr::row_number(),
      q_from_score = pmin(1, .data$cumsum_decoy / .data$cumsum_total)
    ) |>
    dplyr::arrange(dplyr::desc(.data$rank)) |>
    dplyr::mutate(q_from_score = cummin(.data$q_from_score)) |>
    dplyr::arrange(.data$rank) |>
    dplyr::select(-.data$rank, -.data$cumsum_decoy, -.data$cumsum_total)
}

#' Read a Spectronaut TSV report
#'
#' Reads a Spectronaut report exported as tab-separated values (TSV). Uses
#' \code{data.table::fread()} for efficient reading of large files.
#'
#' Spectronaut may use a comma as decimal separator in some locales; set
#' \code{dec=","} when needed.
#'
#' @param path Character scalar. Path to a Spectronaut report TSV file.
#' @param dec Character scalar. Decimal separator passed to \code{fread()} (default \code{"."}).
#' @param select Optional character vector of column names to read; \code{NULL} reads all columns.
#'
#' @return
#' A \link[tibble:tibble]{tibble} containing the columns present in the Spectronaut
#' report (column names depend on the export configuration).
#'
#' @examples
#' if (requireNamespace("data.table", quietly = TRUE)) {
#'   # Create a tiny TSV and read it back
#'   tmp <- tempfile(fileext = ".tsv")
#'   txt <- paste0(
#'     "R.FileName\tEG.PrecursorId\tEG.IsDecoy\tEG.Qvalue\tEG.PEP\tEG.Cscore\n",
#'     "run1\tP1\tFalse\t0.01\t0.02\t120\n",
#'     "run1\tP2\tTrue\tNaN\t0.90\t10\n"
#'   )
#'   writeLines(txt, tmp)
#'   read_spectronaut(tmp)
#' }
#'
#' @export
read_spectronaut <- function(path, dec = ".", select = NULL) {
  if (!requireNamespace("data.table", quietly = TRUE)) {
    rlang::abort("Reading Spectronaut requires 'data.table' package (install.packages('data.table')).")
  }
  if (!file.exists(path)) rlang::abort(paste0("File not found: ", path))

  # Spectronaut exports can be huge; fread is much faster than read.delim
  dt <- data.table::fread(
    path,
    sep = "\t",
    dec = dec,
    select = select,
    showProgress = TRUE
  )

  tibble::as_tibble(dt)
}

#' Read Spectronaut efficiently with column selection
#'
#' Convenience wrapper around \code{\link{read_spectronaut}}. For very large
#' Spectronaut reports, reading only a minimal set of columns can substantially
#' speed up I/O and reduce memory usage.
#'
#' @param path Character scalar. Path to a Spectronaut TSV report.
#' @param minimal Logical. If \code{TRUE} (default), reads only columns typically
#'   needed for precursor-level diagnostics; if \code{FALSE}, reads all columns.
#' @param dec Character scalar. Decimal separator (use \code{","} for some locales).
#'
#' @return
#' A \link[tibble:tibble]{tibble} containing either the selected minimal columns
#' (if \code{minimal=TRUE}) or all columns (if \code{minimal=FALSE}).
#'
#' @examples
#' if (requireNamespace("data.table", quietly = TRUE)) {
#'   tmp <- tempfile(fileext = ".tsv")
#'   txt <- paste0(
#'     "R.FileName\tR.Condition\tEG.PrecursorId\tEG.IsDecoy\tEG.Qvalue\tEG.PEP\tEG.Cscore\n",
#'     "run1\tcondA\tP1\tFalse\t0.01\t0.02\t120\n",
#'     "run1\tcondA\tP2\tTrue\tNaN\t0.90\t10\n"
#'   )
#'   writeLines(txt, tmp)
#'   read_spectronaut_efficient(tmp, minimal = TRUE)
#' }
#'
#' @export
read_spectronaut_efficient <- function(path, minimal = TRUE, dec = ".") {
  if (minimal) {
    select_cols <- c(
      "R.FileName", "R.Condition",
      "EG.PrecursorId", "EG.IsDecoy",
      "EG.Qvalue", "EG.GlobalPrecursorQvalue",
      "EG.PEP", "EG.Cscore", "EG.NormalizedCscore"
    )
    read_spectronaut(path, dec = dec, select = select_cols)
  } else {
    read_spectronaut(path, dec = dec, select = NULL)
  }
}

#' Spectronaut -> run-by-precursor universe (deduplicated within run by \code{EG.PrecursorId})
#'
#' Constructs a run-wise precursor universe from a Spectronaut report. The output
#' is suitable for run-wise FDR diagnostics because each row corresponds to a
#' \code{run} \eqn{\times} \code{precursor} hypothesis.
#'
#' If Spectronaut exports \code{NaN} q-values for decoys (common), the function can
#' optionally recompute q-values from scores per run using target-decoy competition.
#'
#' @param rep Spectronaut report tibble (e.g. returned by \code{\link{read_spectronaut}}
#'   or \code{\link{read_spectronaut_efficient}}).
#' @param q_col Character. q-value column name (default \code{"EG.Qvalue"} for run-wise control).
#' @param run_col Optional character. Run column name; if \code{NULL}, attempts to
#'   detect one of \code{"R.FileName"} or \code{"R.Condition"}.
#' @param pep_col Optional character. PEP column name (default \code{"EG.PEP"}).
#' @param score_col Optional character. Score column name (default \code{"EG.Cscore"}).
#' @param q_max_export Numeric. Export ceiling for q-values (default 1.0), stored in metadata.
#' @param recompute_q Logical. If \code{TRUE} (default) and decoys lack finite q-values,
#'   recompute q-values from \code{score_col} per run.
#' @param id_mode Character. \code{"id"} (default) uses \code{id = EG.PrecursorId} and keeps
#'   \code{run} separate; \code{"runxid"} uses \code{id = "run||precursor"} for global uniqueness.
#' @param unit Character. Unit metadata stored in the returned object.
#' @param scope Character. Scope metadata stored in the returned object.
#' @param q_source Character. Source label stored in metadata.
#'
#' @return
#' A \code{dfdr_tbl} (tibble subclass) with one row per \code{run} \eqn{\times}
#' \code{precursor}. The returned table contains the standard columns required by
#' \code{diagFDR}: \code{id}, \code{is_decoy}, \code{q}, \code{pep}, \code{run}, \code{score}.
#' Metadata (including whether q-values were recomputed) are stored in \code{attr(x, "meta")}.
#'
#' @details
#' When Spectronaut exports decoys with \code{NaN} q-values, recomputing q-values
#' from scores is performed *per run* to respect the run-wise hypothesis structure
#' and to avoid pooling scores across runs that may not be comparable.
#'
#' @examples
#' library(tibble)
#'
#' # Minimal toy Spectronaut-like report
#' rep <- tibble(
#'   R.FileName = c("run1","run1","run1","run2","run2","run2"),
#'   EG.PrecursorId = c("P1","P2","P3","P1","P2","P4"),
#'   EG.IsDecoy = c("False","True","False","False","True","False"),
#'   EG.Qvalue = c(0.01, NaN, 0.02, 0.03, NaN, 0.04),
#'   EG.PEP = c(0.02, 0.9, 0.05, 0.06, 0.95, 0.08),
#'   EG.Cscore = c(120, 10, 80, 100, 5, 60)
#' )
#'
#' x <- spectronaut_runxprecursor(
#'   rep,
#'   q_col = "EG.Qvalue",
#'   run_col = "R.FileName",
#'   recompute_q = TRUE,
#'   id_mode = "runxid"
#' )
#' x
#'
#' @export
spectronaut_runxprecursor <- function(rep,
                                      q_col = "EG.Qvalue",
                                      run_col = NULL,
                                      pep_col = "EG.PEP",
                                      score_col = "EG.Cscore",
                                      q_max_export = 1.0,
                                      recompute_q = TRUE,
                                      id_mode = c("id", "runxid"),
                                      unit = "runxprecursor",
                                      scope = "runwise",
                                      q_source = q_col) {
  spectronaut_require_cols(rep, c("EG.PrecursorId", "EG.IsDecoy"))

  id_mode <- match.arg(id_mode)

  if (is.null(run_col)) run_col <- spectronaut_detect_run_col(rep)
  if (is.na(run_col)) rlang::abort("No run column found (expected R.FileName or R.Condition).")

  # Check if we need to recompute q-values
  needs_recompute <- FALSE
  if (recompute_q && !is.null(score_col) && score_col %in% names(rep)) {
    n_decoy <- sum(rep$EG.IsDecoy == TRUE)
    n_decoy_finite_q <- sum(rep$EG.IsDecoy == TRUE & is.finite(rep[[q_col]]))
    if (n_decoy > 0 && n_decoy_finite_q == 0) {
      needs_recompute <- TRUE
      message("Spectronaut decoys have NaN q-values; recomputing PER RUN from ", score_col)
    }
  }

  has_pep <- !is.null(pep_col) && pep_col %in% names(rep)
  has_score <- !is.null(score_col) && score_col %in% names(rep)

  dd <- rep |>
    dplyr::transmute(
      run = as.character(.data[[run_col]]),
      precursor = as.character(.data[["EG.PrecursorId"]]),
      is_decoy = spectronaut_parse_decoy(.data[["EG.IsDecoy"]]),
      q_original = if (q_col %in% names(rep)) suppressWarnings(as.numeric(.data[[q_col]])) else NA_real_,
      pep = if (has_pep) suppressWarnings(as.numeric(.data[[pep_col]])) else NA_real_,
      score = if (has_score) suppressWarnings(as.numeric(.data[[score_col]])) else NA_real_
    ) |>
    dplyr::filter(!is.na(run), !is.na(precursor))

  # Recompute q-values from scores if needed (PER RUN to respect run-level hypotheses)
  if (needs_recompute) {
    dd <- dd |>
      dplyr::filter(is.finite(score)) |>
      dplyr::group_by(run) |>
      dplyr::group_modify(~ spectronaut_recompute_q_from_score(.x)) |>
      dplyr::ungroup() |>
      dplyr::mutate(q = .data$q_from_score) |>
      dplyr::select(-.data$q_from_score, -(.data$q_original))
    q_source <- paste0(q_source, " (recomputed per run from ", score_col, ")")
  } else {
    dd <- dd |>
      dplyr::mutate(q = .data$q_original) |>
      dplyr::select(-.data$q_original) |>
      dplyr::filter(is.finite(q), q >= 0, q <= 1)
  }

  # Deduplicate within run (in case of duplicates within same run)
  dd <- dd |>
    dplyr::group_by(run, precursor) |>
    dplyr::summarise(
      is_decoy = any(is_decoy),
      q = safe_min(q),
      pep = safe_min(pep),
      score = safe_min(score),
      .groups = "drop"
    ) |>
    dplyr::mutate(
      id = if (id_mode == "runxid") paste(run, precursor, sep = "||") else precursor
    ) |>
    dplyr::select(id, is_decoy, q, pep, run, score)

  as_dfdr_tbl(
    dd,
    unit = unit,
    scope = scope,
    q_source = q_source,
    q_max_export = q_max_export,
    provenance = list(tool = "Spectronaut", run_col = run_col, id_mode = id_mode, recomputed = needs_recompute)
  )
}

#' @importFrom utils head tail
NULL

#' @importFrom rlang .data
NULL

#' Internal: "or else" helper for defaults
#'
#' Returns \code{a} if it is non-\code{NULL} and contains at least one non-\code{NA}
#' value; otherwise returns \code{b}. This is used throughout the package to
#' provide fallbacks for optional metadata fields.
#'
#' @param a,b Objects to choose from.
#'
#' @return \code{a} if available, otherwise \code{b}.
#'
#' @keywords internal
#' @noRd
`%||%` <- function(a, b) {
  if (!is.null(a) && length(a) > 0 && !all(is.na(a))) a else b
}

#' Internal: safe finite minimum
#'
#' Returns the minimum of finite values in \code{x}. Non-finite values
#' (\code{NA}, \code{Inf}, \code{-Inf}) are removed. If no finite values remain,
#' returns \code{NA_real_}.
#'
#' @param x Numeric vector.
#'
#' @return Numeric scalar minimum of finite values, or \code{NA_real_}.
#'
#' @keywords internal
safe_min <- function(x) {
  x <- x[is.finite(x)]
  if (length(x) == 0) return(NA_real_)
  min(x)
}

#' Internal: Jaccard similarity for two ID vectors
#'
#' Computes the Jaccard index \eqn{|A \cap B|/|A \cup B|} on unique values of \code{a}
#' and \code{b}. Returns \code{NA_real_} if either set is empty (or both are empty),
#' which is convenient for diagnostics where the overlap is undefined without
#' discoveries in both sets.
#'
#' @param a,b Vectors representing sets.
#'
#' @return Numeric scalar in \eqn{[0,1]} or \code{NA_real_} if undefined.
#'
#' @keywords internal
jaccard_vec <- function(a, b) {
  a <- unique(a); b <- unique(b)
  if (length(a) == 0 && length(b) == 0) return(NA_real_)
  if (length(a) == 0 || length(b) == 0) return(NA_real_)
  length(intersect(a, b)) / length(union(a, b))
}

#' Internal: Wilson score confidence interval for a binomial proportion
#'
#' Computes a Wilson score interval for \eqn{x/n} at confidence level \code{conf}.
#' Returns \code{c(NA, NA)} when \code{n == 0}.
#'
#' @param x Integer number of successes.
#' @param n Integer number of trials.
#' @param conf Confidence level in \eqn{(0,1)} (default 0.95).
#'
#' @return Numeric vector of length 2: lower and upper confidence bounds.
#'
#' @keywords internal
wilson_ci <- function(x, n, conf = 0.95) {
  if (n == 0) return(c(NA_real_, NA_real_))
  z <- stats::qnorm(1 - (1 - conf) / 2)
  phat <- x / n
  denom <- 1 + z^2 / n
  center <- (phat + z^2 / (2*n)) / denom
  half <- (z * sqrt(phat*(1 - phat)/n + z^2/(4*n^2))) / denom
  c(center - half, center + half)
}

#' Internal: choose a low-confidence q-interval based on export range
#'
#' Chooses an interval for pooled "low-confidence" diagnostics depending on the
#' available maximum q-value \code{qmax}. If \code{qmax} covers the default interval,
#' returns \code{low_default}; otherwise falls back to an interval capped by \code{qmax}.
#'
#' @param qmax Numeric scalar. Maximum available q-value.
#' @param low_default Length-2 numeric vector. Default low-confidence interval.
#' @param low_fallback Length-2 numeric vector. Fallback interval when \code{qmax} is smaller.
#'
#' @return Length-2 numeric vector \code{c(lo, hi)}.
#'
#' @keywords internal
choose_low_conf <- function(qmax,
                            low_default = c(0.2, 0.5),
                            low_fallback = c(0.1, 0.2)) {
  if (is.finite(qmax) && qmax >= low_default[2] - 1e-12) return(low_default)
  c(low_fallback[1], min(low_fallback[2], qmax))
}

#' Internal: make a string safe for filenames
#'
#' Replaces characters that are typically unsafe in filenames with underscores.
#' Keeps letters, digits, dot, underscore, and hyphen.
#'
#' @param x Character vector.
#'
#' @return Character vector of the same length with unsafe characters replaced.
#'
#' @keywords internal
dfdr_safe_filename <- function(x) {
  # replace anything not safe for filenames with "_"
  gsub("[^A-Za-z0-9._-]+", "_", x)
}


#' Safe minimum that handles NA values
#'
#' Returns the minimum of x, with NA values removed.
#' If all values are NA, returns NA_real_.
#'
#' @param x Numeric vector
#' @return Minimum value or NA_real_
#' @keywords internal
safe_min <- function(x) {
  if (all(is.na(x))) return(NA_real_)
  min(x, na.rm = TRUE)
}

#' Internal: safe maximum (NA-robust)
#'
#' Returns \code{max(x, na.rm = TRUE)}. If all values are \code{NA}, returns
#' \code{NA_real_}.
#'
#' @param x Numeric vector.
#'
#' @return Numeric scalar maximum, or \code{NA_real_}.
#'
#' @keywords internal
safe_max <- function(x) {
  if (all(is.na(x))) return(NA_real_)
  max(x, na.rm = TRUE)
}

#' Internal: safe mean (NA-robust)
#'
#' Returns \code{mean(x, na.rm = TRUE)}. If all values are \code{NA}, returns
#' \code{NA_real_}.
#'
#' @param x Numeric vector.
#'
#' @return Numeric scalar mean, or \code{NA_real_}.
#'
#' @keywords internal
safe_mean <- function(x) {
  if (all(is.na(x))) return(NA_real_)
  mean(x, na.rm = TRUE)
}

#' Write diagnostic outputs to a folder
#'
#' Writes tables as CSV and plots as PNG, optionally creates a PPTX (requires officer+rvg).
#'
#' @param diag Output of \code{\link{dfdr_run_all}}.
#' @param out_dir Output directory.
#' @param formats Character vector subset of \code{c("csv","png","pptx")}.
#' @param pptx_path PPTX path (only used if \code{"pptx"} in formats).
#' @param width,height Plot size passed to \code{ggplot2::ggsave}.
#' @param dpi DPI passed to \code{ggplot2::ggsave}.
#'
#' @return Invisibly returns \code{TRUE}.
#' @export
dfdr_write_report <- function(diag,
                               out_dir,
                               formats = c("csv", "png"),
                               pptx_path = file.path(out_dir, "dfdr_plots.pptx"),
                               width = 7.2, height = 4.4, dpi = 180) {
  if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)
  formats <- unique(formats)

  # ---- CSV tables
  if ("csv" %in% formats) {
    tabs <- diag$tables
    # tables that are data frames
    for (nm in names(tabs)) {
      obj <- tabs[[nm]]
      if (is.data.frame(obj)) {
        readr::write_csv(obj, file.path(out_dir, paste0(nm, ".csv")))
      } else if (is.list(obj)) {
        # list of tables (e.g. sumpep per list)
        subdir <- file.path(out_dir, nm)
        dir.create(subdir, showWarnings = FALSE, recursive = TRUE)
        for (k in names(obj)) {
          if (is.data.frame(obj[[k]])) {
            readr::write_csv(obj[[k]], file.path(subdir, paste0(k, ".csv")))
          }
        }
      }
    }
  }

  # ---- PNG plots
  if ("png" %in% formats) {
    pdir <- file.path(out_dir, "plots")
    dir.create(pdir, showWarnings = FALSE, recursive = TRUE)
    for (nm in names(diag$plots)) {
      ggplot2::ggsave(file.path(pdir, paste0(nm, ".png")),
                      diag$plots[[nm]], width = width, height = height, dpi = dpi)
    }
  }

  # ---- PPTX (optional; requires officer+rvg)
  if ("pptx" %in% formats) {
    if (!requireNamespace("officer", quietly = TRUE) ||
        !requireNamespace("rvg", quietly = TRUE)) {
      rlang::abort("PPTX export requires packages 'officer' and 'rvg' in Suggests.")
    }

    doc <- officer::read_pptx()
    for (nm in names(diag$plots)) {
      doc <- officer::add_slide(doc, layout = "Title and Content", master = "Office Theme")
      doc <- officer::ph_with(doc, value = nm, location = officer::ph_location_type(type = "title"))
      doc <- officer::ph_with(doc, value = rvg::dml(ggobj = diag$plots[[nm]]),
                              location = officer::ph_location_type(type = "body"))
    }
    print(doc, target = pptx_path)
  }

  invisible(TRUE)
}

#' P-value calibration diagnostic (ECDF vs uniform; stratified)
#'
#' Computes the empirical CDF \eqn{\hat F(u) = P(p \le u)} on a grid of \code{u}
#' values and summarises departures from uniformity in a decision-relevant region
#' \eqn{u \le u_{\max}}. Intended as a plausibility diagnostic for (pseudo-)p-values
#' (e.g. under the null, \eqn{\hat F(u) \approx u}).
#'
#' @param x A \code{dfdr_tbl} (or data.frame) containing columns \code{id} and \code{p}.
#' @param u_grid Numeric vector in \eqn{(0,1]}. Grid of \code{u} values for evaluating the ECDF.
#' @param u_max Numeric scalar in \eqn{(0,1]}. Upper bound of \code{u} used for
#'   inflation summaries (default 0.1).
#' @param stratify Optional character vector of column names used to stratify
#'   diagnostics (e.g. \code{c("run")}).
#' @param min_n Integer. Minimum number of finite p-values required per stratum.
#'
#' @return
#' A list with components:
#' \describe{
#'   \item{ecdf}{A \link[tibble:tibble]{tibble} with columns \code{stratum},
#'   \code{u}, \code{Fhat}, and \code{n}. There is one row per \code{u} value per
#'   stratum.}
#'   \item{summary}{A tibble with one row per stratum, including \code{n} (number
#'   of finite p-values), \code{max_inflation} (maximum of \code{Fhat(u)-u} for
#'   \code{u <= u_max}), and \code{auc_inflation} (area under the positive part of
#'   \code{Fhat(u)-u} on \eqn{[0,u_{\max}]}, normalised by \code{u_max}). Strata
#'   with \code{n < min_n} are reported with \code{NA} metrics and a \code{note}.}
#' }
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 5000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = sample(c("run1", "run2"), n, replace = TRUE),
#'   # mostly uniform p-values + a small enriched component
#'   p = c(stats::runif(4500), stats::rbeta(500, 0.3, 1))
#' )
#'
#' out <- dfdr_p_calibration(df, stratify = "run", u_max = 0.1, min_n = 200)
#' head(out$ecdf)
#' out$summary
#'
#' @export
dfdr_p_calibration <- function(x,
                               u_grid = c(seq(0.001, 0.1, by = 0.001), seq(0.11, 1, by = 0.01)),
                               u_max = 0.1,
                               stratify = NULL,
                               min_n = 200) {
  x <- .safe_df(x)
  .check_has_cols(x, c("id", "p"))

  if (!is.numeric(u_grid) || any(!is.finite(u_grid)) || any(u_grid <= 0) || any(u_grid > 1)) {
    rlang::abort("`u_grid` must be numeric values in (0,1].")
  }
  u_grid <- sort(unique(u_grid))
  if (!is.numeric(u_max) || length(u_max) != 1 || !is.finite(u_max) || u_max <= 0 || u_max > 1) {
    rlang::abort("`u_max` must be a single number in (0,1].")
  }

  strata_df <- .get_strata_df(x, stratify)
  if (is.null(strata_df)) {
    x$stratum <- "all"
  } else {
    x$stratum <- .make_stratum_id(strata_df)
  }

  out_ecdf <- list()
  out_sum <- list()

  for (s in unique(x$stratum)) {
    xs <- x[x$stratum == s, , drop = FALSE]
    p <- xs$p
    p <- p[is.finite(p)]
    n <- length(p)

    if (n < min_n) {
      out_sum[[s]] <- tibble::tibble(
        stratum = s, n = n,
        max_inflation = NA_real_,
        auc_inflation = NA_real_,
        note = paste0("Skipped: n < ", min_n)
      )
      next
    }

    Fhat <- vapply(u_grid, function(u) mean(p <= u), numeric(1))

    ecdf_tbl <- tibble::tibble(
      stratum = s,
      u = u_grid,
      Fhat = Fhat,
      n = n
    )
    out_ecdf[[s]] <- ecdf_tbl

    sel <- u_grid <= u_max
    infl <- (Fhat - u_grid)
    max_infl <- max(infl[sel], na.rm = TRUE)

    # Positive inflation AUC on [0, u_max], normalized
    u_sel <- u_grid[sel]
    infl_pos <- pmax(infl[sel], 0)
    if (length(u_sel) >= 2) {
      auc <- sum(diff(u_sel) * (head(infl_pos, -1) + tail(infl_pos, -1)) / 2) / max(u_sel)
    } else {
      auc <- NA_real_
    }

    out_sum[[s]] <- tibble::tibble(
      stratum = s,
      n = n,
      max_inflation = max_infl,
      auc_inflation = auc,
      note = NA_character_
    )
  }

  ecdf_all <- if (length(out_ecdf)) dplyr::bind_rows(out_ecdf) else tibble::tibble()
  sum_all  <- dplyr::bind_rows(out_sum)

  list(ecdf = ecdf_all, summary = sum_all)
}

#' Plot p-value calibration (ECDF minus uniform)
#'
#' Plots \eqn{\hat F(u) - u} against \eqn{u} using the \code{ecdf} component
#' returned by \code{\link{dfdr_p_calibration}}. Values above zero indicate
#' potential inflation (excess of small p-values) relative to uniform.
#'
#' @param ecdf_tbl A tibble, typically \code{out$ecdf} from \code{\link{dfdr_p_calibration}}.
#'   Must contain columns \code{stratum}, \code{u}, and \code{Fhat}.
#' @param title Character. Plot title.
#'
#' @return
#' A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 2000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = sample(c("run1", "run2"), n, replace = TRUE),
#'   p = c(stats::runif(1800), stats::rbeta(200, 0.3, 1))
#' )
#' cal <- dfdr_p_calibration(df, stratify = "run", min_n = 100)
#' p <- dfdr_plot_p_calibration(cal$ecdf)
#' p
#'
#' @export
dfdr_plot_p_calibration <- function(ecdf_tbl,
                                    title = "P-value calibration: ECDF(p) - u") {
  .check_has_cols(ecdf_tbl, c("stratum", "u", "Fhat"))
  ggplot2::ggplot(ecdf_tbl, ggplot2::aes(x = .data$u, y = .data$Fhat - .data$u)) +
    ggplot2::geom_hline(yintercept = 0, linetype = 2, color = "grey50") +
    ggplot2::geom_line() +
    ggplot2::facet_wrap(~ stratum, scales = "free_y") +
    ggplot2::labs(x = "u", y = "Fhat(u) - u", title = title)
}


#' P-value calibration plot (cp4p-style), with multiple pi0 reference curves
#'
#' Plots the ECDF of \eqn{1-p} and overlays reference curves derived from
#' \code{cp4p::estim.pi0()} using multiple \eqn{\pi_0} estimation methods.
#' This provides a visual plausibility check for p-value calibration.
#'
#' @param x A \code{dfdr_tbl} (or data.frame) containing columns \code{id},
#'   \code{is_decoy}, and \code{p}.
#' @param sel Character. Subset to use: \code{"all"}, \code{"decoy"}, or \code{"target"}.
#' @param nbins Integer. Passed to \code{cp4p::estim.pi0()}.
#' @param pz Numeric. Passed to \code{cp4p::estim.pi0()}.
#' @param step Numeric. Grid step size for \code{abs = 1 - p}.
#' @param return_data Logical. If \code{TRUE}, return both the plot and the
#'   computed data used to build it.
#'
#' @return
#' If \code{return_data = FALSE} (default), returns a
#' \link[ggplot2:ggplot]{ggplot} object.
#'
#' If \code{return_data = TRUE}, returns a list with components:
#' \describe{
#'   \item{plot}{A ggplot object.}
#'   \item{pi0}{Named numeric vector of \eqn{\pi_0} estimates (one per method).}
#'   \item{data}{A list with \code{main} (ECDF data) and \code{ref} (reference
#'   curve data).}
#' }
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 2000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.9, 0.1)),
#'   p = c(stats::runif(1800), stats::rbeta(200, 0.3, 1))
#' )
#'
#' if (requireNamespace("cp4p", quietly = TRUE)) {
#'   g <- dfdr_plot_p_calibration2(df, sel = "all", return_data = FALSE)
#'   g
#' }
#'
#' @export
dfdr_plot_p_calibration2 <- function(x,
                                     sel = c("all", "decoy", "target"),
                                     nbins = 20,
                                     pz = 0.05,
                                     step = 0.001,
                                     return_data = FALSE) {
  x <- .safe_df(x)
  .check_has_cols(x, c("id", "is_decoy", "p"))

  sel <- match.arg(tolower(sel), c("all", "decoy", "target"))

  xx <- x
  if (sel == "decoy")  xx <- dplyr::filter(xx, .data$is_decoy)
  if (sel == "target") xx <- dplyr::filter(xx, !.data$is_decoy)

  p <- suppressWarnings(as.numeric(xx$p))
  p <- p[is.finite(p) & p >= 0 & p <= 1]

  if (length(p) < 10) rlang::abort("Too few valid p-values to plot calibration.")
  if (all(p == 0) || all(p == 1)) rlang::abort("Degenerate p-values (all 0 or all 1); cannot plot calibration.")

  cmax <- max(p)
  if (!is.finite(cmax) || cmax <= 0) rlang::abort("max(p) must be > 0 for cp4p-style reference curves.")

  one_minus_p <- 1 - p
  Fr <- stats::ecdf(one_minus_p)

  minabs <- max(0, min(one_minus_p) - 0.05)
  abs_grid <- seq(minabs, 1, by = step)
  fc <- Fr(abs_grid)

  df_main <- tibble::tibble(abs = abs_grid, fc = fc)

  # pi0 estimates (ALL methods)
  r <- cp4p::estim.pi0(p = p, pi0.method = "ALL", nbins = nbins, pz = pz)
  pi0_vec <- as.numeric(r$pi0)

  methods <- c("st.spline","st.boot","jiang","histo","langaas","pounds","abh","slim")
  if (length(pi0_vec) != length(methods)) {
    rlang::abort("cp4p::estim.pi0(pi0.method='ALL') did not return 8 estimates as expected.")
  }

  # dr(abs) = pi0 * (abs - 1 + c) / c, truncated at 0
  df_ref <- purrr::map_dfr(seq_along(methods), function(i) {
    dr_vec <- pi0_vec[i] * (abs_grid - 1 + cmax) / cmax
    dr_vec <- pmax(dr_vec, 0)
    tibble::tibble(
      abs = abs_grid,
      dr = dr_vec,
      method = methods[i],
      pi0 = pi0_vec[i]
    )
  })

  pal <- c(
    "st.spline" = "chartreuse2",
    "st.boot"   = "chartreuse4",
    "jiang"     = "darkgreen",
    "histo"     = "darkorange",
    "langaas"   = "darkorange3",
    "pounds"    = "red3",
    "abh"       = "deepskyblue3",
    "slim"      = "blue3"
  )

  g <- ggplot2::ggplot(df_main, ggplot2::aes(x = .data$abs, y = .data$fc)) +
    ggplot2::geom_line(linewidth = 0.9, colour = "black") +
    ggplot2::geom_line(
      data = df_ref,
      ggplot2::aes(x = .data$abs, y = .data$dr, colour = .data$method, linetype = .data$method),
      linewidth = 0.8,
      alpha = 0.9
    ) +
    ggplot2::scale_colour_manual(values = pal, drop = FALSE) +
    ggplot2::coord_cartesian(ylim = c(0, 1.05)) +
    ggplot2::labs(
      title = "Calibration plot (cp4p-style): ECDF of (1-p) with pi0 reference curves",
      subtitle = paste0("Subset: ", sel, " | n=", length(p)),
      x = "1 - p.value",
      y = "Cumulative Distribution Function of 1 - p.value",
      colour = "pi0 method",
      linetype = "pi0 method"
    ) +
    ggplot2::theme_minimal()

  if (isTRUE(return_data)) {
    return(list(plot = g, pi0 = stats::setNames(pi0_vec, methods), data = list(main = df_main, ref = df_ref)))
  }

  g
}

#' Read mzIdentML into a \code{dfdr_tbl} (generic; score-based TDC q-values)
#'
#' Extracts a competed PSM universe (rank-1 by default) from an mzIdentML file,
#' determines target/decoy labels, selects a single numeric PSM score CV term,
#' and reconstructs q-values using target-decoy counting (TDC).
#'
#' This function is intended for workflows where mzIdentML does not provide
#' explicit q-values or PEPs.
#'
#' @param mzid_path Character scalar. Path to an mzIdentML file (\code{.mzid}).
#' @param rank Integer scalar. Which \code{SpectrumIdentificationItem@rank} to use
#'   (default 1).
#' @param score_accession_preference Character vector. Preferred PSI-MS CV accessions
#'   to treat as the primary PSM score, in priority order.
#' @param score_direction One of \code{"auto"}, \code{"lower_better"}, \code{"higher_better"}.
#'   If \code{"auto"}, applies simple rules for common e-value/expectation accessions.
#' @param add_decoy Integer scalar. Additive correction in the TDC FDR estimate:
#'   \eqn{\widehat{FDR} = (D + add\_decoy) / T}. Default 1.
#' @param min_score_coverage Numeric in \eqn{(0,1]}. Minimum fraction of PSMs required to
#'   have the chosen score CV term. Default 1.0 (strict; no missing scores allowed).
#' @param decoy_regex Character scalar. Regex used to infer decoys from protein accessions
#'   if \code{PeptideEvidence@isDecoy} is not informative.
#' @param unit Character. Stored in \code{dfdr_tbl} metadata (default \code{"psm"}).
#' @param scope Character. Stored in \code{dfdr_tbl} metadata (default \code{NA}).
#' @param provenance Named list. Stored in metadata (tool, version, parameters, command, etc.).
#'
#' @return
#' A \code{dfdr_tbl} (tibble subclass) with one row per extracted PSM (at the requested
#' \code{rank}). The returned object contains:
#' \itemize{
#'   \item \code{id}: a PSM identifier derived from run and spectrum ID (\code{"run||spectrumID"});
#'   \item \code{is_decoy}: logical target/decoy label;
#'   \item \code{score}: an internal score where larger values mean better matches;
#'   \item \code{q}: reconstructed monotone q-values from TDC using \code{score} and \code{is_decoy};
#'   \item \code{pep}: \code{NA} (mzIdentML PEPs are not parsed here);
#'   \item \code{run}: run identifier when available.
#' }
#' Metadata are stored in \code{attr(x, "meta")} (including \code{unit}, \code{scope},
#' \code{q_source}, and \code{provenance}).
#'
#' @examples
#' if (requireNamespace("xml2", quietly = TRUE)) {
#'   # Minimal mzIdentML-like file sufficient for diagFDR's parser:
#'   # - 1 target and 1 decoy PSM at rank=1, with a numeric cvParam score
#'   tmp <- tempfile(fileext = ".mzid")
#'   mzid_txt <- paste0(
#'     "<?xml version='1.0' encoding='UTF-8'?>\n",
#'     "<MzIdentML xmlns='http://psidev.info/psi/pi/mzIdentML/1.1'>\n",
#'     "  <SequenceCollection>\n",
#'     "    <DBSequence id='DBSeq_t' accession='PROT1'/>\n",
#'     "    <DBSequence id='DBSeq_d' accession='REV_PROT2'/>\n",
#'     "    <PeptideEvidence id='PE_t' dBSequence_ref='DBSeq_t' isDecoy='false'/>\n",
#'     "    <PeptideEvidence id='PE_d' dBSequence_ref='DBSeq_d' isDecoy='true'/>\n",
#'     "  </SequenceCollection>\n",
#'     "  <DataCollection>\n",
#'     "    <AnalysisData>\n",
#'     "      <SpectrumIdentificationList>\n",
#'     "        <SpectrumIdentificationResult spectraData_ref='runA' spectrumID='scan=1'>\n",
#'     "          <SpectrumIdentificationItem rank='1'>\n",
#'     "            <PeptideEvidenceRef peptideEvidence_ref='PE_t'/>\n",
#'     "            <cvParam accession='MS:1001331' name='X!Tandem:hyperscore' value='50.0'/>\n",
#'     "          </SpectrumIdentificationItem>\n",
#'     "        </SpectrumIdentificationResult>\n",
#'     "        <SpectrumIdentificationResult spectraData_ref='runA' spectrumID='scan=2'>\n",
#'     "          <SpectrumIdentificationItem rank='1'>\n",
#'     "            <PeptideEvidenceRef peptideEvidence_ref='PE_d'/>\n",
#'     "            <cvParam accession='MS:1001331' name='X!Tandem:hyperscore' value='10.0'/>\n",
#'     "          </SpectrumIdentificationItem>\n",
#'     "        </SpectrumIdentificationResult>\n",
#'     "      </SpectrumIdentificationList>\n",
#'     "    </AnalysisData>\n",
#'     "  </DataCollection>\n",
#'     "</MzIdentML>\n"
#'   )
#'   writeLines(mzid_txt, tmp)
#'
#'   x <- read_dfdr_mzid(tmp, rank = 1L, score_direction = "higher_better")
#'   x
#'   range(x$q)
#' }
#'
#' @export
#'
#' @importFrom tibble tibble
#' @importFrom rlang abort
read_dfdr_mzid <- function(
    mzid_path,
    rank = 1L,
    score_accession_preference = c(
      # expectation/e-values first (lower is better)
      "MS:1002257", # Comet:expectation value
      "MS:1001330", # X!Tandem:expect
      "MS:1001328", # OMSSA:evalue
      "MS:1002052", # MS-GF:SpecEValue

      # then higher-better scores
      "MS:1002049", # MS-GF:RawScore
      "MS:1001331", # X!Tandem:hyperscore
      "MS:1001171", # Mascot:score
      "MS:1001950", # PEAKS:peptideScore

      # keep PeptideShaker score last (can saturate)
      "MS:1002466"  # PeptideShaker PSM score
    ),
    score_direction = c("auto", "lower_better", "higher_better"),
    add_decoy = 1L,
    min_score_coverage = 1.0,
    decoy_regex = "(^##|_REVERSED$|^REV_|^DECOY_)",
    unit = "psm",
    scope = NA_character_,
    provenance = list()
) {
  score_direction <- match.arg(score_direction)

  psms <- mzid_parse_psms(
    mzid_path = mzid_path,
    rank = rank,
    score_accession_preference = score_accession_preference,
    min_score_coverage = min_score_coverage,
    decoy_regex = decoy_regex
  )

  if (is.na(psms$score_accession[1])) {
    rlang::abort("No numeric score CV term found in mzIdentML; cannot compute q-values.")
  }

  score_dir <- mzid_score_direction(
    score_accession = psms$score_accession[1],
    score_direction = score_direction
  )

  score_internal <- mzid_to_internal_score(
    score_value = psms$score_value,
    score_direction = score_dir,
    score_accession = psms$score_accession[1]
  )

  # strict requirements for dfdr_tbl
  if (anyNA(psms$is_decoy)) rlang::abort("`is_decoy` contains NA after parsing; please fix upstream.")
  if (!any(psms$is_decoy)) rlang::abort("No decoys detected; cannot compute q-values.")
  if (!any(!psms$is_decoy)) rlang::abort("No targets detected; cannot compute q-values.")
  if (anyNA(score_internal)) rlang::abort("Chosen score is missing (NA) for some PSMs; cannot compute q-values.")
  if (all(is.na(score_internal))) rlang::abort("Score values are all NA; cannot compute q-values.")

  q <- tdc_qvalues(score = score_internal, is_decoy = psms$is_decoy, add_decoy = add_decoy)

  if (any(!is.finite(q))) rlang::abort("Reconstructed q contains non-finite values.")
  if (any(q < 0 | q > 1)) rlang::abort("Reconstructed q not in [0,1].")

  out <- tibble::tibble(
    id = psms$id,
    is_decoy = psms$is_decoy,
    q = q,
    pep = rep(NA_real_, length(q)),
    run = psms$run %||% rep(NA_character_, length(q)),
    score = score_internal
  )

  q_source <- paste0(
    "reconstructed_TDC_from_", psms$score_accession[1],
    " (", psms$score_name[1] %||% "unknown_score_name",
    "; add_decoy=", add_decoy, ")"
  )

  as_dfdr_tbl(
    out,
    unit = unit,
    scope = scope,
    q_source = q_source,
    q_max_export = NA_real_,
    provenance = provenance
  )
}


#' Parse rank-PSMs from mzIdentML (internal)
#'
#' Extracts SpectrumIdentificationItems at the requested rank, derives decoy labels
#' using PeptideEvidence@isDecoy when available, with fallback to DBSequence accession
#' regex inference, and selects a single numeric score term with sufficient coverage.
#'
#' @param mzid_path Character scalar. Path to an mzIdentML file.
#' @param rank Integer scalar. Which `SpectrumIdentificationItem@rank` to extract.
#' @param score_accession_preference Character vector of preferred score accessions.
#' @param min_score_coverage Numeric in (0,1]. Minimum fraction of PSMs required to have
#'   the chosen score CV term.
#' @param decoy_regex Character scalar. Regex for inferring decoys from DBSequence accessions.
#'
#' @return Tibble with columns id, run, is_decoy, score_accession, score_name, score_value.
#' @keywords internal
#'
#' @importFrom xml2 read_xml xml_find_all xml_parent xml_attr
#' @importFrom tibble tibble
#' @importFrom dplyr filter
#' @importFrom rlang abort
#' @return A \link[tibble:tibble]{tibble} with columns \code{id}, \code{run},
#'   \code{is_decoy}, \code{score_accession}, \code{score_name}, \code{score_value}.
#' @keywords internal
mzid_parse_psms <- function(
    mzid_path,
    rank = 1L,
    score_accession_preference = c("MS:1002257"),
    min_score_coverage = 1.0,
    decoy_regex = "(^##|_REVERSED$|^REV_|^DECOY_)"
) {
  if (!is.character(mzid_path) || length(mzid_path) != 1) {
    rlang::abort("`mzid_path` must be a character scalar.")
  }
  if (!file.exists(mzid_path)) rlang::abort("mzIdentML file not found: ", mzid_path)
  if (!is.numeric(min_score_coverage) || length(min_score_coverage) != 1 ||
      !is.finite(min_score_coverage) || min_score_coverage <= 0 || min_score_coverage > 1) {
    rlang::abort("`min_score_coverage` must be a single number in (0,1].")
  }

  doc <- xml2::read_xml(mzid_path)

  # ---- DBSequence lookup: id -> accession (for decoy inference fallback) ----
  dbseq_nodes <- xml2::xml_find_all(doc, ".//*[local-name()='DBSequence']")
  db_lut <- tibble::tibble(
    dbsequence_id = xml2::xml_attr(dbseq_nodes, "id"),
    accession = xml2::xml_attr(dbseq_nodes, "accession")
  )

  # ---- PeptideEvidence lookup: id -> isDecoy + dBSequence_ref ----
  pe_nodes <- xml2::xml_find_all(doc, ".//*[local-name()='PeptideEvidence']")
  if (length(pe_nodes) == 0) {
    rlang::abort("No <PeptideEvidence> nodes found; cannot determine targets/decoys.")
  }

  pe_lut <- tibble::tibble(
    peptideEvidence_ref = xml2::xml_attr(pe_nodes, "id"),
    is_decoy_attr = tolower(xml2::xml_attr(pe_nodes, "isDecoy")) %in% "true",
    dbsequence_ref = xml2::xml_attr(pe_nodes, "dBSequence_ref")
  )

  # join to get accession for each PeptideEvidence
  pe_lut$accession <- db_lut$accession[match(pe_lut$dbsequence_ref, db_lut$dbsequence_id)]

  # ---- Find rank-n SpectrumIdentificationItem nodes ----
  sii_xpath <- sprintf(".//*[local-name()='SpectrumIdentificationItem'][@rank='%s']",
                       as.integer(rank))
  sii_nodes <- xml2::xml_find_all(doc, sii_xpath)
  if (length(sii_nodes) == 0) {
    rlang::abort(paste0("No SpectrumIdentificationItem nodes found for rank=", rank))
  }

  sir_nodes <- xml2::xml_parent(sii_nodes)
  run <- xml2::xml_attr(sir_nodes, "spectraData_ref")
  spectrumID <- xml2::xml_attr(sir_nodes, "spectrumID")
  id <- paste0(run, "||", spectrumID)

  # ---- Evidence refs per SII ----
  pe_refs <- lapply(sii_nodes, function(n) {
    refs <- xml2::xml_find_all(n, ".//*[local-name()='PeptideEvidenceRef']")
    xml2::xml_attr(refs, "peptideEvidence_ref")
  })

  # ---- Decoy determination: attribute first, fallback to accession regex ----
  is_decoy_attr_based <- vapply(pe_refs, function(refs) {
    hit <- pe_lut$is_decoy_attr[match(refs, pe_lut$peptideEvidence_ref)]
    if (length(hit) == 0 || all(is.na(hit))) return(NA)
    any(hit %in% TRUE, na.rm = TRUE)
  }, logical(1))

  has_any_decoy_attr <- any(is_decoy_attr_based %in% TRUE, na.rm = TRUE)

  if (has_any_decoy_attr) {
    # Use isDecoy attribute (standard) if it yields at least one decoy
    is_decoy <- is_decoy_attr_based
  } else {
    # Fallback: infer decoys from DBSequence accession via regex
    is_decoy <- vapply(pe_refs, function(refs) {
      acc <- pe_lut$accession[match(refs, pe_lut$peptideEvidence_ref)]
      acc <- acc[!is.na(acc)]
      if (length(acc) == 0) {
        rlang::abort("Cannot infer decoy status: missing DBSequence accession for some PeptideEvidence.")
      }
      any(grepl(decoy_regex, acc, perl = TRUE))
    }, logical(1))
  }

  if (anyNA(is_decoy)) {
    rlang::abort("`is_decoy` contains NA after parsing/inference.")
  }

  # ---- Extract numeric cvParams per SII ----
  cv_df_list <- lapply(sii_nodes, function(n) {
    cvp <- xml2::xml_find_all(n, ".//*[local-name()='cvParam']")
    if (length(cvp) == 0) {
      return(tibble::tibble(accession = character(), name = character(), value = numeric()))
    }
    df <- tibble::tibble(
      accession = xml2::xml_attr(cvp, "accession"),
      name = xml2::xml_attr(cvp, "name"),
      value = suppressWarnings(as.numeric(xml2::xml_attr(cvp, "value")))
    )
    dplyr::filter(df, !is.na(.data$value))
  })

  # Compute coverage per accession (fraction of SIIs having it)
  acc_per_sii <- lapply(cv_df_list, function(df) unique(df$accession))
  all_acc <- sort(unique(unlist(acc_per_sii)))
  all_acc <- all_acc[!is.na(all_acc)]

  if (length(all_acc) == 0) {
    rlang::abort("No numeric cvParam values found on SpectrumIdentificationItem.")
  }

  coverage <- vapply(all_acc, function(a) {
    mean(vapply(acc_per_sii, function(v) a %in% v, logical(1)))
  }, numeric(1))

  eligible <- all_acc[coverage[all_acc] >= min_score_coverage]

  if (length(eligible) == 0) {
    # Strict behaviour: refuse to compute q if no single score term is widely present
    rlang::abort(
      "No single numeric score cvParam meets min_score_coverage=",
      min_score_coverage, ". ",
      "Cannot compute q-values without dropping rows or mixing score types."
    )
  }

  pref <- intersect(score_accession_preference, eligible)
  if (length(pref) > 0) {
    chosen_acc <- pref[1]
  } else {
    # Choose highest coverage among eligible; break ties by lexical order for determinism
    cov_elig <- coverage[eligible]
    chosen_acc <- eligible[order(cov_elig, decreasing = TRUE, eligible)][1]
  }

  # Pull that score for each SII; must not be missing
  score_value <- rep(NA_real_, length(sii_nodes))
  score_name  <- rep(NA_character_, length(sii_nodes))

  for (i in seq_along(cv_df_list)) {
    df <- cv_df_list[[i]]
    j <- which(df$accession == chosen_acc)
    if (length(j)) {
      score_value[i] <- df$value[j[1]]
      score_name[i]  <- df$name[j[1]]
    }
  }

  if (anyNA(score_value)) {
    rlang::abort(
      "Chosen score accession ", chosen_acc, " is missing for some PSMs (NA score_value). ",
      "Refusing to drop rows or mix score types. Consider lowering min_score_coverage."
    )
  }

  tibble::tibble(
    id = id,
    run = run,
    is_decoy = is_decoy,
    score_accession = chosen_acc,
    score_name = score_name,
    score_value = score_value
  )
}


#' Determine score direction (internal)
#'
#' @param score_accession Character. PSI-MS accession for the score term.
#' @param score_direction One of `"auto"`, `"lower_better"`, `"higher_better"`.
#'
#' @return `"lower_better"` or `"higher_better"`.
#' @keywords internal
mzid_score_direction <- function(score_accession,
                                 score_direction = c("auto", "lower_better", "higher_better")) {
  score_direction <- match.arg(score_direction)
  if (score_direction != "auto") return(score_direction)

  # E-values/expectation values are lower-better
  if (score_accession %in% c(
    "MS:1002257", # Comet:expectation value
    "MS:1001330", # X!Tandem:expect
    "MS:1001328", # OMSSA:evalue
    "MS:1002052"  # MS-GF:SpecEValue
  )) {
    return("lower_better")
  }

  "higher_better"
}


#' Convert raw score to an internal "higher is better" score (internal)
#'
#' @param score_value Numeric vector. Raw score values.
#' @param score_direction Character. `"lower_better"` or `"higher_better"`.
#' @param score_accession Character. Score accession (used for special handling).
#'
#' @return Numeric vector. Larger values mean better matches.
#' @keywords internal
mzid_to_internal_score <- function(score_value, score_direction, score_accession) {
  if (score_direction == "higher_better") return(score_value)

  # Use -log10 for common p/e-value like scores (strictly positive)
  if (score_accession %in% c("MS:1002257", "MS:1001330", "MS:1001328", "MS:1002052")) {
    eps <- .Machine$double.xmin
    return(-log10(pmax(score_value, eps)))
  }

  # Generic fallback
  -score_value
}


#' Compute monotone q-values by target-decoy counting (internal)
#'
#' Computes FDR estimates along the score-sorted list and returns monotone q-values:
#' \eqn{q(i) = \min_{j \ge i} \widehat{FDR}(j)}.
#'
#' @param score Numeric vector. Higher is better.
#' @param is_decoy Logical vector. TRUE=decoy, FALSE=target.
#' @param add_decoy Integer scalar. FDR hat = (D + add_decoy) / T.
#'
#' @return Numeric vector of q-values in `[0,1]`.
#' @keywords internal
tdc_qvalues <- function(score, is_decoy, add_decoy = 1L) {
  if (!is.numeric(score)) rlang::abort("`score` must be numeric.")
  if (!is.logical(is_decoy)) rlang::abort("`is_decoy` must be logical.")
  if (length(score) != length(is_decoy)) rlang::abort("`score` and `is_decoy` must have equal length.")
  if (anyNA(score)) rlang::abort("`score` contains NA; cannot compute q-values.")
  if (anyNA(is_decoy)) rlang::abort("`is_decoy` contains NA; cannot compute q-values.")

  ord <- order(score, decreasing = TRUE)

  dec <- is_decoy[ord]
  tar <- !is_decoy[ord]

  D <- cumsum(dec)
  T <- cumsum(tar)

  fdr_hat <- rep(NA_real_, length(ord))
  ok <- T > 0
  fdr_hat[ok] <- (D[ok] + add_decoy) / T[ok]

  q_ord <- rev(cummin(rev(fdr_hat)))

  q <- rep(NA_real_, length(score))
  q[ord] <- q_ord

  pmin(pmax(q, 0), 1)
}

#' Posterior Error Probability (PEP) reliability and internal PEP calibration error (IPE)
#'
#' Bins identifications by predicted PEP and compares mean predicted PEP to the
#' observed decoy fraction within each bin (internal target-decoy consistency check).
#'
#' @param x A \code{dfdr_tbl} with a non-missing \code{pep} column.
#' @param binwidth Numeric bin width for PEP binning (default 0.05).
#' @param n_min Integer. Minimum bin size to include bins in the IPE summary.
#' @param pep_max Numeric. Maximum mean PEP to include in the IPE summary
#'   (default 0.5).
#'
#' @return
#' A list with components:
#' \describe{
#'   \item{bins}{A \link[tibble:tibble]{tibble} with one row per PEP bin and columns
#'   \code{pep_mean} (mean predicted PEP), \code{decoy_rate} (observed decoy
#'   fraction), and \code{n} (bin size).}
#'   \item{IPE}{Numeric scalar. The internal PEP calibration error (IPE), computed
#'   as a weighted mean absolute deviation between \code{pep_mean} and
#'   \code{decoy_rate} across eligible bins (\code{n >= n_min} and
#'   \code{pep_mean <= pep_max}). \code{NA} if no eligible bins exist.}
#'   \item{params}{A list of parameters used (\code{binwidth}, \code{n_min},
#'   \code{pep_max}).}
#' }
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 5000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   q = pmin(1, rank(-score) / n),         # simple monotone q-like values
#'   pep = pmin(1, pmax(0, stats::runif(n))) # toy PEP in [0,1]
#' )
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")
#'
#' out <- dfdr_pep_reliability(x, binwidth = 0.1, n_min = 50, pep_max = 0.5)
#' out$IPE
#' head(out$bins)
#'
#' @export
dfdr_pep_reliability <- function(x, binwidth = 0.05, n_min = 200, pep_max = 0.5) {
  validate_dfdr_tbl(x)
  if (!"pep" %in% names(x) || all(is.na(x$pep))) rlang::abort("No PEP available (pep is all NA).")

  brks <- seq(0, 1, by = binwidth)
  if (utils::tail(brks, 1) < 1) brks <- c(brks, 1)

  bins <- x |>
    dplyr::filter(is.finite(.data$pep), .data$pep >= 0, .data$pep <= 1) |>
    dplyr::mutate(bin = cut(.data$pep, breaks = brks, include.lowest = TRUE)) |>
    dplyr::group_by(.data$bin) |>
    dplyr::summarise(
      pep_mean = mean(.data$pep),
      decoy_rate = mean(.data$is_decoy),
      n = dplyr::n(),
      .groups = "drop"
    ) |>
    dplyr::filter(is.finite(.data$pep_mean))

  bins_use <- bins |>
    dplyr::filter(.data$n >= n_min, .data$pep_mean <= pep_max)

  IPE <- if (nrow(bins_use) > 0) {
    sum((bins_use$n / sum(bins_use$n)) * abs(bins_use$pep_mean - bins_use$decoy_rate))
  } else {
    NA_real_
  }

  list(
    bins = bins,
    IPE = IPE,
    params = list(binwidth = binwidth, n_min = n_min, pep_max = pep_max)
  )
}

#' Expected number of false targets among accepted identifications
#'
#' Computes \eqn{\sum_{i \in A(\alpha)} \mathrm{PEP}_i} among accepted targets
#' for each threshold \eqn{\alpha}, where acceptance is defined by \code{q <= alpha}.
#'
#' @param x An \code{dfdr_tbl} with a non-missing \code{pep} column.
#' @param alphas Numeric vector of thresholds in \eqn{(0,1]}.
#'
#' @return
#' A \link[tibble:tibble]{tibble} with one row per \code{alpha}. Columns include:
#' \describe{
#'   \item{alpha}{The threshold.}
#'   \item{n_targets}{Number of accepted targets (\code{!is_decoy} and \code{q <= alpha}).}
#'   \item{sum_PEP}{Sum of PEP values among accepted targets (expected false targets).}
#'   \item{mean_PEP}{Mean PEP among accepted targets.}
#' }
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 5000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   q = pmin(1, rank(-score) / n),
#'   pep = stats::runif(n)
#' )
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")
#'
#' dfdr_sumpep(x, alphas = c(0.001, 0.01, 0.05))
#'
#' @export
dfdr_sumpep <- function(x, alphas) {
  validate_dfdr_tbl(x)
  if (all(is.na(x$pep))) rlang::abort("No PEP available (pep is all NA).")

  purrr::map_dfr(alphas, function(a) {
    acc <- x |>
      dplyr::filter(!is_decoy, q <= a, is.finite(pep), pep >= 0, pep <= 1)
    tibble::tibble(
      alpha = a,
      n_targets = nrow(acc),
      sum_PEP = sum(acc$pep, na.rm = TRUE),
      mean_PEP = mean(acc$pep, na.rm = TRUE)
    )
  })
}

#' Equal-chance plausibility by q-value bands
#'
#' Computes decoy fractions in q-value bands and performs a pooled binomial test
#' of whether the decoy fraction is near 0.5 in a mismatch-dominated region
#' (specified by \code{low_conf}).
#'
#' @param x An \code{dfdr_tbl}.
#' @param breaks Numeric vector of q-value cutpoints used to form bands.
#' @param low_conf Length-2 numeric vector giving the pooled test interval
#'   (e.g. \code{c(0.2, 0.5)}).
#' @param min_N Integer. Minimum pooled sample size required for the pooled test
#'   to be considered stable.
#'
#' @return
#' A list with components:
#' \describe{
#'   \item{bands}{A \link[tibble:tibble]{tibble} with one row per q-band, including
#'   \code{n} (band size), \code{n_decoy}, \code{decoy_frac}, and \code{q_mean}.}
#'   \item{pooled}{A one-row tibble with pooled quantities over bands whose
#'   \code{q_mean} lies within \code{low_conf}, including \code{pi_D_hat} (pooled
#'   decoy fraction), a Wilson confidence interval, and a binomial test p-value.
#'   \code{pass_minN} indicates whether \code{min_N} is met. In cases where the
#'   requested banding is not applicable (e.g. \code{qmax} below the smallest
#'   nonzero break), fields are returned as \code{NA} and a \code{note} may be
#'   provided.}
#'   \item{params}{A list of parameters used (\code{breaks}, \code{low_conf}, \code{min_N}).}
#' }
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 8000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.9, 0.1)),
#'   score = rnorm(n),
#'   pep = NA_real_
#' )
#' # Toy q-values in [0, 1]
#' df$q <- stats::runif(n)
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")
#'
#' ec <- dfdr_equal_chance_qbands(x, breaks = c(0, 0.2, 0.5, 1), low_conf = c(0.2, 0.5), min_N = 200)
#' ec$pooled
#'
#' @export
dfdr_equal_chance_qbands <- function(
    x,
    breaks = c(0, 0.01, 0.02, 0.05, 0.1, 0.2, 0.5),
    low_conf = c(0.2, 0.5),
    min_N = 2000
) {
  validate_dfdr_tbl(x)
  meta <- attr(x, "meta") %||% list()
  qmax <- meta$q_max_export %||% NA_real_
  if (!is.finite(qmax)) qmax <- max(x$q, na.rm = TRUE)

  breaks_use <- breaks[breaks <= qmax + 1e-12]
  breaks_use <- unique(sort(breaks_use))

  # --- NEW: graceful handling when not applicable ---
  if (length(breaks_use) < 2) {
    bands <- tibble::tibble(
      qbin = factor(),
      n = integer(),
      n_decoy = integer(),
      decoy_frac = numeric(),
      q_mean = numeric()
    )
    pooled <- tibble::tibble(
      qmax_export = qmax,
      low_lo = low_conf[1],
      low_hi = low_conf[2],
      N_test = 0,
      N_D_test = 0,
      pi_D_hat = NA_real_,
      effect_abs = NA_real_,
      ci95_lo = NA_real_,
      ci95_hi = NA_real_,
      p_value_binom = NA_real_,
      pass_minN = FALSE,
      note = "Not applicable: qmax below smallest nonzero break; provide smaller breaks or export higher q."
    )
    return(list(
      bands = bands,
      pooled = pooled,
      params = list(breaks = breaks, low_conf = low_conf, min_N = min_N)
    ))
  }
  # --- end NEW ---

  if (max(breaks_use) < qmax) breaks_use <- c(breaks_use, qmax)

  dd <- dplyr::mutate(
    dplyr::filter(x, is.finite(.data$q), .data$q >= 0, .data$q <= qmax),
    qbin = cut(.data$q, breaks = breaks_use, include.lowest = TRUE, right = TRUE)
  )

  bands <- dplyr::arrange(
    dplyr::summarise(
      dplyr::group_by(dd, .data$qbin),
      n = dplyr::n(),
      n_decoy = sum(.data$is_decoy),
      decoy_frac = .data$n_decoy / .data$n,
      q_mean = mean(.data$q),
      .groups = "drop"
    ),
    .data$q_mean
  )

  lo <- low_conf[1]; hi <- low_conf[2]
  test_tbl <- dplyr::filter(bands, is.finite(.data$q_mean), .data$q_mean >= lo, .data$q_mean <= hi)
  N_test <- sum(test_tbl$n)
  N_D_test <- sum(test_tbl$n_decoy)
  pi_hat <- ifelse(N_test > 0, N_D_test / N_test, NA_real_)
  ci <- wilson_ci(N_D_test, N_test, conf = 0.95)
  p_binom <- if (N_test > 0) stats::binom.test(N_D_test, N_test, p = 0.5)$p.value else NA_real_

  pooled <- tibble::tibble(
    qmax_export = qmax, low_lo = lo, low_hi = hi,
    N_test = N_test, N_D_test = N_D_test,
    pi_D_hat = pi_hat,
    effect_abs = ifelse(is.finite(pi_hat), abs(pi_hat - 0.5), NA_real_),
    ci95_lo = ci[1], ci95_hi = ci[2],
    p_value_binom = p_binom,
    pass_minN = N_test >= min_N
  )

  list(bands = bands, pooled = pooled, params = list(breaks = breaks, low_conf = low_conf, min_N = min_N))
}

#' Decoy PEP sanity checks
#'
#' Summarises how many decoys receive surprisingly small PEP values. Under a
#' well-calibrated PEP, decoys should typically have high error probabilities;
#' large fractions of decoys below small PEP thresholds can indicate issues with
#' PEP calibration or labeling.
#'
#' @param x A \code{dfdr_tbl} with a non-missing \code{pep} column.
#' @param thresholds Numeric vector of PEP thresholds in \eqn{(0,1]} to summarise.
#'
#' @return
#' A \link[tibble:tibble]{tibble} with one row per \code{threshold}. Columns include:
#' \describe{
#'   \item{threshold}{The PEP cutoff used for counting.}
#'   \item{n_decoy}{Number of decoys with finite PEP in \eqn{(0,1]}.}
#'   \item{n_target}{Number of targets with finite PEP in \eqn{(0,1]}.}
#'   \item{decoy_le}{Count of decoys with \code{pep <= threshold}.}
#'   \item{target_le}{Count of targets with \code{pep <= threshold}.}
#'   \item{frac_decoy_le}{\code{decoy_le / n_decoy}.}
#'   \item{frac_target_le}{\code{target_le / n_target}.}
#' }
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 5000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   q = pmin(1, rank(-score) / n),
#'   pep = stats::runif(n)
#' )
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")
#'
#' dfdr_pep_decoy_sanity(x, thresholds = c(0.01, 0.05, 0.1))
#'
#' @export
dfdr_pep_decoy_sanity <- function(x, thresholds = c(0.01, 0.05, 0.1, 0.2)) {
  validate_dfdr_tbl(x)
  if (!"pep" %in% names(x)) rlang::abort("`x` must contain a `pep` column.")

  pep <- x$pep
  ok <- is.finite(pep) & pep >= 0 & pep <= 1
  pep <- pep[ok]
  is_decoy <- x$is_decoy[ok]

  n_decoy <- sum(is_decoy)
  n_target <- sum(!is_decoy)

  purrr::map_dfr(thresholds, function(th) {
    decoy_le <- sum(is_decoy & pep <= th)
    target_le <- sum((!is_decoy) & pep <= th)

    tibble::tibble(
      threshold = th,
      n_decoy = n_decoy,
      n_target = n_target,
      decoy_le = decoy_le,
      target_le = target_le,
      frac_decoy_le = decoy_le / pmax(n_decoy, 1),
      frac_target_le = target_le / pmax(n_target, 1)
    )
  })
}


#' Target-focused PEP reliability using a TDC-style error proxy
#'
#' Bins identifications by predicted PEP and estimates an error proxy for targets
#' within each bin using decoys as a proxy via a target-decoy-competition (TDC)
#' style ratio \eqn{(D + add\_decoy) / T}.
#'
#' @param x A \code{dfdr_tbl} with columns \code{pep} and \code{is_decoy}.
#' @param breaks Numeric vector of PEP bin edges.
#' @param add_decoy Integer. Additive correction to the decoy count in each bin
#'   (default 0). Use 1 for a conservative small-sample correction.
#'
#' @return
#' A \link[tibble:tibble]{tibble} with one row per PEP bin, including:
#' \describe{
#'   \item{bin}{Formatted bin label.}
#'   \item{n_target}{Number of targets in the bin.}
#'   \item{n_decoy}{Number of decoys in the bin.}
#'   \item{pep_mean_targets}{Mean PEP among targets in the bin.}
#'   \item{pep_mean_all}{Mean PEP among all entries in the bin.}
#'   \item{err_hat_target}{Estimated target error proxy \eqn{(D + add\_decoy)/T}, capped at 1.}
#'   \item{pep_bin_mid}{Midpoint of the PEP bin (useful for plotting).}
#' }
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 8000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.9, 0.1)),
#'   score = rnorm(n),
#'   q = stats::runif(n),
#'   pep = stats::runif(n)
#' )
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")
#'
#' dfdr_pep_reliability_tdc(x, breaks = seq(0, 0.5, by = 0.1), add_decoy = 1L)
#'
#' @export
dfdr_pep_reliability_tdc <- function(x,
                                     breaks = seq(0, 0.5, by = 0.05),
                                     add_decoy = 0L) {
  validate_dfdr_tbl(x)
  if (!"pep" %in% names(x)) rlang::abort("`x` must contain a `pep` column.")

  breaks <- sort(unique(breaks))
  if (length(breaks) < 2) rlang::abort("`breaks` must have at least two values.")

  df <- x |>
    dplyr::filter(is.finite(.data$pep), .data$pep >= min(breaks), .data$pep <= max(breaks)) |>
    dplyr::mutate(
      bin_i = findInterval(.data$pep, vec = breaks, rightmost.closed = TRUE),
      bin_i = dplyr::if_else(.data$bin_i < 1L | .data$bin_i >= length(breaks), NA_integer_, .data$bin_i)
    ) |>
    dplyr::filter(!is.na(.data$bin_i))

  mids <- (breaks[-length(breaks)] + breaks[-1]) / 2

  out <- df |>
    dplyr::group_by(.data$bin_i) |>
    dplyr::summarise(
      pep_lo = breaks[unique(.data$bin_i)],
      pep_hi = breaks[unique(.data$bin_i) + 1L],
      pep_bin_mid = mids[unique(.data$bin_i)],
      n_target = sum(!.data$is_decoy),
      n_decoy = sum(.data$is_decoy),
      pep_mean_targets = mean(.data$pep[!.data$is_decoy], na.rm = TRUE),
      pep_mean_all = mean(.data$pep, na.rm = TRUE),
      err_hat_target = pmin(1, (.data$n_decoy + add_decoy) / pmax(.data$n_target, 1)),
      .groups = "drop"
    ) |>
    dplyr::mutate(
      bin = sprintf("(%.3g, %.3g]", .data$pep_lo, .data$pep_hi)
    ) |>
    dplyr::select(.data$bin, .data$n_target, .data$n_decoy,
                  .data$pep_mean_targets, .data$pep_mean_all,
                  .data$err_hat_target, .data$pep_bin_mid)

  out
}

#' Headline stability diagnostics at a given alpha
#'
#' Computes headline counts at the acceptance threshold corresponding to
#' \code{alpha} (e.g., numbers of accepted targets/decoys) and simple stability
#' diagnostics that quantify how sensitive the result is to small changes in the
#' decoy boundary support.
#'
#' @param x A \code{dfdr_tbl} containing at least \code{id}, \code{is_decoy}, and a
#'   q-value column (see Details).
#' @param alpha Numeric FDR threshold in \eqn{(0,1]} at which headline metrics are
#'   computed.
#' @param k_fixed Integer. Fixed perturbation size used for sensitivity analyses
#'   based on \eqn{D \pm k} around the decoy count at the threshold.
#' @param k_sqrt_mult Numeric. Multiplier for adaptive perturbations of size
#'   \eqn{\lceil \mathrm{mult}\sqrt{D} \rceil}.
#'
#' @details
#' The function assumes that \code{x} contains target/decoy labels in
#' \code{is_decoy} and q-values in column \code{q}. If your input uses another
#' name (e.g., \code{q_value}), rename it to \code{q} before calling this
#' function.
#'
#' @return
#' A one-row \link[tibble:tibble]{tibble} with headline diagnostics evaluated at
#' the specified \code{alpha}. The table includes the number of accepted targets
#' and decoys at the threshold and additional sensitivity/stability summaries
#' derived from perturbing the boundary decoy support (using \code{k_fixed} and
#' \code{k_sqrt_mult}). The returned tibble is intended for reporting and for
#' comparison across workflows.
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 5000
#' x <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   pep = NA_real_
#' )
#'
#' # Construct simple TDC q-values from the score (higher is better)
#' x$q <- diagFDR:::tdc_qvalues(score = x$score, is_decoy = x$is_decoy, add_decoy = 1L)
#' x <- as_dfdr_tbl(x, unit = "psm", scope = "global", q_source = "toy_tdc")
#'
#' dfdr_headline(x, alpha = 0.01)
#'
#' @export
dfdr_headline <- function(x, alpha = 0.01, k_fixed = 10, k_sqrt_mult = 2) {
  validate_dfdr_tbl(x)

  Ta <- sum(!x$is_decoy & x$q <= alpha)
  Da <- sum( x$is_decoy & x$q <= alpha)

  k2 <- ceiling(k_sqrt_mult * sqrt(max(Da, 0)))

  tibble::tibble(
    alpha = alpha,
    T_alpha = as.integer(Ta),
    D_alpha = as.integer(Da),
    FDR_hat = if (Ta > 0) Da / Ta else NA_real_,
    CV_hat  = if (Da > 0) 1 / sqrt(Da) else Inf,
    FDR_minus1 = if (Ta > 0) max(Da - 1, 0) / Ta else NA_real_,
    FDR_plus1  = if (Ta > 0) (Da + 1) / Ta else NA_real_,
    FDR_minusK = if (Ta > 0) max(Da - k_fixed, 0) / Ta else NA_real_,
    FDR_plusK  = if (Ta > 0) (Da + k_fixed) / Ta else NA_real_,
    k2sqrtD = as.integer(k2),
    FDR_minus2sqrtD = if (Ta > 0) max(Da - k2, 0) / Ta else NA_real_,
    FDR_plus2sqrtD  = if (Ta > 0) (Da + k2) / Ta else NA_real_
  )
}

#' Stability curve across alpha values
#'
#' Evaluates \code{\link{dfdr_headline}} over a grid of FDR thresholds
#' (\code{alphas}) to obtain a stability curve that can be plotted or compared
#' across workflows.
#'
#' @param x An \code{dfdr_tbl}.
#' @param alphas Numeric vector of FDR thresholds in \eqn{(0,1]}.
#' @param k_fixed Integer. Fixed +/-K sensitivity interval used by
#'   \code{\link{dfdr_headline}}.
#' @param k_sqrt_mult Numeric. Multiplier for the adaptive +/- ceiling(mult*sqrt(D))
#'   interval used by \code{\link{dfdr_headline}}.
#'
#' @return
#' A \link[tibble:tibble]{tibble} with one row per value of \code{alphas}.
#' Columns are those returned by \code{\link{dfdr_headline}} plus the
#' corresponding \code{alpha}. The table summarizes how headline quantities
#' (e.g., accepted target/decoy counts and sensitivity diagnostics) vary with the
#' chosen FDR cutoff.
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 5000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   pep = NA_real_
#' )
#'
#' df$q <- diagFDR:::tdc_qvalues(df$score, df$is_decoy, add_decoy = 1L)
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy_tdc")
#'
#' dfdr_curve_stability(x, alphas = c(1e-3, 5e-3, 1e-2))
#'
#' @export
dfdr_curve_stability <- function(x, alphas, k_fixed = 10, k_sqrt_mult = 2) {
  validate_dfdr_tbl(x)
  purrr::map_dfr(alphas, ~dfdr_headline(x, alpha = .x, k_fixed = k_fixed, k_sqrt_mult = k_sqrt_mult))
}


#' Local tail support near the cutoff
#'
#' Counts the number of observations (and decoys) in a right-neighborhood of each
#' threshold \eqn{\alpha}, defined as \eqn{(\alpha,\, \alpha + \rho\alpha]} where
#' \eqn{\rho} is \code{win_rel}. This approximates how well supported the decision
#' boundary is by decoys in the immediate tail.
#'
#' @param x An \code{dfdr_tbl}.
#' @param alphas Numeric vector of FDR thresholds in \eqn{(0,1]}.
#' @param win_rel Numeric. Relative window width \eqn{\rho} (default 0.2).
#' @param truncation Character. How to handle cases where
#'   \eqn{\alpha + \rho\alpha} exceeds the available q-value export range
#'   (i.e. \code{q_max_export}). One of \code{"warn_drop"}, \code{"drop"},
#'   \code{"cap"}.
#'
#' @return
#' A \link[tibble:tibble]{tibble} with one row per \code{alpha}. Columns include:
#' \describe{
#'   \item{alpha}{The threshold.}
#'   \item{delta}{The effective window width used (\eqn{\rho\alpha} or capped).}
#'   \item{n_win}{Number of entries with \code{q} in the window.}
#'   \item{D_alpha_win}{Number of decoys in the window.}
#'   \item{decoy_frac_win}{Decoy fraction in the window (\code{D_alpha_win/n_win}).}
#'   \item{upper_used}{Upper endpoint actually used for the window.}
#'   \item{truncated}{Logical indicating whether the requested window was truncated
#'     (or dropped) due to export limits.}
#' }
#' This table is used to assess boundary support: very small \code{n_win} or
#' \code{D_alpha_win} indicates an unstable/poorly-supported cutoff.
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 5000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   pep = NA_real_
#' )
#' df$q <- diagFDR:::tdc_qvalues(df$score, df$is_decoy, add_decoy = 1L)
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy_tdc")
#'
#' dfdr_local_tail(x, alphas = c(1e-3, 5e-3, 1e-2), win_rel = 0.2, truncation = "cap")
#'
#' @export
dfdr_local_tail <- function(x, alphas, win_rel = 0.2,
                             truncation = c("warn_drop", "drop", "cap")) {
  validate_dfdr_tbl(x)
  truncation <- match.arg(truncation)

  meta <- attr(x, "meta") %||% list()
  qmax <- meta$q_max_export %||% NA_real_
  if (!is.finite(qmax)) qmax <- max(x$q, na.rm = TRUE)

  purrr::map_dfr(alphas, function(a) {
    delta <- win_rel * a
    upper <- a + delta

    if (upper <= qmax + 1e-12) {
      win <- x[x$q > a & x$q <= upper, , drop = FALSE]
      return(tibble::tibble(
        alpha = a, delta = delta,
        n_win = nrow(win),
        D_alpha_win = sum(win$is_decoy),
        decoy_frac_win = if (nrow(win) > 0) mean(win$is_decoy) else NA_real_,
        upper_used = upper,
        truncated = FALSE
      ))
    }

    if (truncation %in% c("warn_drop", "drop")) {
      if (truncation == "warn_drop") {
        rlang::warn(paste0(
          "Local tail window exceeds q_max_export (alpha=", a,
          ", upper=", signif(upper, 4),
          ", qmax=", signif(qmax, 4), "). Returning NA."
        ))
      }
      return(tibble::tibble(
        alpha = a, delta = delta,
        n_win = NA_integer_,
        D_alpha_win = NA_integer_,
        decoy_frac_win = NA_real_,
        upper_used = NA_real_,
        truncated = TRUE
      ))
    }

    # cap
    upper2 <- qmax
    win <- x[x$q > a & x$q <= upper2, , drop = FALSE]
    tibble::tibble(
      alpha = a, delta = upper2 - a,
      n_win = nrow(win),
      D_alpha_win = sum(win$is_decoy),
      decoy_frac_win = if (nrow(win) > 0) mean(win$is_decoy) else NA_real_,
      upper_used = upper2,
      truncated = TRUE
    )
  })
}

#' Threshold elasticity (list stability to cutoff perturbation)
#'
#' Computes the stability of the accepted target set under a multiplicative
#' perturbation of the threshold:
#' \deqn{S_\alpha(\epsilon) = J(A(\alpha), A((1+\epsilon)\alpha)),}
#' where \eqn{J} is the Jaccard index and \eqn{A(\alpha)} is the set of accepted
#' target IDs at threshold \eqn{\alpha}.
#'
#' @param x An \code{dfdr_tbl}.
#' @param alphas Numeric vector of FDR thresholds in \eqn{(0,1]}.
#' @param eps Numeric scalar \eqn{\epsilon \ge 0}. Relative perturbation (e.g.
#'   \code{0.2} compares \eqn{\alpha} vs \eqn{1.2\alpha}).
#'
#' @return
#' A \link[tibble:tibble]{tibble} with one row per \code{alpha}. Columns include:
#' \describe{
#'   \item{alpha}{The baseline threshold.}
#'   \item{eps}{The relative perturbation.}
#'   \item{n1}{Number of accepted targets at \code{alpha}.}
#'   \item{n2}{Number of accepted targets at \code{(1+eps)*alpha}.}
#'   \item{jaccard}{Jaccard overlap between the two accepted target sets.}
#' }
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 5000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = "run1",
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   pep = NA_real_
#' )
#'
#' df$q <- diagFDR:::tdc_qvalues(df$score, df$is_decoy, add_decoy = 1L)
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy_tdc")
#'
#' dfdr_elasticity(x, alphas = c(1e-3, 5e-3, 1e-2), eps = 0.2)
#'
#' @export
dfdr_elasticity <- function(x, alphas, eps = 0.2) {
  validate_dfdr_tbl(x)

  purrr::map_dfr(alphas, function(a) {
    A1 <- unique(x$id[!x$is_decoy & x$q <= a])
    A2 <- unique(x$id[!x$is_decoy & x$q <= (1 + eps) * a])
    tibble::tibble(
      alpha = a, eps = eps,
      n1 = length(A1), n2 = length(A2),
      jaccard = jaccard_vec(A1, A2)
    )
  })
}

#' Inter-run Jaccard overlap matrix at a fixed threshold
#'
#' Computes pairwise Jaccard similarity between runs based on accepted target IDs
#' at a fixed threshold \code{alpha}. This can reveal run-to-run disagreement or
#' heterogeneity in identifications.
#'
#' @param x An \code{dfdr_tbl} with a non-missing \code{run} column.
#' @param alpha Numeric FDR threshold in \eqn{(0,1]} used to define accepted
#'   targets (\code{q <= alpha}).
#'
#' @return
#' A \link[tibble:tibble]{tibble} with one row per run pair and columns:
#' \describe{
#'   \item{run1}{Name/identifier of the first run.}
#'   \item{run2}{Name/identifier of the second run.}
#'   \item{jaccard}{Jaccard similarity between accepted target ID sets.}
#' }
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 8000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = sample(c("runA", "runB", "runC"), n, replace = TRUE),
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n),
#'   pep = NA_real_
#' )
#' df$q <- diagFDR:::tdc_qvalues(df$score, df$is_decoy, add_decoy = 1L)
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy_tdc")
#'
#' dfdr_run_jaccard(x, alpha = 0.01)
#'
#' @export
dfdr_run_jaccard <- function(x, alpha = 0.01) {
  validate_dfdr_tbl(x)
  if (all(is.na(x$run))) rlang::abort("`run` is all NA: run-level Jaccard requires a run column.")

  dd <- x |>
    dplyr::filter(!is_decoy, q <= alpha, !is.na(run)) |>
    dplyr::select(run, id) |>
    dplyr::distinct()

  runs <- sort(unique(dd$run))
  sets <- split(dd$id, dd$run)

  out <- expand.grid(run1 = runs, run2 = runs, stringsAsFactors = FALSE)
  out$jaccard <- purrr::map2_dbl(out$run1, out$run2, function(r1, r2) {
    if (r1 == r2) return(1)
    jaccard_vec(sets[[r1]] %||% character(), sets[[r2]] %||% character())
  })
  tibble::as_tibble(out)
}

#' Storey \eqn{\pi_0(\lambda)} tail-uniformity diagnostic
#'
#' Estimates \eqn{\pi_0(\lambda) = \#\{p>\lambda\} / ((1-\lambda)m)} over a grid
#' of \code{lambdas}. This is a plausibility diagnostic for (pseudo-)p-values:
#' under a well-calibrated null, the upper tail should be approximately uniform,
#' leading to stable \eqn{\pi_0(\lambda)} curves.
#'
#' @param x A \code{dfdr_tbl} (or data.frame) containing column \code{p}.
#' @param lambdas Numeric vector in \eqn{[0,1)}. Typical range: 0.5--0.95.
#' @param stratify Optional character vector of column names used to stratify the
#'   diagnostic (e.g. \code{c("run")}).
#' @param min_n Integer. Minimum number of finite p-values required per stratum.
#' @param clamp Logical. If \code{TRUE} (default), clamp \code{pi0_hat} into
#'   \code{[0,1]} for reporting.
#'
#' @return
#' A list with components:
#' \describe{
#'   \item{pi0}{A \link[tibble:tibble]{tibble} with one row per \code{lambda} per
#'   stratum and columns \code{stratum}, \code{lambda}, \code{pi0_hat}, and \code{n}
#'   (the number of finite p-values in the stratum).}
#'   \item{summary}{A tibble with one row per stratum, including \code{n} and
#'   summary statistics of \code{pi0_hat} across \code{lambdas}
#'   (\code{median_pi0}, \code{sd_pi0}, \code{iqr_pi0}). Strata with \code{n < min_n}
#'   are reported with \code{NA} metrics and a \code{note}.}
#' }
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 6000
#' df <- tibble(
#'   run = sample(c("run1", "run2"), n, replace = TRUE),
#'   # mostly uniform p-values + a small enriched component
#'   p = c(stats::runif(5400), stats::rbeta(600, 0.3, 1))
#' )
#'
#' out <- dfdr_pi0_storey(df, stratify = "run", lambdas = seq(0.5, 0.9, by = 0.1), min_n = 500)
#' head(out$pi0)
#' out$summary
#'
#' @export
dfdr_pi0_storey <- function(x,
                            lambdas = seq(0.5, 0.95, by = 0.05),
                            stratify = NULL,
                            min_n = 2000,
                            clamp = TRUE) {
  x <- .safe_df(x)
  .check_has_cols(x, c("p"))

  if (!is.numeric(lambdas) || any(!is.finite(lambdas)) || any(lambdas < 0) || any(lambdas >= 1)) {
    rlang::abort("`lambdas` must be numeric values in [0,1).")
  }
  lambdas <- sort(unique(lambdas))

  strata_df <- .get_strata_df(x, stratify)
  if (is.null(strata_df)) {
    x$stratum <- "all"
  } else {
    x$stratum <- .make_stratum_id(strata_df)
  }

  out_pi0 <- list()
  out_sum <- list()

  for (s in unique(x$stratum)) {
    xs <- x[x$stratum == s, , drop = FALSE]
    p <- xs$p
    p <- p[is.finite(p)]
    m <- length(p)

    if (m < min_n) {
      out_sum[[s]] <- tibble::tibble(
        stratum = s, n = m,
        median_pi0 = NA_real_,
        sd_pi0 = NA_real_,
        iqr_pi0 = NA_real_,
        note = paste0("Skipped: n < ", min_n)
      )
      next
    }

    pi0_hat <- vapply(lambdas, function(lam) {
      sum(p > lam) / ((1 - lam) * m)
    }, numeric(1))

    if (isTRUE(clamp)) pi0_hat <- pmin(pmax(pi0_hat, 0), 1)

    pi0_tbl <- tibble::tibble(
      stratum = s,
      lambda = lambdas,
      pi0_hat = pi0_hat,
      n = m
    )
    out_pi0[[s]] <- pi0_tbl

    out_sum[[s]] <- tibble::tibble(
      stratum = s,
      n = m,
      median_pi0 = stats::median(pi0_hat, na.rm = TRUE),
      sd_pi0 = stats::sd(pi0_hat, na.rm = TRUE),
      iqr_pi0 = stats::IQR(pi0_hat, na.rm = TRUE),
      note = NA_character_
    )
  }

  pi0_all <- if (length(out_pi0)) dplyr::bind_rows(out_pi0) else tibble::tibble()
  sum_all <- dplyr::bind_rows(out_sum)

  list(pi0 = pi0_all, summary = sum_all)
}

#' Plot Storey \eqn{\pi_0(\lambda)} curve
#'
#' Plots \eqn{\pi_0(\lambda)} estimates returned by \code{\link{dfdr_pi0_storey}}.
#' Values closer to 1 suggest many nulls (few true effects), while values far below
#' 1 (especially if unstable across \code{lambda}) may indicate deviations from
#' tail-uniformity.
#'
#' @param pi0_tbl A tibble, typically \code{out$pi0} from \code{\link{dfdr_pi0_storey}}.
#'   Must contain columns \code{stratum}, \code{lambda}, and \code{pi0_hat}.
#' @param title Character. Plot title.
#'
#' @return
#' A \link[ggplot2:ggplot]{ggplot} object.
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' df <- tibble(
#'   stratum = rep("all", 5),
#'   lambda = seq(0.5, 0.9, by = 0.1),
#'   pi0_hat = c(0.95, 0.96, 0.97, 0.98, 0.99),
#'   n = 1000
#' )
#' p <- dfdr_plot_pi0(df)
#' p
#'
#' @export
dfdr_plot_pi0 <- function(pi0_tbl, title = "Storey pi0(lambda) diagnostic") {
  .check_has_cols(pi0_tbl, c("stratum", "lambda", "pi0_hat"))
  ggplot2::ggplot(pi0_tbl, ggplot2::aes(x = .data$lambda, y = .data$pi0_hat)) +
    ggplot2::geom_line() +
    ggplot2::geom_point(size = 1.2) +
    ggplot2::facet_wrap(~ stratum, scales = "free_y") +
    ggplot2::labs(x = expression(lambda), y = expression(hat(pi)[0](lambda)), title = title) +
    ggplot2::ylim(0, 1)
}

#' Write a manifest file describing parameters, list metadata, and warnings
#'
#' Writes a manifest alongside exported outputs to record key run parameters and
#' per-list metadata (unit, scope, q-source, export ceiling, etc.). This helps
#' tie diagnostic outputs to the exact inputs and settings used.
#'
#' If \pkg{jsonlite} is available (suggested dependency), the manifest is written
#' as JSON (\code{*.json}); otherwise a human-readable plain text file
#' (\code{*.txt}) is written.
#'
#' @param diag A list as returned by \code{\link{dfdr_run_all}}.
#' @param out_dir Character scalar. Output directory (created if it does not exist).
#' @param filename Character scalar. Base filename without extension (default \code{"manifest"}).
#'
#' @return
#' The path to the manifest file, returned invisibly. The function is called for
#' its side effect of writing a file to disk.
#'
#' @examples
#' library(tibble)
#'
#' # Create a minimal dfdr_run_all-like object to demonstrate manifest writing
#' df <- tibble(
#'   id = c("1","2","3"),
#'   run = "run1",
#'   is_decoy = c(FALSE, TRUE, FALSE),
#'   score = c(10, 9, 8),
#'   q = c(0.01, 0.02, 0.03),
#'   pep = NA_real_
#' )
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")
#'
#' diag <- list(
#'   objects = list(toy = x),
#'   params = list(alpha_main = 0.01),
#'   warnings = character()
#' )
#'
#' out_dir <- tempdir()
#' dfdr_write_manifest(diag, out_dir = out_dir, filename = "manifest_example")
#'
#' @export
dfdr_write_manifest <- function(diag, out_dir, filename = "manifest") {
  if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)

  # Collect list meta
  obj_meta <- lapply(diag$objects %||% list(), function(x) {
    attr(x, "meta") %||% list()
  })

  meta <- list(
    created = as.character(Sys.time()),
    package = "diagFDR",
    package_version = as.character(utils::packageVersion("diagFDR")),
    params = diag$params %||% list(),
    lists = obj_meta,
    warnings = diag$warnings %||% character()
  )

  if (requireNamespace("jsonlite", quietly = TRUE)) {
    path <- file.path(out_dir, paste0(filename, ".json"))
    jsonlite::write_json(meta, path = path, auto_unbox = TRUE, pretty = TRUE, null = "null")
  } else {
    # fall back to a readable plain text manifest
    path <- file.path(out_dir, paste0(filename, ".txt"))
    lines <- c(
      paste0("created: ", meta$created),
      paste0("package: ", meta$package),
      paste0("version: ", meta$package_version),
      "",
      "params:"
    )
    # params
    if (length(meta$params)) {
      for (nm in names(meta$params)) {
        lines <- c(lines, paste0("  ", nm, ": ", paste(meta$params[[nm]], collapse = ",")))
      }
    } else {
      lines <- c(lines, "  (none)")
    }

    lines <- c(lines, "", "lists:")
    if (length(meta$lists)) {
      for (nm in names(meta$lists)) {
        m <- meta$lists[[nm]]
        lines <- c(lines,
                   paste0("  - name: ", nm),
                   paste0("    unit: ", m$unit %||% NA),
                   paste0("    scope: ", m$scope %||% NA),
                   paste0("    q_source: ", m$q_source %||% NA),
                   paste0("    q_max_export: ", m$q_max_export %||% NA),
                   paste0("    p_source: ", m$p_source %||% NA)
        )
      }
    } else {
      lines <- c(lines, "  (none)")
    }

    lines <- c(lines, "", "warnings:")
    if (length(meta$warnings)) {
      lines <- c(lines, paste0("  - ", meta$warnings))
    } else {
      lines <- c(lines, "  (none)")
    }

    readr::write_lines(lines, path)
  }

  invisible(path)
}

#' Build a headline summary table (one row per list)
#'
#' Convenience helper to assemble a compact, export-ready summary table from the
#' output of \code{\link{dfdr_run_all}}. The function merges headline metrics with
#' selected columns from other diagnostics (e.g. local tail support, equal-chance
#' pooled estimate, and PEP reliability headline if available).
#'
#' @param diag A list as returned by \code{\link{dfdr_run_all}}.
#'
#' @return
#' A \link[tibble:tibble]{tibble} with one row per element of the input list used
#' in \code{\link{dfdr_run_all}} (i.e. one row per \code{list} label). The table
#' is suitable for exporting to a CSV (e.g. \code{summary_headline.csv}).
#'
#' The output typically contains (when available) headline target/decoy counts at
#' \code{alpha_main}, local boundary-support columns (e.g. \code{D_alpha_win},
#' \code{n_win}), equal-chance pooled columns (e.g. \code{pi_D_hat},
#' \code{effect_abs}), and PEP reliability headline quantities (e.g. \code{IPE}).
#' If \code{diag$tables$headline} is missing or empty, an empty tibble is returned.
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 4000
#' df <- tibble(
#'   id = as.character(seq_len(n)),
#'   run = sample(c("run1", "run2"), n, replace = TRUE),
#'   is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'   score = rnorm(n)
#' )
#' df$q <- diagFDR:::tdc_qvalues(df$score, df$is_decoy, add_decoy = 1L)
#' df$pep <- NA_real_
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy_tdc")
#'
#' diag <- dfdr_run_all(
#'   xs = list(toy = x),
#'   alpha_main = 0.01,
#'   compute_pseudo_pvalues = FALSE
#' )
#'
#' dfdr_summary_headline(diag)
#'
#' @export
dfdr_summary_headline <- function(diag) {
  stopifnot(is.list(diag))
  head_tbl <- diag$tables$headline
  if (!is.data.frame(head_tbl) || nrow(head_tbl) == 0) return(tibble::tibble())

  alpha_main <- (diag$params %||% list())$alpha_main %||% NA_real_

  out <- head_tbl

  # ---- local boundary support at alpha_main: D_alpha_win, n_win, decoy_frac_win, truncated
  lt <- diag$tables$local_tail
  if (is.data.frame(lt) && nrow(lt) > 0 && is.finite(alpha_main)) {
    lt_at <- dplyr::filter(lt, alpha == alpha_main) |>
      dplyr::select(dplyr::any_of(c("list","D_alpha_win","n_win","decoy_frac_win","delta","upper_used","truncated")))
    out <- dplyr::left_join(out, lt_at, by = "list")
  }

  # ---- equal-chance pooled (per list already in your outputs)
  ec <- diag$tables$equal_chance_pooled
  if (is.data.frame(ec) && nrow(ec) > 0) {
    # Your table does not currently include list; dfdr_run_all mutates list=.y, so it should.
    # Keep the key columns you showed:
    keep_ec <- intersect(names(ec), c("list","low_lo","low_hi","N_test","pi_D_hat","effect_abs",
                                      "ci95_lo","ci95_hi","p_value_binom","pass_minN","qmax_export"))
    if ("list" %in% keep_ec) out <- dplyr::left_join(out, ec[, keep_ec, drop = FALSE], by = "list")
  }

  # ---- IPE (if present)
  pep_ipe <- diag$tables$pep_IPE
  if (is.data.frame(pep_ipe) && nrow(pep_ipe) > 0) {
    out <- dplyr::left_join(out, pep_ipe, by = "list")
  }

  # Keep a stable column order (adjust as you like)
  preferred <- c(
    "list","alpha",
    "T_alpha","D_alpha","FDR_hat","CV_hat",
    "D_alpha_win","n_win","decoy_frac_win","truncated",
    "FDR_minus1","FDR_plus1","FDR_minus2sqrtD","FDR_plus2sqrtD",
    "pi_D_hat","effect_abs","ci95_lo","ci95_hi","p_value_binom","pass_minN",
    "IPE"
  )
  out |>
    dplyr::select(dplyr::any_of(preferred), dplyr::everything()) |>
    dplyr::arrange(list)
}

#' Write a human-readable \code{README.md} report
#'
#' Produces a lightweight narrative report in Markdown that summarises the most
#' important diagnostics (headline summary, key plots, exported tables) and records
#' per-list metadata (unit, scope, q-source, etc.) from a \code{\link{dfdr_run_all}}
#' result. This is intended to accompany an export folder for auditing and sharing.
#'
#' @param diag A list as returned by \code{\link{dfdr_run_all}}.
#' @param out_dir Character scalar. Output directory (created if it does not exist).
#' @param filename Character scalar. Markdown filename (default \code{"README.md"}).
#'
#' @return
#' The path to the written README file, returned invisibly. The function is called
#' for its side effect of writing a file to disk.
#'
#' @examples
#' library(tibble)
#'
#' # Minimal dfdr_run_all-like object for demonstrating README writing
#' df <- tibble(
#'   id = c("1","2","3"),
#'   run = "run1",
#'   is_decoy = c(FALSE, TRUE, FALSE),
#'   score = c(10, 9, 8),
#'   q = c(0.01, 0.02, 0.03),
#'   pep = NA_real_
#' )
#' x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")
#'
#' diag <- list(
#'   objects = list(toy = x),
#'   params = list(alpha_main = 0.01),
#'   tables = list(headline = tibble(list = "toy", alpha = 0.01)),
#'   warnings = character()
#' )
#'
#' out_dir <- tempdir()
#' dfdr_write_readme(diag, out_dir = out_dir, filename = "README_example.md")
#'
#' @export
dfdr_write_readme <- function(diag, out_dir, filename = "README.md") {
  if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)
  path <- file.path(out_dir, filename)

  summ <- dfdr_summary_headline(diag)

  md_table <- function(df, digits = 4) {
    if (!is.data.frame(df) || nrow(df) == 0) return(c("(no rows)"))
    df_chr <- as.data.frame(lapply(df, function(z) {
      if (is.numeric(z)) format(z, digits = digits, scientific = TRUE) else as.character(z)
    }), stringsAsFactors = FALSE)
    header <- paste0("| ", paste(names(df_chr), collapse = " | "), " |")
    sep    <- paste0("| ", paste(rep("---", ncol(df_chr)), collapse = " | "), " |")
    rows   <- apply(df_chr, 1, function(r) paste0("| ", paste(r, collapse = " | "), " |"))
    c(header, sep, rows)
  }

  alpha_main <- (diag$params %||% list())$alpha_main %||% NA_real_
  lines <- c(
    "# diagFDR report",
    "",
    paste0("- Created: ", as.character(Sys.time())),
    paste0("- diagFDR version: ", as.character(utils::packageVersion("diagFDR"))),
    paste0("- Headline threshold (alpha_main): ", alpha_main),
    "",
    "## Headline summary (Table 6 essentials)",
    "",
    md_table(summ),
    "",
    "## Plots",
    "",
    "Plots are stored in `plots/` (PNG). Key plots include:",
    "- `plots/dalpha.png` (decoy support across alpha)",
    "- `plots/cv.png` (instability proxy across alpha)",
    "- `plots/dwin.png` (local boundary decoy support across alpha)",
    "- `plots/elasticity.png` (Jaccard stability across alpha)",
    "",
    "## Exported tables",
    "",
    "All diagnostic tables are exported as CSV in this folder. The recommended single-file summary is:",
    "- `summary_headline.csv`",
    "",
    "## Metadata",
    ""
  )

  # list metadata
  if (!is.null(diag$objects) && length(diag$objects)) {
    for (nm in names(diag$objects)) {
      meta <- attr(diag$objects[[nm]], "meta") %||% list()
      lines <- c(lines,
                 paste0("### ", nm),
                 paste0("- unit: ", meta$unit %||% "NA"),
                 paste0("- scope: ", meta$scope %||% "NA"),
                 paste0("- q_source: ", meta$q_source %||% "NA"),
                 paste0("- q_max_export: ", meta$q_max_export %||% "NA"),
                 if (!is.null(meta$p_source) && !is.na(meta$p_source)) paste0("- p_source: ", meta$p_source) else NULL,
                 ""
      )
    }
  } else {
    lines <- c(lines, "(No objects found in `diag$objects`.)", "")
  }

  warn <- diag$warnings %||% character()
  lines <- c(lines, "## Warnings / notes", "")
  lines <- c(lines, if (length(warn)) paste0("- ", warn) else "- (none)", "")

  readr::write_lines(lines, path)
  invisible(path)
}

#' Detect DIA-NN run column name
#' @keywords internal
diann_detect_run_col <- function(df, candidates = c("Run", "File.Name", "File.Name.Index")) {
  nm <- names(df)
  hit <- candidates[candidates %in% nm]
  if (length(hit) == 0) return(NA_character_)
  hit[[1]]
}

#' Validate that required DIA-NN columns exist
#' @keywords internal
diann_require_cols <- function(df, cols) {
  miss <- setdiff(cols, names(df))
  if (length(miss)) rlang::abort(paste0("Missing required DIA-NN columns: ", paste(miss, collapse = ", ")))
  invisible(TRUE)
}

#' Read a DIA-NN \code{report.parquet}
#'
#' Reads a DIA-NN \code{report.parquet} file using the \pkg{arrow} package
#' (suggested dependency) and returns it as a tibble.
#'
#' @param path Character scalar. Path to a DIA-NN \code{report.parquet} file.
#'
#' @return
#' A \link[tibble:tibble]{tibble} containing the columns present in the DIA-NN
#' parquet report (column names depend on DIA-NN export settings).
#'
#' @examples
#' if (requireNamespace("arrow", quietly = TRUE)) {
#'   # Create a tiny parquet file and read it back
#'   tmp <- tempfile(fileext = ".parquet")
#'   df <- data.frame(Precursor.Id = c("P1", "P2"), Q.Value = c(0.01, 0.02))
#'   arrow::write_parquet(df, tmp)
#'   out <- read_diann_parquet(tmp)
#'   out
#' }
#'
#' @export
read_diann_parquet <- function(path) {
  if (!requireNamespace("arrow", quietly = TRUE)) {
    rlang::abort("Reading DIA-NN parquet requires the 'arrow' package (install.packages('arrow')).")
  }
  if (!file.exists(path)) rlang::abort(paste0("File not found: ", path))
  arrow::read_parquet(path) |> tibble::as_tibble()
}

#' DIA-NN -> global precursor universe (deduplicated by \code{Precursor.Id})
#'
#' Constructs a precursor-level universe by aggregating a DIA-NN report table to
#' one row per \code{Precursor.Id}. For each precursor, the minimum q-value is
#' used (via \code{safe_min}) and \code{is_decoy} is set to \code{TRUE} if any row
#' in the group is a decoy.
#'
#' @param rep A DIA-NN report tibble (e.g. returned by \code{\link{read_diann_parquet}}).
#' @param q_col Character. Column name for q-values (default \code{"Q.Value"}).
#' @param pep_col Optional character. PEP column name (default \code{"PEP"} if present).
#' @param score_col Optional character. Score column name to retain; if missing, \code{score} is \code{NA}.
#' @param q_max_export Optional numeric export ceiling used in DIA-NN export (e.g. 0.5).
#' @param unit Character. Unit metadata stored in the returned object.
#' @param scope Character. Scope metadata stored in the returned object.
#' @param q_source Character. Label stored in metadata describing the q-value source.
#'
#' @return
#' A \code{dfdr_tbl} (tibble subclass) with one row per precursor and required
#' columns \code{id}, \code{is_decoy}, \code{q}, \code{pep}, \code{run}, \code{score}.
#' Metadata are stored in \code{attr(x, "meta")}.
#'
#' @examples
#' library(tibble)
#'
#' rep <- tibble(
#'   Precursor.Id = c("P1", "P1", "P2"),
#'   Decoy = c(0L, 0L, 1L),
#'   Q.Value = c(0.01, 0.02, 0.03),
#'   PEP = c(0.02, 0.01, 0.9)
#' )
#'
#' x <- diann_global_precursor(rep, q_col = "Q.Value", q_max_export = 0.5)
#' x
#'
#' @export
diann_global_precursor <- function(rep,
                                   q_col = "Q.Value",
                                   pep_col = NULL,
                                   score_col = NULL,
                                   q_max_export = NA_real_,
                                   unit = "precursor",
                                   scope = "global",
                                   q_source = q_col) {
  diann_require_cols(rep, c("Precursor.Id", "Decoy", q_col))

  if (is.null(pep_col)) pep_col <- if ("PEP" %in% names(rep)) "PEP" else NULL

  # score is not consistently named across DIA-NN exports; keep optional
  has_score <- !is.null(score_col) && score_col %in% names(rep)
  has_pep <- !is.null(pep_col) && pep_col %in% names(rep)

  dd <- rep |>
    dplyr::transmute(
      id = as.character(.data[["Precursor.Id"]]),
      is_decoy = as.integer(.data[["Decoy"]]) == 1L,
      q = suppressWarnings(as.numeric(.data[[q_col]])),
      pep = if (has_pep) suppressWarnings(as.numeric(.data[[pep_col]])) else NA_real_,
      run = NA_character_,
      score = if (has_score) suppressWarnings(as.numeric(.data[[score_col]])) else NA_real_
    ) |>
    dplyr::filter(!is.na(id), is.finite(q), q >= 0, q <= 1) |>
    dplyr::group_by(id) |>
    dplyr::summarise(
      is_decoy = any(is_decoy),
      q = safe_min(q),
      pep = safe_min(pep),
      run = NA_character_,
      score = safe_min(score),
      .groups = "drop"
    )

  as_dfdr_tbl(
    dd,
    unit = unit,
    scope = scope,
    q_source = q_source,
    q_max_export = q_max_export,
    provenance = list(tool = "DIA-NN")
  )
}

#' DIA-NN -> run-by-precursor universe (deduplicated within run)
#'
#' Constructs a run-specific universe by aggregating a DIA-NN report table to one
#' row per \code{(run, Precursor.Id)}. The run column is detected automatically
#' unless provided. For each group, the minimum q-value is used and \code{is_decoy}
#' is \code{TRUE} if any row is a decoy.
#'
#' @param rep A DIA-NN report tibble.
#' @param q_col Character. q-value column name (default \code{"Q.Value"}).
#' @param run_col Optional character. Name of the run column. If \code{NULL}, the
#'   function attempts to detect one of \code{"Run"}, \code{"File.Name"}, or
#'   \code{"File.Name.Index"}.
#' @param pep_col Optional character. PEP column name (default \code{"PEP"} if present).
#' @param score_col Optional character. Score column name (if present).
#' @param q_max_export Optional numeric export ceiling (e.g. 0.5), stored as metadata.
#' @param id_mode Character. Either \code{"runxid"} (default) to set
#'   \code{id = paste(run, Precursor.Id, sep="||")} or \code{"id"} to set
#'   \code{id = Precursor.Id} and keep \code{run} separate.
#' @param unit Character. Unit metadata stored in the returned object.
#' @param scope Character. Scope metadata stored in the returned object.
#' @param q_source Character. Label stored in metadata describing the q-value source.
#'
#' @return
#' A \code{dfdr_tbl} with one row per run-by-precursor unit. The returned table
#' includes \code{id}, \code{run}, \code{is_decoy}, \code{q}, \code{pep}, and
#' \code{score}. Metadata are stored in \code{attr(x, "meta")}.
#'
#' @examples
#' library(tibble)
#'
#' rep <- tibble(
#'   Run = c("r1", "r1", "r2", "r2"),
#'   Precursor.Id = c("P1", "P1", "P1", "P2"),
#'   Decoy = c(0L, 0L, 1L, 0L),
#'   Q.Value = c(0.01, 0.02, 0.03, 0.02),
#'   PEP = c(0.02, 0.01, 0.9, 0.05)
#' )
#'
#' x <- diann_runxprecursor(rep, q_col = "Q.Value", run_col = "Run", id_mode = "runxid")
#' x
#'
#' @export
diann_runxprecursor <- function(rep,
                                q_col = "Q.Value",
                                run_col = NULL,
                                pep_col = NULL,
                                score_col = NULL,
                                q_max_export = NA_real_,
                                id_mode = c("id", "runxid"),
                                unit = "runxprecursor",
                                scope = "runwise",
                                q_source = q_col) {
  diann_require_cols(rep, c("Precursor.Id", "Decoy", q_col))

  id_mode <- match.arg(id_mode)

  if (is.null(run_col)) run_col <- diann_detect_run_col(rep)
  if (is.na(run_col)) rlang::abort("No run column found (expected one of Run / File.Name / File.Name.Index).")

  if (is.null(pep_col)) pep_col <- if ("PEP" %in% names(rep)) "PEP" else NULL
  has_pep <- !is.null(pep_col) && pep_col %in% names(rep)
  has_score <- !is.null(score_col) && score_col %in% names(rep)

  dd <- rep |>
    dplyr::transmute(
      run = as.character(.data[[run_col]]),
      precursor = as.character(.data[["Precursor.Id"]]),
      is_decoy = as.integer(.data[["Decoy"]]) == 1L,
      q = suppressWarnings(as.numeric(.data[[q_col]])),
      pep = if (has_pep) suppressWarnings(as.numeric(.data[[pep_col]])) else NA_real_,
      score = if (has_score) suppressWarnings(as.numeric(.data[[score_col]])) else NA_real_
    ) |>
    dplyr::filter(!is.na(run), !is.na(precursor), is.finite(q), q >= 0, q <= 1) |>
    dplyr::group_by(run, precursor) |>
    dplyr::summarise(
      is_decoy = any(is_decoy),
      q = safe_min(q),
      pep = safe_min(pep),
      score = safe_min(score),
      .groups = "drop"
    ) |>
    dplyr::mutate(
      id = if (id_mode == "runxid") paste(run, precursor, sep = "||") else precursor
    ) |>
    dplyr::select(id, is_decoy, q, pep, run, score)

  as_dfdr_tbl(
    dd,
    unit = unit,
    scope = scope,
    q_source = q_source,
    q_max_export = q_max_export,
    provenance = list(tool = "DIA-NN", run_col = run_col, id_mode = id_mode)
  )
}

#' DIA-NN: global precursor list built by minimum run-wise q across runs
#'
#' Constructs a precursor-level table by taking the minimum run-wise q-value
#' across runs for each \code{Precursor.Id}. This mirrors a common scope misuse
#' (aggregating run-wise q-values into a single global list) and is useful for
#' scope-disagreement diagnostics.
#'
#' @param rep A DIA-NN report table (e.g. returned by \code{\link{read_diann_parquet}}).
#' @param run_col Optional character. Name of the run column. If \code{NULL},
#'   attempts to detect it (e.g. \code{"Run"} or \code{"File.Name"}).
#' @param q_col Character. Name of the q-value column to aggregate (default \code{"Q.Value"}).
#' @param pep_col Optional character. Name of the PEP column (default \code{"PEP"} if present).
#' @param score_col Optional character. Name of a score column to carry along (if present).
#' @param q_max_export Optional numeric export ceiling used by the tool (e.g. 0.5),
#'   stored as metadata for truncation-aware diagnostics.
#' @param unit Character. Unit metadata stored in the returned object.
#' @param scope Character. Scope metadata stored in the returned object.
#' @param q_source Character. Source label stored in metadata.
#'
#' @return
#' A \code{dfdr_tbl} with one row per precursor, where \code{q} equals the minimum
#' of the run-wise q-values across runs for that precursor. The returned object is
#' intended for diagnostics (e.g. scope disagreement), not as a recommended FDR procedure.
#'
#' @examples
#' library(tibble)
#'
#' rep <- tibble(
#'   Run = c("r1", "r2", "r1", "r2"),
#'   Precursor.Id = c("P1", "P1", "P2", "P2"),
#'   Decoy = c(0L, 0L, 0L, 1L),
#'   Q.Value = c(0.02, 0.01, 0.05, 0.04),
#'   PEP = c(0.03, 0.02, 0.1, 0.9)
#' )
#'
#' x <- diann_global_minrunq(rep, run_col = "Run", q_col = "Q.Value", q_max_export = 0.5)
#' x
#'
#' @export
diann_global_minrunq <- function(rep,
                                 run_col = NULL,
                                 q_col = "Q.Value",
                                 pep_col = NULL,
                                 score_col = NULL,
                                 q_max_export = NA_real_,
                                 unit = "precursor",
                                 scope = "aggregated",
                                 q_source = paste0("min_run(", q_col, ")")) {
  diann_require_cols(rep, c("Precursor.Id", "Decoy", q_col))

  if (is.null(run_col)) run_col <- diann_detect_run_col(rep)
  if (is.na(run_col)) rlang::abort("No run column found (expected one of Run / File.Name / File.Name.Index).")

  if (is.null(pep_col)) pep_col <- if ("PEP" %in% names(rep)) "PEP" else NULL
  has_pep <- !is.null(pep_col) && pep_col %in% names(rep)
  has_score <- !is.null(score_col) && score_col %in% names(rep)

  dd <- rep |>
    dplyr::transmute(
      run = as.character(.data[[run_col]]),
      id = as.character(.data[["Precursor.Id"]]),
      is_decoy = as.integer(.data[["Decoy"]]) == 1L,
      q = suppressWarnings(as.numeric(.data[[q_col]])),
      pep = if (has_pep) suppressWarnings(as.numeric(.data[[pep_col]])) else NA_real_,
      score = if (has_score) suppressWarnings(as.numeric(.data[[score_col]])) else NA_real_
    ) |>
    dplyr::filter(!is.na(run), !is.na(id), is.finite(q), q >= 0, q <= 1) |>
    dplyr::group_by(id) |>
    dplyr::summarise(
      is_decoy = any(is_decoy),
      q = safe_min(q),     # min over runs
      pep = safe_min(pep), # min PEP over runs (not guaranteed calibrated; still useful as diagnostic)
      run = NA_character_,
      score = safe_min(score),
      .groups = "drop"
    ) |>
    dplyr::select(id, is_decoy, q, pep, run, score)

  as_dfdr_tbl(
    dd,
    unit = unit,
    scope = scope,
    q_source = q_source,
    q_max_export = q_max_export,
    provenance = list(tool = "DIA-NN", run_col = run_col)
  )
}

#' Write diagnostic outputs to a folder
#'
#' Exports the output of \code{\link{dfdr_run_all}} to disk. Depending on selected
#' \code{formats}, this function can:
#' \itemize{
#'   \item write diagnostic tables as CSV;
#'   \item write plots as PNG;
#'   \item create a PowerPoint (\code{.pptx}) containing the plots (requires \pkg{officer} and \pkg{rvg} in Suggests);
#'   \item write a lightweight manifest (\code{\link{dfdr_write_manifest}});
#'   \item write a human-readable \code{README.md} (\code{\link{dfdr_write_readme}});
#'   \item write a compact headline summary table (\code{summary_headline.csv}).
#' }
#'
#' @param diag A list as returned by \code{\link{dfdr_run_all}}.
#' @param out_dir Character scalar. Output directory (created if it does not exist).
#' @param formats Character vector. Subset of
#'   \code{c("csv","png","pptx","manifest","readme","summary")}.
#' @param pptx_path Character scalar. Output path for the PPTX file (only used if
#'   \code{"pptx"} is included in \code{formats}).
#' @param width,height Numeric. Plot size (in inches) passed to \code{ggplot2::ggsave()}
#'   when writing PNGs.
#' @param dpi Numeric. DPI passed to \code{ggplot2::ggsave()}.
#'
#' @return
#' Returns \code{TRUE} invisibly. The function is called for its side effects of
#' writing files to \code{out_dir}.
#'
#' @examples
#' library(tibble)
#'
#' # Create a minimal diag object with one table and one plot
#' diag <- list(
#'   tables = list(headline = tibble(list = "toy", alpha = 0.01, D_alpha = 10)),
#'   plots = list(example_plot = ggplot2::ggplot(tibble(x = 1:3, y = 1:3),
#'                                              ggplot2::aes(x, y)) +
#'                 ggplot2::geom_point()),
#'   objects = list(),
#'   params = list(alpha_main = 0.01),
#'   warnings = character()
#' )
#'
#' out_dir <- tempdir()
#' dfdr_write_report(
#' diag, out_dir = out_dir,
#' formats = c("csv", "png", "summary", "manifest", "readme"))
#'
#' @export
dfdr_write_report <- function(diag,
                              out_dir,
                              formats = c("csv", "png", "manifest", "readme", "summary"),
                              pptx_path = file.path(out_dir, "diagFDR_report.pptx"),
                              width = 7.2, height = 4.4, dpi = 180) {
  if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)
  formats <- unique(formats)

  if ("manifest" %in% formats) dfdr_write_manifest(diag, out_dir)
  if ("readme" %in% formats) dfdr_write_readme(diag, out_dir)
  if ("summary" %in% formats) {
    summ <- dfdr_summary_headline(diag)
    readr::write_csv(summ, file.path(out_dir, "summary_headline.csv"))
  }

  # ---- CSV tables
  if ("csv" %in% formats) {
    tabs <- diag$tables
    for (nm in names(tabs)) {
      obj <- tabs[[nm]]
      nm2 <- dfdr_safe_filename(nm)

      if (is.data.frame(obj)) {
        readr::write_csv(obj, file.path(out_dir, paste0(nm2, ".csv")))
      } else if (is.list(obj)) {
        subdir <- file.path(out_dir, nm2)
        dir.create(subdir, showWarnings = FALSE, recursive = TRUE)
        for (k in names(obj)) {
          k2 <- dfdr_safe_filename(k)
          if (is.data.frame(obj[[k]])) {
            readr::write_csv(obj[[k]], file.path(subdir, paste0(k2, ".csv")))
          }
        }
      }
    }
  }

  # ---- PNG plots
  if ("png" %in% formats) {
    pdir <- file.path(out_dir, "plots")
    dir.create(pdir, showWarnings = FALSE, recursive = TRUE)
    for (nm in names(diag$plots)) {
      nm2 <- dfdr_safe_filename(nm)
      ggplot2::ggsave(file.path(pdir, paste0(nm2, ".png")),
                      diag$plots[[nm]], width = width, height = height, dpi = dpi)
    }
  }

  # ---- PPTX (optional)
  if ("pptx" %in% formats) {
    if (!requireNamespace("officer", quietly = TRUE) ||
        !requireNamespace("rvg", quietly = TRUE)) {
      rlang::abort("PPTX export requires packages 'officer' and 'rvg' in Suggests.")
    }
    doc <- officer::read_pptx()
    for (nm in names(diag$plots)) {
      doc <- officer::add_slide(doc, layout = "Title and Content", master = "Office Theme")
      doc <- officer::ph_with(doc, value = nm, location = officer::ph_location_type(type = "title"))
      doc <- officer::ph_with(doc, value = rvg::dml(ggobj = diag$plots[[nm]]),
                              location = officer::ph_location_type(type = "body"))
    }
    print(doc, target = pptx_path)
  }

  invisible(TRUE)
}

#' BH diagnostics at a headline alpha (threshold, discoveries, boundary support)
#'
#' Computes the Benjamini--Hochberg (BH) rejection threshold \eqn{t_\alpha}, the
#' number of discoveries \eqn{R_\alpha}, and a simple "boundary support" measure:
#' the number of p-values just above \eqn{t_\alpha}. Boundary support is intended
#' to indicate how sensitive the discovery set may be to small perturbations of
#' p-values near the cutoff.
#'
#' @param x A \code{dfdr_tbl} (or data frame) containing columns \code{id} and
#'   \code{p}.
#' @param alpha Numeric BH FDR level in \eqn{(0,1)}.
#' @param boundary Character. One of \code{"mult"} (multiplicative window) or
#'   \code{"add"} (additive window).
#' @param win Numeric. Relative width for the multiplicative boundary window
#'   (default 0.2). Used when \code{boundary = "mult"}, defining the interval
#'   \eqn{(t_\alpha, (1+\mathrm{win})t_\alpha]}.
#' @param delta Numeric. Additive width for the additive boundary window. Required
#'   when \code{boundary = "add"}, defining the interval \eqn{(t_\alpha, t_\alpha+\delta]}.
#'
#' @return
#' A list with two elements:
#' \describe{
#'   \item{headline}{A one-row \link[tibble:tibble]{tibble} containing BH summary
#'   diagnostics, including \code{t_alpha} (the BH threshold), \code{R_alpha} (the
#'   number of rejected hypotheses / discoveries), and \code{N_boundary} (the
#'   number of p-values in a right-neighborhood above the cutoff as determined by
#'   \code{boundary}, \code{win}, and \code{delta}).}
#'   \item{accepted}{A character vector of \code{id} values corresponding to
#'   discoveries (rows with \code{p <= t_alpha}). If no finite BH threshold exists,
#'   this is \code{character(0)}.}
#' }
#'
#' @examples
#' library(tibble)
#'
#' set.seed(1)
#' n <- 5000
#' x <- tibble(
#'   id = as.character(seq_len(n)),
#'   p = c(stats::runif(4500), stats::rbeta(500, 0.3, 1))
#' )
#'
#' out <- dfdr_bh_diagnostics(x, alpha = 0.01, boundary = "mult", win = 0.2)
#' out$headline
#' length(out$accepted)
#'
#' @export
dfdr_bh_diagnostics <- function(x,
                                alpha = 0.01,
                                boundary = c("mult", "add"),
                                win = 0.2,
                                delta = NULL) {
  boundary <- match.arg(boundary)
  x <- .safe_df(x)
  .check_has_cols(x, c("id", "p"))

  if (!is.numeric(alpha) || length(alpha) != 1 || !is.finite(alpha) || alpha <= 0 || alpha >= 1) {
    rlang::abort("`alpha` must be a single number in (0,1).")
  }
  if (boundary == "mult") {
    if (!is.numeric(win) || length(win) != 1 || !is.finite(win) || win <= 0) {
      rlang::abort("`win` must be a single positive number for boundary='mult'.")
    }
  } else {
    if (is.null(delta) || !is.numeric(delta) || length(delta) != 1 || !is.finite(delta) || delta <= 0) {
      rlang::abort("`delta` must be a single positive number for boundary='add'.")
    }
  }

  ok <- is.finite(x$p)
  p <- x$p[ok]
  ids <- x$id[ok]
  m <- length(p)

  t_alpha <- .bh_threshold(p, alpha)

  if (!is.finite(t_alpha)) {
    headline <- tibble::tibble(
      alpha = alpha,
      m = m,
      t_alpha = NA_real_,
      R_alpha = 0L,
      N_boundary = NA_integer_,
      boundary_type = boundary,
      win = if (boundary == "mult") win else NA_real_,
      delta = if (boundary == "add") delta else NA_real_
    )
    return(list(headline = headline, accepted = character()))
  }

  accepted_ids <- ids[p <= t_alpha]
  R_alpha <- length(accepted_ids)

  if (boundary == "mult") {
    upper <- min(1, (1 + win) * t_alpha)
  } else {
    upper <- min(1, t_alpha + delta)
  }
  N_boundary <- sum(p > t_alpha & p <= upper)

  headline <- tibble::tibble(
    alpha = alpha,
    m = m,
    t_alpha = t_alpha,
    R_alpha = as.integer(R_alpha),
    N_boundary = as.integer(N_boundary),
    boundary_type = boundary,
    win = if (boundary == "mult") win else NA_real_,
    delta = if (boundary == "add") delta else NA_real_
  )

  list(headline = headline, accepted = accepted_ids)
}

#' Render a human-readable HTML report from \code{dfdr_run_all} output
#'
#' Renders an HTML report that summarises key diagnostics and embeds plots
#' contained in \code{diag$plots}. The report is intended for interactive review
#' and verifiable reporting.
#'
#' This function requires the \pkg{rmarkdown} package (suggested dependency) and
#' uses the built-in R Markdown template shipped with the package
#' (\code{inst/templates/dfdr_report.Rmd}).
#'
#' @param diag A list as returned by \code{\link{dfdr_run_all}}.
#' @param out_dir Character scalar. Output directory (created if it does not exist).
#' @param filename Character scalar. Output HTML filename (default
#'   \code{"diagFDR_report.html"}).
#' @param self_contained Logical. If \code{TRUE}, embed resources into a single HTML
#'   file (may be larger). Default \code{FALSE}.
#' @param open Logical. If \code{TRUE}, open the report in the default browser
#'   after rendering.
#'
#' @return
#' The path to the rendered HTML file, returned invisibly. The function is called
#' for its side effect of creating an HTML report on disk.
#'
#' @examples
#' # A minimal example that renders a report from a toy dataset.
#' # This example is conditional because rmarkdown is in Suggests.
#' if (requireNamespace("rmarkdown", quietly = TRUE)) {
#'   library(tibble)
#'   tmpdir <- tempdir()
#'
#'   set.seed(1)
#'   n <- 3000
#'   df <- tibble(
#'     id = as.character(seq_len(n)),
#'     run = "run1",
#'     is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
#'     score = rnorm(n),
#'     q = pmin(1, rank(-score) / n),
#'     pep = NA_real_
#'   )
#'   x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")
#'
#'   diag <- dfdr_run_all(
#'     xs = list(toy = x),
#'     alpha_main = 0.01,
#'     compute_pseudo_pvalues = FALSE
#'   )
#'
#'   # Render to a temporary directory (does not open a browser during checks)
#'   dfdr_render_report(diag, out_dir = tmpdir, open = FALSE)
#' }
#'
#' @export
dfdr_render_report <- function(diag, out_dir,
                               filename = "diagFDR_report.html",
                               self_contained = FALSE,
                               open = FALSE) {
  if (!requireNamespace("rmarkdown", quietly = TRUE)) {
    rlang::abort("HTML report rendering requires package 'rmarkdown' (in Suggests).")
  }
  if (!dir.exists(out_dir)) dir.create(out_dir, recursive = TRUE)

  template <- system.file("templates", "dfdr_report.Rmd", package = "diagFDR")
  if (template == "") rlang::abort("Could not find template inst/templates/dfdr_report.Rmd in the installed package.")

  out_file <- file.path(out_dir, filename)

  rmarkdown::render(
    input = template,
    output_file = out_file,
    output_format = rmarkdown::html_document(
      self_contained = self_contained, toc = TRUE, toc_depth = 3, theme = "readable"
    ),
    intermediates_dir = out_dir,
    params = list(diag = diag, out_dir = out_dir),
    envir = new.env(parent = globalenv()),
    quiet = TRUE
  )

  if (isTRUE(open)) utils::browseURL(out_file)
  invisible(out_file)
}

#' Read MaxQuant \code{msms.txt} into a \code{dfdr_tbl} (PSM-level; reconstructed TDC q-values)
#'
#' Reads a MaxQuant \code{msms.txt} file and returns a \code{dfdr_tbl} at the PSM
#' level. q-values are reconstructed by target-decoy counting (TDC) using MaxQuant
#' \code{Score} (higher is better) and the \code{Reverse} column as the decoy
#' indicator (\code{"+"} for decoy).
#'
#' @details
#' Required columns in \code{msms.txt}:
#' \itemize{
#'   \item \code{id} (unique PSM identifier; numeric or character)
#'   \item \code{Raw file} (run name)
#'   \item \code{Reverse} (MaxQuant decoy indicator; \code{"+"} for decoy; blank/NA for target)
#'   \item \code{Score} (MaxQuant score; higher is better)
#'   \item \code{PEP} (posterior error probability; optional in practice, see \code{pep_mode})
#' }
#'
#' The function computes q-values by target-decoy counting (TDC) using \code{Score}
#' and the \code{Reverse} decoy label:
#' \deqn{\widehat{\mathrm{FDR}}(i) = (D(i) + add\_decoy) / T(i), \qquad
#'      q(i) = \min_{j \ge i}\widehat{\mathrm{FDR}}(j)}
#'
#' Contaminants are not treated as decoys. If the column
#' \code{Potential contaminant} exists and \code{exclude_contaminants = TRUE},
#' those rows are removed prior to computing q-values.
#'
#' @param path Character scalar. Path to MaxQuant \code{msms.txt}.
#' @param pep_mode Character. How to handle the \code{PEP} column:
#'   \describe{
#'     \item{\code{"drop"}}{Set \code{pep = NA} for all rows.}
#'     \item{\code{"sanitize"}}{Keep finite PEP in \code{[0,1]}; set other finite values to \code{NA} and warn.}
#'     \item{\code{"strict"}}{Error if any finite PEP is outside \code{[0,1]}.}
#'   }
#' @param exclude_contaminants Logical. If \code{TRUE} and a
#'   \code{Potential contaminant} column exists, rows with \code{"+"} are removed.
#'   Default is \code{TRUE}.
#' @param add_decoy Integer scalar. Additive correction in the TDC estimate:
#'   \eqn{\widehat{\mathrm{FDR}} = (D + add\_decoy)/T}. Default is 1.
#' @param unit Character. Unit stored in the returned object metadata (default \code{"psm"}).
#' @param scope Character. Scope stored in the returned object metadata (default \code{"global"}).
#' @param provenance Named list. Stored in the returned object metadata.
#'
#' @return
#' A \code{dfdr_tbl} (tibble subclass) with one row per PSM and columns:
#' \code{id}, \code{run}, \code{is_decoy}, \code{q}, \code{pep}, \code{score}.
#' The q-values are reconstructed by TDC from \code{Score} and \code{Reverse}.
#' Metadata are stored in \code{attr(x, "meta")} (including \code{unit}, \code{scope},
#' \code{q_source}, and \code{provenance}).
#'
#' @examples
#' if (requireNamespace("readr", quietly = TRUE)) {
#'   # Create a tiny MaxQuant-like msms.txt and read it
#'   tmp <- tempfile(fileext = ".txt")
#'   txt <- paste0(
#'     "id\tRaw file\tReverse\tScore\tPEP\tPotential contaminant\n",
#'     "1\trun1\t\t100\t0.01\t\n",
#'     "2\trun1\t+\t90\t0.80\t\n",
#'     "3\trun1\t\t80\t0.02\t+\n",   # contaminant row removed when exclude_contaminants=TRUE
#'     "4\trun2\t\t70\t0.05\t\n",
#'     "5\trun2\t+\t60\t0.90\t\n"
#'   )
#'   writeLines(txt, tmp)
#'
#'   x <- read_dfdr_maxquant_msms(tmp, pep_mode = "sanitize",
#'         exclude_contaminants = TRUE, add_decoy = 1L)
#'   x
#'   head(x$q)
#' }
#'
#' @export
read_dfdr_maxquant_msms <- function(path,
                                    pep_mode = c("drop", "sanitize", "strict"),
                                    exclude_contaminants = TRUE,
                                    add_decoy = 1L,
                                    unit = "psm",
                                    scope = "global",
                                    provenance = list()) {
  pep_mode <- match.arg(pep_mode)

  if (!is.character(path) || length(path) != 1) {
    rlang::abort("`path` must be a character scalar.")
  }
  if (!file.exists(path)) {
    rlang::abort(paste0("File not found: ", path))
  }

  x <- readr::read_tsv(
    file = path,
    show_col_types = FALSE,
    progress = FALSE,
    na = c("", "NA")
  )

  req <- c("id", "Raw file", "Reverse", "Score", "PEP")
  miss <- setdiff(req, names(x))
  if (length(miss)) {
    rlang::abort(paste0("msms.txt missing required columns: ", paste(miss, collapse = ", ")))
  }

  # Optional contaminant filtering
  if (isTRUE(exclude_contaminants) && ("Potential contaminant" %in% names(x))) {
    x <- dplyr::filter(x, is.na(.data[["Potential contaminant"]]) | .data[["Potential contaminant"]] != "+")
  }

  # Reverse -> is_decoy: "+" => TRUE; NA/blank => FALSE; anything else => error
  reverse_chr <- as.character(x[["Reverse"]])
  bad <- !is.na(reverse_chr) & nzchar(reverse_chr) & reverse_chr != "+"
  if (any(bad)) {
    vals <- sort(unique(reverse_chr[bad]))
    rlang::abort(paste0(
      "Unexpected values in `Reverse` column (expected '+' for decoy, blank/NA for target): ",
      paste(vals, collapse = ", ")
    ))
  }
  is_decoy <- ifelse(!is.na(reverse_chr) & reverse_chr == "+", TRUE, FALSE)

  # Score must be numeric + finite
  score_chr <- as.character(x[["Score"]])
  score <- suppressWarnings(as.numeric(score_chr))
  if (any(!is.finite(score))) {
    bad_sc <- which(!is.finite(score))
    ex <- unique(score_chr[bad_sc])
    ex <- ex[seq_len(min(8, length(ex)))]
    rlang::abort(paste0(
      "`Score` contains non-finite or non-numeric values; cannot compute q-values. Examples: ",
      paste(ex, collapse = ", ")
    ))
  }

  # PEP handling
  pep_chr <- as.character(x[["PEP"]])
  pep <- suppressWarnings(as.numeric(pep_chr))
  pep[!is.finite(pep)] <- NA_real_

  if (pep_mode == "drop") {
    pep <- rep(NA_real_, length(score))
  } else {
    bad_pep <- which(is.finite(pep) & (pep < 0 | pep > 1))
    if (length(bad_pep)) {
      if (pep_mode == "strict") {
        ex <- unique(pep_chr[bad_pep])
        ex <- ex[seq_len(min(8, length(ex)))]
        rlang::abort(paste0(
          "`PEP` contains finite values outside [0,1]. Examples: ",
          paste(ex, collapse = ", ")
        ))
      } else if (pep_mode == "sanitize") {
        n_bad <- length(bad_pep)
        pep[bad_pep] <- NA_real_
        rlang::warn(paste0(
          "PEP values outside [0,1] were set to NA (n=", n_bad, "). ",
          "Proceeding with q-values computed from Score + Reverse."
        ))
      }
    }
  }

  out <- tibble::tibble(
    id = as.character(x[["id"]]),
    run = as.character(x[["Raw file"]]),
    is_decoy = as.logical(is_decoy),
    pep = pep,
    score = score
  )

  if (anyNA(out$id) || any(out$id == "")) rlang::abort("Some `id` values are missing/empty.")
  if (anyNA(out$run) || any(out$run == "")) rlang::abort("Some `Raw file` values are missing/empty.")
  if (anyNA(out$is_decoy)) rlang::abort("`is_decoy` contains NA after parsing Reverse column.")
  if (!any(out$is_decoy)) rlang::abort("No decoys detected (Reverse never '+'); cannot compute q-values.")
  if (!any(!out$is_decoy)) rlang::abort("No targets detected (all Reverse '+'); cannot compute q-values.")

  out$q <- tdc_qvalues(score = out$score, is_decoy = out$is_decoy, add_decoy = add_decoy)

  q_source <- paste0("reconstructed_TDC_from_MaxQuant_Score(add_decoy=", add_decoy, ")")

  as_dfdr_tbl(
    out,
    unit = unit,
    scope = scope,
    q_source = q_source,
    q_max_export = NA_real_,
    provenance = provenance
  )
}

---
title: "diagFDR: MaxQuant diagnostics from msms.txt"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{diagFDR: MaxQuant diagnostics from msms.txt}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5
)
```

This vignette demonstrates how to compute **diagFDR** diagnostics from a **MaxQuant**
`msms.txt` file.

MaxQuant reports a PSM-level score (Andromeda `Score`) and a decoy indicator (`Reverse`).
`diagFDR` can reconstruct **target–decoy counting (TDC)** q-values from the score and then
compute stability and calibration diagnostics on the **competed PSM universe**.

The typical workflow is:

1. Export `msms.txt` from MaxQuant.
2. Read `msms.txt` into a `dfdr_tbl` (PSM-level) using `read_dfdr_maxquant_msms()`.
3. Run `dfdr_run_all()` and inspect headline/stability/calibration diagnostics.
4. Optionally export a report folder (CSV+PNG+manifest+README+summary) and/or render an HTML report.

## Runnable toy example (no external files required)

We start with a simulated dataset that is similar to the `dfdr_tbl` returned by
`read_dfdr_maxquant_msms()`. Any software producing outputs that can be mapped to
columns `id`, `is_decoy`, `q`, `pep`, `run`, and `score` can similarly be used.

```{r toy}
library(diagFDR)

set.seed(10)

n <- 8000
toy <- data.frame(
  id = as.character(seq_len(n)),
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.98, 0.02)),
  run = sample(paste0("run", 1:3), n, replace = TRUE),
  score = c(rnorm(n * 0.98, mean = 7, sd = 1), rnorm(n * 0.02, mean = 5, sd = 1))[seq_len(n)],
  pep = NA_real_
)

# Reconstruct q-values by simple TDC from score (for toy data only):
# Here we mimic a typical "higher score is better" setting.
toy <- toy[order(toy$score, decreasing = TRUE), ]
toy$D_cum <- cumsum(toy$is_decoy)
toy$T_cum <- cumsum(!toy$is_decoy)
toy$FDR_hat <- (toy$D_cum + 1) / pmax(toy$T_cum, 1)
toy$q <- rev(cummin(rev(toy$FDR_hat)))
toy <- toy[, c("id","is_decoy","q","pep","run","score")]

x_toy <- as_dfdr_tbl(
  toy,
  unit = "psm",
  scope = "global",
  q_source = "toy TDC from score",
  q_max_export = 1,
  provenance = list(tool = "toy")
)

diag <- dfdr_run_all(xs = list(MaxQuant_PSM = x_toy), alpha_main = 0.01)
```

### Headline stability at 1%

```{r headline}
diag$tables$headline
```

### Decoy tail support and stability proxy

```{r stability-plots}
diag$plots$dalpha
diag$plots$cv
```

### Local boundary support and elasticity

```{r boundary-stability}
diag$plots$dwin
diag$plots$elasticity
```

### Equal-chance plausibility by q-band

```{r equal-chance}
diag$tables$equal_chance_pooled
diag$plots$equal_chance__MaxQuant_PSM
```

## Real MaxQuant workflow (msms.txt)

The code below shows how to run **diagFDR** directly from a MaxQuant `msms.txt` file.

```{r real-maxquant, eval=FALSE}
library(diagFDR)

mq_path <- "path/to/msms.txt"

# Read msms.txt and reconstruct TDC q-values using MaxQuant Score and Reverse indicator.
# - Reverse == "+" is treated as a decoy indicator.
# - Score is assumed "higher is better".
# - q-values are computed using FDR(i) = (D(i)+add_decoy)/T(i) and q(i)=min_{j>=i} FDR(j).
x_mq <- read_dfdr_maxquant_msms(
  path = mq_path,
  pep_mode = "sanitize",          # or "drop" if PEP contains values >1
  exclude_contaminants = TRUE,
  add_decoy = 1L,
  unit = "psm",
  scope = "global",
  provenance = list(tool = "MaxQuant", file = basename(mq_path))
)

# Run diagnostics
diag <- dfdr_run_all(
  xs = list(MaxQuant_PSM = x_mq),
  alpha_main = 0.01,
  alphas = c(1e-4, 5e-4, 1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
  eps = 0.2,
  win_rel = 0.2,
  truncation = "warn_drop",
  low_conf = c(0.2, 0.5)
)

# Inspect headline diagnostics
diag$tables$headline

# Export tables/plots + human-readable summary
dfdr_write_report(
  diag,
  out_dir = "diagFDR_maxquant_out",
  formats = c("csv", "png", "manifest", "readme", "summary")
)

# Optional: render a single HTML report (requires rmarkdown in Suggests)
dfdr_render_report(diag, out_dir = "diagFDR_maxquant_out")


library(diagFDR)

mq_path <- "path/to/msms.txt"

# Read msms.txt and reconstruct TDC q-values
x_mq <- read_dfdr_maxquant_msms(
  path = mq_path,
  pep_mode = "sanitize",
  exclude_contaminants = TRUE,
  add_decoy = 1L,
  unit = "psm",
  scope = "global",
  provenance = list(tool = "MaxQuant", file = basename(mq_path))
)

# Run diagnostics with automatic pseudo p-value computation
# This will use score_to_pvalue(method="decoy_ecdf") internally
diag <- dfdr_run_all(
  xs = list(MaxQuant_PSM = x_mq),
  alpha_main = 0.01,
  compute_pseudo_pvalues = TRUE,   # generates pseudo p-values from scores
  pseudo_pvalue_method = "decoy_ecdf",  # Most defensible method for arbitrary scores
  p_stratify = "run"  # Optional: stratify p-value diagnostics by run
)

# diag will contain:
# - All standard target-decoy diagnostics
# - PLUS p-value calibration plots and tables
# - PLUS Storey pi0 diagnostics
# - PLUS BH comparison diagnostics

# Inspect headline + p-value calibration
diag$tables$headline
diag$tables$p_calibration_summary

# Export everything
dfdr_write_report(
  diag,
  out_dir = "diagFDR_maxquant_out",
  formats = c("csv", "png", "manifest", "readme", "summary")
)

# Render HTML report (will include p-value diagnostics section)
dfdr_render_report(diag, out_dir = "diagFDR_maxquant_out")
```
##Interpretation of pseudo p-value diagnostics

When compute_pseudo_pvalues = TRUE with method = "decoy_ecdf":

    Pseudo p-values are computed as right-tail probabilities under the decoy score distribution
    π₀ estimate: Estimates the proportion of true nulls; should be stable across λ
    BH comparison: Compares BH-FDR to TDA-FDR; agreement supports consistent FDR control

Important: These are diagnostic pseudo p-values, not guaranteed null p-values. They provide:

    ✓ Additional calibration perspectives
    ✓ Cross-validation between BH and TDA procedures
    ✓ Stratified stability checks (e.g., by run)

Caveat: Pseudo p-value calibration cannot detect scoring pathologies that affect both targets and decoys equally.


## Optional: other way to get p-value / pseudo-p-value diagnostics from MaxQuant scores

MaxQuant `Score` is not a p-value. However, you can construct **pseudo-p-values**
to run p-value-based calibration and BH-stability diagnostics. The most defensible option
for arbitrary scores is to use the empirical decoy null tail (`method = "decoy_ecdf"`),
which maps scores to right-tail probabilities under the decoy distribution.

```{r pseudo-p, eval=FALSE}
# Create pseudo-p-values from the decoy score tail and rerun diagnostics with p-value checks.
x_mq$p <- score_to_pvalue(
  score = x_mq$score,
  method = "decoy_ecdf",
  is_decoy = x_mq$is_decoy
)
attr(x_mq, "meta")$p_source <- "score_to_pvalue(method='decoy_ecdf' on MaxQuant Score)"

diag_p <- dfdr_run_all(
  xs = list(MaxQuant_PSM = x_mq),
  alpha_main = 0.01,
  p_stratify = "run"   # optional stratification if run column is meaningful
)

dfdr_write_report(diag_p, out_dir = "diagFDR_maxquant_out_with_p", formats = c("csv","png","manifest","readme","summary"))
dfdr_render_report(diag_p, out_dir = "diagFDR_maxquant_out_with_p")
```

## Interpretation notes

- **Universe / scope**: `read_dfdr_maxquant_msms()` produces a PSM-level universe; q-values
  are reconstructed by score-based TDC. Protein-level claims require additional inference.

- **Stability**: `D_alpha`, `CV_hat`, and `D_alpha_win` quantify whether the operating cutoff
  is well supported by decoys. Very strict cutoffs can yield sparse decoy tails.

- **Calibration**: equal-chance diagnostics are internal plausibility checks under the target–decoy model.
  Agreement with expectations supports (but does not prove) correct FDR control.

- **Pseudo-p-values** derived from the decoy tail can be used for additional calibration/stability diagnostics,
  but should be interpreted as diagnostic tools unless null calibration is theoretically guaranteed.

---
title: "diagFDR: Verifiable FDR diagnostics report"
output:
  html_document:
    toc: true
    toc_depth: 3
    theme: readable
    css: custom.css
params:
  diag: NULL
  out_dir: NULL
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, message = FALSE, warning = FALSE)

diag <- params$diag
out_dir <- params$out_dir

`%||%` <- function(a, b) if (!is.null(a)) a else b

md_kable <- function(x, digits = 4, escape = FALSE) {
  if (!is.data.frame(x) || nrow(x) == 0) return(knitr::kable(data.frame(note = "(empty)")))
  knitr::kable(x, digits = digits, escape = escape)
}

# -------------------------
# Icon rendering (HTML only)
# -------------------------
icon_ok      <- "&#x2713;"          # ✓
icon_caution <- "&#x26A0;"          # ⚠
icon_problem <- "&#x26A0;&#xFE0F;"  # ⚠️

flag_to_html <- function(s) {
  if (is.null(s) || length(s) == 0 || all(is.na(s))) return("n/a")
  s <- as.character(s)

  if (s %in% c("NA", "n/a")) return("n/a")
  if (identical(s, "OK")) return(icon_ok)
  if (grepl("^PROBLEM", s)) return(paste0(icon_problem, " ", sub("^PROBLEM_?", "", s)))
  if (grepl("^CAUTION", s)) return(paste0(icon_caution, " ", sub("^CAUTION_?", "", s)))
  s
}

status_to_html <- function(s) {
  s <- as.character(s)
  if (is.na(s) || s == "" || s %in% c("NA", "n/a")) return("n/a")
  if (s == "VALID") return(paste0(icon_ok, " Valid"))
  if (s == "CAUTION") return(paste0(icon_caution, " Caution"))
  if (s == "REVIEW_NEEDED") return(paste0(icon_problem, " Review needed"))
  s
}

headline_display <- function(tbl) {
  if (!is.data.frame(tbl) || nrow(tbl) == 0) return(tbl)
  out <- tbl

  flag_cols <- names(out)[grepl("^flag_", names(out))]
  for (cc in flag_cols) {
    out[[cc]] <- vapply(out[[cc]], flag_to_html, character(1))
  }
  if ("status" %in% names(out)) {
    out[["status"]] <- vapply(out[["status"]], status_to_html, character(1))
  }
  out
}

# -------------------------
# Pull tables/plots/metadata
# -------------------------
# Prefer package helper if present
if (exists("dfdr_summary_headline", mode = "function")) {
  summary_tbl <- dfdr_summary_headline(diag)
} else {
  summary_tbl <- diag$tables$headline
}

summary_tbl_disp <- headline_display(summary_tbl)

alpha_main <- (diag$params %||% list())$alpha_main %||% NA_real_
pnames <- names(diag$plots %||% list())

# Extract metadata
meta_tbl <- purrr::imap_dfr(diag$objects %||% list(), function(x, nm) {
  meta <- attr(x, "meta") %||% list()
  tibble::tibble(
    list = nm,
    unit = meta$unit %||% NA_character_,
    scope = meta$scope %||% NA_character_,
    q_source = meta$q_source %||% NA_character_,
    q_max_export = meta$q_max_export %||% NA_real_,
    p_source = meta$p_source %||% NA_character_
  )
})

# helper: choose box class from ASCII flag code
flag_class <- function(flag_value) {
  flag_value <- as.character(flag_value)
  if (is.na(flag_value) || flag_value %in% c("NA", "n/a")) return("metric-caution")
  if (flag_value == "OK") return("metric-ok")
  if (grepl("^PROBLEM", flag_value)) return("metric-problem")
  if (grepl("^CAUTION", flag_value)) return("metric-caution")
  "metric-caution"
}
```

```{css, echo=FALSE}
.alert-success {
  background-color: #d4edda;
  border-left: 4px solid #28a745;
  color: #000000;
  padding: 15px;
  margin: 20px 0;
  border-radius: 4px;
}
.alert-warning {
  background-color: #fff3cd;
  border-left: 4px solid #ffc107;
  color: #000000;
  padding: 15px;
  margin: 20px 0;
  border-radius: 4px;
}
.alert-danger {
  background-color: #f8d7da;
  border-left: 4px solid #dc3545;
  color: #000000;
  padding: 15px;
  margin: 20px 0;
  border-radius: 4px;
}
.alert-info {
  background-color: #d1ecf1;
  border-left: 4px solid #0c5460;
  color: #000000;
  padding: 15px;
  margin: 20px 0;
  border-radius: 4px;
}
.metric-box {
  display: inline-block;
  padding: 8px 12px;
  margin: 5px;
  border-radius: 4px;
  font-weight: bold;
}
.metric-ok { background-color: #d4edda; color: #155724; }
.metric-caution { background-color: #fff3cd; color: #856404; }
.metric-problem { background-color: #f8d7da; color: #721c24; }
details {
  margin: 20px 0;
  padding: 10px;
  border: 1px solid #ddd;
  border-radius: 4px;
}
summary {
  cursor: pointer;
  font-weight: bold;
  padding: 10px;
  background-color: #f8f9fa;
  border-radius: 4px;
}
summary:hover {
  background-color: #e9ecef;
}
```

## Overview

This report was generated from the output of the `dfdr_run_all()` function of the `diagFDR` R package.

- **diagFDR version:** `r as.character(utils::packageVersion("diagFDR"))`
- **Created:** `r as.character(Sys.time())`
- **Headline threshold (alpha_main):** `r alpha_main`
- **Number of lists analyzed:** `r length(diag$objects)`

---

## Executive Summary

```{r executive, results='asis'}
has_flags <- all(c("status", "interpretation") %in% names(summary_tbl))

if (!has_flags) {
  cat('<div class="alert-info">\n')
  cat('<p><em>Automated flagging not available in this version. Please review metrics manually.</em></p>\n')
  cat('</div>\n\n')
} else {

  problems <- summary_tbl |>
    dplyr::filter(status == "REVIEW_NEEDED") |>
    dplyr::pull(list)

  cautions <- summary_tbl |>
    dplyr::filter(status == "CAUTION") |>
    dplyr::pull(list)

  valid <- summary_tbl |>
    dplyr::filter(status == "VALID") |>
    dplyr::pull(list)

  if (length(problems) > 0) {
    cat('<div class="alert-danger">\n')
    cat('<h3>CRITICAL ISSUES DETECTED</h3>\n')
    for (lst in problems) {
      row <- summary_tbl |> dplyr::filter(list == lst)
      row_disp <- summary_tbl_disp |> dplyr::filter(list == lst)

      cat(sprintf('<p><strong>List "%s":</strong> %s</p>\n', lst, row$interpretation))
      cat('<ul>\n')

      if ("flag_FDR" %in% names(row) && !is.na(row$flag_FDR) && row$flag_FDR != "OK") {
        cat(sprintf('<li>FDR: %s (empirical FDR = %.2f%%)</li>\n', row_disp$flag_FDR, row$FDR_hat * 100))
      }
      if ("flag_IPE" %in% names(row) && !is.na(row$flag_IPE) && row$flag_IPE != "OK" && row$flag_IPE != "NA") {
        if ("IPE" %in% names(row) && is.finite(row$IPE)) {
          cat(sprintf('<li>PEP calibration: %s (IPE = %.3f)</li>\n', row_disp$flag_IPE, row$IPE))
        } else {
          cat(sprintf('<li>PEP calibration: %s</li>\n', row_disp$flag_IPE))
        }
      }
      if ("flag_equalchance" %in% names(row) && !is.na(row$flag_equalchance) && row$flag_equalchance != "OK" && row$flag_equalchance != "NA") {
        if ("effect_abs" %in% names(row) && is.finite(row$effect_abs)) {
          cat(sprintf('<li>Equal-chance: %s (|effect| = %.3f)</li>\n', row_disp$flag_equalchance, row$effect_abs))
        } else {
          cat(sprintf('<li>Equal-chance: %s</li>\n', row_disp$flag_equalchance))
        }
      }
      if ("flag_Dalpha" %in% names(row) && !is.na(row$flag_Dalpha) && row$flag_Dalpha != "OK" && row$flag_Dalpha != "NA") {
        cat(sprintf('<li>Decoy support: %s (D_alpha = %d)</li>\n', row_disp$flag_Dalpha, row$D_alpha))
      }
      if ("flag_CV" %in% names(row) && !is.na(row$flag_CV) && row$flag_CV != "OK" && row$flag_CV != "NA") {
        cat(sprintf('<li>Stability: %s (CV = %.3f)</li>\n', row_disp$flag_CV, row$CV_hat))
      }
      cat('</ul>\n')
    }
    cat('</div>\n\n')
  }

  if (length(valid) > 0) {
    cat('<div class="alert-success">\n')
    cat('<h3>VALID LISTS</h3>\n')
    cat('<ul>\n')
    for (lst in valid) {
      row <- summary_tbl |> dplyr::filter(list == lst)
      meta <- meta_tbl |> dplyr::filter(list == lst)
      cat(sprintf(
        '<li><strong>%s</strong> (%s scope, %s unit): FDR=%.2f%%, D_alpha=%d, CV=%.3f</li>\n',
        lst, meta$scope, meta$unit, row$FDR_hat * 100, row$D_alpha, row$CV_hat
      ))
    }
    cat('</ul>\n')
    cat('</div>\n\n')
  }

  if (length(cautions) > 0) {
    cat('<div class="alert-warning">\n')
    cat('<h3>LISTS WITH WARNINGS</h3>\n')
    for (lst in cautions) {
      row <- summary_tbl |> dplyr::filter(list == lst)
      cat(sprintf('<p><strong>List "%s":</strong> %s</p>\n', lst, row$interpretation))
    }
    cat('</div>\n\n')
  }
}
```

### Quick Metrics Dashboard

```{r dashboard, results='asis'}
if (!has_flags) {
  cat('<p><em>Dashboard not available without flagged headline table.</em></p>\n')
} else {
  for (i in seq_len(nrow(summary_tbl))) {
    row <- summary_tbl[i, ]
    row_disp <- summary_tbl_disp[i, ]

    cat('<div style="margin: 15px 0; padding: 10px; border: 1px solid #ddd; border-radius: 5px;">\n')
    cat(sprintf('<h4>%s <span style="float:right;">%s</span></h4>\n', row$list, row_disp$status))

    cat('<div>\n')

    if ("flag_FDR" %in% names(row)) {
      fdr_class <- flag_class(row$flag_FDR)
      cat(sprintf('<span class="metric-box %s">FDR: %.2f%%</span>\n', fdr_class, row$FDR_hat * 100))
    }

    if ("flag_Dalpha" %in% names(row)) {
      d_class <- flag_class(row$flag_Dalpha)
      cat(sprintf('<span class="metric-box %s">D_alpha: %d</span>\n', d_class, row$D_alpha))
    }

    if ("flag_CV" %in% names(row)) {
      cv_class <- flag_class(row$flag_CV)
      cat(sprintf('<span class="metric-box %s">CV: %.3f</span>\n', cv_class, row$CV_hat))
    }

    if ("flag_IPE" %in% names(row) && "IPE" %in% names(row) && is.finite(row$IPE)) {
      ipe_class <- flag_class(row$flag_IPE)
      cat(sprintf('<span class="metric-box %s">IPE: %.3f</span>\n', ipe_class, row$IPE))
    }

    cat('</div>\n')
    cat('</div>\n\n')
  }
}
```

---

## Headline Summary (Table)

```{r headline-simple}
key_cols <- c("list", "T_alpha", "D_alpha", "FDR_hat", "CV_hat")

if ("IPE" %in% names(summary_tbl)) key_cols <- c(key_cols, "IPE")

if (has_flags) {
  flag_cols <- names(summary_tbl)[grepl("^flag_", names(summary_tbl))]
  key_cols <- c(key_cols, flag_cols, "status")
}

display_cols <- key_cols[key_cols %in% names(summary_tbl_disp)]

summary_tbl_disp |>
  dplyr::select(dplyr::all_of(display_cols)) |>
  md_kable()
```

<details>
<summary>Click to show full headline table (all columns)</summary>

```{r headline-full}
md_kable(summary_tbl_disp, digits = 3)
```

</details>

---

## List Metadata (scope / unit / provenance)

```{r metadata}
md_kable(meta_tbl)
```

---

## Score Distributions (Target vs Decoy)

<div class="alert-info">
**Interpretation guide:**

- **Score distributions**: Good separation means target distribution shifted right, minimal overlap with decoys
- **q-value distributions (fallback if no scores)**: Target q-values should be concentrated near 0; decoys more uniform. Large overlap suggests poor discrimination.
- **Very few decoys at high scores / low q-values**: Leads to sparse tail support (check D_alpha)

</div>

```{r score-distributions, results='asis'}
score_plots <- pnames[grepl("^score_dist__", pnames)]

if (length(score_plots) == 0) {
  cat("\n(No score distribution plots found; score column may not be available.)\n\n")
} else {
  for (nm in score_plots) {
    list_name <- sub("^score_dist__", "", nm)
    cat(sprintf('\n### %s\n\n', list_name))
    print(diag$plots[[nm]])
    cat('\n\n')
  }
}
```

---

## Stability Diagnostics

<div class="alert-info">
**What to look for:**

- **D_alpha curve**: Should remain high at operating thresholds (e.g., >50). U-shape indicates granularity at extreme cutoffs.
- **CV curve**: Should be low (e.g., <0.15) at headline threshold. High CV = unstable estimate.
- **D_alpha_win**: Local decoy support near cutoff. If it is low (e.g., <20) indicates sparse boundary support.
- **Elasticity**: High Jaccard index (>0.9) indicates stability.
</div>

### Decoy tail support and instability proxy

```{r stability-plots}
if (!is.null(diag$plots$dalpha)) print(diag$plots$dalpha)
if (!is.null(diag$plots$cv)) print(diag$plots$cv)
```

### Local boundary support (decoys near the cutoff)

```{r dwin-plot}
if (!is.null(diag$plots$dwin)) print(diag$plots$dwin)

lt <- diag$tables$local_tail
if (is.data.frame(lt) && nrow(lt) > 0) {
  cat("\n#### local_tail table (includes `truncated` flags)\n\n")
  md_kable(lt)
}
```

### Threshold elasticity (list sensitivity to cutoff perturbation)

```{r elasticity-plot}
if (!is.null(diag$plots$elasticity)) print(diag$plots$elasticity)
```

---

## Calibration Diagnostics (target-decoy)

<div class="alert-info">
**What to look for:**

- **Equal-chance**: Decoy fraction in each q-band should be ~50%. Large deviations suggest scoring bias.
- **PEP reliability**: Observed error rate should match predicted PEP. IPE > 0.05 indicates miscalibration.
</div>

### Equal-chance plausibility (q-band decoy fractions)

```{r equal-chance-table}
ecp <- diag$tables$equal_chance_pooled
if (is.data.frame(ecp) && nrow(ecp) > 0) md_kable(ecp)
```

```{r equal-chance-plots, results='asis'}
eq_plots <- pnames[grepl("^equal_chance__", pnames)]
if (length(eq_plots) == 0) {
  cat("\n(No equal-chance plots found.)\n")
} else {
  for (nm in eq_plots) {
    list_name <- sub("^equal_chance__", "", nm)
    cat("\n#### ", list_name, "\n\n", sep = "")
    print(diag$plots[[nm]])
    cat("\n\n")
  }
}
```

### PEP reliability (bins + plots)

```{r pep-ipe-table}
ipe <- diag$tables$pep_IPE
if (is.data.frame(ipe) && nrow(ipe) > 0) md_kable(ipe)
```

```{r pep-plots, results='asis'}
pep_plots <- pnames[grepl("^pep_reliability__", pnames)]
if (length(pep_plots) == 0) {
  cat("\n(No PEP reliability plots found; PEP may be unavailable.)\n")
} else {
  for (nm in pep_plots) {
    list_name <- sub("^pep_reliability__", "", nm)
    cat("\n#### ", list_name, "\n\n", sep = "")
    print(diag$plots[[nm]])
    cat("\n\n")
  }
}
```

### PEP sanity: decoy PEP rates + target/decoy PEP densities

```{r pep-sanity-table, results='asis'}
ps <- diag$tables$pep_decoy_sanity
if (is.data.frame(ps) && nrow(ps) > 0) {
  md_kable(ps, digits = 4)
} else {
  cat("\n(No PEP sanity table found.)\n")
}

pep_den_plots <- pnames[grepl("^pep_density__", pnames)]
if (length(pep_den_plots) == 0) {
  cat("\n(No PEP density sanity plots found.)\n")
} else {
  for (nm in pep_den_plots) {
    list_name <- sub("^pep_density__", "", nm)
    cat("\n#### ", list_name, "\n\n", sep = "")
    print(diag$plots[[nm]])
    cat("\n\n")
  }
}
```

### PEP reliability (TDC-style; targets only)

```{r pep-tdc-style, results='asis'}
pt <- diag$tables$pep_reliability_tdc
if (is.data.frame(pt) && nrow(pt) > 0) {
  md_kable(pt, digits = 4)
} else {
  cat("\n(No TDC-style PEP reliability table found.)\n")
}

pep_tdc_plots <- pnames[grepl("^pep_reliability_tdc__", pnames)]
if (length(pep_tdc_plots) == 0) {
  cat("\n(No TDC-style PEP reliability plots found.)\n")
} else {
  for (nm in pep_tdc_plots) {
    list_name <- sub("^pep_reliability_tdc__", "", nm)
    cat("\n#### ", list_name, "\n\n", sep = "")
    print(diag$plots[[nm]])
    cat("\n\n")
  }
}
```

---

## Calibration Diagnostics (p-values)

<div class="alert-info">
This section is shown only when a \texttt{p} column is available (true p-values or pseudo-p-values derived from scores).

**What to look for:**

- **p-value density**: Density of p-values associated to decoy should be uniform between 0 and 1. The density associated to targets should be peaky near 0 and decrease between 0 and 1.
- **Calibration plots**: Straight lines have slopes equal to different estimators of "pi0", i.e. the proportion of true null hypotheses (proportion of false identifications). 
For decoys, the slope should be close to 1 (100% of false identifications). For targets, the closer the estimators are to 0, the lower the proportion of false identifications among the targets.
- **Estimated FDR**: This is a proxy of the FDR based on an estimator of "pi0". It should be 100% for decoys. It should increase with the p-value threshold for targets until the value of the "pi0" estimator when the threshold is 1. 
- **pi0 Storey**: Storey's estimator of the proportion of true null hypotheses (incorrect identifications). Should be stable across lambda (flat line).
- **BH elasticity**: High Jaccard (>0.95) indicates stable BH lists. Low Jaccard at operating thresholds suggests sensitivity to cutoff choice.

</div>

### (Pseudo-)p-value density (targets vs decoys)

```{r p-density-plots, results='asis'}
p_den_plots <- pnames[grepl("^p_density__", pnames)]
if (length(p_den_plots) == 0) {
  cat("\n(No p-density plots found; no `p` column was provided.)\n")
} else {
  for (nm in p_den_plots) {
    list_name <- sub("^p_density__", "", nm)
    cat("\n#### ", list_name, "\n\n", sep = "")
    print(diag$plots[[nm]])
    cat("\n\n")
  }
}
```

```{r p-diagnostics, results='asis'}
p_plots <- pnames[grepl("^Calibration_pvalues__|^Calibration_pvalues_decoys__|^Calibration_pvalues_targets__|^Estimated_fdr__|^Estimated_fdr_decoys__|^Estimated_fdr_targets__|^pi0_Storey__|^BH_elasticity__", pnames)]
if (length(p_plots) == 0) {
  cat("\n(No p-value diagnostics were found; no `p` column was provided.)\n")
} else {
  for (nm in p_plots) {
    cat("\n### ", nm, "\n\n", sep = "")
    print(diag$plots[[nm]])
    cat("\n\n")
  }
}
```

---

## Scope Comparison (if multiple lists)

```{r scope-comparison, results='asis'}
if (length(diag$objects) > 1) {
  cat('<div class="alert-info">\n')
  cat('<strong>Interpretation:</strong>\n')
  cat('<ul>\n')
  cat('<li>J > 0.9: Scopes largely agree</li>\n')
  cat('<li>0.7 < J < 0.9: Moderate disagreement - scope choice matters</li>\n')
  cat('<li>J < 0.7: Strong disagreement - scope is critical</li>\n')
  cat('</ul>\n')
  cat('</div>\n\n')

  if (!is.null(diag$plots$scope_disagreement)) {
    print(diag$plots$scope_disagreement)
  } else {
    cat("\n(Scope disagreement matrix not computed.)\n\n")
  }
} else {
  cat("\n(Only one list provided; scope comparison not applicable.)\n\n")
}
```

---

## Exported Artifacts

- **Tables directory:** `r normalizePath(out_dir, winslash = "/", mustWork = FALSE)`
- **Plots directory:** `r normalizePath(file.path(out_dir, "plots"), winslash = "/", mustWork = FALSE)`

---

## Appendix: Diagnostic Thresholds

The automated flags use these heuristic thresholds:

| Metric | OK | CAUTION | PROBLEM |
|--------|----|---------|---------|
| **D_alpha** | >= 50 | 10--50 | < 10 |
| **CV** | < 0.15 | 0.15--0.30 | > 0.30 |
| **D_alpha_win** | >= 20 | 5--20 | < 5 |
| **IPE** | <= 0.05 | 0.05--0.10 | > 0.10 |
| **FDR_hat** | <= alpha*1.2 | alpha*1.2--alpha*1.5 | > alpha*1.5 |
| **Equal-chance** | |effect| <= 0.10 | 0.10--0.15 | > 0.15 |

These are guidelines, not absolute rules.

---

<div style="text-align: center; margin-top: 40px; padding: 20px; background-color: #f8f9fa; border-top: 2px solid #dee2e6;">
<strong>End of Report</strong><br/>
Generated by diagFDR v`r utils::packageVersion("diagFDR")`
</div>

---
title: "diagFDR: generic PSM diagnostics from mzIdentML (.mzid)"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{diagFDR: generic PSM diagnostics from mzIdentML (.mzid)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5
)
```

This vignette demonstrates how to compute **diagFDR** diagnostics from a generic
**mzIdentML** file (`.mzid`).

Many search engines can export identifications in mzIdentML format. When explicit q-values
and/or PEPs are not present in the mzIdentML, `diagFDR` can:

1. extract a **competed PSM universe** (rank-1 by default),
2. infer target/decoy labels,
3. select a primary numeric score CV term (configurable),
4. reconstruct **TDC q-values** from scores via target--decoy counting (TDC),
5. compute scope/calibration/stability diagnostics.

> Note: mzIdentML is flexible and tool-dependent. If the score CV term or decoy labeling
> is not encoded consistently, you may need to adjust `score_accession_preference`,
> `score_direction`, and/or `decoy_regex`.

## Runnable toy example (no external files required)

This section provides a small *toy* dataset that mimics the **PSM-level competed universe**
returned by `read_dfdr_mzid()` after choosing a score and reconstructing TDC q-values.

```{r toy}
library(diagFDR)

set.seed(3)

n <- 7000
pi_decoy <- 0.03

# Decoy indicator
is_decoy <- sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(1 - pi_decoy, pi_decoy))

# Targets are a mixture: some null-like (close to decoys), some true (higher score)
# This yields realistic separation and non-trivial discoveries at 1% FDR.
is_true_target <- (!is_decoy) & (runif(n) < 0.30)  # 30% of targets are "true"
is_null_target <- (!is_decoy) & (!is_true_target)

score <- numeric(n)
score[is_decoy]       <- rnorm(sum(is_decoy), mean = 0.0, sd = 1.0)
score[is_null_target] <- rnorm(sum(is_null_target), mean = 0.2, sd = 1.0)
score[is_true_target] <- rnorm(sum(is_true_target), mean = 3.0, sd = 1.0)

toy <- data.frame(
  id = paste0("psm", seq_len(n)),
  is_decoy = is_decoy,
  run = sample(paste0("run", 1:4), n, replace = TRUE),
  score = score,
  pep = NA_real_
)

# Score-based TDC q-values (higher score is better)
toy <- toy[order(toy$score, decreasing = TRUE), ]
toy$D_cum <- cumsum(toy$is_decoy)
toy$T_cum <- cumsum(!toy$is_decoy)
toy$FDR_hat <- (toy$D_cum + 1) / pmax(toy$T_cum, 1)
toy$q <- rev(cummin(rev(toy$FDR_hat)))
toy <- toy[, c("id","is_decoy","q","pep","run","score")]

x_toy <- as_dfdr_tbl(
  toy,
  unit = "psm",
  scope = "global",
  q_source = "toy TDC from score",
  q_max_export = 1,
  provenance = list(tool = "toy")
)

diag <- dfdr_run_all(
  xs = list(mzid_PSM = x_toy),
  alpha_main = 0.01,
  alphas = c(1e-4, 5e-4, 1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
  eps = 0.2,
  win_rel = 0.2,
  truncation = "warn_drop",
  low_conf = c(0.2, 0.5)
)
```

### Headline stability at 1%

```{r headline}
diag$tables$headline

if (nrow(diag$tables$headline) > 0 && diag$tables$headline$T_alpha[1] == 0) {
  cat("\nNote: No discoveries at alpha_main for this toy run. ",
      "For demonstration, consider using alpha_main = 0.02.\n", sep = "")
}
```

### Tail support and stability versus threshold

```{r stability-plots}
diag$plots$dalpha
diag$plots$cv
```

### Local boundary support and elasticity

```{r boundary-stability}
diag$plots$dwin
diag$plots$elasticity
```

### Equal-chance plausibility by q-band

```{r equal-chance}
diag$tables$equal_chance_pooled
diag$plots$equal_chance__mzid_PSM
```

## Real mzIdentML workflow (.mzid)

The code below shows how to run **diagFDR** directly from a `.mzid` file.

```{r real-mzid, eval=FALSE}
library(diagFDR)

mzid_path <- "path/to/search_results.mzid"

x_mzid <- read_dfdr_mzid(
  mzid_path = mzid_path,
  rank = 1L,  # competed universe: take rank-1 SpectrumIdentificationItem

  # Choose a score CV term (priority list) and interpret its direction
  score_accession_preference = c(
    "MS:1002257", # MS-GF:RawScore (higher is better)
    "MS:1001330", # Mascot:score (higher is better)
    "MS:1001328", # SEQUEST:xcorr (higher is better)
    "MS:1002052",
    "MS:1002049",
    "MS:1001331",
    "MS:1001171",
    "MS:1001950",
    "MS:1002466"
  ),
  score_direction = "auto",  # or "higher_better"/"lower_better" if auto fails

  # TDC correction: FDR_hat = (D + add_decoy)/T
  add_decoy = 1L,

  # Strict by default: require score for all PSMs (set <1 to allow missing)
  min_score_coverage = 1.0,

  # Fallback decoy inference if PeptideEvidence@isDecoy is not informative
  decoy_regex = "(^##|_REVERSED$|^REV_|^DECOY_)",

  unit = "psm",
  scope = "global",
  provenance = list(file = basename(mzid_path))
)

diag <- dfdr_run_all(
  xs = list(mzid_PSM = x_mzid),
  alpha_main = 0.01,
  alphas = c(1e-4, 5e-4, 1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
  eps = 0.2,
  win_rel = 0.2,
  truncation = "warn_drop",
  low_conf = c(0.2, 0.5)
)

# Export outputs
dfdr_write_report(
  diag,
  out_dir = "diagFDR_mzid_out",
  formats = c("csv", "png", "manifest", "readme", "summary")
)

# Render a single HTML report (requires rmarkdown in Suggests)
dfdr_render_report(diag, out_dir = "diagFDR_mzid_out")
```

## Optional: pseudo-p-values from the decoy score tail

If you want to run the p-value diagnostics (calibration plots, Storey pi0, BH stability),
you can map scores to **pseudo-p-values** using the empirical decoy null tail.

```{r pseudo-p, eval=FALSE}
x_mzid$p <- score_to_pvalue(
  score = x_mzid$score,
  method = "decoy_ecdf",
  is_decoy = x_mzid$is_decoy
)
attr(x_mzid, "meta")$p_source <- "score_to_pvalue(method='decoy_ecdf' on mzid score)"

diag_p <- dfdr_run_all(
  xs = list(mzid_PSM = x_mzid),
  alpha_main = 0.01
)

dfdr_write_report(
  diag_p,
  out_dir = "diagFDR_mzid_out_with_p",
  formats = c("csv","png","manifest","readme","summary")
)
dfdr_render_report(diag_p, out_dir = "diagFDR_mzid_out_with_p")
```

## Interpretation notes / common pitfalls

- **Score selection matters:** mzIdentML files can contain multiple score CV terms.
  If `read_dfdr_mzid()` picks an unexpected score, adjust `score_accession_preference`
  and/or set `score_direction` explicitly.

- **Decoy labeling matters:** ideally, mzIdentML includes `PeptideEvidence@isDecoy`.
  If not, `read_dfdr_mzid()` falls back to `decoy_regex` applied to protein accessions.
  If your pipeline uses a different decoy naming convention, update `decoy_regex`.

- **Missing scores:** if some PSMs are missing the chosen score CV term, you can relax
  `min_score_coverage` (e.g., 0.95) but then interpret results carefully, as the hypothesis
  universe changes.

- **Scope:** this vignette constructs a PSM-level universe. Peptide- or protein-level claims
  require additional aggregation/inference.

---
title: "diagFDR: DIA-NN diagnostics from report.parquet"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{diagFDR: DIA-NN diagnostics from report.parquet}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5
)
```

This vignette demonstrates how to run **diagFDR** on DIA-NN exports and interpret the
key diagnostics in terms of **scope**, **calibration**, and **stability**.

The typical workflow is:

1. Export DIA-NN results with decoys and a permissive q-value ceiling.
2. Read `report.parquet`.
3. Construct one or more *universes* (global precursor list, run×precursor, etc.).
4. Run diagnostics and inspect tables/plots.
5. (Optional) write tables/plots and a human-readable report to disk.

## Recommended DIA-NN export settings

To enable all diagnostics, export:

- decoys: `--report-decoys`
- a permissive export ceiling: `--qvalue 0.5` (or higher)

The q-value ceiling matters because some diagnostics operate in low-confidence regions
(e.g. equal-chance plausibility checks, or local-window support around cutoffs).

## Runnable toy example (no DIA-NN files required)

We start with a small simulated dataset that exercises the diagFDR functions.
Any workflow producing outputs that can be mapped to the columns
`id`, `is_decoy`, `q`, `pep`, `run`, and `score` can be handled similarly.

```{r toy-data}
library(diagFDR)

set.seed(1)

n <- 3000
toy_global <- data.frame(
  id = paste0("P", seq_len(n)),
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.97, 0.03)),
  q = pmin(1, runif(n)^3),       # skew toward small q-values
  pep = NA_real_,
  run = NA_character_,
  score = NA_real_
)

x_global <- as_dfdr_tbl(
  toy_global,
  unit = "precursor",
  scope = "global",
  q_source = "toy",
  q_max_export = 0.5
)

diag <- dfdr_run_all(
  xs = list(global = x_global),
  alpha_main = 0.01,
  alphas = c(1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
  low_conf = c(0.2, 0.5)
)
```

### Headline stability at 1%

```{r headline}
diag$tables$headline
```

### Tail support and stability versus threshold

```{r plots-stability}
diag$plots$dalpha
diag$plots$cv
```

### Local boundary support

```{r plot-dwin}
diag$plots$dwin
```

### Threshold elasticity (list sensitivity to changing alpha)

```{r plot-elasticity}
diag$plots$elasticity
```

### Equal-chance plausibility by q-band

```{r equal-chance}
diag$tables$equal_chance_pooled
diag$plots$equal_chance__global
```

## Real DIA-NN parquet workflow

The following code shows how to run the pipeline on a real DIA-NN `report.parquet`.

```{r real-diann, eval=FALSE}
# Requires arrow 
rep <- read_diann_parquet("path/to/report.parquet")

# (A) Global precursor list using Global.Q.Value
# Recommended for experiment-wide (pooled) lists.
x_global_gq <- diann_global_precursor(
  rep,
  q_col = "Global.Q.Value",
  q_max_export = 0.5,
  unit = "precursor",
  scope = "global",
  q_source = "Global.Q.Value"
)

# (B) Run×precursor universe using run-wise Q.Value
# Recommended for per-run decisions / QC.
x_runx <- diann_runxprecursor(
  rep,
  q_col = "Q.Value",
  q_max_export = 0.5,
  id_mode = "runxid",
  unit = "runxprecursor",
  scope = "runwise",
  q_source = "Q.Value"
)

# (C) Scope misuse comparator: min run-wise q over runs per precursor (anti-pattern)
# Useful for demonstrating/diagnosing scope mismatch.
x_minrun <- diann_global_minrunq(
  rep,
  q_col = "Q.Value",
  q_max_export = 0.5,
  unit = "precursor",
  scope = "aggregated",
  q_source = "min_run(Q.Value)"
)

diag <- dfdr_run_all(
  xs = list(global = x_global_gq, runx = x_runx, minrun = x_minrun),
  alpha_main = 0.01,
  compute_pseudo_pvalues = TRUE  # <-- This adds p-value diagnostics
)

# Compare accepted lists across scopes (Jaccard overlap across alpha)
scope_tbl <- dfdr_scope_disagreement(
  x1 = x_global_gq,
  x2 = x_minrun,
  alphas = c(1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2),
  label1 = "Global.Q.Value",
  label2 = "min_run(Q.Value)"
)

# Write outputs to disk (tables + plots; optionally PPTX)
dfdr_write_report(diag, out_dir = "diagFDR_diann_out", formats = c("csv", "png", "manifest", "readme", "summary"))

# Render a single HTML report (requires rmarkdown in Suggests)
dfdr_render_report(diag, out_dir = "diagFDR_diann_out")
```

## Interpretation notes

- **Scope**: run-wise q-values (`Q.Value`) and global q-values (`Global.Q.Value`)
  do not control the same multiple-testing universe. Constructing experiment-wide lists by
  aggregating run-wise q-values (e.g., taking `min(Q.Value)` across runs) is generally anti-conservative.

- **Stability**: stringent cutoffs can enter a *granular regime* where only a few decoys
  support the boundary. Inspect `D_alpha`, `CV_hat`, and the local boundary support `D_alpha_win`
  before making strong claims at very small alpha.

- **Equal-chance diagnostics** (decoy fractions in low-confidence q-bands) and **PEP reliability**
  are internal consistency checks under target--decoy assumptions; they do not replace external
  validation (e.g. entrapment) when decoy representativeness is uncertain.

---
title: "diagFDR: mokapot diagnostics (competed winners)"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{diagFDR: mokapot diagnostics (competed winners)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5
)
```

This vignette demonstrates how to compute **diagFDR** diagnostics from **mokapot**
PSM exports.

A key choice is to compute diagnostics on **competed winners**: one PSM per
`run + spectrum_id`, selecting the maximum mokapot score among candidates
(targets and decoys). This corresponds to the standard *target–decoy competition*
setting for PSM-level inference.

The workflow is:

1. Read mokapot target and decoy PSM outputs.
2. Construct a competed-winner universe.
3. Run `dfdr_run_all()` and inspect tables/plots.
4. Optionally export results and a human-readable report.

## Runnable toy example (no external files required)

We start with a small simulated dataset that is similar to a competed-winner mokapot output.
Any workflow producing a table that can be mapped to columns
`id`, `is_decoy`, `q`, `pep`, `run`, and `score` can similarly be used.

```{r toy}
library(diagFDR)

set.seed(2)

n <- 6000
toy <- data.frame(
  id = paste0("run1||scan", seq_len(n)),
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.98, 0.02)),
  q = pmin(1, runif(n)^3),                                  # many small q-values
  pep = pmin(1, pmax(0, runif(n)^3 + rnorm(n, sd = 0.02))), # correlated toy PEP
  run = "run1",
  score = rnorm(n)
)

x <- as_dfdr_tbl(
  toy,
  unit = "psm",
  scope = "global",
  q_source = "toy mokapot",
  q_max_export = 1
)

diag <- dfdr_run_all(
  xs = list(mokapot = x),
  alpha_main = 0.01,
  alphas = c(5e-4, 1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
  low_conf = c(0.2, 0.5)
)
```

### Headline stability at 1%

```{r headline}
diag$tables$headline
```

### Decoy tail support and stability proxy

```{r stability-plots}
diag$plots$dalpha
diag$plots$cv
```

### Local boundary support and elasticity

```{r boundary-stability}
diag$plots$dwin
diag$plots$elasticity
```

### Equal-chance plausibility by q-band (internal check)

```{r equal-chance}
diag$tables$equal_chance_pooled
diag$plots$equal_chance__mokapot
```

### PEP reliability and expected errors (ΣPEP)

```{r pep}
diag$tables$pep_IPE
diag$plots$pep_reliability__mokapot
```

```{r sumpep}
# sumpep is a list-of-tibbles (one per list)
diag$tables$sumpep$mokapot
```

## Real mokapot workflow (targets + decoys text exports)

The code below shows how to run **diagFDR** directly from mokapot outputs.

```{r real-mokapot, eval=FALSE}
library(diagFDR)

# Read mokapot outputs (targets + decoys)
raw <- read_mokapot_psms(
  target_path = "path/to/your_output.mokapot.psms.txt",
  decoy_path  = "path/to/your_output.mokapot.decoy.psms.txt"
)

# Construct competed winners (1 per run+spectrum_id; max mokapot score)
x <- mokapot_competed_universe(
  raw,
  id_mode = "runxid",        
  unit = "psm",
  scope = "global",
  q_source = "mokapot q-value",
  q_max_export = 1
)

# Run diagnostics
diag <- dfdr_run_all(
  xs = list(mokapot = x),
  alpha_main = 0.01,
  alphas = c(5e-4, 1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
  low_conf = c(0.2, 0.5)
)

# Write outputs to disk (tables + plots + summary + manifest + README)
dfdr_write_report(
  diag,
  out_dir = "diagFDR_mokapot_out",
  formats = c("csv", "png", "manifest", "readme", "summary")
)

# Render a single HTML report (requires rmarkdown in Suggests)
dfdr_render_report(diag, out_dir = "diagFDR_mokapot_out")
```

## Interpretation notes

- `D_alpha` (tail support) and `D_alpha_win` (local boundary support) quantify how well the
  cutoff is supported by decoys. Very strict thresholds can yield sparse decoy tails and thus
  unstable operating points (large `CV_hat`).

- `S_alpha(eps)` (elasticity) captures list sensitivity to small threshold perturbations.
  Low Jaccard overlap indicates that accepted IDs are sensitive to the chosen cutoff.

- Equal-chance diagnostics (decoy fraction in low-confidence q-bands) and PEP-based diagnostics
  (PEP reliability, ΣPEP) are **internal consistency checks** under target–decoy assumptions.
  They are informative but do not replace external validation strategies (e.g., entrapment) when
  decoy representativeness is uncertain.

---
title: "diagFDR: generic diagnostics from Spectronaut exports"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{diagFDR: generic elution groups diagnostics from Spectronaut (tsv report)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4.5
)
```

This vignette demonstrates how to compute **diagFDR** diagnostics from a Spectronaut export.

## Runnable toy example (no external files required)

This section provides a small *toy* dataset that mimics the **elution group-level universe**
  returned by `read_spectronaut_efficient()`.

```{r toy}
library(diagFDR)

set.seed(3)

n <- 7000
pi_decoy <- 0.03

# Decoy indicator
is_decoy <- sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(1 - pi_decoy, pi_decoy))

# Targets are a mixture: some null-like (close to decoys), some true (higher score)
# This yields realistic separation and non-trivial discoveries at 1% FDR.
is_true_target <- (!is_decoy) & (runif(n) < 0.30)  # 30% of targets are "true"
is_null_target <- (!is_decoy) & (!is_true_target)

score <- numeric(n)
score[is_decoy]       <- rnorm(sum(is_decoy), mean = 0.0, sd = 1.0)
score[is_null_target] <- rnorm(sum(is_null_target), mean = 0.2, sd = 1.0)
score[is_true_target] <- rnorm(sum(is_true_target), mean = 3.0, sd = 1.0)

toy <- data.frame(
  id = paste0("psm", seq_len(n)),
  is_decoy = is_decoy,
  run = sample(paste0("run", 1:4), n, replace = TRUE),
  score = score,
  pep = NA_real_
)

# Score-based TDC q-values (higher score is better)
toy <- toy[order(toy$score, decreasing = TRUE), ]
toy$D_cum <- cumsum(toy$is_decoy)
toy$T_cum <- cumsum(!toy$is_decoy)
toy$FDR_hat <- (toy$D_cum + 1) / pmax(toy$T_cum, 1)
toy$q <- rev(cummin(rev(toy$FDR_hat)))
toy <- toy[, c("id","is_decoy","q","pep","run","score")]

x_toy <- as_dfdr_tbl(
  toy,
  unit = "psm",
  scope = "global",
  q_source = "toy TDC from score",
  q_max_export = 1,
  provenance = list(tool = "toy")
)

diag <- dfdr_run_all(
  xs = list(univ = x_toy),
  alpha_main = 0.01,
  alphas = c(1e-4, 5e-4, 1e-3, 2e-3, 5e-3, 1e-2, 2e-2, 5e-2, 1e-1, 2e-1),
  eps = 0.2,
  win_rel = 0.2,
  truncation = "warn_drop",
  low_conf = c(0.2, 0.5)
)
```

### Headline stability at 1%

```{r headline}
diag$tables$headline

if (nrow(diag$tables$headline) > 0 && diag$tables$headline$T_alpha[1] == 0) {
  cat("\nNote: No discoveries at alpha_main for this toy run. ",
      "For demonstration, consider using alpha_main = 0.02.\n", sep = "")
}
```

### Tail support and stability versus threshold

```{r stability-plots}
diag$plots$dalpha
diag$plots$cv
```

### Local boundary support and elasticity

```{r boundary-stability}
diag$plots$dwin
diag$plots$elasticity
```

### Equal-chance plausibility by q-band

```{r equal-chance}
diag$tables$equal_chance_pooled
diag$plots$equal_chance__mzid_PSM
```

## Application to Spectronaut

The code below shows how to run **diagFDR** directly from an export of Spectronaut at the "elution group" level. The export is assumed to be a "normal" report (not "pivot") that contains peptides characterized by "elution groups". It has the columns "EG.Qvalue", "EG.Cscore" and "EG.IsDecoy".

```{r real-mzid, eval=FALSE}
library(diagFDR)

# Read efficiently (when facing big datasets)
rep <- read_spectronaut_efficient("path/to/search_results.Report-Peptide normal.tsv",
                                  minimal = TRUE, dec = ",")

univ_runwise <- spectronaut_runxprecursor(
  rep,
  q_col = "EG.Qvalue",
  score_col = "EG.Cscore"
)

# Run diagnostics
diag <- dfdr_run_all(
      list(runwise = univ_runwise),
      compute_pseudo_pvalues=TRUE
 )

# Export outputs
dfdr_write_report(
  diag,
  out_dir = "diagFDR_spectronaut_out",
  formats = c("csv", "png", "manifest", "readme", "summary")
)

# Optional: render a single HTML report 
dfdr_render_report(diag, out_dir = "diagFDR_spectronaut_out")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_diann.R
\name{read_diann_parquet}
\alias{read_diann_parquet}
\title{Read a DIA-NN \code{report.parquet}}
\usage{
read_diann_parquet(path)
}
\arguments{
\item{path}{Character scalar. Path to a DIA-NN \code{report.parquet} file.}
}
\value{
A \link[tibble:tibble]{tibble} containing the columns present in the DIA-NN
parquet report (column names depend on DIA-NN export settings).
}
\description{
Reads a DIA-NN \code{report.parquet} file using the \pkg{arrow} package
(suggested dependency) and returns it as a tibble.
}
\examples{
if (requireNamespace("arrow", quietly = TRUE)) {
  # Create a tiny parquet file and read it back
  tmp <- tempfile(fileext = ".parquet")
  df <- data.frame(Precursor.Id = c("P1", "P2"), Q.Value = c(0.01, 0.02))
  arrow::write_parquet(df, tmp)
  out <- read_diann_parquet(tmp)
  out
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{dfdr_plot_p_density_by_decoy}
\alias{dfdr_plot_p_density_by_decoy}
\title{Plot (pseudo-)p-value density by decoy/target}
\usage{
dfdr_plot_p_density_by_decoy(
  x,
  p_max = 1,
  bw = "nrd0",
  adjust = 1,
  title = "(Pseudo-)p-value distribution (density): targets vs decoys",
  log10_x = FALSE,
  eps = 1e-300
)
}
\arguments{
\item{x}{A \code{dfdr_tbl} with columns \code{p} and \code{is_decoy}.}

\item{p_max}{Numeric. Maximum p-value displayed (default 1).}

\item{bw}{Bandwidth passed to \code{ggplot2::geom_density()}.}

\item{adjust}{Numeric smoothing adjustment passed to \code{geom_density()}.}

\item{title}{Character. Plot title.}

\item{log10_x}{Logical. If \code{TRUE}, plot \eqn{-\log_{10}(p)} instead of \code{p}.}

\item{eps}{Numeric. Lower bound used to avoid \code{Inf} when \code{log10_x = TRUE}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Plots density curves of (pseudo-)p-values for targets vs decoys. Optionally
plots \eqn{-\log_{10}(p)} to emphasise small p-values.
}
\examples{
library(tibble)

set.seed(1)
n <- 4000
df <- tibble(
  id = as.character(seq_len(n)),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  q = stats::runif(n, 0, 0.05),
  pep = NA_real_,
  p = stats::runif(n)
)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy", p_source = "toy")
dfdr_plot_p_density_by_decoy(x, log10_x = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_spectronaut_normalEG.R
\name{read_spectronaut}
\alias{read_spectronaut}
\title{Read a Spectronaut TSV report}
\usage{
read_spectronaut(path, dec = ".", select = NULL)
}
\arguments{
\item{path}{Character scalar. Path to a Spectronaut report TSV file.}

\item{dec}{Character scalar. Decimal separator passed to \code{fread()} (default \code{"."}).}

\item{select}{Optional character vector of column names to read; \code{NULL} reads all columns.}
}
\value{
A \link[tibble:tibble]{tibble} containing the columns present in the Spectronaut
report (column names depend on the export configuration).
}
\description{
Reads a Spectronaut report exported as tab-separated values (TSV). Uses
\code{data.table::fread()} for efficient reading of large files.
}
\details{
Spectronaut may use a comma as decimal separator in some locales; set
\code{dec=","} when needed.
}
\examples{
if (requireNamespace("data.table", quietly = TRUE)) {
  # Create a tiny TSV and read it back
  tmp <- tempfile(fileext = ".tsv")
  txt <- paste0(
    "R.FileName\tEG.PrecursorId\tEG.IsDecoy\tEG.Qvalue\tEG.PEP\tEG.Cscore\n",
    "run1\tP1\tFalse\t0.01\t0.02\t120\n",
    "run1\tP2\tTrue\tNaN\t0.90\t10\n"
  )
  writeLines(txt, tmp)
  read_spectronaut(tmp)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_calibration.R
\name{dfdr_pep_reliability_tdc}
\alias{dfdr_pep_reliability_tdc}
\title{Target-focused PEP reliability using a TDC-style error proxy}
\usage{
dfdr_pep_reliability_tdc(x, breaks = seq(0, 0.5, by = 0.05), add_decoy = 0L)
}
\arguments{
\item{x}{A \code{dfdr_tbl} with columns \code{pep} and \code{is_decoy}.}

\item{breaks}{Numeric vector of PEP bin edges.}

\item{add_decoy}{Integer. Additive correction to the decoy count in each bin
(default 0). Use 1 for a conservative small-sample correction.}
}
\value{
A \link[tibble:tibble]{tibble} with one row per PEP bin, including:
\describe{
\item{bin}{Formatted bin label.}
\item{n_target}{Number of targets in the bin.}
\item{n_decoy}{Number of decoys in the bin.}
\item{pep_mean_targets}{Mean PEP among targets in the bin.}
\item{pep_mean_all}{Mean PEP among all entries in the bin.}
\item{err_hat_target}{Estimated target error proxy \eqn{(D + add\_decoy)/T}, capped at 1.}
\item{pep_bin_mid}{Midpoint of the PEP bin (useful for plotting).}
}
}
\description{
Bins identifications by predicted PEP and estimates an error proxy for targets
within each bin using decoys as a proxy via a target-decoy-competition (TDC)
style ratio \eqn{(D + add\_decoy) / T}.
}
\examples{
library(tibble)

set.seed(1)
n <- 8000
df <- tibble(
  id = as.character(seq_len(n)),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.9, 0.1)),
  score = rnorm(n),
  q = stats::runif(n),
  pep = stats::runif(n)
)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")

dfdr_pep_reliability_tdc(x, breaks = seq(0, 0.5, by = 0.1), add_decoy = 1L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pvalue_helpers.R
\name{.make_stratum_id}
\alias{.make_stratum_id}
\title{Internal: build stratum identifiers}
\usage{
.make_stratum_id(strata_df)
}
\arguments{
\item{strata_df}{A data.frame of stratification columns (typically produced by
\code{.get_strata_df}). If \code{NULL}, a single stratum \code{"all"} is used.}
}
\value{
A character vector of stratum identifiers (length \code{nrow(strata_df)} if not
\code{NULL}, otherwise \code{"all"}).
}
\description{
Builds a human-readable stratum identifier per row from a data.frame of
stratification columns. Each identifier is of the form
\code{"col1=... | col2=..."}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mzid.R
\name{mzid_score_direction}
\alias{mzid_score_direction}
\title{Determine score direction (internal)}
\usage{
mzid_score_direction(
  score_accession,
  score_direction = c("auto", "lower_better", "higher_better")
)
}
\arguments{
\item{score_accession}{Character. PSI-MS accession for the score term.}

\item{score_direction}{One of \code{"auto"}, \code{"lower_better"}, \code{"higher_better"}.}
}
\value{
\code{"lower_better"} or \code{"higher_better"}.
}
\description{
Determine score direction (internal)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_dfdr_tbl.R
\name{validate_dfdr_tbl}
\alias{validate_dfdr_tbl}
\title{Validate a \code{dfdr_tbl}}
\usage{
validate_dfdr_tbl(x)
}
\arguments{
\item{x}{A data frame (typically a tibble) to validate.}
}
\value{
The input \code{x} invisibly (unchanged) if validation succeeds. Otherwise an
error is raised describing the first detected problem.
}
\description{
Performs structural checks required by \code{diagFDR} diagnostics. This
function is intended for internal use and is called by constructors and
downstream diagnostics.
}
\details{
The required columns are \code{id}, \code{is_decoy}, \code{q}, \code{pep},
\code{run}, and \code{score}. If a \code{p} column is present, it is also
validated.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pvalue_helpers.R
\name{.jaccard}
\alias{.jaccard}
\title{Internal: Jaccard similarity of two sets}
\usage{
.jaccard(a, b)
}
\arguments{
\item{a}{Vector representing set A.}

\item{b}{Vector representing set B.}
}
\value{
Numeric scalar in \eqn{[0,1]} giving the Jaccard similarity.
}
\description{
Computes the Jaccard index \eqn{|A \cap B| / |A \cup B|} on unique values.
Returns 1 when both sets are empty.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_stability.R
\name{dfdr_local_tail}
\alias{dfdr_local_tail}
\title{Local tail support near the cutoff}
\usage{
dfdr_local_tail(
  x,
  alphas,
  win_rel = 0.2,
  truncation = c("warn_drop", "drop", "cap")
)
}
\arguments{
\item{x}{An \code{dfdr_tbl}.}

\item{alphas}{Numeric vector of FDR thresholds in \eqn{(0,1]}.}

\item{win_rel}{Numeric. Relative window width \eqn{\rho} (default 0.2).}

\item{truncation}{Character. How to handle cases where
\eqn{\alpha + \rho\alpha} exceeds the available q-value export range
(i.e. \code{q_max_export}). One of \code{"warn_drop"}, \code{"drop"},
\code{"cap"}.}
}
\value{
A \link[tibble:tibble]{tibble} with one row per \code{alpha}. Columns include:
\describe{
\item{alpha}{The threshold.}
\item{delta}{The effective window width used (\eqn{\rho\alpha} or capped).}
\item{n_win}{Number of entries with \code{q} in the window.}
\item{D_alpha_win}{Number of decoys in the window.}
\item{decoy_frac_win}{Decoy fraction in the window (\code{D_alpha_win/n_win}).}
\item{upper_used}{Upper endpoint actually used for the window.}
\item{truncated}{Logical indicating whether the requested window was truncated
(or dropped) due to export limits.}
}
This table is used to assess boundary support: very small \code{n_win} or
\code{D_alpha_win} indicates an unstable/poorly-supported cutoff.
}
\description{
Counts the number of observations (and decoys) in a right-neighborhood of each
threshold \eqn{\alpha}, defined as \eqn{(\alpha,\, \alpha + \rho\alpha]} where
\eqn{\rho} is \code{win_rel}. This approximates how well supported the decision
boundary is by decoys in the immediate tail.
}
\examples{
library(tibble)

set.seed(1)
n <- 5000
df <- tibble(
  id = as.character(seq_len(n)),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  pep = NA_real_
)
df$q <- diagFDR:::tdc_qvalues(df$score, df$is_decoy, add_decoy = 1L)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy_tdc")

dfdr_local_tail(x, alphas = c(1e-3, 5e-3, 1e-2), win_rel = 0.2, truncation = "cap")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_spectronaut_normalEG.R
\name{spectronaut_runxprecursor}
\alias{spectronaut_runxprecursor}
\title{Spectronaut -> run-by-precursor universe (deduplicated within run by \code{EG.PrecursorId})}
\usage{
spectronaut_runxprecursor(
  rep,
  q_col = "EG.Qvalue",
  run_col = NULL,
  pep_col = "EG.PEP",
  score_col = "EG.Cscore",
  q_max_export = 1,
  recompute_q = TRUE,
  id_mode = c("id", "runxid"),
  unit = "runxprecursor",
  scope = "runwise",
  q_source = q_col
)
}
\arguments{
\item{rep}{Spectronaut report tibble (e.g. returned by \code{\link{read_spectronaut}}
or \code{\link{read_spectronaut_efficient}}).}

\item{q_col}{Character. q-value column name (default \code{"EG.Qvalue"} for run-wise control).}

\item{run_col}{Optional character. Run column name; if \code{NULL}, attempts to
detect one of \code{"R.FileName"} or \code{"R.Condition"}.}

\item{pep_col}{Optional character. PEP column name (default \code{"EG.PEP"}).}

\item{score_col}{Optional character. Score column name (default \code{"EG.Cscore"}).}

\item{q_max_export}{Numeric. Export ceiling for q-values (default 1.0), stored in metadata.}

\item{recompute_q}{Logical. If \code{TRUE} (default) and decoys lack finite q-values,
recompute q-values from \code{score_col} per run.}

\item{id_mode}{Character. \code{"id"} (default) uses \code{id = EG.PrecursorId} and keeps
\code{run} separate; \code{"runxid"} uses \code{id = "run||precursor"} for global uniqueness.}

\item{unit}{Character. Unit metadata stored in the returned object.}

\item{scope}{Character. Scope metadata stored in the returned object.}

\item{q_source}{Character. Source label stored in metadata.}
}
\value{
A \code{dfdr_tbl} (tibble subclass) with one row per \code{run} \eqn{\times}
\code{precursor}. The returned table contains the standard columns required by
\code{diagFDR}: \code{id}, \code{is_decoy}, \code{q}, \code{pep}, \code{run}, \code{score}.
Metadata (including whether q-values were recomputed) are stored in \code{attr(x, "meta")}.
}
\description{
Constructs a run-wise precursor universe from a Spectronaut report. The output
is suitable for run-wise FDR diagnostics because each row corresponds to a
\code{run} \eqn{\times} \code{precursor} hypothesis.
}
\details{
If Spectronaut exports \code{NaN} q-values for decoys (common), the function can
optionally recompute q-values from scores per run using target-decoy competition.

When Spectronaut exports decoys with \code{NaN} q-values, recomputing q-values
from scores is performed \emph{per run} to respect the run-wise hypothesis structure
and to avoid pooling scores across runs that may not be comparable.
}
\examples{
library(tibble)

# Minimal toy Spectronaut-like report
rep <- tibble(
  R.FileName = c("run1","run1","run1","run2","run2","run2"),
  EG.PrecursorId = c("P1","P2","P3","P1","P2","P4"),
  EG.IsDecoy = c("False","True","False","False","True","False"),
  EG.Qvalue = c(0.01, NaN, 0.02, 0.03, NaN, 0.04),
  EG.PEP = c(0.02, 0.9, 0.05, 0.06, 0.95, 0.08),
  EG.Cscore = c(120, 10, 80, 100, 5, 60)
)

x <- spectronaut_runxprecursor(
  rep,
  q_col = "EG.Qvalue",
  run_col = "R.FileName",
  recompute_q = TRUE,
  id_mode = "runxid"
)
x

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_mokapot.R
\name{mokapot_is_target_to_logical}
\alias{mokapot_is_target_to_logical}
\title{Internal: robust coercion of mokapot \code{is_target} column to logical}
\usage{
mokapot_is_target_to_logical(x)
}
\arguments{
\item{x}{Vector containing mokapot \code{is_target} values (logical, numeric,
or character).}
}
\value{
A logical vector of the same length as \code{x}.
}
\description{
Internal: robust coercion of mokapot \code{is_target} column to logical
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{dfdr_plot_pep_reliability_tdc}
\alias{dfdr_plot_pep_reliability_tdc}
\title{Plot target-focused PEP reliability (TDC-style)}
\usage{
dfdr_plot_pep_reliability_tdc(
  rel_tbl,
  title = "PEP reliability (targets; TDC-style D/T error proxy)"
)
}
\arguments{
\item{rel_tbl}{Output of \code{\link{dfdr_pep_reliability_tdc}}.}

\item{title}{Character. Plot title.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Visualises the output of \code{\link{dfdr_pep_reliability_tdc}} by plotting the
estimated target error proxy \code{err_hat_target} against the mean predicted
target PEP per bin. Point size reflects the number of targets per bin.
}
\examples{
library(tibble)

rel_tbl <- tibble(
  bin = c("(0,0.1]", "(0.1,0.2]", "(0.2,0.3]"),
  n_target = c(800, 500, 200),
  n_decoy = c(20, 30, 40),
  pep_mean_targets = c(0.05, 0.15, 0.25),
  pep_mean_all = c(0.06, 0.17, 0.28),
  err_hat_target = c(0.03, 0.06, 0.20),
  pep_bin_mid = c(0.05, 0.15, 0.25)
)
dfdr_plot_pep_reliability_tdc(rel_tbl)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_mokapot.R
\name{compete_max_score}
\alias{compete_max_score}
\title{Compete winners by maximum score within keys (one per run+spectrum_id)}
\usage{
compete_max_score(df, keys, score_col)
}
\arguments{
\item{df}{A data frame.}

\item{keys}{Character vector of column names defining groups to compete within.}

\item{score_col}{Character scalar giving the score column name.}
}
\value{
A data frame with at most one row per unique combination of \code{keys},
retaining the row with the maximum score (ties broken by taking a single row).
}
\description{
Compete winners by maximum score within keys (one per run+spectrum_id)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/render_report.R
\name{dfdr_render_report}
\alias{dfdr_render_report}
\title{Render a human-readable HTML report from \code{dfdr_run_all} output}
\usage{
dfdr_render_report(
  diag,
  out_dir,
  filename = "diagFDR_report.html",
  self_contained = FALSE,
  open = FALSE
)
}
\arguments{
\item{diag}{A list as returned by \code{\link{dfdr_run_all}}.}

\item{out_dir}{Character scalar. Output directory (created if it does not exist).}

\item{filename}{Character scalar. Output HTML filename (default
\code{"diagFDR_report.html"}).}

\item{self_contained}{Logical. If \code{TRUE}, embed resources into a single HTML
file (may be larger). Default \code{FALSE}.}

\item{open}{Logical. If \code{TRUE}, open the report in the default browser
after rendering.}
}
\value{
The path to the rendered HTML file, returned invisibly. The function is called
for its side effect of creating an HTML report on disk.
}
\description{
Renders an HTML report that summarises key diagnostics and embeds plots
contained in \code{diag$plots}. The report is intended for interactive review
and verifiable reporting.
}
\details{
This function requires the \pkg{rmarkdown} package (suggested dependency) and
uses the built-in R Markdown template shipped with the package
(\code{inst/templates/dfdr_report.Rmd}).
}
\examples{
# A minimal example that renders a report from a toy dataset.
# This example is conditional because rmarkdown is in Suggests.
if (requireNamespace("rmarkdown", quietly = TRUE)) {
  library(tibble)
  tmpdir <- tempdir()

  set.seed(1)
  n <- 3000
  df <- tibble(
    id = as.character(seq_len(n)),
    run = "run1",
    is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
    score = rnorm(n),
    q = pmin(1, rank(-score) / n),
    pep = NA_real_
  )
  x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")

  diag <- dfdr_run_all(
    xs = list(toy = x),
    alpha_main = 0.01,
    compute_pseudo_pvalues = FALSE
  )

  # Render to a temporary directory (does not open a browser during checks)
  dfdr_render_report(diag, out_dir = tmpdir, open = FALSE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/score_to_pvalue.R
\name{add_pseudo_pvalues_to_list}
\alias{add_pseudo_pvalues_to_list}
\title{Add pseudo p-values to \code{dfdr_tbl} objects that have scores but no p-values}
\usage{
add_pseudo_pvalues_to_list(xs, method = "decoy_ecdf")
}
\arguments{
\item{xs}{Named list of \code{dfdr_tbl} objects.}

\item{method}{Character. Method passed to \code{\link{score_to_pvalue}} (default \code{"decoy_ecdf"}).}
}
\value{
A list of \code{dfdr_tbl} objects of the same length/order as \code{xs}, with a
\code{p} column added where applicable.
}
\description{
Iterates over a named list of \code{dfdr_tbl} objects and, where a finite \code{p}
column is missing but \code{score} and \code{is_decoy} are available, adds a new
\code{p} column computed by \code{\link{score_to_pvalue}}. Updates the metadata
attribute \code{p_source}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mzid.R
\name{mzid_parse_psms}
\alias{mzid_parse_psms}
\title{Parse rank-PSMs from mzIdentML (internal)}
\usage{
mzid_parse_psms(
  mzid_path,
  rank = 1L,
  score_accession_preference = c("MS:1002257"),
  min_score_coverage = 1,
  decoy_regex = "(^##|_REVERSED$|^REV_|^DECOY_)"
)
}
\arguments{
\item{mzid_path}{Character scalar. Path to an mzIdentML file.}

\item{rank}{Integer scalar. Which \code{SpectrumIdentificationItem@rank} to extract.}

\item{score_accession_preference}{Character vector of preferred score accessions.}

\item{min_score_coverage}{Numeric in (0,1]. Minimum fraction of PSMs required to have
the chosen score CV term.}

\item{decoy_regex}{Character scalar. Regex for inferring decoys from DBSequence accessions.}
}
\value{
Tibble with columns id, run, is_decoy, score_accession, score_name, score_value.

A \link[tibble:tibble]{tibble} with columns \code{id}, \code{run},
\code{is_decoy}, \code{score_accession}, \code{score_name}, \code{score_value}.
}
\description{
Extracts SpectrumIdentificationItems at the requested rank, derives decoy labels
using PeptideEvidence@isDecoy when available, with fallback to DBSequence accession
regex inference, and selects a single numeric score term with sufficient coverage.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{wilson_ci}
\alias{wilson_ci}
\title{Internal: Wilson score confidence interval for a binomial proportion}
\usage{
wilson_ci(x, n, conf = 0.95)
}
\arguments{
\item{x}{Integer number of successes.}

\item{n}{Integer number of trials.}

\item{conf}{Confidence level in \eqn{(0,1)} (default 0.95).}
}
\value{
Numeric vector of length 2: lower and upper confidence bounds.
}
\description{
Computes a Wilson score interval for \eqn{x/n} at confidence level \code{conf}.
Returns \code{c(NA, NA)} when \code{n == 0}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_spectronaut_normalEG.R
\name{spectronaut_detect_run_col}
\alias{spectronaut_detect_run_col}
\title{Detect Spectronaut run column name}
\usage{
spectronaut_detect_run_col(
  df,
  candidates = c("R.FileName", "R.Condition", "File.Name")
)
}
\description{
Detect Spectronaut run column name
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{dfdr_plot_dalpha}
\alias{dfdr_plot_dalpha}
\title{Plot decoy support \eqn{D_\alpha} versus \code{alpha}}
\usage{
dfdr_plot_dalpha(
  stab_tbl,
  xlab = "alpha (log10)",
  title = "Decoy support D_alpha vs alpha"
)
}
\arguments{
\item{stab_tbl}{A stability table (e.g. from \code{\link{dfdr_curve_stability}})
containing columns \code{alpha}, \code{D_alpha}, and \code{list}.}

\item{xlab}{Character. X-axis label.}

\item{title}{Character. Plot title.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Plots the number of accepted decoys at each threshold \code{alpha}. This is a
key stability indicator: very small \eqn{D_\alpha} implies granular/unstable
behaviour of target-decoy based estimates.
}
\examples{
library(tibble)

stab_tbl <- tibble(
  list = rep(c("A", "B"), each = 4),
  alpha = rep(c(1e-3, 2e-3, 5e-3, 1e-2), times = 2),
  D_alpha = c(5, 8, 20, 40, 2, 3, 6, 10)
)
dfdr_plot_dalpha(stab_tbl)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_diann.R
\name{diann_runxprecursor}
\alias{diann_runxprecursor}
\title{DIA-NN -> run-by-precursor universe (deduplicated within run)}
\usage{
diann_runxprecursor(
  rep,
  q_col = "Q.Value",
  run_col = NULL,
  pep_col = NULL,
  score_col = NULL,
  q_max_export = NA_real_,
  id_mode = c("id", "runxid"),
  unit = "runxprecursor",
  scope = "runwise",
  q_source = q_col
)
}
\arguments{
\item{rep}{A DIA-NN report tibble.}

\item{q_col}{Character. q-value column name (default \code{"Q.Value"}).}

\item{run_col}{Optional character. Name of the run column. If \code{NULL}, the
function attempts to detect one of \code{"Run"}, \code{"File.Name"}, or
\code{"File.Name.Index"}.}

\item{pep_col}{Optional character. PEP column name (default \code{"PEP"} if present).}

\item{score_col}{Optional character. Score column name (if present).}

\item{q_max_export}{Optional numeric export ceiling (e.g. 0.5), stored as metadata.}

\item{id_mode}{Character. Either \code{"runxid"} (default) to set
\code{id = paste(run, Precursor.Id, sep="||")} or \code{"id"} to set
\code{id = Precursor.Id} and keep \code{run} separate.}

\item{unit}{Character. Unit metadata stored in the returned object.}

\item{scope}{Character. Scope metadata stored in the returned object.}

\item{q_source}{Character. Label stored in metadata describing the q-value source.}
}
\value{
A \code{dfdr_tbl} with one row per run-by-precursor unit. The returned table
includes \code{id}, \code{run}, \code{is_decoy}, \code{q}, \code{pep}, and
\code{score}. Metadata are stored in \code{attr(x, "meta")}.
}
\description{
Constructs a run-specific universe by aggregating a DIA-NN report table to one
row per \code{(run, Precursor.Id)}. The run column is detected automatically
unless provided. For each group, the minimum q-value is used and \code{is_decoy}
is \code{TRUE} if any row is a decoy.
}
\examples{
library(tibble)

rep <- tibble(
  Run = c("r1", "r1", "r2", "r2"),
  Precursor.Id = c("P1", "P1", "P1", "P2"),
  Decoy = c(0L, 0L, 1L, 0L),
  Q.Value = c(0.01, 0.02, 0.03, 0.02),
  PEP = c(0.02, 0.01, 0.9, 0.05)
)

x <- diann_runxprecursor(rep, q_col = "Q.Value", run_col = "Run", id_mode = "runxid")
x

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_stability.R
\name{dfdr_curve_stability}
\alias{dfdr_curve_stability}
\title{Stability curve across alpha values}
\usage{
dfdr_curve_stability(x, alphas, k_fixed = 10, k_sqrt_mult = 2)
}
\arguments{
\item{x}{An \code{dfdr_tbl}.}

\item{alphas}{Numeric vector of FDR thresholds in \eqn{(0,1]}.}

\item{k_fixed}{Integer. Fixed +/-K sensitivity interval used by
\code{\link{dfdr_headline}}.}

\item{k_sqrt_mult}{Numeric. Multiplier for the adaptive +/- ceiling(mult*sqrt(D))
interval used by \code{\link{dfdr_headline}}.}
}
\value{
A \link[tibble:tibble]{tibble} with one row per value of \code{alphas}.
Columns are those returned by \code{\link{dfdr_headline}} plus the
corresponding \code{alpha}. The table summarizes how headline quantities
(e.g., accepted target/decoy counts and sensitivity diagnostics) vary with the
chosen FDR cutoff.
}
\description{
Evaluates \code{\link{dfdr_headline}} over a grid of FDR thresholds
(\code{alphas}) to obtain a stability curve that can be plotted or compared
across workflows.
}
\examples{
library(tibble)

set.seed(1)
n <- 5000
df <- tibble(
  id = as.character(seq_len(n)),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  pep = NA_real_
)

df$q <- diagFDR:::tdc_qvalues(df$score, df$is_decoy, add_decoy = 1L)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy_tdc")

dfdr_curve_stability(x, alphas = c(1e-3, 5e-3, 1e-2))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_readme.R
\name{dfdr_write_readme}
\alias{dfdr_write_readme}
\title{Write a human-readable \code{README.md} report}
\usage{
dfdr_write_readme(diag, out_dir, filename = "README.md")
}
\arguments{
\item{diag}{A list as returned by \code{\link{dfdr_run_all}}.}

\item{out_dir}{Character scalar. Output directory (created if it does not exist).}

\item{filename}{Character scalar. Markdown filename (default \code{"README.md"}).}
}
\value{
The path to the written README file, returned invisibly. The function is called
for its side effect of writing a file to disk.
}
\description{
Produces a lightweight narrative report in Markdown that summarises the most
important diagnostics (headline summary, key plots, exported tables) and records
per-list metadata (unit, scope, q-source, etc.) from a \code{\link{dfdr_run_all}}
result. This is intended to accompany an export folder for auditing and sharing.
}
\examples{
library(tibble)

# Minimal dfdr_run_all-like object for demonstrating README writing
df <- tibble(
  id = c("1","2","3"),
  run = "run1",
  is_decoy = c(FALSE, TRUE, FALSE),
  score = c(10, 9, 8),
  q = c(0.01, 0.02, 0.03),
  pep = NA_real_
)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")

diag <- list(
  objects = list(toy = x),
  params = list(alpha_main = 0.01),
  tables = list(headline = tibble(list = "toy", alpha = 0.01)),
  warnings = character()
)

out_dir <- tempdir()
dfdr_write_readme(diag, out_dir = out_dir, filename = "README_example.md")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_diann.R
\name{diann_global_precursor}
\alias{diann_global_precursor}
\title{DIA-NN -> global precursor universe (deduplicated by \code{Precursor.Id})}
\usage{
diann_global_precursor(
  rep,
  q_col = "Q.Value",
  pep_col = NULL,
  score_col = NULL,
  q_max_export = NA_real_,
  unit = "precursor",
  scope = "global",
  q_source = q_col
)
}
\arguments{
\item{rep}{A DIA-NN report tibble (e.g. returned by \code{\link{read_diann_parquet}}).}

\item{q_col}{Character. Column name for q-values (default \code{"Q.Value"}).}

\item{pep_col}{Optional character. PEP column name (default \code{"PEP"} if present).}

\item{score_col}{Optional character. Score column name to retain; if missing, \code{score} is \code{NA}.}

\item{q_max_export}{Optional numeric export ceiling used in DIA-NN export (e.g. 0.5).}

\item{unit}{Character. Unit metadata stored in the returned object.}

\item{scope}{Character. Scope metadata stored in the returned object.}

\item{q_source}{Character. Label stored in metadata describing the q-value source.}
}
\value{
A \code{dfdr_tbl} (tibble subclass) with one row per precursor and required
columns \code{id}, \code{is_decoy}, \code{q}, \code{pep}, \code{run}, \code{score}.
Metadata are stored in \code{attr(x, "meta")}.
}
\description{
Constructs a precursor-level universe by aggregating a DIA-NN report table to
one row per \code{Precursor.Id}. For each precursor, the minimum q-value is
used (via \code{safe_min}) and \code{is_decoy} is set to \code{TRUE} if any row
in the group is a decoy.
}
\examples{
library(tibble)

rep <- tibble(
  Precursor.Id = c("P1", "P1", "P2"),
  Decoy = c(0L, 0L, 1L),
  Q.Value = c(0.01, 0.02, 0.03),
  PEP = c(0.02, 0.01, 0.9)
)

x <- diann_global_precursor(rep, q_col = "Q.Value", q_max_export = 0.5)
x

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pi0_storey.R
\name{dfdr_plot_pi0}
\alias{dfdr_plot_pi0}
\title{Plot Storey \eqn{\pi_0(\lambda)} curve}
\usage{
dfdr_plot_pi0(pi0_tbl, title = "Storey pi0(lambda) diagnostic")
}
\arguments{
\item{pi0_tbl}{A tibble, typically \code{out$pi0} from \code{\link{dfdr_pi0_storey}}.
Must contain columns \code{stratum}, \code{lambda}, and \code{pi0_hat}.}

\item{title}{Character. Plot title.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Plots \eqn{\pi_0(\lambda)} estimates returned by \code{\link{dfdr_pi0_storey}}.
Values closer to 1 suggest many nulls (few true effects), while values far below
1 (especially if unstable across \code{lambda}) may indicate deviations from
tail-uniformity.
}
\examples{
library(tibble)

set.seed(1)
df <- tibble(
  stratum = rep("all", 5),
  lambda = seq(0.5, 0.9, by = 0.1),
  pi0_hat = c(0.95, 0.96, 0.97, 0.98, 0.99),
  n = 1000
)
p <- dfdr_plot_pi0(df)
p

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mzid.R
\name{read_dfdr_mzid}
\alias{read_dfdr_mzid}
\title{Read mzIdentML into a \code{dfdr_tbl} (generic; score-based TDC q-values)}
\usage{
read_dfdr_mzid(
  mzid_path,
  rank = 1L,
  score_accession_preference = c("MS:1002257", "MS:1001330", "MS:1001328", "MS:1002052",
    "MS:1002049", "MS:1001331", "MS:1001171", "MS:1001950", "MS:1002466"),
  score_direction = c("auto", "lower_better", "higher_better"),
  add_decoy = 1L,
  min_score_coverage = 1,
  decoy_regex = "(^##|_REVERSED$|^REV_|^DECOY_)",
  unit = "psm",
  scope = NA_character_,
  provenance = list()
)
}
\arguments{
\item{mzid_path}{Character scalar. Path to an mzIdentML file (\code{.mzid}).}

\item{rank}{Integer scalar. Which \code{SpectrumIdentificationItem@rank} to use
(default 1).}

\item{score_accession_preference}{Character vector. Preferred PSI-MS CV accessions
to treat as the primary PSM score, in priority order.}

\item{score_direction}{One of \code{"auto"}, \code{"lower_better"}, \code{"higher_better"}.
If \code{"auto"}, applies simple rules for common e-value/expectation accessions.}

\item{add_decoy}{Integer scalar. Additive correction in the TDC FDR estimate:
\eqn{\widehat{FDR} = (D + add\_decoy) / T}. Default 1.}

\item{min_score_coverage}{Numeric in \eqn{(0,1]}. Minimum fraction of PSMs required to
have the chosen score CV term. Default 1.0 (strict; no missing scores allowed).}

\item{decoy_regex}{Character scalar. Regex used to infer decoys from protein accessions
if \code{PeptideEvidence@isDecoy} is not informative.}

\item{unit}{Character. Stored in \code{dfdr_tbl} metadata (default \code{"psm"}).}

\item{scope}{Character. Stored in \code{dfdr_tbl} metadata (default \code{NA}).}

\item{provenance}{Named list. Stored in metadata (tool, version, parameters, command, etc.).}
}
\value{
A \code{dfdr_tbl} (tibble subclass) with one row per extracted PSM (at the requested
\code{rank}). The returned object contains:
\itemize{
\item \code{id}: a PSM identifier derived from run and spectrum ID (\code{"run||spectrumID"});
\item \code{is_decoy}: logical target/decoy label;
\item \code{score}: an internal score where larger values mean better matches;
\item \code{q}: reconstructed monotone q-values from TDC using \code{score} and \code{is_decoy};
\item \code{pep}: \code{NA} (mzIdentML PEPs are not parsed here);
\item \code{run}: run identifier when available.
}
Metadata are stored in \code{attr(x, "meta")} (including \code{unit}, \code{scope},
\code{q_source}, and \code{provenance}).
}
\description{
Extracts a competed PSM universe (rank-1 by default) from an mzIdentML file,
determines target/decoy labels, selects a single numeric PSM score CV term,
and reconstructs q-values using target-decoy counting (TDC).
}
\details{
This function is intended for workflows where mzIdentML does not provide
explicit q-values or PEPs.
}
\examples{
if (requireNamespace("xml2", quietly = TRUE)) {
  # Minimal mzIdentML-like file sufficient for diagFDR's parser:
  # - 1 target and 1 decoy PSM at rank=1, with a numeric cvParam score
  tmp <- tempfile(fileext = ".mzid")
  mzid_txt <- paste0(
    "<?xml version='1.0' encoding='UTF-8'?>\n",
    "<MzIdentML xmlns='http://psidev.info/psi/pi/mzIdentML/1.1'>\n",
    "  <SequenceCollection>\n",
    "    <DBSequence id='DBSeq_t' accession='PROT1'/>\n",
    "    <DBSequence id='DBSeq_d' accession='REV_PROT2'/>\n",
    "    <PeptideEvidence id='PE_t' dBSequence_ref='DBSeq_t' isDecoy='false'/>\n",
    "    <PeptideEvidence id='PE_d' dBSequence_ref='DBSeq_d' isDecoy='true'/>\n",
    "  </SequenceCollection>\n",
    "  <DataCollection>\n",
    "    <AnalysisData>\n",
    "      <SpectrumIdentificationList>\n",
    "        <SpectrumIdentificationResult spectraData_ref='runA' spectrumID='scan=1'>\n",
    "          <SpectrumIdentificationItem rank='1'>\n",
    "            <PeptideEvidenceRef peptideEvidence_ref='PE_t'/>\n",
    "            <cvParam accession='MS:1001331' name='X!Tandem:hyperscore' value='50.0'/>\n",
    "          </SpectrumIdentificationItem>\n",
    "        </SpectrumIdentificationResult>\n",
    "        <SpectrumIdentificationResult spectraData_ref='runA' spectrumID='scan=2'>\n",
    "          <SpectrumIdentificationItem rank='1'>\n",
    "            <PeptideEvidenceRef peptideEvidence_ref='PE_d'/>\n",
    "            <cvParam accession='MS:1001331' name='X!Tandem:hyperscore' value='10.0'/>\n",
    "          </SpectrumIdentificationItem>\n",
    "        </SpectrumIdentificationResult>\n",
    "      </SpectrumIdentificationList>\n",
    "    </AnalysisData>\n",
    "  </DataCollection>\n",
    "</MzIdentML>\n"
  )
  writeLines(mzid_txt, tmp)

  x <- read_dfdr_mzid(tmp, rank = 1L, score_direction = "higher_better")
  x
  range(x$q)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{dfdr_safe_filename}
\alias{dfdr_safe_filename}
\title{Internal: make a string safe for filenames}
\usage{
dfdr_safe_filename(x)
}
\arguments{
\item{x}{Character vector.}
}
\value{
Character vector of the same length with unsafe characters replaced.
}
\description{
Replaces characters that are typically unsafe in filenames with underscores.
Keeps letters, digits, dot, underscore, and hyphen.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_p_calibration.R
\name{dfdr_plot_p_calibration}
\alias{dfdr_plot_p_calibration}
\title{Plot p-value calibration (ECDF minus uniform)}
\usage{
dfdr_plot_p_calibration(ecdf_tbl, title = "P-value calibration: ECDF(p) - u")
}
\arguments{
\item{ecdf_tbl}{A tibble, typically \code{out$ecdf} from \code{\link{dfdr_p_calibration}}.
Must contain columns \code{stratum}, \code{u}, and \code{Fhat}.}

\item{title}{Character. Plot title.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Plots \eqn{\hat F(u) - u} against \eqn{u} using the \code{ecdf} component
returned by \code{\link{dfdr_p_calibration}}. Values above zero indicate
potential inflation (excess of small p-values) relative to uniform.
}
\examples{
library(tibble)

set.seed(1)
n <- 2000
df <- tibble(
  id = as.character(seq_len(n)),
  run = sample(c("run1", "run2"), n, replace = TRUE),
  p = c(stats::runif(1800), stats::rbeta(200, 0.3, 1))
)
cal <- dfdr_p_calibration(df, stratify = "run", min_n = 100)
p <- dfdr_plot_p_calibration(cal$ecdf)
p

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{dfdr_plot_elasticity}
\alias{dfdr_plot_elasticity}
\title{Plot threshold elasticity (Jaccard) versus \code{alpha}}
\usage{
dfdr_plot_elasticity(
  el_tbl,
  xlab = "alpha (log10)",
  title = "Threshold elasticity (Jaccard) vs alpha"
)
}
\arguments{
\item{el_tbl}{An elasticity table (e.g. from \code{\link{dfdr_elasticity}})
containing columns \code{alpha}, \code{jaccard}, and \code{list}.}

\item{xlab}{Character. X-axis label.}

\item{title}{Character. Plot title.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Plots Jaccard overlap values (as returned by \code{\link{dfdr_elasticity}})
against \code{alpha}. Lower overlap indicates greater sensitivity of the
accepted set to small changes of the threshold.
}
\examples{
library(tibble)

el_tbl <- tibble(
  list = rep(c("A", "B"), each = 4),
  alpha = rep(c(1e-3, 2e-3, 5e-3, 1e-2), times = 2),
  jaccard = c(0.95, 0.93, 0.90, 0.88, 0.92, 0.90, 0.86, 0.82)
)
dfdr_plot_elasticity(el_tbl)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/score_to_pvalue.R
\name{score_to_pvalue}
\alias{score_to_pvalue}
\title{Convert identification scores to p-values or pseudo-p-values}
\usage{
score_to_pvalue(
  score,
  method = c("mascot", "neglog10p", "evalue", "rank", "decoy_ecdf"),
  is_decoy = NULL,
  eps = .Machine$double.xmin,
  clamp = TRUE,
  ties = c("average", "first", "random", "max", "min")
)
}
\arguments{
\item{score}{Numeric vector. Identification scores. Higher-is-better is assumed for
\code{method = "rank"} and \code{method = "decoy_ecdf"} (flip sign upstream if needed).}

\item{method}{Character. Conversion method:
\describe{
\item{\code{"mascot"}}{Treat \code{score} as Mascot score \eqn{S=-10\log_{10}(p)}; returns \eqn{p=10^{-S/10}}.}
\item{\code{"neglog10p"}}{Generic \eqn{S=-10\log_{10}(p)}; same transform as \code{"mascot"}.}
\item{\code{"evalue"}}{Treat \code{score} as an expectation/e-value-like quantity and return it as a p-like value.}
\item{\code{"rank"}}{Rank-based pseudo-p-values \eqn{p_i=\mathrm{rank}(-s_i)/(n+1)}.}
\item{\code{"decoy_ecdf"}}{Decoy-ECDF pseudo-p-values from the empirical right-tail of decoy scores;
requires \code{is_decoy}.}
}}

\item{is_decoy}{Optional logical vector (same length as \code{score}). Required for
\code{method = "decoy_ecdf"}. TRUE=decoy, FALSE=target.}

\item{eps}{Numeric scalar > 0. Lower bound to avoid exact 0 (default \code{.Machine$double.xmin}).}

\item{clamp}{Logical. If \code{TRUE} (default), clamp output to \code{[0,1]}.}

\item{ties}{Character. Tie-handling for \code{method="rank"}; passed to \code{rank()}.}
}
\value{
A numeric vector of p-values/pseudo-p-values of the same length as \code{score}.
Non-finite scores yield \code{NA_real_}. For \code{method="decoy_ecdf"}, returned
values estimate the decoy right-tail probability \eqn{P(S_{decoy}\ge s)} (with a
small +1 smoothing), which can be interpreted as a null-calibrated pseudo-p-value
when decoys provide a good null sample.
}
\description{
Converts identification scores to:
\itemize{
\item \strong{p-values} when the score has a known p-value definition (e.g. Mascot score),
\item \strong{pseudo-p-values} when no such definition exists (e.g. MaxQuant/Andromeda scores).
}
}
\details{
Pseudo-p-values are useful as inputs to plausibility checks (e.g. calibration/ECDF diagnostics),
but they are not guaranteed to be valid null p-values unless the method is explicitly null-based
(e.g. \code{method = "decoy_ecdf"} with a reliable decoy set).
}
\examples{
set.seed(1)
score <- rnorm(10)

# Mascot-like transformation: S = -10 log10(p)
S <- c(10, 20, 30)
score_to_pvalue(S, method = "mascot")

# Rank-based pseudo-p-values (higher score => smaller p)
score_to_pvalue(score, method = "rank")

# Decoy-ECDF pseudo-p-values (requires decoys)
n <- 200
score2 <- c(rnorm(n, mean = 0), rnorm(n, mean = 1))  # targets shifted higher
is_decoy <- c(rep(TRUE, n), rep(FALSE, n))
p2 <- score_to_pvalue(score2, method = "decoy_ecdf", is_decoy = is_decoy)
summary(p2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dfdr_summary_headline.R
\name{dfdr_summary_headline}
\alias{dfdr_summary_headline}
\title{Build a headline summary table (one row per list)}
\usage{
dfdr_summary_headline(diag)
}
\arguments{
\item{diag}{A list as returned by \code{\link{dfdr_run_all}}.}
}
\value{
A \link[tibble:tibble]{tibble} with one row per element of the input list used
in \code{\link{dfdr_run_all}} (i.e. one row per \code{list} label). The table
is suitable for exporting to a CSV (e.g. \code{summary_headline.csv}).

The output typically contains (when available) headline target/decoy counts at
\code{alpha_main}, local boundary-support columns (e.g. \code{D_alpha_win},
\code{n_win}), equal-chance pooled columns (e.g. \code{pi_D_hat},
\code{effect_abs}), and PEP reliability headline quantities (e.g. \code{IPE}).
If \code{diag$tables$headline} is missing or empty, an empty tibble is returned.
}
\description{
Convenience helper to assemble a compact, export-ready summary table from the
output of \code{\link{dfdr_run_all}}. The function merges headline metrics with
selected columns from other diagnostics (e.g. local tail support, equal-chance
pooled estimate, and PEP reliability headline if available).
}
\examples{
library(tibble)

set.seed(1)
n <- 4000
df <- tibble(
  id = as.character(seq_len(n)),
  run = sample(c("run1", "run2"), n, replace = TRUE),
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n)
)
df$q <- diagFDR:::tdc_qvalues(df$score, df$is_decoy, add_decoy = 1L)
df$pep <- NA_real_
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy_tdc")

diag <- dfdr_run_all(
  xs = list(toy = x),
  alpha_main = 0.01,
  compute_pseudo_pvalues = FALSE
)

dfdr_summary_headline(diag)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{dfdr_plot_fdrhat_pi0}
\alias{dfdr_plot_fdrhat_pi0}
\title{Plot Storey-style estimated FDR curve: \eqn{\hat\pi_0 m t / R(t)}}
\usage{
dfdr_plot_fdrhat_pi0(
  x,
  sel = c("all", "target", "decoy"),
  pi0.method = "pounds",
  nbins = 20,
  pz = 0.05,
  t_grid = 10^seq(-6, 0, length.out = 400),
  r_min = 1,
  cap_one = TRUE
)
}
\arguments{
\item{x}{A \code{dfdr_tbl} (or data.frame) containing columns \code{id},
\code{p}, and \code{is_decoy}.}

\item{sel}{Character. Subset to use: \code{"all"}, \code{"target"}, or \code{"decoy"}.}

\item{pi0.method}{Character. Method passed to \code{cp4p::estim.pi0()} (not \code{"ALL"}).}

\item{nbins}{Integer. Passed to \code{cp4p::estim.pi0()}.}

\item{pz}{Numeric. Passed to \code{cp4p::estim.pi0()}.}

\item{t_grid}{Numeric vector of thresholds in \eqn{(0,1]} at which to evaluate the curve.}

\item{r_min}{Numeric. Minimum value used to determine when to compute the curve
to avoid division by zero at tiny \code{t}.}

\item{cap_one}{Logical. If \code{TRUE}, caps \code{fdr_hat} at 1.}
}
\value{
A list with components:
\describe{
\item{plot}{A \link[ggplot2:ggplot]{ggplot} object.}
\item{data}{A \link[tibble:tibble]{tibble} with columns \code{t}, \code{R}, and \code{fdr_hat}.}
\item{pi0_hat}{Numeric scalar \eqn{\hat\pi_0}.}
\item{m}{Integer. Number of finite p-values used.}
}
}
\description{
Given p-values (or pseudo-p-values), estimates \eqn{\pi_0} using
\code{cp4p::estim.pi0()} and plots
\deqn{\widehat{\mathrm{FDR}}(t) = \hat\pi_0 \, m \, t / R(t),}
where \eqn{R(t) = \#\{p \le t\}} and \eqn{m} is the number of finite p-values.
}
\examples{
library(tibble)

if (requireNamespace("cp4p", quietly = TRUE)) {
  set.seed(1)
  n <- 4000
  df <- tibble(
    id = as.character(seq_len(n)),
    is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
    p = c(stats::runif(3600), stats::rbeta(400, 0.3, 1))
  )
  out <- dfdr_plot_fdrhat_pi0(df, sel = "all", pi0.method = "pounds", nbins = 10)
  out$pi0_hat
  out$plot
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_dfdr_tbl.R
\name{as_dfdr_tbl}
\alias{as_dfdr_tbl}
\title{Create a \code{dfdr_tbl}}
\usage{
as_dfdr_tbl(
  x,
  unit = NA_character_,
  scope = NA_character_,
  q_source = NA_character_,
  q_max_export = NA_real_,
  p_source = NA_character_,
  provenance = list()
)
}
\arguments{
\item{x}{A data.frame with columns \code{id}, \code{is_decoy}, \code{q},
\code{pep}, \code{run}, \code{score} (where \code{pep}/\code{run}/\code{score}
may be \code{NA} if unavailable). Optionally may contain \code{p}.}

\item{unit}{Character. Statistical unit, e.g. \code{"psm"}, \code{"precursor"},
\code{"peptide"}, \code{"protein"}.}

\item{scope}{Character. Scope of FDR control, e.g. \code{"runwise"},
\code{"global"}, \code{"aggregated"}.}

\item{q_source}{Character. Description of where q-values came from (e.g. a
column name or tool).}

\item{q_max_export}{Numeric. Optional export ceiling used to generate q (if known).}

\item{p_source}{Optional character describing where \code{p} came from.}

\item{provenance}{Named list carrying provenance information (tool, version,
parameters, command, etc.).}
}
\value{
An S3 object of class \code{dfdr_tbl} that inherits from
\link[tibble:tibble]{tbl_df} (and \code{data.frame}). The returned object:
\itemize{
\item contains the validated input data with \code{is_decoy} coerced to
logical (\code{TRUE}/\code{FALSE});
\item carries metadata in \code{attr(x, "meta")} with entries \code{unit},
\code{scope}, \code{q_source}, \code{q_max_export}, \code{p_source}, and
\code{provenance}.
}
}
\description{
Coerces input to a tibble, standardizes the \code{is_decoy} column to logical,
validates required columns and types, attaches metadata, and returns an S3
object of class \code{dfdr_tbl} for downstream diagnostics.
}
\examples{
library(tibble)

df <- tibble(
  id = as.character(1:6),
  run = c("run1","run1","run1","run2","run2","run2"),
  is_decoy = c(FALSE, FALSE, TRUE, FALSE, TRUE, FALSE),
  score = c(10, 9, 8, 7, 6, 5),
  q = c(0.01, 0.02, 0.02, 0.05, 0.06, 0.07),
  pep = c(0.01, 0.02, NA, 0.05, NA, 0.07)
)

x <- as_dfdr_tbl(
  df,
  unit = "psm",
  scope = "global",
  q_source = "toy_q",
  provenance = list(tool = "toy")
)

x
attr(x, "meta")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{dfdr_plot_cv}
\alias{dfdr_plot_cv}
\title{Plot \code{CV_hat} versus \code{alpha}}
\usage{
dfdr_plot_cv(
  stab_tbl,
  xlab = "alpha (log10)",
  title = "Stability proxy CV_hat vs alpha"
)
}
\arguments{
\item{stab_tbl}{A stability table (e.g. from \code{\link{dfdr_curve_stability}})
with columns \code{alpha}, \code{CV_hat}, and \code{list}.}

\item{xlab}{Character. X-axis label.}

\item{title}{Character. Plot title.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Plots a stability proxy (\code{CV_hat}) against the threshold \code{alpha} for
one or more lists.
}
\examples{
library(tibble)

stab_tbl <- tibble(
  list = rep(c("A", "B"), each = 4),
  alpha = rep(c(1e-3, 2e-3, 5e-3, 1e-2), times = 2),
  CV_hat = c(0.30, 0.22, 0.15, 0.12, 0.40, 0.28, 0.20, 0.18)
)
dfdr_plot_cv(stab_tbl)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_mokapot.R
\name{mokapot_competed_universe}
\alias{mokapot_competed_universe}
\title{Create a competed-winner universe from mokapot PSM outputs}
\usage{
mokapot_competed_universe(
  raw,
  run_col = "run",
  spectrum_id_col = "spectrum_id",
  score_col = "mokapot score",
  q_col = "mokapot q-value",
  pep_col = "mokapot PEP",
  id_mode = c("runxid", "id"),
  unit = "psm",
  scope = "global",
  q_source = "mokapot q-value",
  q_max_export = 1
)
}
\arguments{
\item{raw}{A combined mokapot table, typically the output of
\code{\link{read_mokapot_psms}}.}

\item{run_col}{Character. Column name for run (default \code{"run"}).}

\item{spectrum_id_col}{Character. Column name for spectrum identifier (default
\code{"spectrum_id"}).}

\item{score_col}{Character. Column name for mokapot score used for competition
(default \code{"mokapot score"}; higher is better).}

\item{q_col}{Character. Column name for mokapot q-value (default
\code{"mokapot q-value"}).}

\item{pep_col}{Character. Column name for mokapot PEP (default
\code{"mokapot PEP"}).}

\item{id_mode}{Character. If \code{"runxid"} (default), sets \code{id} to
\code{"run||spectrum_id"}. If \code{"id"}, sets \code{id = spectrum_id}.}

\item{unit}{Character. Unit metadata stored in the returned object.}

\item{scope}{Character. Scope metadata stored in the returned object.}

\item{q_source}{Character. Source label stored in metadata.}

\item{q_max_export}{Numeric. Export ceiling for q-values (typically 1 for mokapot).}
}
\value{
An object of class \code{dfdr_tbl} (a tibble subclass) with one row per competed
\code{run + spectrum_id}. The returned table contains the standard columns
required by \code{diagFDR}: \code{id}, \code{is_decoy}, \code{q}, \code{pep},
\code{run}, and \code{score}. Metadata are stored in \code{attr(x, "meta")}.
}
\description{
Constructs a "postprocessor universe" by selecting (competing) the single best
scoring entry (target or decoy) per \code{run + spectrum_id} from mokapot PSM
outputs. This yields one row per spectrum per run and can be used for downstream
target-decoy diagnostics.
}
\examples{
library(tibble)

raw <- tibble(
  run = c("r1", "r1"),
  spectrum_id = c("1001", "1001"),
  `mokapot score` = c(2.0, 1.0),      # target wins
  `mokapot q-value` = c(0.01, 0.5),
  `mokapot PEP` = c(0.02, 0.6),
  is_target = c(TRUE, FALSE)
)

x <- mokapot_competed_universe(raw, id_mode = "runxid")
x

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_scope.R
\name{dfdr_scope_disagreement}
\alias{dfdr_scope_disagreement}
\title{Scope disagreement between two lists}
\usage{
dfdr_scope_disagreement(
  x1,
  x2,
  alphas,
  label1 = "A",
  label2 = "B",
  normalize_ids = TRUE
)
}
\arguments{
\item{x1}{First \code{dfdr_tbl}.}

\item{x2}{Second \code{dfdr_tbl}.}

\item{alphas}{Numeric vector of thresholds in \eqn{(0,1]}.}

\item{label1}{Character label for \code{x1} in the output (default \code{"A"}).}

\item{label2}{Character label for \code{x2} in the output (default \code{"B"}).}

\item{normalize_ids}{Logical. If \code{TRUE} (default), extracts a "base" ID by
stripping any run prefix of the form \code{"<run>||<id>"} (everything up to
and including \code{"||"}). Set to \code{FALSE} if IDs are already comparable
between \code{x1} and \code{x2}.}
}
\value{
A \link[tibble:tibble]{tibble} with one row per \code{alpha} and columns:
\describe{
\item{alpha}{The threshold.}
\item{label1,label2}{The labels identifying the two inputs.}
\item{n1,n2}{Numbers of accepted target IDs in \code{x1} and \code{x2} at \code{alpha}.}
\item{jaccard}{Jaccard similarity between the two accepted target ID sets.}
}
}
\description{
Computes the Jaccard overlap of accepted target IDs between two
\code{dfdr_tbl} objects across a set of thresholds. This is useful for comparing
results obtained under different FDR scopes (e.g. run-wise vs global) or from
different pipelines, provided that both inputs use compatible \code{id}
semantics.
}
\examples{
library(tibble)

set.seed(1)
n <- 4000

# Two "lists" with slightly different q-values for the same base IDs
base_ids <- as.character(seq_len(n))

df1 <- tibble(
  id = paste0("run1||", base_ids),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  q = stats::runif(n, 0, 0.05),
  pep = NA_real_
)

df2 <- tibble(
  id = base_ids,              # no run prefix here
  run = "all",
  is_decoy = df1$is_decoy,    # same decoy labels for simplicity
  score = df1$score + rnorm(n, sd = 0.2),
  q = pmin(1, df1$q * 1.2),   # slightly perturbed q-values
  pep = NA_real_
)

x1 <- as_dfdr_tbl(df1, unit = "psm", scope = "runwise", q_source = "toy")
x2 <- as_dfdr_tbl(df2, unit = "psm", scope = "global",  q_source = "toy")

dfdr_scope_disagreement(x1, x2, alphas = c(0.005, 0.01, 0.02), normalize_ids = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mzid.R
\name{tdc_qvalues}
\alias{tdc_qvalues}
\title{Compute monotone q-values by target-decoy counting (internal)}
\usage{
tdc_qvalues(score, is_decoy, add_decoy = 1L)
}
\arguments{
\item{score}{Numeric vector. Higher is better.}

\item{is_decoy}{Logical vector. TRUE=decoy, FALSE=target.}

\item{add_decoy}{Integer scalar. FDR hat = (D + add_decoy) / T.}
}
\value{
Numeric vector of q-values in \verb{[0,1]}.
}
\description{
Computes FDR estimates along the score-sorted list and returns monotone q-values:
\eqn{q(i) = \min_{j \ge i} \widehat{FDR}(j)}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_stability.R
\name{dfdr_headline}
\alias{dfdr_headline}
\title{Headline stability diagnostics at a given alpha}
\usage{
dfdr_headline(x, alpha = 0.01, k_fixed = 10, k_sqrt_mult = 2)
}
\arguments{
\item{x}{A \code{dfdr_tbl} containing at least \code{id}, \code{is_decoy}, and a
q-value column (see Details).}

\item{alpha}{Numeric FDR threshold in \eqn{(0,1]} at which headline metrics are
computed.}

\item{k_fixed}{Integer. Fixed perturbation size used for sensitivity analyses
based on \eqn{D \pm k} around the decoy count at the threshold.}

\item{k_sqrt_mult}{Numeric. Multiplier for adaptive perturbations of size
\eqn{\lceil \mathrm{mult}\sqrt{D} \rceil}.}
}
\value{
A one-row \link[tibble:tibble]{tibble} with headline diagnostics evaluated at
the specified \code{alpha}. The table includes the number of accepted targets
and decoys at the threshold and additional sensitivity/stability summaries
derived from perturbing the boundary decoy support (using \code{k_fixed} and
\code{k_sqrt_mult}). The returned tibble is intended for reporting and for
comparison across workflows.
}
\description{
Computes headline counts at the acceptance threshold corresponding to
\code{alpha} (e.g., numbers of accepted targets/decoys) and simple stability
diagnostics that quantify how sensitive the result is to small changes in the
decoy boundary support.
}
\details{
The function assumes that \code{x} contains target/decoy labels in
\code{is_decoy} and q-values in column \code{q}. If your input uses another
name (e.g., \code{q_value}), rename it to \code{q} before calling this
function.
}
\examples{
library(tibble)

set.seed(1)
n <- 5000
x <- tibble(
  id = as.character(seq_len(n)),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  pep = NA_real_
)

# Construct simple TDC q-values from the score (higher is better)
x$q <- diagFDR:::tdc_qvalues(score = x$score, is_decoy = x$is_decoy, add_decoy = 1L)
x <- as_dfdr_tbl(x, unit = "psm", scope = "global", q_source = "toy_tdc")

dfdr_headline(x, alpha = 0.01)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{safe_min}
\alias{safe_min}
\title{Internal: safe finite minimum}
\usage{
safe_min(x)

safe_min(x)
}
\arguments{
\item{x}{Numeric vector}
}
\value{
Numeric scalar minimum of finite values, or \code{NA_real_}.

Minimum value or NA_real_
}
\description{
Returns the minimum of finite values in \code{x}. Non-finite values
(\code{NA}, \code{Inf}, \code{-Inf}) are removed. If no finite values remain,
returns \code{NA_real_}.

Returns the minimum of x, with NA values removed.
If all values are NA, returns NA_real_.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pvalue_helpers.R
\name{.bh_threshold}
\alias{.bh_threshold}
\title{Internal: Benjamini--Hochberg rejection threshold}
\usage{
.bh_threshold(p, alpha)
}
\arguments{
\item{p}{Numeric vector of p-values (only finite values are used).}

\item{alpha}{Numeric scalar in \eqn{(0,1)}.}
}
\value{
Numeric scalar BH threshold \eqn{t_\alpha}, or \code{NA_real_} if no
rejection threshold exists.
}
\description{
Computes the BH cutoff \eqn{t_\alpha} for a vector of p-values \code{p} at level
\code{alpha}. Returns \code{NA} if no hypotheses are rejected (or no finite p-values).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pvalue_helpers.R
\name{.get_strata_df}
\alias{.get_strata_df}
\title{Internal: extract stratification columns}
\usage{
.get_strata_df(x, stratify = NULL)
}
\arguments{
\item{x}{A data.frame / tibble.}

\item{stratify}{Optional character vector of column names used for stratification.}
}
\value{
If \code{stratify} is \code{NULL} or empty, returns \code{NULL}. Otherwise
returns a data.frame with the selected stratification columns (in the order
given by \code{stratify}). Throws an error if any requested columns are missing.
}
\description{
Returns the subset of \code{x} corresponding to columns named in \code{stratify}.
Used by stratified diagnostics.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{dfdr_plot_dwin}
\alias{dfdr_plot_dwin}
\title{Plot local decoy support \code{D_alpha_win} versus \code{alpha}}
\usage{
dfdr_plot_dwin(
  dwin_tbl,
  xlab = "alpha (log10)",
  title = "Local decoy support D_alpha_win vs alpha"
)
}
\arguments{
\item{dwin_tbl}{A local-tail table (e.g. from \code{\link{dfdr_local_tail}})
containing columns \code{alpha}, \code{D_alpha_win}, and \code{list}.}

\item{xlab}{Character. X-axis label.}

\item{title}{Character. Plot title.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Plots the number of decoys in a right-neighborhood above each threshold
(as computed by \code{\link{dfdr_local_tail}}). This approximates how well
supported the boundary is by nearby decoys.
}
\examples{
library(tibble)

dwin_tbl <- tibble(
  list = rep(c("A", "B"), each = 4),
  alpha = rep(c(1e-3, 2e-3, 5e-3, 1e-2), times = 2),
  D_alpha_win = c(1, 2, 6, 15, 0, 1, 2, 4)
)
dfdr_plot_dwin(dwin_tbl)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{dfdr_plot_equal_chance}
\alias{dfdr_plot_equal_chance}
\title{Plot equal-chance plausibility by q-value bands}
\usage{
dfdr_plot_equal_chance(
  bands_tbl,
  title = "Equal-chance plausibility: decoy fraction by q-band"
)
}
\arguments{
\item{bands_tbl}{A q-band table (e.g. \code{out$bands} from
\code{\link{dfdr_equal_chance_qbands}}) containing columns \code{q_mean},
\code{decoy_frac}, and \code{n}.}

\item{title}{Character. Plot title.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Visualises decoy fractions by q-value band (as produced by
\code{\link{dfdr_equal_chance_qbands}}). A horizontal reference at 0.5 indicates
the expected decoy fraction under an "equal-chance" region assumption.
}
\examples{
library(tibble)

bands_tbl <- tibble(
  q_mean = c(0.05, 0.15, 0.30, 0.45),
  decoy_frac = c(0.10, 0.20, 0.45, 0.52),
  n = c(2000, 1500, 800, 400)
)
dfdr_plot_equal_chance(bands_tbl)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_stability.R
\name{dfdr_elasticity}
\alias{dfdr_elasticity}
\title{Threshold elasticity (list stability to cutoff perturbation)}
\usage{
dfdr_elasticity(x, alphas, eps = 0.2)
}
\arguments{
\item{x}{An \code{dfdr_tbl}.}

\item{alphas}{Numeric vector of FDR thresholds in \eqn{(0,1]}.}

\item{eps}{Numeric scalar \eqn{\epsilon \ge 0}. Relative perturbation (e.g.
\code{0.2} compares \eqn{\alpha} vs \eqn{1.2\alpha}).}
}
\value{
A \link[tibble:tibble]{tibble} with one row per \code{alpha}. Columns include:
\describe{
\item{alpha}{The baseline threshold.}
\item{eps}{The relative perturbation.}
\item{n1}{Number of accepted targets at \code{alpha}.}
\item{n2}{Number of accepted targets at \code{(1+eps)*alpha}.}
\item{jaccard}{Jaccard overlap between the two accepted target sets.}
}
}
\description{
Computes the stability of the accepted target set under a multiplicative
perturbation of the threshold:
\deqn{S_\alpha(\epsilon) = J(A(\alpha), A((1+\epsilon)\alpha)),}
where \eqn{J} is the Jaccard index and \eqn{A(\alpha)} is the set of accepted
target IDs at threshold \eqn{\alpha}.
}
\examples{
library(tibble)

set.seed(1)
n <- 5000
df <- tibble(
  id = as.character(seq_len(n)),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  pep = NA_real_
)

df$q <- diagFDR:::tdc_qvalues(df$score, df$is_decoy, add_decoy = 1L)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy_tdc")

dfdr_elasticity(x, alphas = c(1e-3, 5e-3, 1e-2), eps = 0.2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_stability.R
\name{dfdr_run_jaccard}
\alias{dfdr_run_jaccard}
\title{Inter-run Jaccard overlap matrix at a fixed threshold}
\usage{
dfdr_run_jaccard(x, alpha = 0.01)
}
\arguments{
\item{x}{An \code{dfdr_tbl} with a non-missing \code{run} column.}

\item{alpha}{Numeric FDR threshold in \eqn{(0,1]} used to define accepted
targets (\code{q <= alpha}).}
}
\value{
A \link[tibble:tibble]{tibble} with one row per run pair and columns:
\describe{
\item{run1}{Name/identifier of the first run.}
\item{run2}{Name/identifier of the second run.}
\item{jaccard}{Jaccard similarity between accepted target ID sets.}
}
}
\description{
Computes pairwise Jaccard similarity between runs based on accepted target IDs
at a fixed threshold \code{alpha}. This can reveal run-to-run disagreement or
heterogeneity in identifications.
}
\examples{
library(tibble)

set.seed(1)
n <- 8000
df <- tibble(
  id = as.character(seq_len(n)),
  run = sample(c("runA", "runB", "runC"), n, replace = TRUE),
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  pep = NA_real_
)
df$q <- diagFDR:::tdc_qvalues(df$score, df$is_decoy, add_decoy = 1L)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy_tdc")

dfdr_run_jaccard(x, alpha = 0.01)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_calibration.R
\name{dfdr_sumpep}
\alias{dfdr_sumpep}
\title{Expected number of false targets among accepted identifications}
\usage{
dfdr_sumpep(x, alphas)
}
\arguments{
\item{x}{An \code{dfdr_tbl} with a non-missing \code{pep} column.}

\item{alphas}{Numeric vector of thresholds in \eqn{(0,1]}.}
}
\value{
A \link[tibble:tibble]{tibble} with one row per \code{alpha}. Columns include:
\describe{
\item{alpha}{The threshold.}
\item{n_targets}{Number of accepted targets (\code{!is_decoy} and \code{q <= alpha}).}
\item{sum_PEP}{Sum of PEP values among accepted targets (expected false targets).}
\item{mean_PEP}{Mean PEP among accepted targets.}
}
}
\description{
Computes \eqn{\sum_{i \in A(\alpha)} \mathrm{PEP}_i} among accepted targets
for each threshold \eqn{\alpha}, where acceptance is defined by \code{q <= alpha}.
}
\examples{
library(tibble)

set.seed(1)
n <- 5000
df <- tibble(
  id = as.character(seq_len(n)),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  q = pmin(1, rank(-score) / n),
  pep = stats::runif(n)
)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")

dfdr_sumpep(x, alphas = c(0.001, 0.01, 0.05))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{dfdr_plot_score_distributions}
\alias{dfdr_plot_score_distributions}
\title{Plot target vs decoy score distributions}
\usage{
dfdr_plot_score_distributions(x, title = "Target vs Decoy Score Distributions")
}
\arguments{
\item{x}{A \code{dfdr_tbl}.}

\item{title}{Character. Plot title.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Plots density distributions of scores (if available) or q-values (fallback) for
targets vs decoys. This provides a quick sanity check that decoys are shifted
toward lower scores (or higher q-values).
}
\examples{
library(tibble)

set.seed(1)
n <- 4000
df <- tibble(
  id = as.character(seq_len(n)),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  q = pmin(1, rank(-score) / n),
  pep = NA_real_
)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")
dfdr_plot_score_distributions(x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_spectronaut_normalEG.R
\name{spectronaut_parse_decoy}
\alias{spectronaut_parse_decoy}
\title{Parse Spectronaut boolean decoy column}
\usage{
spectronaut_parse_decoy(x)
}
\description{
Parse Spectronaut boolean decoy column
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_spectronaut_normalEG.R
\name{read_spectronaut_efficient}
\alias{read_spectronaut_efficient}
\title{Read Spectronaut efficiently with column selection}
\usage{
read_spectronaut_efficient(path, minimal = TRUE, dec = ".")
}
\arguments{
\item{path}{Character scalar. Path to a Spectronaut TSV report.}

\item{minimal}{Logical. If \code{TRUE} (default), reads only columns typically
needed for precursor-level diagnostics; if \code{FALSE}, reads all columns.}

\item{dec}{Character scalar. Decimal separator (use \code{","} for some locales).}
}
\value{
A \link[tibble:tibble]{tibble} containing either the selected minimal columns
(if \code{minimal=TRUE}) or all columns (if \code{minimal=FALSE}).
}
\description{
Convenience wrapper around \code{\link{read_spectronaut}}. For very large
Spectronaut reports, reading only a minimal set of columns can substantially
speed up I/O and reduce memory usage.
}
\examples{
if (requireNamespace("data.table", quietly = TRUE)) {
  tmp <- tempfile(fileext = ".tsv")
  txt <- paste0(
    "R.FileName\tR.Condition\tEG.PrecursorId\tEG.IsDecoy\tEG.Qvalue\tEG.PEP\tEG.Cscore\n",
    "run1\tcondA\tP1\tFalse\t0.01\t0.02\t120\n",
    "run1\tcondA\tP2\tTrue\tNaN\t0.90\t10\n"
  )
  writeLines(txt, tmp)
  read_spectronaut_efficient(tmp, minimal = TRUE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{jaccard_vec}
\alias{jaccard_vec}
\title{Internal: Jaccard similarity for two ID vectors}
\usage{
jaccard_vec(a, b)
}
\arguments{
\item{a, b}{Vectors representing sets.}
}
\value{
Numeric scalar in \eqn{[0,1]} or \code{NA_real_} if undefined.
}
\description{
Computes the Jaccard index \eqn{|A \cap B|/|A \cup B|} on unique values of \code{a}
and \code{b}. Returns \code{NA_real_} if either set is empty (or both are empty),
which is convenient for diagnostics where the overlap is undefined without
discoveries in both sets.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_calibration.R
\name{dfdr_pep_decoy_sanity}
\alias{dfdr_pep_decoy_sanity}
\title{Decoy PEP sanity checks}
\usage{
dfdr_pep_decoy_sanity(x, thresholds = c(0.01, 0.05, 0.1, 0.2))
}
\arguments{
\item{x}{A \code{dfdr_tbl} with a non-missing \code{pep} column.}

\item{thresholds}{Numeric vector of PEP thresholds in \eqn{(0,1]} to summarise.}
}
\value{
A \link[tibble:tibble]{tibble} with one row per \code{threshold}. Columns include:
\describe{
\item{threshold}{The PEP cutoff used for counting.}
\item{n_decoy}{Number of decoys with finite PEP in \eqn{(0,1]}.}
\item{n_target}{Number of targets with finite PEP in \eqn{(0,1]}.}
\item{decoy_le}{Count of decoys with \code{pep <= threshold}.}
\item{target_le}{Count of targets with \code{pep <= threshold}.}
\item{frac_decoy_le}{\code{decoy_le / n_decoy}.}
\item{frac_target_le}{\code{target_le / n_target}.}
}
}
\description{
Summarises how many decoys receive surprisingly small PEP values. Under a
well-calibrated PEP, decoys should typically have high error probabilities;
large fractions of decoys below small PEP thresholds can indicate issues with
PEP calibration or labeling.
}
\examples{
library(tibble)

set.seed(1)
n <- 5000
df <- tibble(
  id = as.character(seq_len(n)),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  q = pmin(1, rank(-score) / n),
  pep = stats::runif(n)
)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")

dfdr_pep_decoy_sanity(x, thresholds = c(0.01, 0.05, 0.1))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BH_elasticity.R
\name{dfdr_plot_bh_elasticity}
\alias{dfdr_plot_bh_elasticity}
\title{Plot BH elasticity (Jaccard) vs alpha}
\usage{
dfdr_plot_bh_elasticity(
  el_tbl,
  xlab = "alpha (log10)",
  title = "BH elasticity: Jaccard(alpha, (1+eps)alpha)"
)
}
\arguments{
\item{el_tbl}{A tibble as returned by \code{\link{dfdr_bh_elasticity}}.}

\item{xlab}{Character. X-axis label.}

\item{title}{Character. Plot title.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Convenience plotting function for the output of \code{\link{dfdr_bh_elasticity}}.
The x-axis is \code{log10(alpha)} and the y-axis is the Jaccard overlap between
BH discovery sets at \code{alpha} and \code{(1+eps)*alpha}.
}
\examples{
library(tibble)

set.seed(1)
n <- 2000
x <- tibble(
  id = as.character(seq_len(n)),
  p = c(stats::runif(1800), stats::rbeta(200, 0.3, 1))
)

el <- dfdr_bh_elasticity(x, alphas = c(1e-3, 5e-3, 1e-2, 2e-2), eps = 0.2)
p <- dfdr_plot_bh_elasticity(el)
p

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_spectronaut_normalEG.R
\name{spectronaut_recompute_q_from_score}
\alias{spectronaut_recompute_q_from_score}
\title{Recompute q-values from scores using target-decoy competition}
\usage{
spectronaut_recompute_q_from_score(df)
}
\arguments{
\item{df}{Data frame with columns: is_decoy, score}
}
\value{
Data frame with added column: q_from_score
}
\description{
When Spectronaut exports NaN q-values for decoys, we can recompute them
from scores. Higher scores are better (more confident).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pvalue_helpers.R
\name{.safe_df}
\alias{.safe_df}
\title{Internal: validate data-frame input}
\usage{
.safe_df(x)
}
\arguments{
\item{x}{Object to check.}
}
\value{
Returns \code{x} unchanged if it is a data.frame; otherwise throws an error.
}
\description{
Asserts that \code{x} is a data.frame (including tibble). Used to provide a
consistent error message for user-facing functions that accept data frames.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_dfdr_tbl.R
\name{print.dfdr_tbl}
\alias{print.dfdr_tbl}
\title{Print a \code{dfdr_tbl}}
\usage{
\method{print}{dfdr_tbl}(x, ...)
}
\arguments{
\item{x}{A \code{dfdr_tbl}.}

\item{...}{Passed to the next print method.}
}
\value{
Returns \code{x} invisibly (called for its printing side effect).
}
\description{
Prints a compact header with attached metadata (unit, scope, q-source, etc.)
and then falls back to the default tibble/data.frame print method.
}
\examples{
library(tibble)

df <- tibble(
  id = as.character(1:3),
  run = "run1",
  is_decoy = c(FALSE, TRUE, FALSE),
  score = c(10, 9, 8),
  q = c(0.01, 0.02, 0.03),
  pep = c(0.01, NA, 0.03)
)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy_q")
x

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_p_calibration.R
\name{dfdr_plot_p_calibration2}
\alias{dfdr_plot_p_calibration2}
\title{P-value calibration plot (cp4p-style), with multiple pi0 reference curves}
\usage{
dfdr_plot_p_calibration2(
  x,
  sel = c("all", "decoy", "target"),
  nbins = 20,
  pz = 0.05,
  step = 0.001,
  return_data = FALSE
)
}
\arguments{
\item{x}{A \code{dfdr_tbl} (or data.frame) containing columns \code{id},
\code{is_decoy}, and \code{p}.}

\item{sel}{Character. Subset to use: \code{"all"}, \code{"decoy"}, or \code{"target"}.}

\item{nbins}{Integer. Passed to \code{cp4p::estim.pi0()}.}

\item{pz}{Numeric. Passed to \code{cp4p::estim.pi0()}.}

\item{step}{Numeric. Grid step size for \code{abs = 1 - p}.}

\item{return_data}{Logical. If \code{TRUE}, return both the plot and the
computed data used to build it.}
}
\value{
If \code{return_data = FALSE} (default), returns a
\link[ggplot2:ggplot]{ggplot} object.

If \code{return_data = TRUE}, returns a list with components:
\describe{
\item{plot}{A ggplot object.}
\item{pi0}{Named numeric vector of \eqn{\pi_0} estimates (one per method).}
\item{data}{A list with \code{main} (ECDF data) and \code{ref} (reference
curve data).}
}
}
\description{
Plots the ECDF of \eqn{1-p} and overlays reference curves derived from
\code{cp4p::estim.pi0()} using multiple \eqn{\pi_0} estimation methods.
This provides a visual plausibility check for p-value calibration.
}
\examples{
library(tibble)

set.seed(1)
n <- 2000
df <- tibble(
  id = as.character(seq_len(n)),
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.9, 0.1)),
  p = c(stats::runif(1800), stats::rbeta(200, 0.3, 1))
)

if (requireNamespace("cp4p", quietly = TRUE)) {
  g <- dfdr_plot_p_calibration2(df, sel = "all", return_data = FALSE)
  g
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write_manifest.R
\name{dfdr_write_manifest}
\alias{dfdr_write_manifest}
\title{Write a manifest file describing parameters, list metadata, and warnings}
\usage{
dfdr_write_manifest(diag, out_dir, filename = "manifest")
}
\arguments{
\item{diag}{A list as returned by \code{\link{dfdr_run_all}}.}

\item{out_dir}{Character scalar. Output directory (created if it does not exist).}

\item{filename}{Character scalar. Base filename without extension (default \code{"manifest"}).}
}
\value{
The path to the manifest file, returned invisibly. The function is called for
its side effect of writing a file to disk.
}
\description{
Writes a manifest alongside exported outputs to record key run parameters and
per-list metadata (unit, scope, q-source, export ceiling, etc.). This helps
tie diagnostic outputs to the exact inputs and settings used.
}
\details{
If \pkg{jsonlite} is available (suggested dependency), the manifest is written
as JSON (\code{*.json}); otherwise a human-readable plain text file
(\code{*.txt}) is written.
}
\examples{
library(tibble)

# Create a minimal dfdr_run_all-like object to demonstrate manifest writing
df <- tibble(
  id = c("1","2","3"),
  run = "run1",
  is_decoy = c(FALSE, TRUE, FALSE),
  score = c(10, 9, 8),
  q = c(0.01, 0.02, 0.03),
  pep = NA_real_
)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")

diag <- list(
  objects = list(toy = x),
  params = list(alpha_main = 0.01),
  warnings = character()
)

out_dir <- tempdir()
dfdr_write_manifest(diag, out_dir = out_dir, filename = "manifest_example")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_mokapot.R
\name{read_mokapot_psms}
\alias{read_mokapot_psms}
\title{Read mokapot PSM text outputs (targets + decoys) and combine}
\usage{
read_mokapot_psms(target_path, decoy_path)
}
\arguments{
\item{target_path}{Character scalar. Path to the mokapot target PSM table
(e.g. \code{*.mokapot.psms.txt}).}

\item{decoy_path}{Character scalar. Path to the mokapot decoy PSM table
(e.g. \code{*.mokapot.decoy.psms.txt}).}
}
\value{
A \link[tibble:tibble]{tibble} containing all rows from the target and decoy
tables. Column names and content are determined by mokapot.
}
\description{
Reads the mokapot PSM output tables for targets and decoys (typically
\code{*.mokapot.psms.txt} and \code{*.mokapot.decoy.psms.txt}) and concatenates
them into a single table.
}
\examples{
if (requireNamespace("readr", quietly = TRUE)) {
  # Create tiny example mokapot-like target/decoy tables and read them back
  tf1 <- tempfile(fileext = ".txt")
  tf2 <- tempfile(fileext = ".txt")
  txt1 <- "run\tspectrum_id\tmokapot score\tmokapot q-value\tmokapot PEP
  \tis_target\nr1\t1\t2.0\t0.01\t0.02\tTRUE\n"
  txt2 <- "run\tspectrum_id\tmokapot score\tmokapot q-value\tmokapot PEP
  \tis_target\nr1\t1\t1.0\t0.50\t0.60\tFALSE\n"
  writeLines(txt1, tf1)
  writeLines(txt2, tf2)

  raw <- read_mokapot_psms(tf1, tf2)
  raw
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_diann.R
\name{diann_detect_run_col}
\alias{diann_detect_run_col}
\title{Detect DIA-NN run column name}
\usage{
diann_detect_run_col(df, candidates = c("Run", "File.Name", "File.Name.Index"))
}
\description{
Detect DIA-NN run column name
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{choose_low_conf}
\alias{choose_low_conf}
\title{Internal: choose a low-confidence q-interval based on export range}
\usage{
choose_low_conf(qmax, low_default = c(0.2, 0.5), low_fallback = c(0.1, 0.2))
}
\arguments{
\item{qmax}{Numeric scalar. Maximum available q-value.}

\item{low_default}{Length-2 numeric vector. Default low-confidence interval.}

\item{low_fallback}{Length-2 numeric vector. Fallback interval when \code{qmax} is smaller.}
}
\value{
Length-2 numeric vector \code{c(lo, hi)}.
}
\description{
Chooses an interval for pooled "low-confidence" diagnostics depending on the
available maximum q-value \code{qmax}. If \code{qmax} covers the default interval,
returns \code{low_default}; otherwise falls back to an interval capped by \code{qmax}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_diann.R
\name{diann_require_cols}
\alias{diann_require_cols}
\title{Validate that required DIA-NN columns exist}
\usage{
diann_require_cols(df, cols)
}
\description{
Validate that required DIA-NN columns exist
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{dfdr_plot_scope_disagreement_matrix}
\alias{dfdr_plot_scope_disagreement_matrix}
\title{Plot scope disagreement as a Jaccard overlap heatmap}
\usage{
dfdr_plot_scope_disagreement_matrix(xs, alpha = 0.01)
}
\arguments{
\item{xs}{Named list of \code{dfdr_tbl} objects.}

\item{alpha}{Numeric threshold used to define accepted targets (\code{q <= alpha}).}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object (heatmap).
}
\description{
Computes accepted target ID sets for each list at a fixed \code{alpha} and
visualises pairwise Jaccard overlaps as a heatmap. IDs are compared at the
"base" level by stripping any \code{"run||"} prefix.
}
\examples{
library(tibble)

set.seed(1)
n <- 2000

make_x <- function(label) {
  df <- tibble(
    id = paste0("run1||", as.character(seq_len(n))),
    run = "run1",
    is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
    score = rnorm(n),
    q = stats::runif(n, 0, 0.05),
    pep = NA_real_
  )
  as_dfdr_tbl(df, unit = "psm", scope = label, q_source = "toy")
}

xs <- list(A = make_x("A"), B = make_x("B"), C = make_x("C"))
dfdr_plot_scope_disagreement_matrix(xs, alpha = 0.01)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_calibration.R
\name{dfdr_pep_reliability}
\alias{dfdr_pep_reliability}
\title{Posterior Error Probability (PEP) reliability and internal PEP calibration error (IPE)}
\usage{
dfdr_pep_reliability(x, binwidth = 0.05, n_min = 200, pep_max = 0.5)
}
\arguments{
\item{x}{A \code{dfdr_tbl} with a non-missing \code{pep} column.}

\item{binwidth}{Numeric bin width for PEP binning (default 0.05).}

\item{n_min}{Integer. Minimum bin size to include bins in the IPE summary.}

\item{pep_max}{Numeric. Maximum mean PEP to include in the IPE summary
(default 0.5).}
}
\value{
A list with components:
\describe{
\item{bins}{A \link[tibble:tibble]{tibble} with one row per PEP bin and columns
\code{pep_mean} (mean predicted PEP), \code{decoy_rate} (observed decoy
fraction), and \code{n} (bin size).}
\item{IPE}{Numeric scalar. The internal PEP calibration error (IPE), computed
as a weighted mean absolute deviation between \code{pep_mean} and
\code{decoy_rate} across eligible bins (\code{n >= n_min} and
\code{pep_mean <= pep_max}). \code{NA} if no eligible bins exist.}
\item{params}{A list of parameters used (\code{binwidth}, \code{n_min},
\code{pep_max}).}
}
}
\description{
Bins identifications by predicted PEP and compares mean predicted PEP to the
observed decoy fraction within each bin (internal target-decoy consistency check).
}
\examples{
library(tibble)

set.seed(1)
n <- 5000
df <- tibble(
  id = as.character(seq_len(n)),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  q = pmin(1, rank(-score) / n),         # simple monotone q-like values
  pep = pmin(1, pmax(0, stats::runif(n))) # toy PEP in [0,1]
)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")

out <- dfdr_pep_reliability(x, binwidth = 0.1, n_min = 50, pep_max = 0.5)
out$IPE
head(out$bins)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_maxquant_msms.R
\name{read_dfdr_maxquant_msms}
\alias{read_dfdr_maxquant_msms}
\title{Read MaxQuant \code{msms.txt} into a \code{dfdr_tbl} (PSM-level; reconstructed TDC q-values)}
\usage{
read_dfdr_maxquant_msms(
  path,
  pep_mode = c("drop", "sanitize", "strict"),
  exclude_contaminants = TRUE,
  add_decoy = 1L,
  unit = "psm",
  scope = "global",
  provenance = list()
)
}
\arguments{
\item{path}{Character scalar. Path to MaxQuant \code{msms.txt}.}

\item{pep_mode}{Character. How to handle the \code{PEP} column:
\describe{
\item{\code{"drop"}}{Set \code{pep = NA} for all rows.}
\item{\code{"sanitize"}}{Keep finite PEP in \code{[0,1]}; set other finite values to \code{NA} and warn.}
\item{\code{"strict"}}{Error if any finite PEP is outside \code{[0,1]}.}
}}

\item{exclude_contaminants}{Logical. If \code{TRUE} and a
\code{Potential contaminant} column exists, rows with \code{"+"} are removed.
Default is \code{TRUE}.}

\item{add_decoy}{Integer scalar. Additive correction in the TDC estimate:
\eqn{\widehat{\mathrm{FDR}} = (D + add\_decoy)/T}. Default is 1.}

\item{unit}{Character. Unit stored in the returned object metadata (default \code{"psm"}).}

\item{scope}{Character. Scope stored in the returned object metadata (default \code{"global"}).}

\item{provenance}{Named list. Stored in the returned object metadata.}
}
\value{
A \code{dfdr_tbl} (tibble subclass) with one row per PSM and columns:
\code{id}, \code{run}, \code{is_decoy}, \code{q}, \code{pep}, \code{score}.
The q-values are reconstructed by TDC from \code{Score} and \code{Reverse}.
Metadata are stored in \code{attr(x, "meta")} (including \code{unit}, \code{scope},
\code{q_source}, and \code{provenance}).
}
\description{
Reads a MaxQuant \code{msms.txt} file and returns a \code{dfdr_tbl} at the PSM
level. q-values are reconstructed by target-decoy counting (TDC) using MaxQuant
\code{Score} (higher is better) and the \code{Reverse} column as the decoy
indicator (\code{"+"} for decoy).
}
\details{
Required columns in \code{msms.txt}:
\itemize{
\item \code{id} (unique PSM identifier; numeric or character)
\item \code{Raw file} (run name)
\item \code{Reverse} (MaxQuant decoy indicator; \code{"+"} for decoy; blank/NA for target)
\item \code{Score} (MaxQuant score; higher is better)
\item \code{PEP} (posterior error probability; optional in practice, see \code{pep_mode})
}

The function computes q-values by target-decoy counting (TDC) using \code{Score}
and the \code{Reverse} decoy label:
\deqn{\widehat{\mathrm{FDR}}(i) = (D(i) + add\_decoy) / T(i), \qquad
     q(i) = \min_{j \ge i}\widehat{\mathrm{FDR}}(j)}

Contaminants are not treated as decoys. If the column
\code{Potential contaminant} exists and \code{exclude_contaminants = TRUE},
those rows are removed prior to computing q-values.
}
\examples{
if (requireNamespace("readr", quietly = TRUE)) {
  # Create a tiny MaxQuant-like msms.txt and read it
  tmp <- tempfile(fileext = ".txt")
  txt <- paste0(
    "id\tRaw file\tReverse\tScore\tPEP\tPotential contaminant\n",
    "1\trun1\t\t100\t0.01\t\n",
    "2\trun1\t+\t90\t0.80\t\n",
    "3\trun1\t\t80\t0.02\t+\n",   # contaminant row removed when exclude_contaminants=TRUE
    "4\trun2\t\t70\t0.05\t\n",
    "5\trun2\t+\t60\t0.90\t\n"
  )
  writeLines(txt, tmp)

  x <- read_dfdr_maxquant_msms(tmp, pep_mode = "sanitize",
        exclude_contaminants = TRUE, add_decoy = 1L)
  x
  head(x$q)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_all.R
\name{dfdr_run_all}
\alias{dfdr_run_all}
\title{Run a standard set of FDR QC diagnostics}
\usage{
dfdr_run_all(
  xs,
  alpha_main = 0.01,
  alphas = c(1e-04, 5e-04, 0.001, 0.002, 0.005, 0.01, 0.02, 0.05, 0.1, 0.2),
  eps = 0.2,
  win_rel = 0.2,
  truncation = "warn_drop",
  k_fixed = 10,
  k_sqrt_mult = 2,
  qband_breaks = c(0, 0.01, 0.02, 0.05, 0.1, 0.2, 0.5),
  low_conf = c(0.2, 0.5),
  min_N_equalchance = 2000,
  pep_binwidth = 0.05,
  pep_n_min = 200,
  pep_max = 0.5,
  pep_sanity_thresholds = c(0.01, 0.05, 0.1, 0.2),
  pep_density_max = 0.5,
  pep_rel_tdc_breaks = seq(0, 0.5, by = 0.05),
  pep_rel_tdc_add_decoy = 0L,
  compute_pseudo_pvalues = TRUE,
  pseudo_pvalue_method = "decoy_ecdf",
  p_u_grid = c(seq(0.001, 0.1, by = 0.001), seq(0.11, 1, by = 0.01)),
  p_u_max = 0.1,
  p_stratify = NULL,
  p_min_n_calib = 200,
  p_lambdas = seq(0.5, 0.95, by = 0.05),
  p_min_n_pi0 = 2000,
  bh_boundary = c("mult", "add"),
  bh_win = 0.2,
  bh_delta = NULL
)
}
\arguments{
\item{xs}{Named list of \code{dfdr_tbl} objects.}

\item{alpha_main}{Numeric scalar in \eqn{(0,1]}. Headline threshold (e.g. 0.01).}

\item{alphas}{Numeric vector of thresholds in \eqn{(0,1]} used to build curves.}

\item{eps}{Numeric scalar. Relative perturbation used in elasticity (e.g. 0.2).}

\item{win_rel}{Numeric scalar. Relative window width for local tail support (e.g. 0.2).}

\item{truncation}{Character. Truncation policy for local tail windows; see \code{\link{dfdr_local_tail}}.}

\item{k_fixed}{Integer. Fixed +/-K used in sensitivity intervals.}

\item{k_sqrt_mult}{Numeric. Multiplier for adaptive +/-\eqn{\lceil mult\sqrt{D}\rceil} intervals.}

\item{qband_breaks}{Numeric vector. Cutpoints for q-band equal-chance diagnostic.}

\item{low_conf}{Length-2 numeric vector. Pooled low-confidence region for the equal-chance test.}

\item{min_N_equalchance}{Integer. Minimum pooled N for equal-chance test.}

\item{pep_binwidth}{Numeric. Bin width for PEP reliability bins.}

\item{pep_n_min}{Integer. Minimum bin size to include PEP bins in IPE summary/plots.}

\item{pep_max}{Numeric. Max mean PEP included in IPE summary.}

\item{pep_sanity_thresholds}{Numeric vector. Thresholds for decoy PEP sanity summaries.}

\item{pep_density_max}{Numeric. Max PEP displayed in target/decoy density plot.}

\item{pep_rel_tdc_breaks}{Numeric vector. Breaks used for the TDC-style PEP reliability bins.}

\item{pep_rel_tdc_add_decoy}{Integer. Additive correction used in D/T for TDC-style reliability.}

\item{compute_pseudo_pvalues}{Logical. If \code{TRUE}, attempt to compute pseudo-p-values
when \code{p} is missing.}

\item{pseudo_pvalue_method}{Character. Method passed to \code{\link{score_to_pvalue}}
(default \code{"decoy_ecdf"}).}

\item{p_u_grid}{Numeric vector in \eqn{(0,1]}. Grid for p-value ECDF calibration diagnostics.}

\item{p_u_max}{Numeric scalar in \eqn{(0,1]}. Upper bound of \code{u} used for p-value inflation summaries.}

\item{p_stratify}{Optional character vector of columns for stratified p-value diagnostics.}

\item{p_min_n_calib}{Integer. Minimum n for p-value calibration per stratum.}

\item{p_lambdas}{Numeric vector in \eqn{[0,1)}. Lambdas for Storey \eqn{\pi_0(\lambda)} diagnostic.}

\item{p_min_n_pi0}{Integer. Minimum n for \eqn{\pi_0(\lambda)} diagnostic per stratum.}

\item{bh_boundary}{Character. Boundary window type for BH diagnostics (\code{"mult"} or \code{"add"}).}

\item{bh_win}{Numeric. Relative window size for BH boundary support when \code{bh_boundary="mult"}.}

\item{bh_delta}{Numeric. Additive window size when \code{bh_boundary="add"}.}
}
\value{
A list with components:
\describe{
\item{tables}{A named list of result tables (tibbles) for the computed diagnostics
(e.g. \code{headline}, \code{stability}, \code{local_tail}, \code{elasticity},
and optional PEP/p-value/BH-related tables).}
\item{plots}{A named list of \link[ggplot2:ggplot]{ggplot} objects produced from
the diagnostics.}
\item{objects}{The input \code{xs}, possibly augmented with computed pseudo-p-values
if \code{compute_pseudo_pvalues=TRUE}.}
\item{params}{A list of parameter values used to compute the diagnostics.}
}
}
\description{
Runs stability, boundary-support, and plausibility diagnostics on a named list
of \code{dfdr_tbl} objects and returns both summary tables and plots.
}
\details{
Diagnostics include (depending on available columns):
\itemize{
\item Target/decoy headline diagnostics at \code{alpha_main} (counts, FDR estimate, stability proxies)
\item Stability curves across \code{alphas}
\item Local tail decoy support near each \code{alpha}
\item Elasticity (Jaccard overlap under threshold perturbation)
\item PEP reliability / IPE and PEP sanity summaries (if \code{pep} is present)
\item Equal-chance by q-bands (always attempted; may be not applicable if q-range is truncated)
\item (Pseudo-)p-value calibration and Storey \eqn{\pi_0(\lambda)} (if \code{p} is present or computed)
\item BH diagnostics (if \code{p} is present)
}
}
\examples{
\donttest{
library(tibble)

set.seed(1)
n <- 2000
df <- tibble(
  id = as.character(seq_len(n)),
  run = sample(c("run1", "run2"), n, replace = TRUE),
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  # Toy q/pep/p for demonstration purposes
  q = pmin(1, rank(-score) / n),
  pep = stats::runif(n),
  p = stats::runif(n)
)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy", p_source = "toy")

diag <- dfdr_run_all(
  xs = list(toy = x),
  alpha_main = 0.01,
  compute_pseudo_pvalues = FALSE
)

names(diag$tables)
names(diag$plots)
diag$tables$headline
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_write.R, R/write_report.R
\name{dfdr_write_report}
\alias{dfdr_write_report}
\title{Write diagnostic outputs to a folder}
\usage{
dfdr_write_report(
  diag,
  out_dir,
  formats = c("csv", "png", "manifest", "readme", "summary"),
  pptx_path = file.path(out_dir, "diagFDR_report.pptx"),
  width = 7.2,
  height = 4.4,
  dpi = 180
)

dfdr_write_report(
  diag,
  out_dir,
  formats = c("csv", "png", "manifest", "readme", "summary"),
  pptx_path = file.path(out_dir, "diagFDR_report.pptx"),
  width = 7.2,
  height = 4.4,
  dpi = 180
)
}
\arguments{
\item{diag}{A list as returned by \code{\link{dfdr_run_all}}.}

\item{out_dir}{Character scalar. Output directory (created if it does not exist).}

\item{formats}{Character vector. Subset of
\code{c("csv","png","pptx","manifest","readme","summary")}.}

\item{pptx_path}{Character scalar. Output path for the PPTX file (only used if
\code{"pptx"} is included in \code{formats}).}

\item{width, height}{Numeric. Plot size (in inches) passed to \code{ggplot2::ggsave()}
when writing PNGs.}

\item{dpi}{Numeric. DPI passed to \code{ggplot2::ggsave()}.}
}
\value{
Invisibly returns \code{TRUE}.

Returns \code{TRUE} invisibly. The function is called for its side effects of
writing files to \code{out_dir}.
}
\description{
Writes tables as CSV and plots as PNG, optionally creates a PPTX (requires officer+rvg).

Exports the output of \code{\link{dfdr_run_all}} to disk. Depending on selected
\code{formats}, this function can:
\itemize{
\item write diagnostic tables as CSV;
\item write plots as PNG;
\item create a PowerPoint (\code{.pptx}) containing the plots (requires \pkg{officer} and \pkg{rvg} in Suggests);
\item write a lightweight manifest (\code{\link{dfdr_write_manifest}});
\item write a human-readable \code{README.md} (\code{\link{dfdr_write_readme}});
\item write a compact headline summary table (\code{summary_headline.csv}).
}
}
\examples{
library(tibble)

# Create a minimal diag object with one table and one plot
diag <- list(
  tables = list(headline = tibble(list = "toy", alpha = 0.01, D_alpha = 10)),
  plots = list(example_plot = ggplot2::ggplot(tibble(x = 1:3, y = 1:3),
                                             ggplot2::aes(x, y)) +
                ggplot2::geom_point()),
  objects = list(),
  params = list(alpha_main = 0.01),
  warnings = character()
)

out_dir <- tempdir()
dfdr_write_report(
diag, out_dir = out_dir,
formats = c("csv", "png", "summary", "manifest", "readme"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_spectronaut_normalEG.R
\name{spectronaut_require_cols}
\alias{spectronaut_require_cols}
\title{Validate that required Spectronaut columns exist}
\usage{
spectronaut_require_cols(df, cols)
}
\description{
Validate that required Spectronaut columns exist
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_p_calibration.R
\name{dfdr_p_calibration}
\alias{dfdr_p_calibration}
\title{P-value calibration diagnostic (ECDF vs uniform; stratified)}
\usage{
dfdr_p_calibration(
  x,
  u_grid = c(seq(0.001, 0.1, by = 0.001), seq(0.11, 1, by = 0.01)),
  u_max = 0.1,
  stratify = NULL,
  min_n = 200
)
}
\arguments{
\item{x}{A \code{dfdr_tbl} (or data.frame) containing columns \code{id} and \code{p}.}

\item{u_grid}{Numeric vector in \eqn{(0,1]}. Grid of \code{u} values for evaluating the ECDF.}

\item{u_max}{Numeric scalar in \eqn{(0,1]}. Upper bound of \code{u} used for
inflation summaries (default 0.1).}

\item{stratify}{Optional character vector of column names used to stratify
diagnostics (e.g. \code{c("run")}).}

\item{min_n}{Integer. Minimum number of finite p-values required per stratum.}
}
\value{
A list with components:
\describe{
\item{ecdf}{A \link[tibble:tibble]{tibble} with columns \code{stratum},
\code{u}, \code{Fhat}, and \code{n}. There is one row per \code{u} value per
stratum.}
\item{summary}{A tibble with one row per stratum, including \code{n} (number
of finite p-values), \code{max_inflation} (maximum of \code{Fhat(u)-u} for
\code{u <= u_max}), and \code{auc_inflation} (area under the positive part of
\code{Fhat(u)-u} on \eqn{[0,u_{\max}]}, normalised by \code{u_max}). Strata
with \code{n < min_n} are reported with \code{NA} metrics and a \code{note}.}
}
}
\description{
Computes the empirical CDF \eqn{\hat F(u) = P(p \le u)} on a grid of \code{u}
values and summarises departures from uniformity in a decision-relevant region
\eqn{u \le u_{\max}}. Intended as a plausibility diagnostic for (pseudo-)p-values
(e.g. under the null, \eqn{\hat F(u) \approx u}).
}
\examples{
library(tibble)

set.seed(1)
n <- 5000
df <- tibble(
  id = as.character(seq_len(n)),
  run = sample(c("run1", "run2"), n, replace = TRUE),
  # mostly uniform p-values + a small enriched component
  p = c(stats::runif(4500), stats::rbeta(500, 0.3, 1))
)

out <- dfdr_p_calibration(df, stratify = "run", u_max = 0.1, min_n = 200)
head(out$ecdf)
out$summary

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flag_headline.R
\name{flag_headline}
\alias{flag_headline}
\title{Flag diagnostic values based on heuristic thresholds}
\usage{
flag_headline(headline_tbl)
}
\arguments{
\item{headline_tbl}{A data.frame/tibble of headline diagnostics. It may contain
one or multiple rows. If some expected columns are missing, they are created
and filled with \code{NA}.}
}
\value{
A \link[tibble:tibble]{tibble} (or data frame) with the same rows as
\code{headline_tbl} and additional columns:
\describe{
\item{flag_Dalpha}{Flag based on \code{D_alpha} (decoy support).}
\item{flag_CV}{Flag based on \code{CV_hat} (stability/variability).}
\item{flag_Dwin}{Flag based on \code{D_alpha_win} (local tail support).}
\item{flag_IPE}{Flag based on \code{IPE} (internal PEP calibration error).}
\item{flag_FDR}{Flag comparing \code{FDR_hat} to the nominal \code{alpha}.}
\item{flag_equalchance}{Flag based on \code{effect_abs} (equal-chance deviation).}
\item{status}{Overall triage status: \code{"VALID"}, \code{"CAUTION"}, or
\code{"REVIEW_NEEDED"}.}
\item{interpretation}{A short human-readable interpretation string.}
}
}
\description{
Adds simple, ASCII-only flags and a coarse overall status to a headline
diagnostics table (typically produced by \code{\link{dfdr_headline}} or the
\code{headline} table inside \code{\link{dfdr_run_all}}). The heuristics are
intended for quick triage in reports; they do not replace manual review.
}
\details{
Rendering as icons (if desired) can be handled downstream (e.g. in an HTML
template); this function returns plain text labels for portability.
}
\examples{
library(tibble)

headline_tbl <- tibble(
  list = c("A", "B"),
  alpha = 0.01,
  D_alpha = c(5, 80),
  CV_hat = c(0.35, 0.10),
  D_alpha_win = c(2, 30),
  IPE = c(0.12, 0.02),
  FDR_hat = c(0.02, 0.009),
  effect_abs = c(0.18, 0.05)
)

flag_headline(headline_tbl)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pi0_storey.R
\name{dfdr_pi0_storey}
\alias{dfdr_pi0_storey}
\title{Storey \eqn{\pi_0(\lambda)} tail-uniformity diagnostic}
\usage{
dfdr_pi0_storey(
  x,
  lambdas = seq(0.5, 0.95, by = 0.05),
  stratify = NULL,
  min_n = 2000,
  clamp = TRUE
)
}
\arguments{
\item{x}{A \code{dfdr_tbl} (or data.frame) containing column \code{p}.}

\item{lambdas}{Numeric vector in \eqn{[0,1)}. Typical range: 0.5--0.95.}

\item{stratify}{Optional character vector of column names used to stratify the
diagnostic (e.g. \code{c("run")}).}

\item{min_n}{Integer. Minimum number of finite p-values required per stratum.}

\item{clamp}{Logical. If \code{TRUE} (default), clamp \code{pi0_hat} into
\code{[0,1]} for reporting.}
}
\value{
A list with components:
\describe{
\item{pi0}{A \link[tibble:tibble]{tibble} with one row per \code{lambda} per
stratum and columns \code{stratum}, \code{lambda}, \code{pi0_hat}, and \code{n}
(the number of finite p-values in the stratum).}
\item{summary}{A tibble with one row per stratum, including \code{n} and
summary statistics of \code{pi0_hat} across \code{lambdas}
(\code{median_pi0}, \code{sd_pi0}, \code{iqr_pi0}). Strata with \code{n < min_n}
are reported with \code{NA} metrics and a \code{note}.}
}
}
\description{
Estimates \eqn{\pi_0(\lambda) = \#\{p>\lambda\} / ((1-\lambda)m)} over a grid
of \code{lambdas}. This is a plausibility diagnostic for (pseudo-)p-values:
under a well-calibrated null, the upper tail should be approximately uniform,
leading to stable \eqn{\pi_0(\lambda)} curves.
}
\examples{
library(tibble)

set.seed(1)
n <- 6000
df <- tibble(
  run = sample(c("run1", "run2"), n, replace = TRUE),
  # mostly uniform p-values + a small enriched component
  p = c(stats::runif(5400), stats::rbeta(600, 0.3, 1))
)

out <- dfdr_pi0_storey(df, stratify = "run", lambdas = seq(0.5, 0.9, by = 0.1), min_n = 500)
head(out$pi0)
out$summary

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{safe_max}
\alias{safe_max}
\title{Internal: safe maximum (NA-robust)}
\usage{
safe_max(x)
}
\arguments{
\item{x}{Numeric vector.}
}
\value{
Numeric scalar maximum, or \code{NA_real_}.
}
\description{
Returns \code{max(x, na.rm = TRUE)}. If all values are \code{NA}, returns
\code{NA_real_}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diag_calibration.R
\name{dfdr_equal_chance_qbands}
\alias{dfdr_equal_chance_qbands}
\title{Equal-chance plausibility by q-value bands}
\usage{
dfdr_equal_chance_qbands(
  x,
  breaks = c(0, 0.01, 0.02, 0.05, 0.1, 0.2, 0.5),
  low_conf = c(0.2, 0.5),
  min_N = 2000
)
}
\arguments{
\item{x}{An \code{dfdr_tbl}.}

\item{breaks}{Numeric vector of q-value cutpoints used to form bands.}

\item{low_conf}{Length-2 numeric vector giving the pooled test interval
(e.g. \code{c(0.2, 0.5)}).}

\item{min_N}{Integer. Minimum pooled sample size required for the pooled test
to be considered stable.}
}
\value{
A list with components:
\describe{
\item{bands}{A \link[tibble:tibble]{tibble} with one row per q-band, including
\code{n} (band size), \code{n_decoy}, \code{decoy_frac}, and \code{q_mean}.}
\item{pooled}{A one-row tibble with pooled quantities over bands whose
\code{q_mean} lies within \code{low_conf}, including \code{pi_D_hat} (pooled
decoy fraction), a Wilson confidence interval, and a binomial test p-value.
\code{pass_minN} indicates whether \code{min_N} is met. In cases where the
requested banding is not applicable (e.g. \code{qmax} below the smallest
nonzero break), fields are returned as \code{NA} and a \code{note} may be
provided.}
\item{params}{A list of parameters used (\code{breaks}, \code{low_conf}, \code{min_N}).}
}
}
\description{
Computes decoy fractions in q-value bands and performs a pooled binomial test
of whether the decoy fraction is near 0.5 in a mismatch-dominated region
(specified by \code{low_conf}).
}
\examples{
library(tibble)

set.seed(1)
n <- 8000
df <- tibble(
  id = as.character(seq_len(n)),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.9, 0.1)),
  score = rnorm(n),
  pep = NA_real_
)
# Toy q-values in [0, 1]
df$q <- stats::runif(n)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")

ec <- dfdr_equal_chance_qbands(x, breaks = c(0, 0.2, 0.5, 1), low_conf = c(0.2, 0.5), min_N = 200)
ec$pooled

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{dfdr_plot_pep_density_by_decoy}
\alias{dfdr_plot_pep_density_by_decoy}
\title{Plot PEP density by decoy/target}
\usage{
dfdr_plot_pep_density_by_decoy(
  x,
  pep_max = 0.5,
  bw = "nrd0",
  adjust = 1,
  title = "PEP distribution (density): targets vs decoys"
)
}
\arguments{
\item{x}{A \code{dfdr_tbl} with a \code{pep} column.}

\item{pep_max}{Numeric. Maximum PEP displayed (default 0.5).}

\item{bw}{Bandwidth passed to \code{ggplot2::geom_density()}.}

\item{adjust}{Numeric smoothing adjustment passed to \code{geom_density()}.}

\item{title}{Character. Plot title.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Plots density curves of PEP values for targets vs decoys, truncated to
\code{pep <= pep_max}.
}
\examples{
library(tibble)

set.seed(1)
n <- 4000
df <- tibble(
  id = as.character(seq_len(n)),
  run = "run1",
  is_decoy = sample(c(FALSE, TRUE), n, replace = TRUE, prob = c(0.95, 0.05)),
  score = rnorm(n),
  q = stats::runif(n, 0, 0.05),
  pep = stats::runif(n)
)
x <- as_dfdr_tbl(df, unit = "psm", scope = "global", q_source = "toy")
dfdr_plot_pep_density_by_decoy(x, pep_max = 0.5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_mzid.R
\name{mzid_to_internal_score}
\alias{mzid_to_internal_score}
\title{Convert raw score to an internal "higher is better" score (internal)}
\usage{
mzid_to_internal_score(score_value, score_direction, score_accession)
}
\arguments{
\item{score_value}{Numeric vector. Raw score values.}

\item{score_direction}{Character. \code{"lower_better"} or \code{"higher_better"}.}

\item{score_accession}{Character. Score accession (used for special handling).}
}
\value{
Numeric vector. Larger values mean better matches.
}
\description{
Convert raw score to an internal "higher is better" score (internal)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{dfdr_plot_pep_reliability}
\alias{dfdr_plot_pep_reliability}
\title{Plot PEP reliability}
\usage{
dfdr_plot_pep_reliability(bins_tbl, title = "PEP reliability")
}
\arguments{
\item{bins_tbl}{A PEP reliability bins table (e.g. \code{out$bins} from
\code{\link{dfdr_pep_reliability}}) containing columns \code{pep_mean},
\code{decoy_rate}, and \code{n}.}

\item{title}{Character. Plot title.}
}
\value{
A \link[ggplot2:ggplot]{ggplot} object.
}
\description{
Visualises PEP reliability by plotting the observed decoy fraction against the
mean predicted PEP in each bin (from \code{\link{dfdr_pep_reliability}}). Point
size reflects bin counts.
}
\examples{
library(tibble)

bins_tbl <- tibble(
  pep_mean = c(0.05, 0.15, 0.25, 0.35),
  decoy_rate = c(0.06, 0.14, 0.30, 0.40),
  n = c(500, 400, 250, 120)
)
dfdr_plot_pep_reliability(bins_tbl)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BH_diagnostics.R
\name{dfdr_bh_diagnostics}
\alias{dfdr_bh_diagnostics}
\title{BH diagnostics at a headline alpha (threshold, discoveries, boundary support)}
\usage{
dfdr_bh_diagnostics(
  x,
  alpha = 0.01,
  boundary = c("mult", "add"),
  win = 0.2,
  delta = NULL
)
}
\arguments{
\item{x}{A \code{dfdr_tbl} (or data frame) containing columns \code{id} and
\code{p}.}

\item{alpha}{Numeric BH FDR level in \eqn{(0,1)}.}

\item{boundary}{Character. One of \code{"mult"} (multiplicative window) or
\code{"add"} (additive window).}

\item{win}{Numeric. Relative width for the multiplicative boundary window
(default 0.2). Used when \code{boundary = "mult"}, defining the interval
\eqn{(t_\alpha, (1+\mathrm{win})t_\alpha]}.}

\item{delta}{Numeric. Additive width for the additive boundary window. Required
when \code{boundary = "add"}, defining the interval \eqn{(t_\alpha, t_\alpha+\delta]}.}
}
\value{
A list with two elements:
\describe{
\item{headline}{A one-row \link[tibble:tibble]{tibble} containing BH summary
diagnostics, including \code{t_alpha} (the BH threshold), \code{R_alpha} (the
number of rejected hypotheses / discoveries), and \code{N_boundary} (the
number of p-values in a right-neighborhood above the cutoff as determined by
\code{boundary}, \code{win}, and \code{delta}).}
\item{accepted}{A character vector of \code{id} values corresponding to
discoveries (rows with \code{p <= t_alpha}). If no finite BH threshold exists,
this is \code{character(0)}.}
}
}
\description{
Computes the Benjamini--Hochberg (BH) rejection threshold \eqn{t_\alpha}, the
number of discoveries \eqn{R_\alpha}, and a simple "boundary support" measure:
the number of p-values just above \eqn{t_\alpha}. Boundary support is intended
to indicate how sensitive the discovery set may be to small perturbations of
p-values near the cutoff.
}
\examples{
library(tibble)

set.seed(1)
n <- 5000
x <- tibble(
  id = as.character(seq_len(n)),
  p = c(stats::runif(4500), stats::rbeta(500, 0.3, 1))
)

out <- dfdr_bh_diagnostics(x, alpha = 0.01, boundary = "mult", win = 0.2)
out$headline
length(out$accepted)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BH_elasticity.R
\name{dfdr_bh_elasticity}
\alias{dfdr_bh_elasticity}
\title{BH list elasticity (Jaccard) across alpha values}
\usage{
dfdr_bh_elasticity(
  x,
  alphas = c(1e-04, 5e-04, 0.001, 0.002, 0.005, 0.01, 0.02, 0.05),
  eps = 0.2
)
}
\arguments{
\item{x}{A \code{dfdr_tbl} (or data frame) with columns \code{id} and \code{p}.
Only finite \code{p}-values are used.}

\item{alphas}{Numeric vector of BH levels in \eqn{(0,1)}.}

\item{eps}{Numeric scalar \eqn{\epsilon > 0}. Relative perturbation (default
0.2), i.e. compares \code{alpha} vs \code{(1+eps)*alpha}.}
}
\value{
A \link[tibble:tibble]{tibble} with one row per element of \code{alphas}. Columns include:
\describe{
\item{alpha}{The nominal BH level.}
\item{alpha_perturbed}{The perturbed level \code{min((1+eps)*alpha, 1)}.}
\item{t_alpha}{BH rejection threshold at \code{alpha} (NA if no rejections).}
\item{t_alpha_perturbed}{BH rejection threshold at \code{alpha_perturbed} (NA if none).}
\item{R_alpha}{Number of discoveries at \code{alpha}.}
\item{R_alpha_perturbed}{Number of discoveries at \code{alpha_perturbed}.}
\item{jaccard}{Jaccard similarity between the two discovery ID sets.}
}
This output is intended to quantify how sensitive BH discoveries are to small
changes in the chosen FDR level.
}
\description{
Computes the stability (elasticity) of Benjamini--Hochberg (BH) discovery sets
under a multiplicative perturbation of the nominal FDR level. For each
\code{alpha} in \code{alphas}, the function compares the BH discovery set at
\code{alpha} to the discovery set at \code{(1+eps)*alpha} using the Jaccard
index.
}
\examples{
library(tibble)

set.seed(1)
n <- 5000
x <- tibble(
  id = as.character(seq_len(n)),
  # mixture: mostly null p-values + a small enriched component
  p = c(stats::runif(4500), stats::rbeta(500, 0.3, 1))
)

dfdr_bh_elasticity(x, alphas = c(1e-3, 5e-3, 1e-2, 2e-2), eps = 0.2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io_diann.R
\name{diann_global_minrunq}
\alias{diann_global_minrunq}
\title{DIA-NN: global precursor list built by minimum run-wise q across runs}
\usage{
diann_global_minrunq(
  rep,
  run_col = NULL,
  q_col = "Q.Value",
  pep_col = NULL,
  score_col = NULL,
  q_max_export = NA_real_,
  unit = "precursor",
  scope = "aggregated",
  q_source = paste0("min_run(", q_col, ")")
)
}
\arguments{
\item{rep}{A DIA-NN report table (e.g. returned by \code{\link{read_diann_parquet}}).}

\item{run_col}{Optional character. Name of the run column. If \code{NULL},
attempts to detect it (e.g. \code{"Run"} or \code{"File.Name"}).}

\item{q_col}{Character. Name of the q-value column to aggregate (default \code{"Q.Value"}).}

\item{pep_col}{Optional character. Name of the PEP column (default \code{"PEP"} if present).}

\item{score_col}{Optional character. Name of a score column to carry along (if present).}

\item{q_max_export}{Optional numeric export ceiling used by the tool (e.g. 0.5),
stored as metadata for truncation-aware diagnostics.}

\item{unit}{Character. Unit metadata stored in the returned object.}

\item{scope}{Character. Scope metadata stored in the returned object.}

\item{q_source}{Character. Source label stored in metadata.}
}
\value{
A \code{dfdr_tbl} with one row per precursor, where \code{q} equals the minimum
of the run-wise q-values across runs for that precursor. The returned object is
intended for diagnostics (e.g. scope disagreement), not as a recommended FDR procedure.
}
\description{
Constructs a precursor-level table by taking the minimum run-wise q-value
across runs for each \code{Precursor.Id}. This mirrors a common scope misuse
(aggregating run-wise q-values into a single global list) and is useful for
scope-disagreement diagnostics.
}
\examples{
library(tibble)

rep <- tibble(
  Run = c("r1", "r2", "r1", "r2"),
  Precursor.Id = c("P1", "P1", "P2", "P2"),
  Decoy = c(0L, 0L, 0L, 1L),
  Q.Value = c(0.02, 0.01, 0.05, 0.04),
  PEP = c(0.03, 0.02, 0.1, 0.9)
)

x <- diann_global_minrunq(rep, run_col = "Run", q_col = "Q.Value", q_max_export = 0.5)
x

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pvalue_helpers.R
\name{.check_has_cols}
\alias{.check_has_cols}
\title{Internal: assert required columns exist}
\usage{
.check_has_cols(x, cols)
}
\arguments{
\item{x}{A data.frame / tibble.}

\item{cols}{Character vector of required column names.}
}
\value{
Returns \code{invisible(TRUE)} if all columns are present; otherwise throws an error.
}
\description{
Checks that a data frame contains the required column names.
}
\keyword{internal}
